/**
 * The Hotloader is capable of loading *all* scenes at once,
 * favoring a single, optimized loading period over several smaller ones at scene-transition time.
 * Note that the Titlescreen scene is not loaded by this class.
 */
function HotLoader() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Resolve function for loading promise.
    var loadResolve;
    // Reject function for loading promise.
    var loadReject;

    // ID of the div containing the hot load in progress warning.
    var hotLoadWarningId = "hotloadwarning";
    var hotLoadProgressId = "hotloadprogress";
    var hotLoadProgressNumberId = "hotloadprogressnumber";
    var compiledFileLocation = "scenes/compiled/AllScenes_compiled.js";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Stores the resolve and reject functions for use later.
     * @param {function} resolve Load resolve function.
     * @param {function} reject Load reject function.
     */
    function storeLoadResolve(resolve, reject) {
        loadResolve = function () {
            $(`#${hotLoadWarningId}`).hide(400);
            $(`#${hotLoadProgressId}`).hide(400);
            resolve();
        };
        loadReject = function (reason) {
            $(`#${hotLoadWarningId}`).hide(400);
            $(`#${hotLoadProgressId}`).hide(400);
            reject(reason);
        };
    }

    /**
     * Updates the UI when download progress happens.
     * @param {Event} evt The event triggered by the XHTML request.
     */
    function compiledHotLoadingProgressUpdate(evt) {
        if (evt.lengthComputable) {
            var percentComplete = Math.round((evt.loaded / evt.total) * 100);
            $(`#${hotLoadProgressNumberId}`).text(percentComplete);
        }
    }

    /**
     * Creates an XMLHttpRequest with a registered listener that will listen to changes in download progress and update the UI accordingly.
     */
    function compiledHotLoadingProgressListen() {
        var xhr = new window.XMLHttpRequest();
        xhr.addEventListener("progress", compiledHotLoadingProgressUpdate);
        return xhr;
    }

    /**
     * Signals something fucked up.
     * Should be called when loading fails.
     */
    function hotLoadFailure() {
        loadReject(`Failed to load scenes. Perhaps there's something wrong with your internet connection? Clear your browser cache and try again.`);
    }

    /**
     * Hotloads the compiled version of all scenes.
     * @param {string} compiledFile A path to the compiled file.
     */
    function compiledHotLoading(compiledFile) {
        $.ajax({
            url: compiledFile,
            dataType: "script",
            xhr: compiledHotLoadingProgressListen
        }).done(loadResolve).fail(hotLoadFailure);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.loaded = new Promise(storeLoadResolve);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Hotloads all the required game files asynchronously by adding references to them to the DOM.
     */
    this.hotLoad = function () {
        $(`#${hotLoadWarningId}`).show(400);
        $(`#${hotLoadProgressId}`).show(400);
        compiledHotLoading(compiledFileLocation);
    };

    /**
     * Performs regular loading.
     */
    this.regularLoad = function () {
        loadResolve();
    };

    /**
     * Performs regular or hotloading depending on the settings.
     */
    this.hotLoadIfRequired = function () {
        if (global.settings.getHotLoad()) {
            that.hotLoad();
        } else {
            that.regularLoad();
        }
    };
}

