/**
 * Backwards-compatible theme loader.
 * @param {boolean} careAboutSettings If true, this ThemeLoader will care about settings in use.
 */
function ThemeLoader(careAboutSettings) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // All known themes.
    var themes = new Themes(careAboutSettings);
    // Generator to turn Theme objects into CSS.
    var generator = new ThemeGenerator();
    // What the default theme is called.
    var defaultThemeName = "defaulttheme";

    // Stores loaded themes by name.
    var themesByName = {};

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Given the name of a theme, returns the id for the css tag containing that theme.
     * @param {string} name The name of the scene.
     * @returns {string} The id of the css tag. 
     */
    function nameToId(name) {
        return `ThemeLoader-${name}-Style`;
    }

    /**
     * Adds the passed CSS to the DOM.
     * @param {string} name The name of the scene.
     * @param {string} css The CSS to add.
     */
    function addCss(name, css) {
        var sheet = document.createElement('style');
        sheet.innerHTML = css;
        sheet.id = nameToId(name);
        document.head.appendChild(sheet);
    }

    /**
     * Updates the custom theme.
     */
    function updateCustomTheme() {
        that.unload(defaultThemeName);
        themesByName[defaultThemeName] = true;
        loadCustomTheme();
    }

    /**
     * Registers custom theme settings listener.
     */
    function registerListeners() {
        global.settings.customThemeListener.register(updateCustomTheme);
    }

    /**
     * Loads the custom theme from settings.
     */
    function loadCustomTheme() {
        var defaultTheme = new Theme();
        defaultTheme.load(global.settings.getCustomTheme());
        var css = generator.generate(defaultThemeName, defaultTheme);
        addCss(defaultThemeName, css);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Generates and adds the specified theme if it has not yet been added.
     * @param {string} theme The name of the theme to add.
     */
    this.load = function (theme) {
        if (!theme || themesByName[theme] || !themes[theme]) {
            return;
        }
        themesByName[theme] = true;
        var themeObject = themes[theme];
        var css = generator.generate(theme, themeObject);
        addCss(theme, css);
    };

    /**
     * Unloads the specified theme. Should probably not be used. Ever.
     * @param {string} theme The name of the theme to remove.
     */
    this.unload = function (theme) {
        delete themesByName[theme];
        $("#" + nameToId(theme)).remove();
    };

    /**
     * Initialization function. Only needed if the themeloader should care about settings.
     */
    this.init = function () {
        themes.init();
        themesByName[defaultThemeName] = true; // Disable unnecessary empty theme loading.
        if (careAboutSettings && global.settings.getCustomThemeEnabled()) {
            loadCustomTheme();
            registerListeners();
        }
    };
}
