// res\js\audio\AudioFile.js

/**
 * This class represents a single AudioFile, regardless of whether it has been loaded or not.
 * @param {AudioContext} context The audio context.
 * @param {string} url The URL from which to load the file.
 * @param {GainNode} masterVolumeGain The master volume gain node, connected to the audio output.
 * @param {GainNode} effectsVolumeGain The effects volume gain node, connected to the master gain node.
 * @param {GainNode} backgroundVolumeGain The background volume gain node, connected to the master gain node.
 * @param {int} defaultVolume The default volume for the audio file, from 1 to 100.
 */
function AudioFile(context, url, masterVolumeGain, effectsVolumeGain, backgroundVolumeGain, defaultVolume) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Default playing volume.
    var ownVolume = defaultVolume || 50;
    ownVolume = ownVolume / 100;

    // Whether or not the AudioFile is currently loading.
    var loading = false;
    // Whether or not the AudioFile has successfully loaded and decoded.
    var loaded = false;
    // Whether or not there was an error during the loading of the AudioFile.
    var error = false;

    // Promise that will resolve when loading of the file succeeds.
    var loadPromise;
    // Resolve function for the promise.
    var loadPromiseResolve;
    // Reject function for the promise.
    var loadPromiseReject;

    // The audio buffer formed by decoding the loaded file.
    var audioBuffer;

    // Whether or not the looping audio is playing.
    var looping = false;

    // Whether or not the sound is queued to play a single time when loaded.
    var queuedPlay = false;

    // Whether or not a loop is queued to start when th
    var queuedLoop = false;

    /**
     * An object of all currently playing (not looping) nodes mapped to their id.
     */
    var currentlyPlayingNodes = {};
    var currentlyPlayingMaxId = 0;

    // The current loop node, which can be accessed to stop it.
    var loopNode;

    // The loop node's additional gain, which can be used to fade out the loop.
    var loopGain = context.createGain();
    loopGain.connect(backgroundVolumeGain);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Function called when the audio data has finished decoding.
     * @param {binary} buffer The decoded buffer formed from the audio file.
     */
    function audioDataDecoded(buffer) {
        audioBuffer = buffer;
        loaded = true;
        loading = false;
        loadPromiseResolve();
    }

    /**
     * Called when the audio file is loaded.
     * "this" refers to the XMLHttpRequest used to load the file.
     */
    function fileLoaded() {
        context.decodeAudioData(this.response, audioDataDecoded, loadError);
    }

    /**
     * Function called when audio has failed to load or decode.
     */
    function loadError() {
        error = true;
        loading = false;
        loadPromiseReject();
    }

    /**
     * Creates the promise used to queue loops.
     */
    function createPromise() {
        loadPromise = new Promise(function (resolve, reject) {
            loadPromiseResolve = resolve;
            loadPromiseReject = reject;
        });
        loadPromise.then(processQueues);
    }

    /**
     * Creates and starts an XMLHttpRequest to fetch the audio data.
     */
    function startRequest() {
        var request = new XMLHttpRequest();
        request.open('GET', url);
        request.responseType = 'arraybuffer';
        request.onload = fileLoaded;
        request.onerror = loadError;
        request.send();
    }

    /**
     * Processes both the play and the loop load queue.
     */
    function processQueues() {
        if (queuedPlay) {
            that.play();
        }
        if (queuedLoop) {
            queuedLoop = false;
            startLoop();
        }
    }

    /**
     * Creates a volume gain set to the specified volume.
     * @param {int} volume Volume ranging from 0-1.
     * @returns {GainNode} The gain. Unconnected.
     */
    function createVolumeGain(volume) {
        var gain = context.createGain();
        gain.gain.setValueAtTime(volume, context.currentTime);
        return gain;
    }

    /**
     * Adds an object containing the node and the gain to the object of all currently playing nodes.
     * Also registers the onend function that will remove the node from the object when it finishes.
     * @param {SourceNode} node The node to add.
     * @param {GainNode} gain The gain to add.
     * @returns {int} The id the node was assigned.
     */
    function addToPlayingNodes(node, gain) {
        var id = currentlyPlayingMaxId;
        var object = {node: node, gain: gain};
        currentlyPlayingNodes[id] = object;
        node.onended = function (event) {
            removeFromPlayingNodes(id);
        };
        ++currentlyPlayingMaxId;
        return id;
    }

    /**
     * Removes a node to the object of all currently playing nodes.
     * @param {int} id The id of the node to remove.
     */
    function removeFromPlayingNodes(id) {
        delete currentlyPlayingNodes[id];
    }

    /**
     * Returns true if the playing nodes object contains the passed id.
     * @param {int} id The id to check for.
     * @returns {boolean} True if the playing nodes object contains the passed id.
     */
    function playingNodesHas(id) {
        return currentlyPlayingNodes.hasOwnProperty(id);
    }

    /**
     * Gets the node at the given id and returns it.
     * @param {int} id The id to get.
     * @returns {object} The object containing both the node and the gain, if it exists, or undefined otherwise.
     */
    function playingNodesGet(id) {
        return currentlyPlayingNodes[id];
    }

    /**
     * Starts looping the audio.
     * Always starts with a fade.
     */
    function startLoop() {
        var source = context.createBufferSource();
        source.buffer = audioBuffer;
        source.loop = true;
        source.connect(loopGain);
        loopGain.gain.setValueAtTime(0.001, context.currentTime);
        loopGain.gain.exponentialRampToValueAtTime(ownVolume, context.currentTime + that.fadeTime);
        source.start();
        loopNode = source;
        looping = true;
    }

    /**
     * Queues the audio file to start looping when loaded.
     */
    function queueLoop() {
        queuedLoop = true;
    }

    /**
     * Immediately stops an ongoing loop.
     */
    function stopOngoingLoop() {
        loopNode.stop(context.currentTime);
        looping = false;
    }

    /**
     * Cancels a queued audio file loop.
     */
    function stopQueuedLoop() {
        queuedLoop = false;
    }

    /**
     * Fades out an ongoing loop.
     */
    function fadeOngoingLoop() {
        loopGain.gain.exponentialRampToValueAtTime(0.001, context.currentTime + that.fadeTime);
        loopNode.stop(context.currentTime + that.fadeTime);
        looping = false;
    }

    /**
     * Cancels a queued audio file loop.
     */
    function fadeQueuedLoop() {
        stopQueuedLoop();
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // URL the audio file was loaded from. Readonly.
    this.url = url;

    // Time required for looping music to start and stop fading.
    this.fadeTime = 1;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Causes this AudioFile to start loading.
     * Does nothing if already loading or loaded or failed to load.
     */
    this.load = function () {
        if (loading || loaded || error) {
            return;
        }
        loading = true;
        createPromise();
        startRequest();
    };

    /**
     * Plays this audio file a single time.
     * Returns the ID of the playing node.
     * Does nothing if the audio file has not yet finished loading.
     * @returns {int} The id of the playing node, which can be used to stop it.
     */
    this.play = function () {
        if (!loaded) {
            return;
        }
        var volumeGain = createVolumeGain(ownVolume);
        volumeGain.connect(effectsVolumeGain);
        var source = context.createBufferSource();
        source.buffer = audioBuffer;
        source.connect(volumeGain);
        source.start();
        return addToPlayingNodes(source, volumeGain);
    };

    /**
     * Stops the playback referenced by the id, if such a playback exists.
     * @param {int} id The id of the play to stop.
     */
    this.stop = function (id) {
        if (!playingNodesHas(id)) {
            return;
        }
        var object = playingNodesGet(id);
        if (!object) {
            return;
        }
        var node = object.node;
        if (!node) {
            return;
        }
        node.stop(context.currentTime);
    };

    /**
     * Stops all currently playing sounds apart from the loop.
     */
    this.stopAll = function () {
        for (var id in currentlyPlayingNodes) {
            if (currentlyPlayingNodes.hasOwnProperty(id)) {
                that.stop(id);
            }
        }
    };

    /**
     * Fades the playback referenced by the id, if such a playback exists.
     * @param {int} id The id of the play to fade out.
     */
    this.fade = function (id) {
        if (!playingNodesHas(id)) {
            return;
        }
        var object = playingNodesGet(id);
        if (!object) {
            return;
        }
        var node = object.node;
        var gain = object.gain;
        if (!node || !gain) {
            return;
        }
        gain.gain.exponentialRampToValueAtTime(0.001, context.currentTime + that.fadeTime);
        node.stop(context.currentTime + that.fadeTime);
    };

    /**
     * Fades all currently playing sounds apart from the loop.
     */
    this.fadeAll = function () {
        for (var id in currentlyPlayingNodes) {
            if (currentlyPlayingNodes.hasOwnProperty(id)) {
                that.fade(id);
            }
        }
    };

    /**
     * Plays this audio file a single time after it has loaded, or immediately if it has already loaded.
     */
    this.playWhenLoaded = function () {
        if (loaded) {
            return that.play();
        }
        queuedPlay = true;
        return currentlyPlayingMaxId;
    };

    /**
     * Cancels playing an audio file when loaded.
     * Can only be cancelled before the file has finished loaded.
     * If the file has already loaded, this function has no effect.
     */
    this.cancelPlayWhenLoaded = function () {
        queuedPlay = false;
    };

    /**
     * Plays this audio file over and over again.
     * Does nothing if the audio file is already looping.
     * Waits for the audio to load before attempting to play it.
     */
    this.loop = function () {
        if (error || looping || queuedLoop) {
            return;
        }
        if (loaded) {
            startLoop();
        } else {
            queueLoop();
        }
    };

    /**
     * Immediately stops this looping audio file.
     * Does nothing if the audio file is not looping.
     * Prevents audio files that have not yet loaded, but have already received the instruction to loop, from looping.
     * Has no effect on audio started by the play() function.
     */
    this.stopLoop = function () {
        if (error || (!queuedLoop && !looping)) {
            return;
        }
        if (loaded) {
            stopOngoingLoop();
        } else {
            stopQueuedLoop();
        }
    };

    /**
     * Fades out this looping audio file.
     * Does nothing if the audio file is not looping.
     * Prevents audio files that have not yet loaded, but have already received the instruction to loop, from looping.
     * Has no effect on audio started by the play() function.
     */
    this.fadeLoop = function () {
        if (error || (!queuedLoop && !looping)) {
            return;
        }
        if (loaded) {
            fadeOngoingLoop();
        } else {
            fadeQueuedLoop();
        }
    };
}

// END res\js\audio\AudioFile.js

// res\js\audio\AudioManager.js

/**
 * Manages loading and playing of audio elements.
 * Its init() function should be called immediately upon successful project load.
 */
function AudioManager() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Object of all indices which are currently looping.
    var loopingIndices = {};

    // Object of all indices which may be currently playing.
    var playingIndices = {};

    // All currently in use indices referenced by URL.
    var urlToIndex = {};

    // Context used to play loaded audio.
    var context;

    // A gain which will always be the last chain in the audio link to which all other sounds connect.
    var masterVolumeGain;

    // A gain which is tuned to the effects volume level, and linked to the master volume gain.
    var effectsVolumeGain;

    // A gain which is tuned to the background music volume level, and linked to the master volume gain.
    var backgroundVolumeGain;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets the passed gain to the given volume.
     * Does nothing if the gain does not exist.
     * @param {GainNode} gain The gain to set the volume of.
     * @param {int} volume The volume, ranging from 0 to 100.
     */
    function setVolume(gain, volume) {
        if (!gain) {
            return;
        }
        volume = volume / 100;
        gain.gain.setValueAtTime(volume, context.currentTime);
    }

    /**
     * Sets the master volume to the given amount.
     * @param {int} volume The master volume, ranging from 0 to 100.
     */
    function setMasterVolume(volume) {
        setVolume(masterVolumeGain, volume);
    }

    /**
     * Sets the effects volume to the given amount.
     * @param {int} volume The effects volume, ranging from 0 to 100.
     */
    function setEffectsVolume(volume) {
        setVolume(effectsVolumeGain, volume);
    }

    /**
     * Sets the background volume to the given amount.
     * @param {int} volume The background volume, ranging from 0 to 100.
     */
    function setBackgroundVolume(volume) {
        setVolume(backgroundVolumeGain, volume);
    }

    /**
     * Adds a workaround to a shitty google autoplay policy decision.
     * See https://developers.google.com/web/updates/2017/09/autoplay-policy-changes#webaudio
     */
    function googleWorkaround() {
        if (!context || !that.audioEnabled) {
            return;
        }

        var resumeAudio = function () {
            if (context.state === "suspended") {
                context.resume();
            }
            document.removeEventListener("click", resumeAudio);
        };

        document.addEventListener("click", resumeAudio);
    }

    /**
     * Creates the audio context, if audio is enabled.
     * Disables audio and logs a warning to the console if the web audio api is not supported.
     * May cause a warning message which should be ignored. (Thanks, Google. Fuck you.)
     */
    function createContext() {
        if (!that.audioEnabled) {
            return;
        }

        try {
            window.AudioContext = window.AudioContext || window.webkitAudioContext;
            context = new AudioContext();
        } catch (e) {
            console.log('Web Audio API is not supported in this browser');
            that.audioEnabled = false;
        }

        googleWorkaround();
    }

    /**
     * Creates the master volume gain, and connects it to the output.
     * Does nothing if there is no context or if audio is disabled.
     */
    function createMasterVolumeGain() {
        if (!context || !that.audioEnabled) {
            return;
        }
        masterVolumeGain = context.createGain();
        setMasterVolume(global.settings.getMasterVolume());
        masterVolumeGain.connect(context.destination);
    }

    /**
     * Creates the effects volume gain, and connects it to the master volume gain.
     * Does nothing if there is no context or if audio is disabled.
     */
    function createEffectsVolumeGain() {
        if (!context || !that.audioEnabled) {
            return;
        }
        effectsVolumeGain = context.createGain();
        setEffectsVolume(global.settings.getEffectsVolume());
        effectsVolumeGain.connect(masterVolumeGain);
    }

    /**
     * Creates the background music volume gain, and connects it to the output.
     * Does nothing if there is no context or if audio is disabled.
     */
    function createBackgroundVolumeGain() {
        if (!context || !that.audioEnabled) {
            return;
        }
        backgroundVolumeGain = context.createGain();
        setBackgroundVolume(global.settings.getBackgroundVolume());
        backgroundVolumeGain.connect(masterVolumeGain);
    }

    /**
     * Creats and loads an AudioFile with the given URL.
     * @param {string} url URL of the audio file to load.
     * @param {int} defaultVolume The default volume for the audio file, from 1 to 100.
     * @returns {AudioFile} The AudioFile, after it has started (but not necessarily completed) loading.
     */
    function loadAudioFile(url, defaultVolume) {
        var file = new AudioFile(context, url, masterVolumeGain, effectsVolumeGain, backgroundVolumeGain, defaultVolume);
        file.load();
        return file;
    }

    /**
     * Adds an AudioFile to the audioFiles array, adds a mapping from the url to the index in the urlToIndex object, and then returns the index.
     * @param {AudioFile} file The AudioFile to add.
     * @returns {int} The index of the AudioFile in the audioFiles array.
     */
    function addAudioFile(file) {
        var index = that.audioFiles.length;
        that.audioFiles.push(file);
        urlToIndex[file.url] = index;
        return index;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // Array of all loaded (or load-in-progress) audio files.
    this.audioFiles = [];

    // Whether or not audio is enabled at all.
    this.audioEnabled = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the audio manager.
     */
    this.init = function () {
        that.audioEnabled = global.settings.getAudioEnabled();
        createContext();
        createMasterVolumeGain();
        createEffectsVolumeGain();
        createBackgroundVolumeGain();
        global.settings.masterVolumeListener.register(setMasterVolume);
        global.settings.effectsVolumeListener.register(setEffectsVolume);
        global.settings.backgroundVolumeListener.register(setBackgroundVolume);
    };

    /**
     * Asynchronously loads the audio file referenced by the given URL.
     * @param {string} url URL of the audio file to load.
     * @param {int} defaultVolume The default volume for the audio file, from 1 to 100.
     * @returns {int} An index to the sound for use in all further audio operations, or -1 if audio playing is disabled.
     *                This index may also be used to directly access the AudioFile, although all looping operations MUST happen via the AudioManager.
     */
    this.load = function (url, defaultVolume) {
        if (!that.audioEnabled) {
            return -1;
        }
        if (urlToIndex.hasOwnProperty(url)) {
            return urlToIndex[url];
        }
        return addAudioFile(loadAudioFile(url, defaultVolume));
    };

    /**
     * Plays the audio file referenced by the index a single time.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * Does nothing if the audio has not yet loaded.
     * @param {int} index A sound-index obtained by the load() function.
     * @returns {int} An index which can be used to refer to the audio track later. -1 if audio is disabled.
     */
    this.play = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return -1;
        }
        playingIndices[index] = true;
        return that.audioFiles[index].play();
    };

    /**
     * Plays the audio file referenced by the index a single time after it is loaded.
     * Plays instantly if the audio file has already loaded.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * @param {int} index A sound-index obtained by the load() function.
     * @returns {int} An index which can be used to refer to the audio track later. -1 if audio is disabled.
     */
    this.playWhenLoaded = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return -1;
        }
        playingIndices[index] = true;
        return that.audioFiles[index].playWhenLoaded();
    };

    /**
     * Stops a single playback of the given index.
     * @param {int} index Index of the audio file, returned on creation.
     * @param {int} track Index of the playback, returned on playing.
     */
    this.stop = function (index, track) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        that.audioFiles[index].stop(track);
    };

    /**
     * Fades a single playback of the given index.
     * @param {int} index Index of the audio file, returned on creation.
     * @param {int} track Index of the playback, returned on playing.
     */
    this.fade = function (index, track) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        that.audioFiles[index].fade(track);
    };

    /**
     * Cancels playing the audio file referenced by the index when loaded.
     * Can only be cancelled before the file has finished loaded.
     * If the file has already loaded, this function has no effect.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * @param {int} index A sound-index obtained by the load() function.
     */
    this.cancelPlayWhenLoaded = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        that.audioFiles[index].cancelPlayWhenLoaded();
    };

    /**
     * Plays the audio file referenced by the index over and over again.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * Does nothing if the audio file is already looping.
     * Waits for the audio to load before attempting to play it.
     * @param {int} index A sound-index obtained by the load() function.
     */
    this.loop = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        loopingIndices[index] = true;
        that.audioFiles[index].loop();
    };

    /**
     * Immediately stops the looping audio file referenced by the index.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * Does nothing if the audio file is not looping.
     * Has no effect on audio started by the play() function.
     * @param {int} index A sound-index obtained by the load() function.
     */
    this.stopLoop = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        delete loopingIndices[index];
        that.audioFiles[index].stopLoop();
    };

    /**
     * Smoothly fades out the looping audio file referenced by the index.
     * Does nothing if the index is out of bounds (such as if audio is disabled entirely).
     * Does nothing if the audio file is not looping.
     * Has no effect on audio started by the play() function.
     * @param {int} index A sound-index obtained by the load() function.
     */
    this.fadeLoop = function (index) {
        if (index < 0 || index >= that.audioFiles.length) {
            return;
        }
        delete loopingIndices[index];
        that.audioFiles[index].fadeLoop();
    };

    /**
     * Fades out all looping audio currently playing.
     */
    this.fadeAllLoopingAudio = function () {
        for (var index in loopingIndices) {
            if (loopingIndices.hasOwnProperty(index)) {
                that.fadeLoop(index);
            }
        }
        loopingIndices = {};
    };

    /**
     * Fades out all audio currently playing, looping or otherwise.
     */
    this.fadeAllAudio = function () {
        that.fadeAllLoopingAudio();
        for (var index in playingIndices) {
            if (playingIndices.hasOwnProperty(index)) {
                that.audioFiles[index].fadeAll();
            }
        }
        playingIndices = {};
    };
}

// END res\js\audio\AudioManager.js

// res\js\character\body\Body.js

/**
 * The Body class holds information relevant to the kobold's body,
 * including breast size, dick size and whether or not the kobold has a cunt.
 */
function Body() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Revises the kobold's gender based on whether or not he/she has a dick,
     * has a cunt and/or has breasts.
     */
    function reviseGender() {
        that.gender = GenderEnum.NONE;

        if (that.dick) {
            that.gender = GenderEnum.MALE;
        }

        if (that.cunt) {
            if (global.settings.getFlatChestedFemale()) {
                that.gender = GenderEnum.FEMALE;
            } else {
                that.gender = GenderEnum.CUNTBOY;
            }
        }

        if (that.dick && that.cunt) {
            that.gender = GenderEnum.MALEHERM;
        }

        if (that.cunt && that.breasts) {
            that.gender = GenderEnum.FEMALE;
        }

        if (that.dick && that.breasts) {
            that.gender = GenderEnum.SHEMALE;
        }

        if (that.dick && that.cunt && that.breasts) {
            that.gender = GenderEnum.HERMAPHRODITE;
        }
    }

    /**
     * Revises the kobold's features based on his/her gender.
     */
    function reviseFeatures() {
        if (that.gender == GenderEnum.MALE) {
            if (!that.dick) {
                that.dick = SizeEnum.SMALLEST;
            }
            that.breasts = SizeEnum.NONE;
            that.cunt = false;
            that.lactating = false;
        }

        if (that.gender == GenderEnum.FEMALE) {
            if (!that.cunt) {
                that.cunt = true;
            }
            if (!that.breasts) {
                that.breasts = SizeEnum.SMALLEST;
            }
            that.dick = SizeEnum.NONE;
        }

        if (that.gender == GenderEnum.SHEMALE) {
            if (!that.dick) {
                that.dick = SizeEnum.SMALLEST;
            }
            if (!that.breasts) {
                that.breasts = SizeEnum.SMALLEST;
            }
            that.cunt = false;
        }

        if (that.gender == GenderEnum.HERMAPHRODITE) {
            if (!that.dick) {
                that.dick = SizeEnum.SMALLEST;
            }
            if (!that.breasts) {
                that.breasts = SizeEnum.SMALLEST;
            }
            if (!that.cunt) {
                that.cunt = true;
            }
        }

        if (that.gender == GenderEnum.CUNTBOY) {
            if (!that.cunt) {
                that.cunt = true;
            }
            that.breasts = SizeEnum.NONE;
            that.cunt = false;
            that.lactating = false;
        }

        if (that.gender == GenderEnum.MALEHERM) {
            if (!that.dick) {
                that.dick = SizeEnum.SMALLEST;
            }
            if (!that.cunt) {
                that.cunt = true;
            }
            that.breasts = SizeEnum.NONE;
            that.lactating = false;
        }

        if (that.gender == GenderEnum.NONE) {
            that.dick = SizeEnum.NONE();
            that.breasts = SizeEnum.NONE();
            that.cunt = false;
            that.lactating = false;
        }
    }

    /**
     * Makes sure the passed size is valid.
     * @param {SizeEnum} size The size to make valid.
     * @returns {SizeEnum} The size, made valid. Guaranteed to be greater than or equal to SizeEnum.NONE and smaller than or equal to SizeEnum.HUGE.
     */
    function makeSizeValid(size) {
        var validSize = size || SizeEnum.NONE;
        validSize = Math.min(validSize, SizeEnum.HUGE);
        validSize = Math.max(validSize, SizeEnum.NONE);
        return validSize;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // The gender of the kobold's body.
    this.gender = GenderEnum.NONE;
    // Whether or not the kobold has a dick.
    this.dick = SizeEnum.NONE;
    // Whether or not the kobold has breasts.
    this.breasts = SizeEnum.NONE;
    this.lactating = false;
    // Whether or not the kobold has a cunt.
    this.cunt = false;

    this.analVirgin = true;
    this.vaginalVirgin = true;
    this.oralVirgin = true;
    this.cockslitVirgin = true;
    this.dickVirgin = true;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Returns the name of the kobold's gender.
     * @returns {string} The name of the kobold's gender.
     */
    this.getGenderName = function () {
        return GenderEnum.properties[that.gender].name;
    };

    /**
     * Returns the path to the icon of the kobold's gender.
     * @returns {string} The path to the icon of the kobold's gender.
     */
    this.getGenderIcon = function () {
        return GenderEnum.properties[that.gender].icon;
    };

    // TODO: pronouns and shit.

    /**
     * Grows the kobold's dick to the specified size and revises the kobold's 
     * gender if necessary.
     * @param {SizeEnum} size How big the kobold's (new) dick should be.
     */
    this.growDick = function (size) {
        size = makeSizeValid(size);
        that.dick = size;
        reviseGender();
    };

    /**
     * Grows the kobold's breasts to the specified size and revises the kobold's 
     * gender if necessary.
     * @param {SizeEnum} size How big the kobold's (new) breasts should be.
     */
    this.growBreasts = function (size) {
        size = makeSizeValid(size);
        that.breasts = size;
        reviseGender();
    };

    /**
     * Gives the kobold a cunt and revises the kobold's gender if necessary.
     */
    this.growCunt = function () {
        this.cunt = true;
        reviseGender();
    };

    /*
     * Removes the kobold's dick and revises its gender.
     */
    this.removeDick = function () {
        this.dick = SizeEnum.NONE;
        reviseGender();
    };

    /*
     * Removes the kobold's breasts and revises its gender.
     */
    this.removeBreasts = function () {
        this.breasts = SizeEnum.NONE;
        that.lactating = false;
        reviseGender();
    };

    /*
     * Removes the kobold's cunt and revises its gender.
     */
    this.removeCunt = function () {
        this.cunt = false;
        reviseGender();
    };

    /*
     * Sets the kobold's gender and revises his/her body features.
     * Any new body features will be of the smallest kind.
     * @param {GenderEnum} newGender The kobold's new gender.
     */
    this.setGender = function (newGender) {
        this.gender = newGender;
        reviseFeatures();
    };

    /**
     * Returns a value greater than 0 indicating the size of the dick if this gender has a dick, 0 otherwise.
     * @returns {boolean} A value greater than 0 indicating the size of the dick if this gender has a dick, 0 otherwise.
     */
    this.hasDick = function () {
        return that.dick;
    };

    /**
     * Returns a value greater than 0 indicating the size of the breasts if this gender has breasts, 0 otherwise.
     * @returns {boolean} A value greater than 0 indicating the size of the breasts if this gender has breasts, 0 otherwise.
     */
    this.hasBreasts = function () {
        return that.breasts;
    };

    /**
     * Returns true if this gender has a cunt, false otherwise.
     * @returns {boolean} True if this gender has a cunt, false otherwise.
     */
    this.hasCunt = function () {
        return that.cunt;
    };

    /**
     * Causes the kobold to lactate.
     */
    this.lactate = function () {
        that.lactating = true;
    };

    /**
     * Causes the kobold to stop lactating.
     */
    this.stopLactation = function () {
        that.lactating = false;
    };

    /**
     * Returns true if the kobold is a giving, anal, oral, cockslit and vaginal virgin and false otherwise.
     * @returns {boolean} True if the kobold is a giving, anal, oral, cockslit and vaginal virgin, and false otherwise.
     */
    this.virgin = function () {
        return that.analVirgin && that.vaginalVirgin && that.oralVirgin && that.cockslitVirgin && that.dickVirgin;
    };

    /**
     * Returns whether the kobold is, at heart, a boy or a girl.
     * @returns {GenderEnum} GenderEnum.MALE if the kobold is a boy, GenderEnum.FEMALE if the kobold is a girl.
     */
    this.getBaseGender = function () {
        return GenderEnum.properties[that.gender].baseGender;
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Body-stripped} body A Body object, stripped of private members and all functions.
     */
    this.load = function (body) {
        that.gender = body.gender;
        that.dick = body.dick;
        that.breasts = body.breasts;
        that.lactating = body.lactating;
        that.cunt = body.cunt;
        that.analVirgin = body.analVirgin;
        that.vaginalVirgin = body.vaginalVirgin;
        that.oralVirgin = body.oralVirgin;
        that.cockslitVirgin = body.cockslitVirgin;
        that.dickVirgin = body.dickVirgin;
    };
}

// END res\js\character\body\Body.js

// res\js\character\body\GenderEnum.js

/**
 * Enum representing all kinds of genders.
 */
var GenderEnum = {
    // Guaranteed support.
    MALE: 1,
    FEMALE: 2,
    // Optional additions.
    SHEMALE: 3,
    HERMAPHRODITE: 4,
    CUNTBOY: 5,
    // Probably unsupported.
    MALEHERM: 6,
    NONE: 7
};

GenderEnum.properties = {
    1: {symbol: "M", icon:"res/img/genders/male.png", name: "Male", baseGender: GenderEnum.MALE},
    2: {symbol: "F", icon:"res/img/genders/female.png", name: "Female", baseGender: GenderEnum.FEMALE},
    3: {symbol: "S", icon:"res/img/genders/shemale.png", name: "Shemale", baseGender: GenderEnum.FEMALE},
    4: {symbol: "H", icon:"res/img/genders/herm.png", name: "Hermaphrodite", baseGender: GenderEnum.FEMALE},
    5: {symbol: "C", icon:"res/img/genders/cuntboy.png", name: "Cuntboy", baseGender: GenderEnum.MALE},
    6: {symbol: "W", icon:"res/img/genders/mherm.png", name: "Male hermaphrodite", baseGender: GenderEnum.MALE},
    7: {symbol: "N", icon:"res/img/genders/none.png", name: "None", baseGender: GenderEnum.MALE}
};

// END res\js\character\body\GenderEnum.js

// res\js\character\body\SizeEnum.js

/**
 * The SizeEnum is capable of expressing the size of various parts of the kobold's body.
 */
var SizeEnum = {
    NONE: 0,
    SMALLEST: 1,
    SMALL: 2,
    AVERAGE: 3,
    LARGE: 4,
    LARGER: 5,
    HUGE: 6
};

SizeEnum.properties = {
    
};

// END res\js\character\body\SizeEnum.js

// res\js\character\Character.js

/**
 * The Character class represents the main character of the game.
 */
function Character() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.name = "Moe";
    this.body = new Body();
    this.stats = new Stats();
    this.status = new Status();
    this.history = new History();
    this.inventory = new Inventory();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Character-stripped} kobold A Character object, stripped of private members and all functions.
     */
    this.load = function (kobold) {
        that.name = kobold.name;
        that.body.load(kobold.body);
        that.stats.load(kobold.stats);
        that.status.load(kobold.status);
        that.history.load(kobold.history);
        that.inventory.load(kobold.inventory);
    };
}

// END res\js\character\Character.js

// res\js\character\history\consumable\ConsumableHistory.js

/**
 * Contains a list of all the consumables the kobold has used, and how many times it has used them.
 */
function ConsumableHistory() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // Consumable id : how much of that item has been consumed
    this.consumed = {

    };

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Logs the usage of a consumable with the passed item id.
     * @param {int} id The consumed item's id.
     */
    this.use = function (id) {
        that.consumed[id] = that.consumed[id] ? that.consumed[id] : 0;
        that.consumed[id]++;
    };

    this.load = function (consumableHistory) {
        if (consumableHistory && consumableHistory.consumed) {
            that.consumed = consumableHistory.consumed;
        } else {
            that.consumed = {};
        }
    };
}

// END res\js\character\history\consumable\ConsumableHistory.js

// res\js\character\history\History.js

/**
 * The History class represents a kobold's history.
 * In essence, it's a key-value store to be used by scenes to store
 * values at the time of reading, and to log choices made.
 */
function History() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // Scene-specific storage.
    this.scenes = {};
    // Shared storage.
    this.shared = new SharedHistory();
    // The ids of all scenes seen so far, in order.
    this.sceneStack = [];
    // History of consumable consumption.
    this.consumable = new ConsumableHistory();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Adds the scene with the passed id to the scene stack.
     * Should be called during the load function of all scenes.
     * @param {string} id The id of the scene.
     */
    this.addToSceneStack = function (id) {
        if (that.sceneStack.length < 1) {
            that.sceneStack.push(id);
            return;
        }
        var lastScene = that.sceneStack[that.sceneStack.length - 1];
        if (lastScene === id) {
            return;
        }
        that.sceneStack.push(id);
    };

    /**
     * Checks if the scene stack contains the scene with the passed id.
     * @param {string} id The id of the scene.
     * @returns {boolean} True if the scene stack contains the scene with the passed id, false otherwise.
     */
    this.sceneStackHas = function (id) {
        return that.sceneStack.includes(id);
    };

    /**
     * Returns true if the shared storage has the specified key, false otherwise.
     * @param {string} key They key to look for.
     * @returns {boolean} True if the key exists, false otherwise.
     */
    this.hasShared = function (key) {
        return that.shared.hasOwnProperty(key);
    };

    /**
     * Gets the shared storage.
     * @returns {object} Shared scene storage.
     */
    this.getShared = function () {
        return that.shared;
    };

    /**
     * Sets the shared storage object.
     * @param {object} newStorage The new storage object.
     */
    this.setShared = function (newStorage) {
        that.shared = newStorage;
    };

    /**
     * Returns true if the passed scene has a history object, false otherwise.
     * @param {string} sceneName The name of the scene to check the history for.
     */
    this.exists = function (sceneName) {
        return that.scenes.hasOwnProperty(sceneName);
    };

    /**
     * Returns whether or not the passed scene's history object has the specified key.
     * @param {string} sceneName The scene of which to check the history.
     * @param {string} key The key to look for.
     */
    this.has = function (sceneName, key) {
        return that.exists(sceneName) && that.scenes[sceneName].hasOwnProperty(key);
    };

    /**
     * Fetches the history object of the passed scene and returns it.
     * Returns an empty object if the scene does not have a history object associated with it.
     * @param {string} sceneName The name of the scene to get the history for.
     */
    this.get = function (sceneName) {
        if (that.exists(sceneName)) {
            return that.scenes[sceneName];
        }
        return {};
    };

    /**
     * Gets a specified key of a scene.
     * @param {string} sceneName The scene name for which to get the key.
     * @param {string} key The key to get.
     * @returns {object} The resulting object or undefined if the key has not yet been set.
     */
    this.getKey = function (sceneName, key) {
        return that.get(sceneName)[key];
    };

    /**
     * Sets the history object for the passed scene.
     * @param {string} sceneName The name of the scene to set the history for.
     * @param {object} history The new history object.
     */
    this.set = function (sceneName, history) {
        that.scenes[sceneName] = history;
    };

    /**
     * Sets a specified key of a scene.
     * @param {string} sceneName The scene name for which to set the key.
     * @param {string} key The key to set.
     * @param {object} setTo What to set the key to.
     */
    this.setKey = function (sceneName, key, setTo) {
        var history = that.get(sceneName);
        history[key] = setTo;
        that.set(sceneName, history);
    };

    /**
     * Unsets a specified key of a scene.
     * @param {string} sceneName The scene name for which to set the key.
     * @param {string} key The key to set.
     */
    this.unsetKey = function (sceneName, key) {
        var history = that.get(sceneName);
        delete history[key];
        that.set(sceneName, history);
    };

    /**
     * Performs the function if the specified key is not yet set,
     * then sets the key to setTo. Does nothing if the key is already set.
     * @param {string} sceneName The name of the scene.
     * @param {string} key The key to check for.
     * @param {anything} setTo What to set the key to after performing the function.
     * @param {type} fun The function that will be performed should the key not be set yet.
     */
    this.doIfNotSet = function (sceneName, key, setTo, fun) {
        if (!that.has(sceneName, key)) {
            fun();
            var history = that.get(sceneName);
            history[key] = setTo;
            global.kobold.history.set(sceneName, history);
        }
    };

    /**
     * Sets the specified key to setTo if not yet set.
     * @param {string} sceneName The name of the scene.
     * @param {string} key The key to check for.
     * @param {anything} setTo What to set the key to if not yet set.
     */
    this.setIfNotSet = function (sceneName, key, setTo) {
        that.doIfNotSet(sceneName, key, setTo, function () {});
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {History-stripped} history A History object, stripped of private members and all functions.
     */
    this.load = function (history) {
        that.scenes = history.scenes;
        that.shared.load(history.shared);
        that.sceneStack = history.sceneStack;
        that.consumable.load(history.consumable);
    };
}

// END res\js\character\history\History.js

// res\js\character\history\shared\caravanbandits\CaravanBandits.js

/**
 * The kobold's relation with the bandits guarding the money transport, 
 * the first encounter in the forest.
 */
function CaravanBandits() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.knowledge = CaravanBanditsKnowledgeEnum.GUARDS;
    
    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {CaravanBandits-stripped} caravanBandits A CaravanBandits object, stripped of private members and all functions.
     */
    this.load = function (caravanBandits) {
        if(!caravanBandits) {
            that.load(new CaravanBandits());
            return;
        } 
        that.knowledge = caravanBandits.knowledge;
    };
}

// END res\js\character\history\shared\caravanbandits\CaravanBandits.js

// res\js\character\history\shared\caravanbandits\CaravanBanditsKnowledgeEnum.js

/**
 * Enum representing the kobold's level of knowledge of the caravan bandits.
 */
var CaravanBanditsKnowledgeEnum = {
    GUARDS: 1,          // The kobold thinks nothing is off about them.
    SUSPICIOUS: 2,      // The kobold is suspicious of their actions.
    BANDITS: 3,         // The kobold knows they're bandits.
    BARONBANDITS: 4     // The kobold knows they're bandits hired by the baron.
};

// END res\js\character\history\shared\caravanbandits\CaravanBanditsKnowledgeEnum.js

// res\js\character\history\shared\cassidy\Cassidy.js

/**
 * The kobold's relation with Cassidy, first encounterable in the hamlet.
 */
function Cassidy() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.dead = false;
    this.capturedby = false;
    this.koboldsfreed = false;
    this.tableraped = false;
    this.agree = false;
    this.sex = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Cassidy-stripped} cassidy A Cassidy object, stripped of private members and all functions.
     */
    this.load = function (cassidy) {
        if (!cassidy) {
            that.load(new Cassidy());
            return;
        }
        that.dead = cassidy.dead;
        that.capturedby = cassidy.capturedby;
        that.koboldsfreed = cassidy.koboldsfreed;
        that.tableraped = cassidy.tableraped;
        that.agree = cassidy.agree;
        that.sex = cassidy.sex;
    };
}

// END res\js\character\history\shared\cassidy\Cassidy.js

// res\js\character\history\shared\debtcollector\DebtCollector.js

/**
 * The debt collector is a fat half-pig half-kobold hybrid the kobold can meet.
 */
function DebtCollector() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    this.status = DebtCollectorEnum.UNMET;
    
    this.paid = 0;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {DragonessPrincess-stripped} dragonessPrincess A DragonessPrincess object, stripped of private members and all functions.
     */
    this.load = function (debtCollector) {
        if (!debtCollector) {
            that.load(new DebtCollector());
            return;
        }
        that.status = debtCollector.status;
        that.paid = debtCollector.paid;
    };
}

// END res\js\character\history\shared\debtcollector\DebtCollector.js

// res\js\character\history\shared\debtcollector\DebtCollectorEnum.js

var DebtCollectorEnum = {
    UNMET: 1,       // Kobold didn't meet the debt collector.
    AMBUSHED: 2,    // Kobold attempted to ambush the debt collector but got knocked out.
    UNPAID: 3,      // Kobold hid money from the debt collector.
    PAID: 4,        // Kobold paid the debt collector.
    REFUSED: 5      // Kobold refused to pay and got beat up.
};

// END res\js\character\history\shared\debtcollector\DebtCollectorEnum.js

// res\js\character\history\shared\deepwoods\DeepwoodsShared.js

/**
 * What the kobold did in the deepwoods.
 */
function DeepwoodsShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.tentaclePlant = new TentaclePlantShared();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A Shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        if (!shared) {
            that.load(new DeepwoodsShared());
            return;
        }
        that.tentaclePlant.load(shared.tentaclePlant);
    };
}

// END res\js\character\history\shared\deepwoods\DeepwoodsShared.js

// res\js\character\history\shared\deepwoods\TentaclePlantShared.js

/**
 * The kobold's interactions with the tentacle plant.
 */
function TentaclePlantShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.encountered = false; // Kobold encountered the tentacle plant.
    this.sucked = false; // Kobold sucked the tentacle plant.
    this.fucked = false; // Kobold was fucked by the tentacle plant.
    this.enjoyed = false; // Kobold enjoyed getting fucked by the tentacle plant.
    this.sapKnowledge = false; // Kobold knows about the tentacle plant's sap.
    this.sapTasted = false; // Kobold tasted the tentacle plant's sap.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A Shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        var defaults = new TentaclePlantShared();
        if (!shared) {
            that.load(new TentaclePlantShared());
            return;
        }
        that.encountered = shared.encountered || defaults.encountered;
        that.sucked = shared.sucked || defaults.sucked;
        that.fucked = shared.fucked || defaults.fucked;
        that.enjoyed = shared.enjoyed || defaults.enjoyed;
        that.sapKnowledge = shared.sapKnowledge || defaults.sapKnowledge;
        that.sapTasted = shared.sapTasted || defaults.sapTasted;
    };
}

// END res\js\character\history\shared\deepwoods\TentaclePlantShared.js

// res\js\character\history\shared\dragonessprincess\DragonessPrincess.js

/**
 * The kobold's relation with the Dragoness Princess, first encounterable in the
 * forest.
 */
function DragonessPrincess() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.relation = DragonessPrincessEnum.UNMET;
    this.savedBy = DragonessPrincessSavedBy.NOTHING;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {DragonessPrincess-stripped} dragonessPrincess A DragonessPrincess object, stripped of private members and all functions.
     */
    this.load = function (dragonessPrincess) {
        if(!dragonessPrincess) {
            that.load(new DragonessPrincess());
            return;
        }
        var defaults = new DragonessPrincess();
        that.relation = dragonessPrincess.relation || defaults.relation;
        that.savedBy = dragonessPrincess.savedBy || defaults.savedBy;
    };
}

// END res\js\character\history\shared\dragonessprincess\DragonessPrincess.js

// res\js\character\history\shared\dragonessprincess\DragonessPrincessEnum.js

/**
 * Enum representing the kobold's relationship with the dragoness princess.
 */
var DragonessPrincessEnum = {
    UNMET: 1,           // Haven't seen her yet.
    UNHELPED: 2,        // Did not help her (but was seen by her).
    HELPED: 3,          // Helped her post-rape.
    SLOPPY_SECONDS: 4,  // Fucked her after the gang did.
    SAVED: 5,           // Saved from the gang prior to rape.
    RAN_AWAY: 6         // Your intervention allowed her to run.
};

// END res\js\character\history\shared\dragonessprincess\DragonessPrincessEnum.js

// res\js\character\history\shared\dragonessprincess\DragonessPrincessSavedBy.js

/**
 * Enum representing the kobold's relationship with the dragoness princess.
 */
var DragonessPrincessSavedBy = {
    NOTHING: 1,
    SCARY_NOISE: 2,
    PAYING_GANG: 3,
    PERSUASION: 4
};

// END res\js\character\history\shared\dragonessprincess\DragonessPrincessSavedBy.js

// res\js\character\history\shared\enslavement\daytwo\EnslavementNoonTwoShared.js

/**
 * What the kobold did during the second noon of enslavement.
 */
function EnslavementNoonTwoShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.blackmailed = false; // Whether or not the kobold was blackmailed with the mouth gag (no recruitment, threat of violence, no protection from the human).
    this.makeUpOffer = false; // Whether or not the kobold was given the make up offer (no recruitment, threat of violence if it doesn't cooperate, human goes easy on it).
    this.simpleOffer = false; // Whether or not the kobold was given the simple offer (recruitment, no violence, no human).
    this.insulted = false; // Whether or not the kobold insulted Fenrir.
    this.attacked = false; // Whether or not the kobold attempted to attack Fenrir.
    this.kissedBalls = false; // Whether or not the kobold ended up kissing the wolf's balls. If true, the offer was accepted.
    this.kissedBallsEager = false; // Whether or not the kobold eagerly kissed the wolf's balls. If true, the offer was accepted.

    this.gagged = false; // Whether or not the kobold was gagged by the wolf.
    this.gagOpened = false; // Whether or not the kobold opened its mouth for the wolf's gag.

    this.badRefuse = false; // Whether or not the kobold chose to refuse to bathe while being a bad kobold, causing Fenrir to nearly attack the kobold.
    this.goodRefuse = false; // Whether or not the kobold chose to refuse to bathe while being a good kobold, causing Fenrir to get chewed out by the caravan master.
    this.badDrink = false; // Whether or not the kobold drank from the trough while being a bad kobold.
    this.goodDrink = false; // Whether or not the kobold drank from the trough while being a good kobold.
    this.badBathe = false; // Whether or not the kobold chose to bathe while being a bad kobold, causing Fenrir to get chewed out by the caravan master.
    this.goodBathe = false; // Whether or not the kobold chose to bathe while being a good kobold, causing Fenrir to leave the kobold to bathe alone.

    this.injected = false; // Whether or not the kobold was injected by the wolf.
    this.tookMPlus = false; // Whether or not the kobold took Mplus (regardless of injected or not).
    this.tookFemPlus = false; // Whether or not the kobold took FemPlus (regardless of injected or not).

    this.stageSmiled = false; // Whether or not the kobold smiled on stage.
    this.stagePosed = false; // Whether or not the koobld posed on stage.

    this.boughtBySeki = false; // Whether or not the kobold ended up being bought by Seki.

    this.complianceRating = 0; // Compliance rating for noon two. A number between one and ten signifying how well the kobold behaved.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Adds the passed number to the current compliance rating.
     * @param {integer} toAdd How much to add (positive) or subtract (negative).
     */
    this.alterComplianceRating = function (toAdd) {
        that.complianceRating = that.complianceRating + toAdd;
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        if (!shared) {
            that.load(new EnslavementNoonTwoShared());
            return;
        }
        that.blackmailed = shared.blackmailed || false;
        that.makeUpOffer = shared.makeUpOffer || false;
        that.simpleOffer = shared.simpleOffer || false;
        that.insulted = shared.insulted || false;
        that.attacked = shared.attacked || false;
        that.kissedBalls = shared.kissedBalls || false;
        that.kissedBallsEager = shared.kissedBallsEager || false;

        that.badRefuse = shared.badRefuse || false;
        that.goodRefuse = shared.goodRefuse || false;
        that.badDrink = shared.badDrink || false;
        that.goodDrink = shared.goodDrink || false;
        that.badBathe = shared.badBathe || false;
        that.goodBathe = shared.goodBathe || false;

        that.gagged = shared.gagged || false;
        that.gagOpened = shared.gagOpened || false;

        that.injected = shared.injected || false;
        that.tookMPlus = shared.tookMPlus || false;
        that.tookFemPlus = shared.tookFemPlus || false;

        that.stageSmiled = shared.stageSmiled || false;
        that.stagePosed = shared.stagePosed || false;
        that.boughtBySeki = shared.boughtBySeki || false;

        that.complianceRating = shared.complianceRating || 0;
    };
}

// END res\js\character\history\shared\enslavement\daytwo\EnslavementNoonTwoShared.js

// res\js\character\history\shared\enslavement\EnslavementNightOneShared.js

/**
 * What the kobold did during the first night of enslavement.
 */
function EnslavementNightOneShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nameStated = false; // Whether or not the kobold stated its name.
    this.knockedOut = false; // Whether or not the kobold was knocked out.
    this.escapedBindings = false; // Whether or not the kobold was knocked out.
    this.complianceRating = 0; // Compliance rating for night one. A number between one and ten signifying how well the kobold behaved.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {EnslavementNightOneShared-stripped} nightOne A EnslavementNightOneShared object, stripped of private members and all functions.
     */
    this.load = function (nightOne) {
        if (!nightOne) {
            that.load(new EnslavementNightOneShared());
            return;
        }
        that.nameStated = nightOne.nameStated || false;
        that.knockedOut = nightOne.knockedOut || false;
        that.escapedBindings = nightOne.escapedBindings || false;
        that.complianceRating = nightOne.complianceRating || 0;
    };
}

// END res\js\character\history\shared\enslavement\EnslavementNightOneShared.js

// res\js\character\history\shared\enslavement\EnslavementShared.js

/**
 * What the kobold did during enslavement.
 */
function EnslavementShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.beesting = false; // Whether or not the kobold was stung by bees in the forest.
    this.captured = false; // Whether or not the kobold was enslaved.
    this.captureFucked = false; // Whether or not the kobold was fucked during capture.
    this.captureEnjoyed = false; // Whether or not the kobold enjoyed getting fucked during capture.
    
    this.nightOne = new EnslavementNightOneShared(); // First night of enslavement: caravan confinement.
    this.noonTwo = new EnslavementNoonTwoShared(); // Second noon of enslavement: hamlet auction.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {EnslavementShared-stripped} enslavement A EnslavementShared object, stripped of private members and all functions.
     */
    this.load = function (enslavement) {
        if (!enslavement) {
            that.load(new EnslavementShared());
            return;
        }
        that.beesting = enslavement.beesting || false;
        that.captured = enslavement.captured || false;
        that.captureFucked = enslavement.captureFucked || false;
        that.captureEnjoyed = enslavement.captureEnjoyed || false;
        that.nightOne.load(enslavement.nightOne);
        that.noonTwo.load(enslavement.noonTwo);
    };
    
    /**
     * Sums up all the separate compliance ratings and returns the results.
     * @returns {int} The total compliance rating.
     */
    this.totalComplianceRating = function() {
        return that.nightOne.complianceRating + that.noonTwo.complianceRating;
    };
}

// END res\js\character\history\shared\enslavement\EnslavementShared.js

// res\js\character\history\shared\farmerhouse\FarmerHouseShared.js

/**
 * What the kobold did in the farmer house.
 */
function FarmerHouseShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.went = false;
    this.failed = false;
    this.stole = false;
    this.raped = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {FarmerHouseShared-stripped} farmerhouse A FarmerHouseShared object, stripped of private members and all functions.
     */
    this.load = function (farmerhouse) {
        if (!farmerhouse) {
            that.load(new FarmerHouseShared());
            return;
        }
        that.went = farmerhouse.went;
        that.failed = farmerhouse.failed;
        that.stole = farmerhouse.stole;
        that.raped = farmerhouse.raped;
    };
}

// END res\js\character\history\shared\farmerhouse\FarmerHouseShared.js

// res\js\character\history\shared\home\HomeStatus.js

/**
 * The status of the Kobold's home.
 */
function HomeStatus() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.status = HomeStatusEnum.UNSAVED;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {HomeStatus-stripped} home A HomeStatus object, stripped of private members and all functions.
     */
    this.load = function (home) {
        if(!home) {
            that.load(new HomeStatus());
            return;
        } 
        that.status = home.status;
    };
}

// END res\js\character\history\shared\home\HomeStatus.js

// res\js\character\history\shared\home\HomeStatusEnum.js

/**
 * Enum representing the status of the kobold's home.
 */
var HomeStatusEnum = {
    UNSAVED: 1,
    SAVED: 2
};

// END res\js\character\history\shared\home\HomeStatusEnum.js

// res\js\character\history\shared\seki\Seki.js

/**
 * The kobold's relation with Seki, first encounterable in the hamlet.
 */
function Seki() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.knowsDying = false;
    this.hamletHadMoney = false;
    this.moneySpent = 0;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {DragonessPrincess-stripped} dragonessPrincess A DragonessPrincess object, stripped of private members and all functions.
     */
    this.load = function (seki) {
        if(!seki) {
            that.load(new Seki());
            return;
        }
        that.knowsDying = seki.knowsDying;
        that.hamletHadMoney = seki.hamletHadMoney;
        that.moneySpent = seki.moneySpent;
    };
}

// END res\js\character\history\shared\seki\Seki.js

// res\js\character\history\shared\SharedHistory.js

/**
 * The SharedHistory class contains the kobold's shared, structured history of 
 * interactions with important characters.
 */
function SharedHistory() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.caravanBandits = new CaravanBandits();
    this.tenPawGang = new TenPawGang();
    this.debtCollector = new DebtCollector();
    this.dragonessPrincess = new DragonessPrincess();
    this.home = new HomeStatus();
    this.seki = new Seki();
    this.cassidy = new Cassidy();
    this.farmerhouse = new FarmerHouseShared();
    this.varanar = new VaranarShared();
    this.enslavement = new EnslavementShared();
    this.well = new HamletWellShared();
    this.deepwoods = new DeepwoodsShared();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {SharedHistory-stripped} sharedHistory A SharedHistory object, stripped of private members and all functions.
     */
    this.load = function (sharedHistory) {
        if (!sharedHistory) {
            that.load(new SharedHistory());
            return;
        }
        that.caravanBandits.load(sharedHistory.caravanBandits);
        that.tenPawGang.load(sharedHistory.tenPawGang);
        that.debtCollector.load(sharedHistory.debtCollector);
        that.dragonessPrincess.load(sharedHistory.dragonessPrincess);
        that.home.load(sharedHistory.home);
        that.seki.load(sharedHistory.seki);
        that.cassidy.load(sharedHistory.cassidy);
        that.farmerhouse.load(sharedHistory.farmerhouse);
        that.varanar.load(sharedHistory.varanar);
        that.enslavement.load(sharedHistory.enslavement);
        that.well.load(sharedHistory.well);
        that.deepwoods.load(sharedHistory.deepwoods);
    };
}

// END res\js\character\history\shared\SharedHistory.js

// res\js\character\history\shared\tenpawgang\TenPawGang.js

/**
 * The kobold's relation with the Ten Paw Gang, a band of five kobolds
 * first encounterable in the forest.
 */
function TenPawGang() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.relation = TenPawGangEnum.UNMET;
    this.footsie = TenPawGangFootsieEnum.NODIFFERENT;
    this.usedRock = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {TenPawGang-stripped} tenPawGang A TenPawGang object, stripped of private members and all functions.
     */
    this.load = function (tenPawGang) {
        if(!tenPawGang) {
            that.load(new TenPawGang());
            return;
        }
        
        that.relation = tenPawGang.relation;
        that.footsie = tenPawGang.footsie;
        that.usedRock = tenPawGang.usedRock;
    };
}

// END res\js\character\history\shared\tenpawgang\TenPawGang.js

// res\js\character\history\shared\tenpawgang\TenPawGangEnum.js

/**
 * Enum representing the kobold's relationship with the Ten Paw Gang.
 */
var TenPawGangEnum = {
    UNMET: 1,           // Haven't seen them yet.
    DEEPLY_OBSERVED: 2, // Have watched them rape dragoness.
    STOLEN_FROM: 3,     // Have stolen from them.
    SPOOKED: 4,         // Have spooked them.
    PAID: 5,            // Paid them to leave the dragoness alone.
    EAGER_THREESOME: 6, // Eagerly sucked and jerked two of them.
    RELUCTANT_THREESOME: 7,  // Reluctantly jerked and got abused by two of them.
    EAGER_GANGBANG: 8,  // Eagerly fucked the entire group.
    RELUCTANT_GANGBANG: 9,  // Hated getting fucked by the entire group.
    GANGRAPE: 10        // Protested and got raped by the leader.
};

// END res\js\character\history\shared\tenpawgang\TenPawGangEnum.js

// res\js\character\history\shared\tenpawgang\TenPawGangFootsieEnum.js

/**
 * Enum representing the kobold's relationship with Footsie.
 */
var TenPawGangFootsieEnum = {
    NODIFFERENT: 1,             // No special interactions.
    OBSERVED: 2,                // Observed his interest in feet.
    MASTURBATION_INDULGED: 3,   // Indulged his mutual masturbation request.
    MASTURBATION_DECLINED: 4    // Denied his mutual masturbation request.
};

// END res\js\character\history\shared\tenpawgang\TenPawGangFootsieEnum.js

// res\js\character\history\shared\varanar\daythree\AfternoonThreeShared.js

/**
 * What the kobold did during the third afternoon of Varanar.
 */
function AfternoonThreeShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    this.massageParlor = false; // Went to the massage parlor.
    this.parlorBargainSuccess = false; // Successfully bargained the price down at the massage parlor.
    this.parlorRan = false; // Ran from the parlor.
    this.parlorFought = false; // Fought Tychus at the parlor.
    this.parlorKilled = false; // Killed Tychus at the parlor.
    this.parlorMassaged = false; // Massaged at the parlor.
    this.parlorArmMassage = false; // Massaged arms at the parlor.
    this.parlorFootMassage = false; // Massaged feet at the parlor.
    this.parlorBackMassage = false; // Massaged back at the parlor.
    this.parlorFreed = false; // Freed kobolds from the parlor. If combined with killed, the kobolds hate Moe.
    this.parlorKnockedOut = false; // Knocked out at the parlor.
    this.parlorInjured = false; // Injured by the post-massage at the parlor.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A Shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        var defaults = new AfternoonThreeShared();
        if (!shared) {
            that.load(defaults);
            return;
        }
        for (var key in that) {
            if (typeof that[key] === "function") {
                continue;
            }
            that[key] = shared[key] || defaults[key];
        }
        for (var key in shared) {
            if (that[key] || !shared[key]) { // Already processed or don't need to process.
                continue;
            }
            that[key] = shared[key];
        }
    };
}

// END res\js\character\history\shared\varanar\daythree\AfternoonThreeShared.js

// res\js\character\history\shared\varanar\daythree\DayThreeShared.js

/**
 * What the kobold did during the third day of Varanar.
 */
function DayThreeShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.morning = new MorningThreeShared();
    this.noon = new NoonThreeShared();
    this.afternoon = new AfternoonThreeShared();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A Shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        if (!shared) {
            that.load(new DayThreeShared());
            return;
        }
        that.morning.load(shared.morning);
        that.noon.load(shared.noon);
        that.afternoon.load(shared.afternoon);
    };
}

// END res\js\character\history\shared\varanar\daythree\DayThreeShared.js

// res\js\character\history\shared\varanar\daythree\MorningThreeShared.js

/**
 * What the kobold did during the third morning of Varanar.
 */
function MorningThreeShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.thievingAttempt = false; // Kobold attempted to steal from main character.
    this.stolenFrom = false; // Main character was stolen from.
    this.eggedKobold = false; // Threw egg at kobold.
    this.declinedSex = false; // Main character declined sex.
    this.fuckedFemaleKobold = false; // Main character had a dick.
    this.fuckedMaleKobold = false; // Main character did not have a dick.
    this.cameInside = false; // Kobold came inside of main character.
    this.oral = false; // Main character did not have a dick and sucked the kobold off.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {MorningThreeShared-stripped} morningThree A MorningThreeShared object, stripped of private members and all functions.
     */
    this.load = function (morningThree) {
        if (!morningThree) {
            that.load(new MorningThreeShared());
            return;
        }
        that.thievingAttempt = morningThree.thievingAttempt || false;
        that.stolenFrom = morningThree.stolenFrom || false;
        that.eggedKobold = morningThree.eggedKobold || false;
        that.declinedSex = morningThree.declinedSex || false;
        that.fuckedFemaleKobold = morningThree.fuckedFemaleKobold || false;
        that.fuckedMaleKobold = morningThree.fuckedMaleKobold || false;
        that.cameInside = morningThree.cameInside || false;
        that.oral = morningThree.oral || false;
    };
}

// END res\js\character\history\shared\varanar\daythree\MorningThreeShared.js

// res\js\character\history\shared\varanar\daythree\NoonThreeShared.js

/**
 * What the kobold did during the third noon of Varanar.
 */
function NoonThreeShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.served = false; // Served at the soup kitchen.
    this.ate = false; // Ate at the soup kitchen.
    this.stole = false; // Stole the sheep's ring.
    this.gaveHouse = false; // Gave the sheep Cassidy's house.
    this.gaveHouseMentioned = false; // Told the sheep about Cassidy's corpse
    this.comforted = false; // Whether or not the kobold fucked the sheep.
    this.donated = 0; // How much gold the kobold donated to the sheep.

    // -------------------- 

    this.moneySpent = 0; // How much gold the kobold spent.
    this.moneyReceived = 0; // How much gold the kobold received.
    this.jerked = false; // Jerked the dragon's dick.
    this.jerkedEager = false; // Liked jerking the dragon's dick.
    this.sucked = false;  // Sucked the dragon's dick.
    this.suckedEager = false; // Liked sucking the dragon's dick.
    this.fucked = false; // Fucked the dragon.
    this.fuckedEager = false; // Liked fucking the dragon.

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Shared-stripped} shared A Shared object, stripped of private members and all functions.
     */
    this.load = function (shared) {
        var defaults = new NoonThreeShared();
        if (!shared) {
            that.load(defaults);
            return;
        }
        that.served = shared.served || defaults.served;
        that.ate = shared.ate || defaults.ate;
        that.stole = shared.stole || defaults.stole;
        that.gaveHouse = shared.gaveHouse || defaults.gaveHouse;
        that.gaveHouseMentioned = shared.gaveHouseMentioned || defaults.gaveHouseMentioned;
        that.comforted = shared.comforted || defaults.comforted;
        that.donated = shared.donated || defaults.donated;


        that.moneySpent = shared.moneySpent || defaults.moneySpent;
        that.moneyReceived = shared.moneyReceived || defaults.moneyReceived;
        that.jerked = shared.moneySpent || defaults.jerked;
        that.jerkedEager = shared.moneySpent || defaults.jerkedEager;
        that.sucked = shared.sucked || defaults.sucked;
        that.suckedEager = shared.moneySpent || defaults.suckedEager;
        that.fucked = shared.moneySpent || defaults.fucked;
        that.fuckedEager = shared.moneySpent || defaults.fuckedEager;
    };
}

// END res\js\character\history\shared\varanar\daythree\NoonThreeShared.js

// res\js\character\history\shared\varanar\NightTwoShared.js

/**
 * What the kobold did during the second night of Varanar.
 */
function NightTwoShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.arrivalEnum = {
        REGULAR: 1,
        RIVER: 2
    };
    this.arrival = that.arrivalEnum.REGULAR;
    this.alleyAttack = false;
    this.alleyPiss = false;
    this.beatUp = false;
    this.pissSlut = false;
    this.fuckSlut = false;
    this.fetishSlut = false;
    this.fought = false;
    this.buttKiss = false;
    this.flaccidOral = false;
    this.forced = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {NightTwoShared-stripped} nightTwo A NightTwoShared object, stripped of private members and all functions.
     */
    this.load = function (nightTwo) {
        if (!nightTwo) {
            that.load(new NightTwoShared());
            return;
        }
        that.arrival = nightTwo.arrival || that.arrivalEnum.REGULAR;
        that.alleyAttack = nightTwo.alleyAttack || false;
        that.alleyPiss = nightTwo.alleyPiss || false;
        that.beatUp = nightTwo.beatUp || false;
        that.pissSlut = nightTwo.pissSlut || false;
        that.fuckSlut = nightTwo.fuckSlut || false;
        that.fetishSlut = nightTwo.fetishSlut || false;
        that.fought = nightTwo.fought || false;
        that.buttKiss = nightTwo.buttKiss || false;
        that.flaccidOral = nightTwo.flaccidOral || false;
        that.flaccidOral = nightTwo.forced || false;
    };
}

// END res\js\character\history\shared\varanar\NightTwoShared.js

// res\js\character\history\shared\varanar\VaranarShared.js

/**
 * What the kobold did in Varanar.
 */
function VaranarShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nightTwo = new NightTwoShared();
    this.dayThree = new DayThreeShared();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {VaranarShared-stripped} varanarShared A VaranarShared object, stripped of private members and all functions.
     */
    this.load = function (varanarShared) {
        if (!varanarShared) {
            that.load(new VaranarShared());
            return;
        }
        that.nightTwo.load(varanarShared.nightTwo);
        that.dayThree.load(varanarShared.dayThree);
    };
}

// END res\js\character\history\shared\varanar\VaranarShared.js

// res\js\character\history\shared\well\HamletWellShared.js

/**
 * What the kobold was up to in the well and the cave.
 */
function HamletWellShared() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.drankWater = false;
    this.drowned = false;
    this.fell = false;

    this.metBoys = false;

    this.scrapedThigh = false;
    this.escapedClimb = false;
    this.escapedRope = false;

    this.basiliskMet = false; // Saw the basilisk
    this.basiliskFought = false; // Fought the basilisk
    this.basiliskKilled = false; // Killed the basilisk
    this.basiliskFriend = false; // Befriended the basilisk

    this.basiliskPetrified = false; // Got petrified by basilisk
    this.basiliskFucked = false; // Got fucked by basilisk
    this.basiliskPenetrated = false; // Got penetrated by basilisk
    this.basiliskEnjoyed = false; // Enjoyed getting fucked by basilisk

    this.basiliskFingerDislocated = false; // Dislocated a finger fighting the basilisk

    this.basiliskDominated = false; // Dominated basilisk
    this.basiliskPeed = false; // Peed on basilisk
    this.basiliskOral = false; // Sucked / licked by basilisk
    this.basiliskDomFucked = false; // Fucked basilisk

    this.basiliskMirror = false; // Petrified the basilisk using a mirror
    this.basiliskRode = false; // Whether or not the basilisk was ridden
    this.basiliskRodeCameInside = false; // Whether or not the basilisk came inside of the kobold after being ridden

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {HamletWellShared-stripped} well A HamletWellShared object, stripped of private members and all functions.
     */
    this.load = function (well) {
        if (!well) {
            that.load(new HamletWellShared());
            return;
        }
        that.drankWater = well.drankWater || false;
        that.drowned = well.drowned || false;
        that.fell = well.fell || false;


        that.metBoys = well.metBoys || false;
        if (!that.metBoys && global.kobold.history.getKey("HamletWell", "help")) {
            that.metBoys = true;
        }

        that.scrapedThigh = well.scrapedThigh || false;
        that.escapedClimb = well.escapedClimb || false;
        that.escapedRope = well.escapedRope || false;

        that.basiliskMet = well.basiliskMet || false;
        that.basiliskFought = well.basiliskFought || false;
        that.basiliskKilled = well.basiliskKilled || false;
        that.basiliskFriend = well.basiliskFriend || false;

        that.basiliskPetrified = well.basiliskPetrified || false;
        that.basiliskFucked = well.basiliskFucked || false;
        that.basiliskPenetrated = well.basiliskPenetrated || false;
        that.basiliskEnjoyed = well.basiliskEnjoyed || false;

        that.basiliskFingerDislocated = well.basiliskFingerDislocated || false;

        that.basiliskDominated = well.basiliskDominated || false;
        that.basiliskPeed = well.basiliskPeed || false;
        that.basiliskOral = well.basiliskOral || false;
        that.basiliskDomFucked = well.basiliskDomFucked || false;

        that.basiliskMirror = well.basiliskMirror || false;
        that.basiliskRode = well.basiliskRode || false;
        that.basiliskRodeCameInside = well.basiliskRodeCameInside || false;
    };
}

// END res\js\character\history\shared\well\HamletWellShared.js

// res\js\character\inventory\bag\Bag.js

/**
 * The Bag contains all of the kobold's consumables and miscellaneous items.
 */
function Bag() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Converts an object of id: {amount, item} tuples into a new list of 
     * ItemAmount objects and returns that list.
     * @param {object} list The object of tuples to convert.
     * @returns {array} The list converted to an array of ItemAmounts.
     */
    function listToItemAmount(list) {
        var output = [];
        for (var key in list) {
            if (list.hasOwnProperty(key)) {
                var amount = list[key].amount;
                var item = constructItem(key);
                output.push(new ItemAmount(amount, item));
            }
        }
        return output;
    }

    /**
     * Fetches the id from the passed object.
     * If the passed object is already a number, returns it.
     * @param {object | int} objectOrId The object or the id.
     * @returns {int} The id.
     */
    function getItemId(objectOrId) {
        if (typeof (objectOrId) === "object") {
            return objectOrId.id;
        }
        return objectOrId;
    }

    /**
     * Converts the passed object or id into an object.
     * If it is already an object, it is returned.
     * If it is an item id, its constructor is fetched and called.
     * @param {object | ItemEnum} objectOrId The object or the id.
     * @returns {object} The (potentially constructed) object.
     */
    function convertToItem(objectOrId) {
        if (typeof objectOrId === "object") {
            return objectOrId;
        }
        return constructItem(objectOrId);
    }

    /**
     * Construct an item via its itemId and returns the item.
     * @param {ItemEnum} itemId The id of the item.
     * @returns {object} The constructed item.
     */
    function constructItem(itemId) {
        return ItemEnum.properties[itemId].create();
    }

    /**
     * Constructs an entry into the miscs of consumables objects.
     * @param {int} amount How much of the item the kobold has.
     * @param {object} item The properly-constructed item.
     * @returns {object} The created entry.
     */
    function constructEntry(amount, item) {
        return {"amount": amount, "item": item};
    }

    /**
     * Converts a JSON-loaded objects to an object containing properly-constructed items and returns them.
     * @param {object} list The JSON-loaded object.
     * @returns {object} The constructed object.
     */
    function loadList(list) {
        var output = {};
        for (var key in list) {
            if (list.hasOwnProperty(key)) {
                var amount = list[key].amount;
                var item = constructItem(key);
                output[key] = constructEntry(amount, item);
            }
        }
        return output;
    }


    // PUBLIC MEMBERS \\        this.public = "test";

    // Item-ID mapped objects containing objects with an "amount" and an "item" member.
    this.miscs = {};
    this.consumables = {};

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};


    /**
     * Adds the specified mount of the passed miscellaneous item to the kobold's bag.
     * @param {Miscellaneous | int} misc The miscellaneous item to add or the miscellaneous item's id.
     * @param {int} amount How many of the miscellaneous item to add. 1 assumed if not passed.
     */
    this.addMisc = function (misc, amount) {
        if (!amount) {
            amount = 1;
        }
        var id = getItemId(misc);
        if (that.miscs[id]) {
            that.miscs[id].amount += amount;
        } else {
            var item = convertToItem(misc);
            that.miscs[id] = constructEntry(amount, item);
        }
    };

    /**
     * Returns how many of the passed miscelleaneous item the kobold has.
     * @param {Miscellaneous | int} misc The miscellaneous item to check or the miscellaneous item's id.
     * @returns {int} How many of the miscellaneous item the kobold has.
     */
    this.hasMisc = function (misc) {
        var id = getItemId(misc);
        if (that.miscs[id]) {
            return that.miscs[id].amount;
        }
        return 0;
    };

    /**
     * Removes the specified amount of the passed miscellaneous item from the kobold's bag.
     * @param {Miscellaneous | int} misc The miscellaneous item to remove or the miscellaneous item's id.
     * @param {int} amount How many of the miscellaneous item to remove. 1 is assumed if not passed.
     */
    this.removeMisc = function (misc, amount) {
        if (!amount) {
            amount = 1;
        }
        var id = getItemId(misc);
        if (!that.miscs[id]) {
            return;
        }
        that.miscs[id].amount -= amount;
        if (that.miscs[id].amount <= 0) {
            delete that.miscs[id];
        }
    };


    /**
     * Removes all of the passed miscellaneous item from the kobold's bag.
     * @param {Miscellaneous | int} misc The miscellaneous item to remove or the miscellaneous item's id.
     */
    this.removeAllMisc = function (misc) {
        var id = getItemId(misc);
        delete miscs[id];
    };

    /**
     * Adds the specified amount of the passed consumable to the kobold's bag.
     * @param {Consumable | int} consumable The consumable to add or the consumable's item id.
     * @param {int} amount How many of the consumable to add. 1 assumed if not passed.
     */
    this.addConsumable = function (consumable, amount) {
        if (!amount) {
            amount = 1;
        }
        var id = getItemId(consumable);
        if (that.consumables[id]) {
            that.consumables[id].amount += amount;
        } else {
            var item = convertToItem(consumable);
            that.consumables[id] = constructEntry(amount, item);
        }
    };

    /**
     * Returns how many of the passed consumable the kobold has.
     * @param {Consumable | int} consumable The consumable to check or the consumable's item id.
     * @returns {int} How many of the consumable the kobold has.
     */
    this.hasConsumable = function (consumable) {
        var id = getItemId(consumable);
        if (that.consumables[id]) {
            return that.consumables[id].amount;
        }
        return 0;
    };

    /**
     * Removes the specified amount of passed consumable from the kobold's bag.
     * @param {Consumable | int} consumable The consumable to remove or the consumable's item id.
     * @param {int} amount How many of the consumable to remove. 1 is assumed if not passed.
     */
    this.removeConsumable = function (consumable, amount) {
        if (!amount) {
            amount = 1;
        }
        var id = getItemId(consumable);
        if (!that.consumables[id]) {
            return;
        }
        that.consumables[id].amount -= amount;
        if (that.consumables[id].amount <= 0) {
            delete that.consumables[id];
        }
    };

    /**
     * Removes all of the passed consumable from the kobold's bag.
     * @param {Consumable | int} consumable The consumable to remove or the consumable's item id.
     */
    this.removeAllConsumable = function (consumable) {
        var id = getItemId(consumable);
        delete consumables[id];
    };

    /**
     * Removes all miscellaneous items from the kobold's bag.
     */
    this.emptyMiscs = function () {
        that.miscs = {};
    };

    /**
     * Removes all consumables from the kobold's bag.
     */
    this.emptyConsumables = function () {
        that.consumables = {};
    };

    /**
     * Empties the bag, removing everything from it.
     */
    this.emptyBag = function () {
        that.emptyMiscs();
        that.emptyConsumables();
    };

    /**
     * Gets an array of ItemAmount objects representing all of the kobold's consumables. 
     * @returns {array} An array of ItemAmount objects representing all of the kobold's consumables. 
     */
    this.getConsumablesList = function () {
        return listToItemAmount(that.consumables);
    };

    /**
     * Gets an array of ItemAmount objects representing all of the kobold's miscellaneous items. 
     * @returns {array} An array of ItemAmount objects representing all of the kobold's miscellaneous items. 
     */
    this.getMiscItemsList = function () {
        return listToItemAmount(that.miscs);
    };
    
    /**
     * Returns true if the kobold has any misc or consumable item, false otherwise.
     * @returns {boolean} True if the kobold has any misc or consumable item, false otherwise.
     */
    this.hasAnyItem = function () {
        return Object.keys(that.miscs).length || Object.keys(that.consumables).length;
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Bag-stripped} bag A Bag object, stripped of private members and all functions.
     */
    this.load = function (bag) {
        that.miscs = loadList(bag.miscs);
        that.consumables = loadList(bag.consumables);
    };
}

// END res\js\character\inventory\bag\Bag.js

// res\js\character\inventory\bag\NumbersEnum.js

/**
 * Transforms numbers to text.
 * Used in the sexual history of stats.
 */
var NumbersEnum = {
    1: "one",
    2: "two",
    3: "three",
    4: "four",
    5: "five",
    6: "six",
    7: "seven",
    8: "eight",
    9: "nine",
    10: "ten",
    11: "eleven",
    12: "twelve",
    13: "thirteen",
    14: "fourteen",
    15: "fifteen",
    16: "sixteen",
    17: "seventeen",
    18: "eighteen",
    19: "nineteen",
    20: "twenty",
    21: "twenty-one",
    22: "twenty-two",
    23: "twenty-three",
    24: "twenty-four",
    25: "twenty-five",
    26: "twenty-six",
    27: "twenty-seven",
    28: "twenty-eight",
    29: "twenty-nine",
    30: "thirty",
    31: "thirty-one",
    32: "thirty-two",
    33: "thirty-three",
    34: "thirty-four",
    35: "thirty-five",
    36: "thirty-six",
    37: "thirty-seven",
    38: "thirty-eight",
    39: "thirty-nine",
    40: "fourty",
    41: "fourty-one",
    42: "fourty-two",
    43: "fourty-three",
    44: "fourty-four",
    45: "fourty-five",
    46: "fourty-six",
    47: "fourty-seven",
    48: "fourty-eight",
    49: "fourty-nine",
    50: "fifty",
    51: "fifty-one",
    52: "fifty-two",
    53: "fifty-three",
    54: "fifty-four",
    55: "fifty-five",
    56: "fifty-six",
    57: "fifty-seven",
    58: "fifty-eight",
    59: "fifty-nine",
    60: "sixty",
    61: "sixty-one",
    62: "sixty-two",
    63: "sixty-three",
    64: "sixty-four",
    65: "sixty-five",
    66: "sixty-six",
    67: "sixty-seven",
    68: "sixty-eight",
    69: "sixty-nine",
    70: "seventy",
    71: "seventy-one",
    72: "seventy-two",
    73: "seventy-three",
    74: "seventy-four",
    75: "seventy-five",
    76: "seventy-six",
    77: "seventy-seven",
    78: "seventy-eight",
    79: "seventy-nine",
    80: "eighty",
    81: "eighty-one",
    82: "eighty-two",
    83: "eighty-three",
    84: "eighty-four",
    85: "eighty-five",
    86: "eighty-six",
    87: "eighty-seven",
    88: "eighty-eight",
    89: "eighty-nine",
    90: "ninety",
    91: "ninety-one",
    92: "ninety-two",
    93: "ninety-three",
    94: "ninety-four",
    95: "ninety-five",
    96: "ninety-six",
    97: "ninety-seven",
    98: "ninety-eight",
    99: "ninety-nine",
    max: 100
};

NumbersEnum.convert = function (number) {
    if(number >= max) {
        return number;
    }
    return NumbersEnum[number];
};

// END res\js\character\inventory\bag\NumbersEnum.js

// res\js\character\inventory\equipment\Equipment.js

/**
 * The Equipment class holds all the things the kobold is currently wearing,
 * as well as some convenience functions which aggregate equipment-functionality
 * into a more usable format.
 */
function Equipment() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Turns a slot-enum into the appropriate member of this class and returns it.
     * @param {SlotEnum} slot The slot to convert.
     * @returns {string} The name of the member.
     */
    function getMember(slot) {
        return SlotEnum.properties[slot].member;
    }

    /**
     * Returns the item as a properly-constructed and loaded object, or null if the item is null.
     * @param {object} item The JSON-built item. 
     * @returns {object} The item as a properly-constructed and loaded object, or null if the item is null.
     */
    function itemOrNull(item) {
        if (item) {
            var properlyConstructedItem = ItemEnum.properties[item.id].create();
            properlyConstructedItem.load(item);
            return properlyConstructedItem;
        }
        return null;
    }

    /**
     * Converts the passed object or id into an object.
     * If it is already an object, it is returned.
     * If it is an item id, its constructor is fetched and called.
     * @param {object | ItemEnum} objectOrId The object or the id.
     * @returns {object} The (potentially constructed) object.
     */
    function convertToItem(objectOrId) {
        if (typeof objectOrId === "object") {
            return objectOrId;
        }
        return constructItem(objectOrId);
    }

    /**
     * Construct an item via its itemId and returns the item.
     * @param {ItemEnum} itemId The id of the item.
     * @returns {object} The constructed item.
     */
    function constructItem(itemId) {
        return ItemEnum.properties[itemId].create();
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.bra = null;
    this.helmet = null;
    this.pants = null;
    this.shirt = null;
    this.shoes = null;
    this.underwear = null;
    this.weapon = null;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Strips the item in the passed slot, setting it to null.
     * @param {SlotEnum} slot The slot to strip.
     */
    this.strip = function (slot) {
        var member = getMember(slot);
        that[member] = null;
    };

    /**
     * Removes all equipment from the kobold. Should be called AFTER fetching
     * and storing details of the stripping for display purposes.
     */
    this.stripNaked = function () {
        that.bra = null;
        that.helmet = null;
        that.pants = null;
        that.shirt = null;
        that.shoes = null;
        that.underwear = null;
        that.weapon = null;
    };

    /**
     * Returns true if the kobold is completely naked (including shoes and helmet).
     * @returns {boolean} True if the kobold is completely naked. False otherwise.
     */
    this.completelyNaked = function () {
        return !that.bra && !that.helmet && !that.pants && !that.shirt && !that.shoes && !that.underwear;
    };

    /**
     * Returns true if all of the kobold's most important parts are naked.
     * Important parts are: bra, pants, shirt and underwear.
     * @returns {boolean} True if all of the kobold's most important parts are naked.
     */
    this.importantNaked = function () {
        return !that.bra && !that.pants && !that.shirt && !that.underwear;
    };

    /**
     * Wears the passed piece of equipment in the appropriate slot.
     * Removes any previously worn piece of gear in that slot.
     * @param {Gear | ItemEnum} gear The piece of gear to wear. May be the id of the item instead.
     */
    this.wear = function (gear) {
        if (!gear) {
            return;
        }
        gear = convertToItem(gear);
        var slot = gear.slot;
        var member = getMember(slot);
        that[member] = gear;
    };

    /**
     * Returns true if the kobold is wearing a piece of equipment in the passed slot.
     * @param {SlotEnum} slot The slot to check.
     * @returns {boolean} True if the kobold is wearing something in the slot, false otherwise.
     */
    this.isWearing = function (slot) {
        var member = getMember(slot);
        return that[member] !== null;
    };

    /**
     * Returns true if the kobold is visibly wearing something not intended for its gender.
     * @returns {boolean} True if the kobold is wearing something not intended for its gender. False otherwise.
     */
    this.isCrossdressing = function () {
        var braWrongGender = that.bra && that.bra.wrongGender();
        var helmetWrongGender = that.helmet && that.helmet.wrongGender();
        var pantsWrongGender = that.pants && that.pants.wrongGender();
        var shirtWrongGender = that.shirt && that.shirt.wrongGender();
        var shoesWrongGender = that.shoes && that.shoes.wrongGender();
        var underwearWrongGender = that.underwear && that.underwear.wrongGender();

        if (that.pants && that.pants.coversGroin()) {
            underwearWrongGender = false;
        }
        if (that.shirt && that.shirt.coversChest()) {
            braWrongGender = false;
        }

        return braWrongGender || helmetWrongGender || pantsWrongGender || shirtWrongGender || shoesWrongGender || underwearWrongGender;
    };

    /**
     * Returns true if the kobold's groin is exposed and false otherwise.
     * @returns {boolean} True if the kobold's groin is exposed, false otherwise.
     */
    this.groinExposed = function () {
        var pantsCovered = false;
        if (that.pants && that.pants.coversGroin()) {
            pantsCovered = true;
        }
        var underwearCovered = false;
        if (that.underwear && that.underwear.coversGroin()) {
            underwearCovered = true;
        }
        var shirtCovered = false;
        if (that.shirt && that.shirt.coversGroin()) {
            shirtCovered = true;
        }
        return !pantsCovered && !underwearCovered && !shirtCovered;
    };

    /**
     * Returns true if the kobold's rear is exposed and false otherwise.
     * @returns {boolean} True if the kobold's rear is exposed, false otherwise.
     */
    this.rearExposed = function () {
        var pantsCovered = false;
        if (that.pants && that.pants.coversRear()) {
            pantsCovered = true;
        }
        var underwearCovered = false;
        if (that.underwear && that.underwear.coversRear()) {
            underwearCovered = true;
        }
        var shirtCovered = false;
        if (that.shirt && that.shirt.coversRear()) {
            shirtCovered = true;
        }
        return !pantsCovered && !underwearCovered && !shirtCovered;
    };

    /**
     * Returns true if the kobold's chest is exposed and false otherwise.
     * @returns {boolean} True if the kobold's chest is exposed, false otherwise.
     */
    this.chestExposed = function () {
        var shirtCovered = that.shirt && that.shirt.coversChest();
        var braCovered = that.bra !== null;
        return !shirtCovered && !braCovered;
    };

    /**
     * Rebuilds a piece of gear stored in the history object.
     * @param {object} object The history-saved object.
     * @returns {Gear} The gear constructed from the history-saved object. Null if the object is null.
     */
    this.buildGearFromHistory = function (object) {
        return itemOrNull(object);
    };

    /**
     * Cleans all gear currently being worn.
     */
    this.cleanAll = function () {
        that.pants && that.pants.clean();
        that.shirt && that.shirt.clean();
        that.underwear && that.underwear.clean();
    };

    /**
     * Calculates the total worth of all of the equipment the kobold is wearing (including its weaponry).
     */
    this.totalWorth = function () {
        var braValue = (that.bra && that.bra.price()) || 0;
        var helmetValue = (that.helmet && that.helmet.price()) || 0;
        var pantsValue = (that.pants && that.pants.price()) || 0;
        var shirtValue = (that.shirt && that.shirt.price()) || 0;
        var shoesValue = (that.shoes && that.shoes.price()) || 0;
        var underwearValue = (that.underwear && that.underwear.price()) || 0;
        var weaponValue = (that.weapon && that.weapon.price()) || 0;
        return braValue + helmetValue + pantsValue + shirtValue + shoesValue + underwearValue + weaponValue;
    };
    
    /**
     * Calculates the total armor value of all of the equipment the kobold is wearing (excluding its weaponry).
     */
    this.totalArmor = function () {
        var braValue = (that.bra && that.bra.defence()) || 0;
        var helmetValue = (that.helmet && that.helmet.defence()) || 0;
        var pantsValue = (that.pants && that.pants.defence()) || 0;
        var shirtValue = (that.shirt && that.shirt.defence()) || 0;
        var shoesValue = (that.shoes && that.shoes.defence()) || 0;
        var underwearValue = (that.underwear && that.underwear.defence()) || 0;
        return braValue + helmetValue + pantsValue + shirtValue + shoesValue + underwearValue;
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Equipment-stripped} equipment An Equipment object, stripped of private members and all functions.
     */
    this.load = function (equipment) {
        that.bra = itemOrNull(equipment.bra);
        that.helmet = itemOrNull(equipment.helmet);
        that.pants = itemOrNull(equipment.pants);
        that.shirt = itemOrNull(equipment.shirt);
        that.shoes = itemOrNull(equipment.shoes);
        that.underwear = itemOrNull(equipment.underwear);
        that.weapon = itemOrNull(equipment.weapon);
    };
}

// END res\js\character\inventory\equipment\Equipment.js

// res\js\character\inventory\equipment\SlotEnum.js

/**
 * Enum representing the kobold's equipment slots.
 */
var SlotEnum = {
    BRA: 1,
    HELMET: 2,
    PANTS: 3,
    SHIRT: 4,
    SHOES: 5,
    UNDERWEAR: 6,
    WEAPON: 7
};

SlotEnum.properties = {
    1: {name: "Chest", member: "bra"},
    2: {name: "Head", member: "helmet"},
    3: {name: "Legs", member: "pants"},
    4: {name: "Upper body", member: "shirt"},
    5: {name: "Feet", member: "shoes"},
    6: {name: "Groin", member: "underwear"},
    7: {name: "Hands", member: "weapon"}
};

// END res\js\character\inventory\equipment\SlotEnum.js

// res\js\character\inventory\Inventory.js

/**
 * The Inventory class represents a kobold's inventory and equipment.
 * It contains the amount of coins a kobold is carrying, as well as the kobold's
 * bag of things and the stuff the kobold is currently wearing.
 */
function Inventory() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the body of a tooltip for items.
     * @param {object} item The properly-constructed item.
     * @returns {string} The body of the tooltip.
     */
    function createToolTipBody(item) {
        return item.description();
    }

    /**
     * Creates a tooltip header with the passed name.
     * @param {string} name The name of the item to create a header for.
     * @returns {string} The header as a HTML string.
     */
    function createToolTipHeader(name) {
        return `${name}`;
    }

    /**
     * Creates a footer for a tooltip of the passed consumable.
     * @param {object} consumable The properly-constructed consumable.
     * @returns {string} The footer.
     */
    function createConsumableToolTipFooter(consumable) {
        return `<div class="toolbottomprice">${consumable.price()} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Creates a footer for a tooltip of the passed piece of gear.
     * @param {object} gear The properly-constructed piece of gear.
     * @returns {string} The footer.
     */
    function createGearToolTipFooter(gear) {
        var footer = `<div class="toolbottomprice">${gear.price()} ${global.ui.icons.coin()}</div>`;
        footer += `<div class="toolbottomdefence">${gear.defence()} ${global.ui.icons.shield()}</div>`;
        footer += `<div class="toolbottomoffence">${gear.offence()} ${global.ui.icons.sword()}</div>`;
        return footer;
    }

    /**
     * Creates a tooltip footer for miscellaneous items.
     * @param {object} misc The properly-constructed miscellaneous item.
     * @returns {string} The tooltip footer.
     */
    function createMiscToolTipFooter(misc) {
        return `<div class="toolbottomprice">${misc.price()} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Creates a tooltip footer for the passed item.
     * @param {object} item The properly-constructed item.
     * @returns {String} The tooltip footer.
     */
    function createToolTipFooter(item) {
        switch (item.type) {
            case ItemTypeEnum.CONSUMABLE:
                return createConsumableToolTipFooter(item);
            case ItemTypeEnum.GEAR:
                return createGearToolTipFooter(item);
            default:
                return createMiscToolTipFooter(item);
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.bag = new Bag();
    this.equipment = new Equipment();

    // How much money the kobold has.
    this.coins = 0;

    // Whether or not the kobold is in a safe zone (can use consumables). READ-ONLY
    this.safeZone = false;
    // Whether or not the kobold is in a sell zone (can sell items). READ-ONLY
    this.sellZone = false;
    // Sell price modifier for items. Should always be reset to 1.0 after shops.
    this.sellMod = 1.0;
    // Specific function which can be used to listen to the selling of objects.
    // Must be manually reset to null after usage.
    this.sellListener = null;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds a bold-faced item name which will reveal a tooltip when moused over.
     * @param {object | itemEnum} item The item for which to build a tooltip.
     * @param {boolean} caps Whether or not the item's name should be capitalized. May be omitted.
     * @returns {string} The constructed item name with tooltip.
     */
    this.createItemWithTooltip = function (item, caps) {
        if (!item) {
            return "";
        }
        var mouseOver = `<span class="bold">${item.name(caps)}</span>`;
        return that.createItemTooltip(mouseOver, item);
    };

    /**
     * Builds an item tooltip around the specified mouseover target.
     * @param {string} mouseOver The item which should produce the tooltip on mouseover.
     * @param {object} item The item for which to build a tooltip.
     * @returns {string} The constructed item name with tooltip.
     */
    this.createItemTooltip = function (mouseOver, item) {
        if (!item) {
            return "";
        }
        var body = createToolTipBody(item);
        var header = createToolTipHeader(item.name(true));
        var footer = createToolTipFooter(item);
        var style = item.toolTipStyle();
        return global.ui.makeToolTip(mouseOver, body, header, footer, style);
    };

    /**
     * Gets a list of all items and consumables mixed together in ItemAmount objects.
     * @returns {array} An array of ItemAmount objects representing all consumables and misc items in the kobold's bag.
     */
    this.getItemList = function () {
        var misc = that.bag.getMiscItemsList();
        var cons = that.bag.getConsumablesList();
        var both = misc.concat(cons);
        return both;
    };

    /**
     * Adds the passed, properly-constructed item to the kobold's inventory.
     * @param {object} item The properly constructed item.
     */
    this.addItem = function (item) {
        if (item.type === ItemTypeEnum.GEAR) {
            that.equipment.wear(item);
        } else if (item.type === ItemTypeEnum.CONSUMABLE) {
            that.bag.addConsumable(item);
        } else {
            that.bag.addMisc(item);
        }
    };

    /**
     * Makes an item out of the passed item id.
     * @param {ItemEnum} itemId The id of the item. 
     * @returns {object} The properly constructed item.
     */
    this.constructItem = function (itemId) {
        return ItemEnum.properties[itemId].create();
    };

    /**
     * Makes an item out of the passed JSONified item stripped of functions.
     * @param {object} jsonItem The item parsed from JSON to an object.
     * @returns {object} The properly constructed item.
     */
    this.constructItemFromMemory = function (jsonItem) {
        if (jsonItem.type === ItemTypeEnum.CONSUMABLE || jsonItem.type === ItemTypeEnum.MISC) {
            return that.constructItem(jsonItem.id);
        } else {
            return that.equipment.buildGearFromHistory(jsonItem);
        }
    };

    /**
     * Properly constructs an array of ItemAmount objects read from memory,
     * ensuring that all of the items within as well as the ItemAmounts themselves are properly initialized.
     * @param {array} list An array of JSONified ItemAmount objects.
     * @returns {array} An array of deeply constructed ItemAmount objects.
     */
    this.constructItemAmountListFromMemory = function (list) {
        var properArray = [];
        for (var i = 0; i < list.length; i++) {
            var jsonElement = list[i];
            var amount = jsonElement.amount;
            var jsonItem = jsonElement.item;
            var item = that.constructItemFromMemory(jsonItem);
            properArray.push(new ItemAmount(amount, item));
        }
        return properArray;
    };

    /**
     * Enters a safe zone.
     * Sets the safeZone variable and may cause UI changes.
     */
    this.enterSafeZone = function () {
        that.safeZone = true;
        global.ui.tabs.inventory.startBlinking();
    };

    /**
     * Exits a safe zone.
     * Unsets the safeZone variable and may cause UI changes.
     */
    this.exitSafeZone = function () {
        that.safeZone = false;
        if (!that.sellZone) {
            global.ui.tabs.inventory.stopBlinking();
        }
    };

    /**
     * Enters a sell zone.
     * Sets the sellZone variable and may cause UI changes.
     */
    this.enterSellZone = function () {
        that.sellZone = true;
        global.ui.tabs.inventory.startBlinking();
    };

    /**
     * Exits a sell zone.
     * Unsets the sellZone variable and may cause UI changes.
     */
    this.exitSellZone = function () {
        that.sellZone = false;
        if (!that.safeZone) {
            global.ui.tabs.inventory.stopBlinking();
        }
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Inventory-stripped} inventory An Inventory object, stripped of private members and all functions.
     */
    this.load = function (inventory) {
        that.bag.load(inventory.bag);
        that.equipment.load(inventory.equipment);
        that.coins = inventory.coins;
        inventory.safeZone ? that.enterSafeZone() : that.exitSafeZone();
        inventory.sellZone ? that.enterSellZone() : that.exitSellZone();
        if (inventory.sellMod) {
            that.sellMod = inventory.sellMod;
        }
    };
}

// END res\js\character\inventory\Inventory.js

// res\js\character\inventory\ItemAmount.js

/**
 * Class representing an item and how many of that item the kobold possesses.
 * @param {int} amountPassed How many of the item the kobold has.
 * @param {object} itemPassed The well-constructed item.
 */
function ItemAmount(amountPassed, itemPassed){
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.amount = amountPassed;
    this.item = itemPassed;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\character\inventory\ItemAmount.js

// res\js\character\stats\Stats.js

/**
 * The Stats class contains all the stats of the kobold. This includes:
 *      -Agility
 *      -Charisma
 *      -Endurance
 *      -Intelligence
 *      -Luck
 *      -Strength
 */
function Stats() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var max = 10;
    var min = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Fetches and returns the name of the member of the passed stat.
     * @param {StatsEnum} stat The stat of which to get the member name.
     * @returns {string} The name of the public member.
     */
    function getMember(stat){
        return StatsEnum.properties[stat].member;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.agility = 5;
    this.charisma = 5;
    this.endurance = 5;
    this.intelligence = 5;
    this.luck = 5;
    this.strength = 5;
    
    this.statMin = min;
    this.statMax = max;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Gets the value of the passed stat.
     * @param {StatsEnum} stat The stat to get.
     * @returns {int} The value of the stat.
     */
    this.get = function(stat) {
        return that[getMember(stat)];
    };
    
    /**
     * Sets the value of the passed stat.
     * @param {StatsEnum} stat The stat to set.
     * @param {int} value The new value to set the stat to.
     */
    this.set = function(stat, value) {
        that[getMember(stat)] = value;
    };
    
    /**
     * Increases the value of the passed stat, to a maximum value.
     * @param {StatsEnum} stat The stat to increase.
     * @param {int} value How much to increase it.
     */
    this.increaseStat = function(stat, value) {
        var curr = that.get(stat);
        var increased = curr + value;
        if(increased > max) {
            increased = max;
        }
        that.set(stat, increased);
    };
    
    /**
     * Decreases the value of the passed stat, to a minimum value.
     * @param {StatsEnum} stat The stat to decrease.
     * @param {int} value How much to decrease it.
     */
    this.decreaseStat = function(stat, value) {
        var curr = that.get(stat);
        var decreased = curr - value;
        if(decreased < min) {
            decreased = min;
        }
        that.set(stat, decreased);
    };
    
    /**
     * Gets the name of the passed stat.
     * @param {StatsEnum} stat The stat to get the name of.
     * @returns {int} The name of the stat.
     */
    this.getName = function(stat) {
        return StatsEnum.properties[stat].name;
    };
    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Stats-stripped} stats A Stats object, stripped of private members and all functions.
     */
    this.load = function (stats) {
        that.agility = stats.agility;
        that.charisma = stats.charisma;
        that.endurance = stats.endurance;
        that.intelligence = stats.intelligence;
        that.luck = stats.luck;
        that.strength = stats.strength;
    };
}

// END res\js\character\stats\Stats.js

// res\js\character\stats\StatsEnum.js

/**
 * The kobold's various stats.
 */
var StatsEnum = {
    AGILITY: 1,
    CHARISMA: 2,
    ENDURANCE: 3,
    INTELLIGENCE: 4,
    LUCK: 5,
    STRENGTH: 6
};

StatsEnum.properties = {
    1: {name: "Agility", member: "agility"},
    2: {name: "Charisma", member: "charisma"},
    3: {name: "Endurance", member: "endurance"},
    4: {name: "Intelligence", member: "intelligence"},
    5: {name: "Luck", member: "luck"},
    6: {name: "Strength", member: "strength"}
};

// END res\js\character\stats\StatsEnum.js

// res\js\character\status\MentalHealth.js

/*
 * The MentalHealth class keeps track of the kobold's mental health.
 */
function MentalHealth() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets the lasthit member, if the reason specified is defined.
     * Else, sets the lasthit member to "Unknown".
     * @param {string} hit The last hit.
     */
    function setLastHit(hit) {
        if(hit){
            that.lasthit = hit;
        } else {
            that.lasthit = "Unknown";
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.health = 100;
    this.min = 0;
    this.max = 100;
    this.lasthit = "Unknown";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Gets the kobold's mental health.
     * @returns {int} The mental health of the kobold.
     */
    this.get = function() {
        return that.health;
    };
    
    /**
     * Sets the kobold's mental health.
     * @param {int} value The new value to set the kobold's mental health.
     */
    this.set = function(value) {
        that.health = value;
    };
    
    /**
     * Increases the kobold's mental health, to a maximum value.
     * @param {int} value How much to increase it.
     * @param {string} reason What happened to cause the change.
     */
    this.increase = function(value, reason) {
        setLastHit(reason);
        var increased = that.health + value;
        if(increased > that.max) {
            increased = that.max;
        }
        that.set(increased);
    };
    
    /**
     * Decreases the kobold's mental health, to a minimum value.
     * @param {int} value How much to decrease it.
     * @param {string} reason What happened to cause the change.
     */
    this.decrease = function(value, reason) {
        setLastHit(reason);
        var decreased = that.health - value;
        if(decreased < that.min) {
            decreased = that.min;
        }
        that.set(decreased);
    };
    

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {MentalHealth-stripped} mentalHealth A MentalHealth object, stripped of private members and all functions.
     */
    this.load = function (mentalHealth) {
        that.health = mentalHealth.health;
        that.min = mentalHealth.min;
        that.max = mentalHealth.max;
        that.lasthit = mentalHealth.lasthit;
    };
}

// END res\js\character\status\MentalHealth.js

// res\js\character\status\PhysicalHealth.js

/*
 * The PhysicalHealth class keeps track of the kobold's physical health.
 */
function PhysicalHealth() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Sets the lasthit member, if the reason specified is defined.
     * Else, sets the lasthit member to "Unknown".
     * @param {string} hit The last hit.
     */
    function setLastHit(hit) {
        if(hit){
            that.lasthit = hit;
        } else {
            that.lasthit = "Unknown";
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.health = 100;
    this.min = 0;
    this.max = 100;
    this.lasthit = "Unknown";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Gets the kobold's physical health.
     * @returns {int} The physical health of the kobold.
     */
    this.get = function() {
        return that.health;
    };
    
    /**
     * Sets the kobold's physical health.
     * @param {int} value The new value to set the kobold's physical health.
     */
    this.set = function(value) {
        that.health = value;
    };
    
    /**
     * Increases the kobold's physical health, to a maximum value.
     * @param {int} value How much to increase it.
     * @param {string} reason What happened to cause the change.
     */
    this.increase = function(value, reason) {
        setLastHit(reason);
        var increased = that.health + value;
        if(increased > that.max) {
            increased = that.max;
        }
        that.set(increased);
    };
    
    /**
     * Decreases the kobold's physical health, to a minimum value.
     * @param {int} value How much to decrease it.
     * @param {string} reason What happened to cause the change.
     */
    this.decrease = function(value, reason) {
        setLastHit(reason);
        var decreased = that.health - value;
        if(decreased < that.min) {
            decreased = that.min;
        }
        that.set(decreased);
    };
    

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {PhysicalHealth-stripped} physicalHealth A PhysicalHealth object, stripped of private members and all functions.
     */
    this.load = function (physicalHealth) {
        that.health = physicalHealth.health;
        that.min = physicalHealth.min;
        that.max = physicalHealth.max;
        that.lasthit = physicalHealth.lasthit;
    };
}

// END res\js\character\status\PhysicalHealth.js

// res\js\character\status\SexualExertion.js

/*
 * The SexualExertion class keeps track of the kobold's sexual exertion.
 */
function SexualExertion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Sets the lasthit member, if the reason specified is defined.
     * Else, sets the lasthit member to "Unknown".
     * @param {string} hit The last hit.
     */
    function setLastHit(hit) {
        if(hit){
            that.lasthit = hit;
        } else {
            that.lasthit = "Unknown";
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.health = 0;
    this.min = 0;
    this.max = 100;
    this.lasthit = "Unknown";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Gets the kobold's sexual exertion.
     * @returns {int} The sexual exertion of the kobold.
     */
    this.get = function() {
        return that.health;
    };
    
    /**
     * Sets the kobold's sexual exertion.
     * @param {int} value The new value to set the kobold's sexual exertion.
     */
    this.set = function(value) {
        that.health = value;
    };
    
    /**
     * Increases the kobold's sexual exertion, to a maximum value.
     * @param {int} value How much to increase it.
     * @param {string} reason What happened to cause the change.
     */
    this.increase = function(value, reason) {
        setLastHit(reason);
        var increased = that.health + value;
        if(increased > that.max) {
            increased = that.max;
        }
        that.set(increased);
    };
    
    /**
     * Decreases the kobold's sexual exertion, to a minimum value.
     * @param {int} value How much to decrease it.
     * @param {string} reason What happened to cause the change.
     */
    this.decrease = function(value, reason) {
        setLastHit(reason);
        var decreased = that.health - value;
        if(decreased < that.min) {
            decreased = that.min;
        }
        that.set(decreased);
    };
    

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {SexualExertion-stripped} exertion A SexualExertion object, stripped of private members and all functions.
     */
    this.load = function (exertion) {
        that.health = exertion.health;
        that.min = exertion.min;
        that.max = exertion.max;
        that.lasthit = exertion.lasthit;
    };
}

// END res\js\character\status\SexualExertion.js

// res\js\character\status\Stamina.js

/*
 * The Stamina class keeps track of the kobold's stamina.
 */
function Stamina() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Sets the lasthit member, if the reason specified is defined.
     * Else, sets the lasthit member to "Unknown".
     * @param {string} hit The last hit.
     */
    function setLastHit(hit) {
        if(hit){
            that.lasthit = hit;
        } else {
            that.lasthit = "Unknown";
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.health = 100;
    this.min = 0;
    this.max = 100;
    this.lasthit = "Unknown";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Gets the kobold's stamina.
     * @returns {int} The stamina of the kobold.
     */
    this.get = function() {
        return that.health;
    };
    
    /**
     * Sets the kobold's stamina.
     * @param {int} value The new value to set the kobold's stamina.
     */
    this.set = function(value) {
        that.health = value;
    };
    
    /**
     * Increases the kobold's stamina, to a maximum value.
     * @param {int} value How much to increase it.
     * @param {string} reason What happened to cause the change.
     */
    this.increase = function(value, reason) {
        setLastHit(reason);
        var increased = that.health + value;
        if(increased > that.max) {
            increased = that.max;
        }
        that.set(increased);
    };
    
    /**
     * Decreases the kobold's stamina, to a minimum value.
     * @param {int} value How much to decrease it.
     * @param {string} reason What happened to cause the change.
     */
    this.decrease = function(value, reason) {
        setLastHit(reason);
        var decreased = that.health - value;
        if(decreased < that.min) {
            decreased = that.min;
        }
        that.set(decreased);
    };
    

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Stamina-stripped} stamina A Stamina object, stripped of private members and all functions.
     */
    this.load = function (stamina) {
        that.health = stamina.health;
        that.min = stamina.min;
        that.max = stamina.max;
        that.lasthit = stamina.lasthit;
    };
}

// END res\js\character\status\Stamina.js

// res\js\character\status\Status.js

/*
 * The Status class keeps track of the kobold's physical and mental health,
 * fatigue and sexual exertion.
 */
function Status() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.physical = new PhysicalHealth();
    this.mental = new MentalHealth();
    this.stamina = new Stamina();
    this.exertion = new SexualExertion();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Status-stripped} status A Status object, stripped of private members and all functions.
     */
    this.load = function (status) {
        that.physical.load(status.physical);
        that.mental.load(status.mental);
        that.stamina.load(status.stamina);
        that.exertion.load(status.exertion);
    };
}

// END res\js\character\status\Status.js

// res\js\controllers\AudioController.js

/**
 * Controller for audio.
 * 
 * @param {string} id The id of the scene.
 * @param {string} url URL of the audio file. Also the key used to log playing.
 * @param {int} volume The volume of the audio file.
 */
function AudioController(id, url, volume) {
    var that = this;
    this.audioIndex = global.audio.load(url, volume);

    this.play = function () {
        return global.audio.playWhenLoaded(that.audioIndex);
    };
    this.playOnce = function () {
        var trackNumber = -1;
        global.kobold.history.doIfNotSet(id, url, true, function () {
            trackNumber = global.audio.playWhenLoaded(that.audioIndex);
        });
        return trackNumber;
    };
    this.playSettingsDependent = function () {
        if (global.settings.getAudioInMenu()) {
            return that.playOnce();
        } else {
            return that.play();
        }
    };
    this.stop = function (index) {
        global.audio.cancelPlayWhenLoaded(that.audioIndex);
        global.audio.stop(that.audioIndex, index);
    };
    this.loop = function () {
        global.audio.loop(that.audioIndex);
    };
    this.stopLoop = function () {
        global.audio.stopLoop(that.audioIndex);
    };
    this.fadeLoop = function () {
        global.audio.fadeLoop(that.audioIndex);
    };
}

// END res\js\controllers\AudioController.js

// res\js\controllers\ChangeController.js

/**
 * Choice controller with added change function.
 * 
 * Capabilities:
 * -change
 * -set
 * -get
 * -has
 * -unset
 * -continue
 * -continued
 * 
 * @param {string} id Id of the scene.
 * @param {string} key Key of this choice.
 * @param {function} changeFn Change function, called once.
 * @param {function} changeNoteFn Change note function. Should return a global.ui.change.buildNotification() result.
 */
function ChangeController(id, key, changeFn, changeNoteFn) {
    ChoiceController.call(this, id, key);
    this.change = function () {
        var changeId = key + "Change";
        global.kobold.history.doIfNotSet(id, changeId, true, changeFn);
        return changeNoteFn();
    };
}

// END res\js\controllers\ChangeController.js

// res\js\controllers\ChoiceController.js

/**
 * Controller for a single choice, regardless of whether it's a continue or a set/get.
 * 
 * Capabilities:
 * -set
 * -get
 * -has
 * -unset
 * -continue
 * -continued
 * 
 * @param {string} id The id of the scene.
 * @param {string} key The key of this choice.
 */
function ChoiceController(id, key) {
    ContinueController.call(this, id, key);
    this.set = function (choice) {
        global.kobold.history.setKey(id, key, choice);
    };
    this.get = function () {
        return global.kobold.history.getKey(id, key);
    };
    this.has = function () {
        return global.kobold.history.has(id, key);
    };
}

// END res\js\controllers\ChoiceController.js

// res\js\controllers\ContinueController.js

/**
 * Controller for a single choiceless segment.
 * 
 * Capabilities:
 * -unset
 * -continue
 * -continued
 * 
 * @param {string} id The id of the scene.
 * @param {string} key The key of this choice.
 */
function ContinueController(id, key) {
    this.continue = function () {
        global.kobold.history.setKey(id, key, true);
    };
    this.continued = function () {
        return global.kobold.history.has(id, key);
    };
    this.unset = function () {
        global.kobold.history.unsetKey(id, key);
    };
}

// END res\js\controllers\ContinueController.js

// res\js\controllers\MiscController.js

/**
 * Controller containing a wealth of single-fetched information.
 * All unset- functions remove the single-fetched information entry, allowing things to be fetched once more.
 * 
 * Capabilities:
 * -gold
 * -strip
 * -pants (wearPants, removePants)
 * -bra (wearBra, removeBra)
 * -shirt (wearShirt, removeShirt)
 * -underwear (wearUnderwear, removeUnderwear)
 * -helmet (wearHelmet, removeHelmet)
 * -shoes (wearShoes, removeShoes)
 * -weapon (wearWeapon, removeWeapon)
 * 
 * -nude
 * -importantNude
 * -groinExposed
 * -chestExposed
 * -crossdressing
 * -equipmentWorth
 * -totalArmor
 * 
 * -hasConsumable
 * -hasMisc
 * -hasAnything
 * 
 * -gender
 * -baseGender
 * -dick
 * -dickVirgin
 * -cockslitVirgin
 * -cunt
 * -cuntVirgin
 * -analVirgin
 * -oralVirgin
 * -boobs
 * -lactating
 * -name
 * 
 * -health
 * -unsetHealth
 * -sanity
 * -unsetSanity
 * -stamina
 * -unsetStamina
 * -exertion
 * -unsetExertion
 * 
 * -agility
 * -unsetAgility
 * -charisma
 * -unsetCharisma
 * -endurance
 * -unsetEndurance
 * -intelligence
 * -unsetIntelligence
 * -luck
 * -unsetLuck
 * -strength
 * -unsetStrength
 * -unsetAll
 * 
 * @param {string} id id of the scene using this controller.
 * @param {string} key Controller-specific key. Default empty string.
 * @param {string} separator Separator between key and function-key.
 */
function MiscController(id, key, separator) {
    var that = this;
    key = key || "";
    separator = separator || "";

    this.gold = function () {
        var goldId = key + separator + "gold";
        global.kobold.history.setIfNotSet(id, goldId, global.kobold.inventory.coins);
        return global.kobold.history.getKey(id, goldId);
    };

    this.strip = function () {
        var stripId = key + separator + "strip";
        global.kobold.history.doIfNotSet(id, stripId, true, function () {
            global.kobold.inventory.equipment.stripNaked();
        });
    };

    this.pants = function () {
        var pantsId = key + separator + "pants";
        global.kobold.history.setIfNotSet(id, pantsId, global.kobold.inventory.equipment.pants);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, pantsId));
    };

    this.wearPants = function (bottom) {
        var pantsId = key + separator + "wearPants";
        global.kobold.history.doIfNotSet(id, pantsId, true, function () {
            global.kobold.inventory.equipment.wear(bottom);
        });
    };

    this.removePants = function () {
        var pantsId = key + separator + "removePants";
        global.kobold.history.doIfNotSet(id, pantsId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.PANTS);
        });
    };

    this.bra = function () {
        var braId = key + separator + "bra";
        global.kobold.history.setIfNotSet(id, braId, global.kobold.inventory.equipment.bra);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, braId));
    };

    this.wearBra = function (top) {
        var braId = key + separator + "wearBra";
        global.kobold.history.doIfNotSet(id, braId, true, function () {
            global.kobold.inventory.equipment.wear(top);
        });
    };

    this.removeBra = function () {
        var braId = key + separator + "removeBra";
        global.kobold.history.doIfNotSet(id, braId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.BRA);
        });
    };

    this.shirt = function () {
        var shirtId = key + separator + "shirt";
        global.kobold.history.setIfNotSet(id, shirtId, global.kobold.inventory.equipment.shirt);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, shirtId));
    };

    this.wearShirt = function (top) {
        var shirtId = key + separator + "wearShirt";
        global.kobold.history.doIfNotSet(id, shirtId, true, function () {
            global.kobold.inventory.equipment.wear(top);
        });
    };

    this.removeShirt = function () {
        var shirtId = key + separator + "removeShirt";
        global.kobold.history.doIfNotSet(id, shirtId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.SHIRT);
        });
    };

    this.underwear = function () {
        var underwearId = key + separator + "underwear";
        global.kobold.history.setIfNotSet(id, underwearId, global.kobold.inventory.equipment.underwear);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, underwearId));
    };

    this.wearUnderwear = function (undies) {
        var underwearId = key + separator + "wearUnderwear";
        global.kobold.history.doIfNotSet(id, underwearId, true, function () {
            global.kobold.inventory.equipment.wear(undies);
        });
    };

    this.removeUnderwear = function () {
        var underwearId = key + separator + "removeUnderwear";
        global.kobold.history.doIfNotSet(id, underwearId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.UNDERWEAR);
        });
    };

    this.helmet = function () {
        var helmetId = key + separator + "helmet";
        global.kobold.history.setIfNotSet(id, helmetId, global.kobold.inventory.equipment.helmet);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, helmetId));
    };

    this.wearHelmet = function (helm) {
        var helmetId = key + separator + "wearHelmet";
        global.kobold.history.doIfNotSet(id, helmetId, true, function () {
            global.kobold.inventory.equipment.wear(helm);
        });
    };

    this.removeHelmet = function () {
        var helmetId = key + separator + "removeHelmet";
        global.kobold.history.doIfNotSet(id, helmetId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.HELMET);
        });
    };

    this.shoes = function () {
        var shoesId = key + separator + "shoes";
        global.kobold.history.setIfNotSet(id, shoesId, global.kobold.inventory.equipment.shoes);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, shoesId));
    };

    this.wearShoes = function (footwear) {
        var shoesId = key + separator + "wearShoes";
        global.kobold.history.doIfNotSet(id, shoesId, true, function () {
            global.kobold.inventory.equipment.wear(footwear);
        });
    };

    this.removeShoes = function () {
        var shoesId = key + separator + "removeShoes";
        global.kobold.history.doIfNotSet(id, shoesId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.SHOES);
        });
    };

    this.weapon = function () {
        var weaponId = key + separator + "weapon";
        global.kobold.history.setIfNotSet(id, weaponId, global.kobold.inventory.equipment.weapon);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, weaponId));
    };

    this.wearWeapon = function (weapon) {
        var weaponId = key + separator + "wearWeapon";
        global.kobold.history.doIfNotSet(id, weaponId, true, function () {
            global.kobold.inventory.equipment.wear(weapon);
        });
    };

    this.removeWeapon = function () {
        var weaponId = key + separator + "removeWeapon";
        global.kobold.history.doIfNotSet(id, weaponId, true, function () {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
        });
    };


    this.nude = function () {
        var nudeId = key + separator + "nude";
        global.kobold.history.setIfNotSet(id, nudeId, global.kobold.inventory.equipment.completelyNaked());
        return global.kobold.history.getKey(id, nudeId);
    };

    this.importantNude = function () {
        var importantNudeId = key + separator + "importantNude";
        global.kobold.history.setIfNotSet(id, importantNudeId, global.kobold.inventory.equipment.importantNaked());
        return global.kobold.history.getKey(id, importantNudeId);
    };

    this.groinExposed = function () {
        var groinExposedId = key + separator + "groinExposed";
        global.kobold.history.setIfNotSet(id, groinExposedId, global.kobold.inventory.equipment.groinExposed());
        return global.kobold.history.getKey(id, groinExposedId);
    };

    this.chestExposed = function () {
        var chestExposedId = key + separator + "chestExposed";
        global.kobold.history.setIfNotSet(id, chestExposedId, global.kobold.inventory.equipment.chestExposed());
        return global.kobold.history.getKey(id, chestExposedId);
    };

    this.crossdressing = function () {
        var crossdressingId = key + separator + "crossdressing";
        global.kobold.history.setIfNotSet(id, crossdressingId, global.kobold.inventory.equipment.isCrossdressing());
        return global.kobold.history.getKey(id, crossdressingId);
    };

    this.equipmentWorth = function () {
        var equipmentWorthId = key + separator + "equipmentWorth";
        global.kobold.history.setIfNotSet(id, equipmentWorthId, global.kobold.inventory.equipment.totalWorth());
        return global.kobold.history.getKey(id, equipmentWorthId);
    };

    this.totalArmor = function () {
        var totalArmorId = key + separator + "totalArmor";
        global.kobold.history.setIfNotSet(id, totalArmorId, global.kobold.inventory.equipment.totalArmor());
        return global.kobold.history.getKey(id, totalArmorId);
    };


    this.hasConsumable = function (itemId) {
        var hasItemId = key + separator + "hasConsumable" + itemId;
        global.kobold.history.setIfNotSet(id, hasItemId, global.kobold.inventory.bag.hasConsumable(itemId));
        return global.kobold.history.getKey(id, hasItemId);
    };

    this.hasMisc = function (itemId) {
        var hasItemId = key + separator + "hasMisc" + itemId;
        global.kobold.history.setIfNotSet(id, hasItemId, global.kobold.inventory.bag.hasMisc(itemId));
        return global.kobold.history.getKey(id, hasItemId);
    };
    
    this.hasAnything = function () {
        var hasAnythingId = key + separator + "hasAnything";
        global.kobold.history.setIfNotSet(id, hasAnythingId, global.kobold.inventory.bag.hasAnyItem());
        return global.kobold.history.getKey(id, hasAnythingId);
    };


    this.gender = function () {
        var genderId = key + separator + "gender";
        global.kobold.history.setIfNotSet(id, genderId, global.kobold.body.gender);
        return global.kobold.history.getKey(id, genderId);
    };

    this.baseGender = function () {
        var baseGenderId = key + separator + "baseGender";
        global.kobold.history.setIfNotSet(id, baseGenderId, global.kobold.body.getBaseGender());
        return global.kobold.history.getKey(id, baseGenderId);
    };

    this.dick = function () {
        var dickId = key + separator + "dick";
        global.kobold.history.setIfNotSet(id, dickId, global.kobold.body.dick);
        return global.kobold.history.getKey(id, dickId);
    };

    this.dickVirgin = function () {
        var dickVirginId = key + separator + "dickVirgin";
        global.kobold.history.setIfNotSet(id, dickVirginId, global.kobold.body.dickVirgin);
        return global.kobold.history.getKey(id, dickVirginId);
    };

    this.cockslitVirgin = function () {
        var cockslitVirginId = key + separator + "cockslitVirgin";
        global.kobold.history.setIfNotSet(id, cockslitVirginId, global.kobold.body.cockslitVirgin);
        return global.kobold.history.getKey(id, cockslitVirginId);
    };

    this.cunt = function () {
        var cuntId = key + separator + "cunt";
        global.kobold.history.setIfNotSet(id, cuntId, global.kobold.body.cunt);
        return global.kobold.history.getKey(id, cuntId);
    };

    this.cuntVirgin = function () {
        var cuntVirginId = key + separator + "cuntVirgin";
        global.kobold.history.setIfNotSet(id, cuntVirginId, global.kobold.body.vaginalVirgin);
        return global.kobold.history.getKey(id, cuntVirginId);
    };

    this.analVirgin = function () {
        var analVirginId = key + separator + "analVirgin";
        global.kobold.history.setIfNotSet(id, analVirginId, global.kobold.body.analVirgin);
        return global.kobold.history.getKey(id, analVirginId);
    };

    this.oralVirgin = function () {
        var oralVirginId = key + separator + "oralVirgin";
        global.kobold.history.setIfNotSet(id, oralVirginId, global.kobold.body.oralVirgin);
        return global.kobold.history.getKey(id, oralVirginId);
    };

    this.boobs = function () {
        var boobsId = key + separator + "boobs";
        global.kobold.history.setIfNotSet(id, boobsId, global.kobold.body.breasts);
        return global.kobold.history.getKey(id, boobsId);
    };

    this.lactating = function () {
        var lactatingId = key + separator + "lactating";
        global.kobold.history.setIfNotSet(id, lactatingId, global.kobold.body.lactating);
        return global.kobold.history.getKey(id, lactatingId);
    };

    this.name = function () {
        var nameId = key + separator + "name";
        global.kobold.history.setIfNotSet(id, nameId, global.kobold.name);
        return global.kobold.history.getKey(id, nameId);
    };


    this.health = function () {
        var healthId = key + separator + "health";
        global.kobold.history.setIfNotSet(id, healthId, global.kobold.status.physical.get());
        return global.kobold.history.getKey(id, healthId);
    };

    this.unsetHealth = function () {
        var healthId = key + separator + "health";
        global.kobold.history.unsetKey(id, healthId);
    };

    this.sanity = function () {
        var sanityId = key + separator + "sanity";
        global.kobold.history.setIfNotSet(id, sanityId, global.kobold.status.mental.get());
        return global.kobold.history.getKey(id, sanityId);
    };

    this.unsetSanity = function () {
        var sanityId = key + separator + "sanity";
        global.kobold.history.unsetKey(id, sanityId);
    };

    this.stamina = function () {
        var staminaId = key + separator + "stamina";
        global.kobold.history.setIfNotSet(id, staminaId, global.kobold.status.stamina.get());
        return global.kobold.history.getKey(id, staminaId);
    };

    this.unsetStamina = function () {
        var staminaId = key + separator + "stamina";
        global.kobold.history.unsetKey(id, staminaId);
    };

    this.exertion = function () {
        var exertionId = key + separator + "exertion";
        global.kobold.history.setIfNotSet(id, exertionId, global.kobold.status.exertion.get());
        return global.kobold.history.getKey(id, exertionId);
    };

    this.unsetExertion = function () {
        var exertionId = key + separator + "exertion";
        global.kobold.history.unsetKey(id, exertionId);
    };


    this.agility = function () {
        var agilityId = key + separator + "agility";
        global.kobold.history.setIfNotSet(id, agilityId, global.kobold.stats.get(StatsEnum.AGILITY));
        return global.kobold.history.getKey(id, agilityId);
    };

    this.unsetAgility = function () {
        var agilityId = key + separator + "agility";
        global.kobold.history.unsetKey(id, agilityId);
    };

    this.charisma = function () {
        var charismaId = key + separator + "charisma";
        global.kobold.history.setIfNotSet(id, charismaId, global.kobold.stats.get(StatsEnum.CHARISMA));
        return global.kobold.history.getKey(id, charismaId);
    };

    this.unsetCharisma = function () {
        var charismaId = key + separator + "charisma";
        global.kobold.history.unsetKey(id, charismaId);
    };

    this.endurance = function () {
        var enduranceId = key + separator + "endurance";
        global.kobold.history.setIfNotSet(id, enduranceId, global.kobold.stats.get(StatsEnum.ENDURANCE));
        return global.kobold.history.getKey(id, enduranceId);
    };

    this.unsetEndurance = function () {
        var enduranceId = key + separator + "endurance";
        global.kobold.history.unsetKey(id, enduranceId);
    };

    this.intelligence = function () {
        var intelligenceId = key + separator + "intelligence";
        global.kobold.history.setIfNotSet(id, intelligenceId, global.kobold.stats.get(StatsEnum.INTELLIGENCE));
        return global.kobold.history.getKey(id, intelligenceId);
    };

    this.unsetIntelligence = function () {
        var intelligenceId = key + separator + "intelligence";
        global.kobold.history.unsetKey(id, intelligenceId);
    };

    this.luck = function () {
        var luckId = key + separator + "luck";
        global.kobold.history.setIfNotSet(id, luckId, global.kobold.stats.get(StatsEnum.LUCK));
        return global.kobold.history.getKey(id, luckId);
    };

    this.unsetLuck = function () {
        var luckId = key + separator + "luck";
        global.kobold.history.unsetKey(id, luckId);
    };

    this.strength = function () {
        var strengthId = key + separator + "strength";
        global.kobold.history.setIfNotSet(id, strengthId, global.kobold.stats.get(StatsEnum.STRENGTH));
        return global.kobold.history.getKey(id, strengthId);
    };

    this.unsetStrength = function () {
        var strengthId = key + separator + "strength";
        global.kobold.history.unsetKey(id, strengthId);
    };

    this.unsetAll = function () {
        that.unsetHealth();
        that.unsetSanity();
        that.unsetStamina();
        that.unsetExertion();
        that.unsetAgility();
        that.unsetCharisma();
        that.unsetEndurance();
        that.unsetIntelligence();
        that.unsetLuck();
        that.unsetStrength();
    };
}

// END res\js\controllers\MiscController.js

// res\js\controllers\SceneController.js

/**
 * A basic controller for scenes.
 * Contains functions virtually every scene should need.
 * It is recommended to always inherit base scene controllers from the SceneController.
 * Uses the "autosave" and "continue" tokens.
 * 
 * Capabilities:
 * -autosave
 * -fakeAutosave
 * -continue
 * -continued
 * 
 * @param {string} id The id of the scene.
 */
function SceneController(id) {
    /*
     * Autosaves the game if necessary.
     */
    this.autosave = function () {
        global.kobold.history.doIfNotSet(id, "autosave", true, function () {
            global.saveManager.autoSave();
        });
    };
    
    /*
     * Sets the autosave key without actually autosaving.
     */
    this.fakeAutosave = function () {
        global.kobold.history.setIfNotSet(id, "autosave", true);
    };

    /**
     * Signals the scene has been continued.
     */
    this.continue = function () {
        global.kobold.history.setKey(id, "continue", true);
    };

    /**
     * Returns whether or not the scene has continued.
     * @returns {boolean} True if the scene has continued, false otherwise.
     */
    this.continued = function () {
        return global.kobold.history.has(id, "continue");
    };
}

// END res\js\controllers\SceneController.js

// res\js\controllers\UndoController.js

/**
 * Change controller with undo functionality.
 * 
 * Capabilities:
 * -change
 * -undo
 * -set
 * -get
 * -has
 * -unset
 * -continue
 * -continued
 * 
 * Choices are inherently unsettable.
 * This class offers change undo functionality.
 * A full choice undo requires calling both the unset as well as the undo function.
 * @param {string} id Id of the scene.
 * @param {string} key Key of this choice.
 * @param {function} changeFn Change function, called once.
 * @param {function} changeNoteFn Change note function. Should return a global.ui.change.buildNotification() result.
 * @param {function} undoFn Undo change function. Should undo everything done in the change function.
 */
function UndoController(id, key, changeFn, changeNoteFn, undoFn) {
    ChangeController.call(this, id, key, changeFn, changeNoteFn, undoFn);
    this.undo = function () {
        var changeId = key + "Change";
        if (global.kobold.history.has(id, changeId)) {
            global.kobold.history.unsetKey(id, changeId);
            undoFn && undoFn();
        }
    };
}

// END res\js\controllers\UndoController.js

// res\js\Global.js

/**
 * This JavaScript file contains code that will be called upon the start of 
 * Kobold Adventure. It includes definitions of critical global objects, which
 * may be referred to throughout the project. The initialization of the global
 * object should occur as soon as possible.
 */

function Global() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.ui = new UiController();
    this.audio = new AudioManager();
    this.hotLoader = new HotLoader();
    this.sceneLoader = new SceneLoader();
    this.saveManager = new SaveManager();
    this.parser = new KoboldParser();
    this.settings = new Settings();

    // The id of the current scene. Needed for saving.
    this.currentScene = "";
    // The name of the current scene.
    this.currentSceneName = "";
    // The folder of the current scene.
    this.currentSceneFolder = "";

    // The most important character around: the kobold.
    this.kobold = new Character();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes all global objects.
     */
    this.init = function () {
        that.audio.init(); // Init audio first so UI can access audio features.
        that.ui.init();
    };

    /**
     * Displays an error message.
     * Should only be done in terminal cases.
     * @param {string} errorMessage The error message to display.
     */
    this.error = function (errorMessage) {
        alert(errorMessage);
    };

    /**
     * Ends the current scene, resetting the scroll level to zero.
     */
    this.sceneEnd = function () {
        that.ui.sceneScroll = 0;
        that.ui.resetScroll();
    };
}

var global = new Global();

// END res\js\Global.js

// res\js\items\consumables\food\Apple.js

/**
 * The Apple class represents an apple that can be consumed by the kobold,
 * which increases its statuses.
 */
function Apple() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Apple";
    var itemDescription = "A large, reddish apple. Delicious! Eat it.";
    var baseValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You chomp down on the apple.
                            It's nice and juicy on the inside.
                            Ripe, but not [i]too[/i] ripe.
                            Sweetened to near-perfection.
                        [/p]
                        [p]
                            You savor the fruity flavor while chewing off bite after bite.
                            Before long, the apple is gone.
                            You toss away the core, having heard that it contains some kind of poison.
                            You are left feeling just a bit less hungry than you were before,
                            and a little better in general.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.APPLE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 160px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(5, "Ate an apple");
        global.kobold.status.mental.increase(5, "Ate an apple");
        global.kobold.status.stamina.increase(5, "Ate an apple");
        global.kobold.status.exertion.decrease(5, "Ate an apple");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +5), 
            new Change("Food", "Sanity", +5),
            new Change("Food", "Stamina", +5),
            new Change("Food", "Sexual exhaustion", -5, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\Apple.js

// res\js\items\consumables\food\Bread.js

/**
 * The Bread class represents a meal that can be consumed by the kobold,
 * which increases its statuses.
 */
function Bread() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Bread";
    var itemDescription = "A thick loaf of bread.";
    var baseValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You bring the bread up to your lips, and [b]tear[/b] into it,
                            ripping off over half of the loaf in a single bite.
                            While exceedingly dry, and lacking flavored condiments,
                            the staple food for many species across the world still tastes better than what you [i]usually[/i] eat.
                            As you munch on the food, you ponder the immense gap between the average living conditions of humans and kobolds.
                        [/p]
                        [p]
                            You gobble down the rest of the bread, crumbs falling to the ground en masse.
                            It may not be very [i]satisfying[/i] to eat, but it still manages to fill you up rather nicely.
                            By the time it's all gone, you feel slightly energized, all though not nearly as much as a [b]full[/b] meal would have left you.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BREAD;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(3, "Ate bread");
        global.kobold.status.mental.increase(3, "Ate bread");
        global.kobold.status.stamina.increase(3, "Ate bread");
        global.kobold.status.exertion.decrease(3, "Ate bread");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +3), 
            new Change("Food", "Sanity", +3),
            new Change("Food", "Stamina", +3),
            new Change("Food", "Sexual exhaustion", -3, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\Bread.js

// res\js\items\consumables\food\Butter.js

/**
 * The Butter class represents a meal that can be consumed by the kobold,
 * which increases its statuses.
 */
function Butter() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Butter";
    var itemDescription = "A stick of butter, wrapped in a preserving combination of paper and linen.";
    var baseValue = 10;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            Butter.
                            Churned milk.
                            Usually used as a spread to add flavor to bread, as a condiment, or as grease in a frying pan.
                            Sadly, you don't have the luxury of a stove, so you'll be eating yours as-is.
                        [/p]
                        [p]
                            You bring the stick of butter to your muzzle, and give it a little lick.
                            The taste is alright, but you aren't sure about eating the entire thing.
                            Still, if you just let it linger, it might spoil.
                            Suppressing the natural urge [b]not[/b] to shove pure [b]fat[/b] down your face, you take a large bite.
                        [/p]
                        [p]
                            The butter tastes surprisingly good.
                            It melts in your mouth like snow on a hot summer day, saturating your tastebuds with its slightly salty flavor.
                            While you [b]know[/b] this is bad for you, you swallow down and take a second bite, and then a third, and a fourth.
                            Before you regain your self-control, the entire stick of butter is gone, save for some of it stuck to your lips, which you eagerly lick clean.
                            Your guilty pleasure has left you in greater risk of heart conditions, but at least it was [i]somewhat[/i] nourishing.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BUTTER;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.decrease(2, "Ate butter");
        global.kobold.status.mental.increase(2, "Ate butter");
        global.kobold.status.stamina.increase(2, "Ate butter");
        global.kobold.status.exertion.decrease(2, "Ate butter");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", -2), 
            new Change("Food", "Sanity", +2),
            new Change("Food", "Stamina", +2),
            new Change("Food", "Sexual exhaustion", -2, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\Butter.js

// res\js\items\consumables\food\Carrot.js

/**
 * The Carrot class represents a vegetable that can be consumed by the kobold,
 * which increases its statuses.
 */
function Carrot() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Carrot";
    var itemDescription = "A taste-lacking vegetable. Good for your eyes.";
    var baseValue = 5;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You bite into the carrot,
                            tearing off the tip using your sharp teeth.
                            It's hard, it's cold, and it doesn't taste like much,
                            but at least it's food.
                            You thoroughly chew the carrot-piece, before swallowing it down and taking another bite.
                        [/p]
                        [p]
                            The coolness of the vegetable makes it feel quite [i]refreshing[/i].
                            You can see why rabbits and other animals love these things so much.
                            You chow down nibble after nibble, until you reach the end of the orange stick.
                            Overall, while not too [i]satisfying[/i] to eat, and not nearly enough to sate your hunger completely,
                            it provided some much-needed nutrition, energy for you to keep on trucking, be it towards glory or defeat.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.CARROT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 175px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(5, "Ate a carrot");
        global.kobold.status.mental.increase(3, "Ate a carrot");
        global.kobold.status.stamina.increase(5, "Ate a carrot");
        global.kobold.status.exertion.decrease(5, "Ate a carrot");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +5),
            new Change("Food", "Sanity", +3),
            new Change("Food", "Stamina", +5),
            new Change("Food", "Sexual exhaustion", -5, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Consumable-stripped} consumable A Consumable object, stripped of private members and all functions.
     */
    this.load = function (consumable) {

    };
}

// END res\js\items\consumables\food\Carrot.js

// res\js\items\consumables\food\Fish.js

/**
 * The Fish class represents a raw fish that can be consumed by the kobold,
 * which increases its statuses.
 */
function Fish() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Fish";
    var itemDescription = "A raw fish, of questionable age. At least it's not [i]completely[/i] rotten yet.";
    var baseValue = 3;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You sniff the fish.
                            It doesn't smell too good.
                            It's been at least a few days since this has been caught, and you don't think it's been kept in the best of conditions since.
                            Steeling your mind by remembering that you've had [b]far[/b] worse meals than this,
                            you part your lips, and sink your teeth into the dead, aquatical creature.
                        [/p]
                        [p]
                            You [b]tear[/b] off over half of the spoilt corpse's side,
                            chewing through the tough, unsavory scales.
                            Your first bite having ripped off the tough exterior,
                            your second dip allows you to tear into the squishy insides, where all the good bits are.
                            You yank out a few bloody organs along with some of the meat, chew the mixture thoroughly, then swallow down.
                        [/p]
                        [p]
                            Definitely [b]long[/b] overdue,
                            but you'd rather have a spoiled meal than none at all.
                            You power on, chewing through the insides, making sure not to swallow any of the small bones.
                            Nearing the end of your meal, you start to feel kind of [i]unwell[/i].
                            You decide not to eat the remaining few pieces, instead discarding the rotten, half-eaten fish.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.FISH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.decrease(2, "Ate a raw fish");
        global.kobold.status.mental.decrease(2, "Ate a raw fish");
        global.kobold.status.stamina.increase(3, "Ate a raw fish");
        global.kobold.status.exertion.decrease(3, "Ate a raw fish");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", -2),
            new Change("Food", "Sanity", -2),
            new Change("Food", "Stamina", +3),
            new Change("Food", "Sexual exhaustion", -3, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Consumable-stripped} consumable A Consumable object, stripped of private members and all functions.
     */
    this.load = function (consumable) {

    };
}

// END res\js\items\consumables\food\Fish.js

// res\js\items\consumables\food\Ham.js

/**
 * The Ham class represents a meal that can be consumed by the kobold,
 * which increases its statuses.
 */
function Ham() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Ham";
    var itemDescription = "A chunk of ham, with several slices missing.";
    var baseValue = 8;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            A good old piece of [b]meat[/b].
                            You can't remember the last time you had something like this.
                            You bring the preserved swine-leg to your mouth, and bite off a large piece of it.
                            Salty and well-seasoned, if a little too stringy for your tastes.
                            Beats eating raw meat off of an animal carcass, though.
                        [/p]
                        [p]
                            You thoroughly chew the mouthful you bit off, swallowing it before chomping down once again.
                            You settle into a slow rhythm of biting, savoring, processing and swallowing chunk after chunk of ham.
                            All too soon, the entire piece of meat is gone.
                            While a bit monotonous, you can't say you [b]didn't[/b] enjoy it.
                            In fact, it was [b]very[/b] good compared to other stuff you've eaten!
                            Overall, you've had a nice, filling, energy-packed meal that has left you lusting for more.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.HAM;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(6, "Ate ham");
        global.kobold.status.mental.increase(6, "Ate ham");
        global.kobold.status.stamina.increase(6, "Ate ham");
        global.kobold.status.exertion.decrease(6, "Ate ham");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +6), 
            new Change("Food", "Sanity", +6),
            new Change("Food", "Stamina", +6),
            new Change("Food", "Sexual exhaustion", -6, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\Ham.js

// res\js\items\consumables\food\HeartyMeal.js

/**
 * The HeartyMeal class represents a meal that can be consumed by the kobold,
 * which increases its statuses.
 */
function HeartyMeal() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Hearty Meal";
    var itemDescription = "A basket containing bread, meat, some cheese and a small drink of water. An entire meal to go.";
    var baseValue = 12;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You open up the small basket, and fetch out a loaf of bread,
                            on which you put a slice of cheese and a slice of ham.
                            You take a moment to eagerly lick your lips.
                            The sandwich you just made is quite a feast compared to what you usually eat.
                            And you don't intend to let any of it go to waste.
                        [/p]
                        [p]
                            You bite down and tear off an entire mouthful of the delicious mixture of meat, cheese and bread.
                            The flavor, while ordinary to most people who are [i]used[/i] to eating regular food,
                            is absolutely [b]divine[/b] to a kobold like yourself who's used to eating garbage,
                            dead animals left behind by hunters and whatever fish you manage to catch in the river.
                        [/p]
                        [p]
                            After a few more big bites, the appetizing sandwich is, sadly enough, all gone.
                            You swallow the final mouthful, before reaching into the basket once more, this time fetching the small bottle of water.
                            Gulping the cool liquid down, you clear the insides of your muzzle of any remaining bits and pieces of food.
                        [/p]
                        [p]
                            You've had a very nice meal, which has left you feeling much better than before.
                            You contently rub your well-filled stomach, and with a smile on your face,
                            you prepare yourself to continue your adventure.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.HEARTY_MEAL;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(10, "Ate a hearty meal");
        global.kobold.status.mental.increase(10, "Ate a hearty meal");
        global.kobold.status.stamina.increase(10, "Ate a hearty meal");
        global.kobold.status.exertion.decrease(10, "Ate a hearty meal");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +10), 
            new Change("Food", "Sanity", +10),
            new Change("Food", "Stamina", +10),
            new Change("Food", "Sexual exhaustion", -10, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\HeartyMeal.js

// res\js\items\consumables\food\Plum.js

/**
 * The Plum class represents a fruit that can be consumed by the kobold,
 * which increases its statuses.
 */
function Plum() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Plum";
    var itemDescription = "A medium-sized, purple fruit, oddly with [i]no stem[/i]. Still, it's tasty. Eat it!";
    var baseValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You bite into the succulent, purple fruit, taking out a large chunk of delicious, ripe, juicy goodness.
                            Its juices splatter along the sides of your maw.
                            Tastes pretty darn great!
                            You delight in the flavor while chewing the mulchy goodness,
                            before swallowing it down.
                        [/p]
                        [p]
                            Your first mouthful is soon followed by a second, and a third.
                            Oddly enough, besides having no stem, this particular plum also lacks a core.
                            Must be some kind of special breed.
                            Without the core to worry about, you stuff the rest of the mangled fruit into your mouth,
                            [b]devouring[/b] it whole, letting not even a single bit go to waste.
                            The odd plum didn't do much to sate your hunger, but it was very yummy nonetheless!
                            It leaves you in a [i]slightly[/i] happier mood.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.PLUM;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 175px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(2, "Ate a plum");
        global.kobold.status.mental.increase(5, "Ate a plum");
        global.kobold.status.stamina.increase(2, "Ate a plum");
        global.kobold.status.exertion.decrease(2, "Ate a plum");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +2),
            new Change("Food", "Sanity", +5),
            new Change("Food", "Stamina", +2),
            new Change("Food", "Sexual exhaustion", -2, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Consumable-stripped} consumable A Consumable object, stripped of private members and all functions.
     */
    this.load = function (consumable) {

    };
}

// END res\js\items\consumables\food\Plum.js

// res\js\items\consumables\food\Sandwich.js

/**
 * The HeartyMeal class represents a meal that can be consumed by the kobold,
 * which increases its statuses.
 */
function Sandwich() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Sandwich";
    var itemDescription = "A ham-sandwich. With butter. Yummy.";
    var baseValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You tear into the sandwich, biting off nearly half of it in one go.
                            The butter tastes surprisingly [b]good[/b].
                            You suspect it was hand-made.
                            The ham, while pretty good itself, pales in comparison.
                            The entire thing ends up tasting pretty darn great!
                        [/p]
                        [p]
                            You eagerly chomp down again and again, but sadly the entire thing is gone before you manage to sate your hunger completely.
                            You lick your fingers clean of any residual crumbs and a smidgen of butter that had escaped the bread.
                            That was quite an excellent pick-me-up!
                            Much better than what you [i]usually[/i] eat, and far more nutritional to boot.
                            You feel energized and ready for more adventure!
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SANDWICH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(6, "Ate a sandwich");
        global.kobold.status.mental.increase(6, "Ate a sandwich");
        global.kobold.status.stamina.increase(6, "Ate a sandwich");
        global.kobold.status.exertion.decrease(6, "Ate a sandwich");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +6), 
            new Change("Food", "Sanity", +6),
            new Change("Food", "Stamina", +6),
            new Change("Food", "Sexual exhaustion", -6, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\food\Sandwich.js

// res\js\items\consumables\food\TeaBiscuit.js

/**
 * The TeaBiscuit class represents a biscuit that can be consumed by the kobold,
 * which increases its statuses.
 */
function TeaBiscuit() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Tea Biscuit";
    var itemDescription = "A delicious, sugary treat. Usually consumed while drinking hot beverages, but good to eat on its own as well.";
    var baseValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You bring the vanilla biscuit to your muzzle,
                            its delightful scent reaching your nostrils long before the crunchy goodness itself passes your lips.
                            You bite off over half of the cookie in one go, its delectable flavor spreading across your tastebuds.
                            Yummy!
                        [/p]
                        [p]
                            Sadly, it's a rather small wafer.
                            Small enough that after your initial bite, you can fit the entirety of the rest into your mouth.
                            After a bit of chewing and subsequent swallowing, the biscuit's all gone, but luckily, its taste lingers for a few minutes more.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.TEA_BISCUIT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 210px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(5, "Ate a biscuit");
        global.kobold.status.mental.increase(5, "Ate a biscuit");
        global.kobold.status.stamina.increase(5, "Ate a biscuit");
        global.kobold.status.exertion.decrease(5, "Ate a biscuit");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +5), 
            new Change("Food", "Sanity", +5),
            new Change("Food", "Stamina", +5),
            new Change("Food", "Sexual exhaustion", -5, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {TeaBiscuit-stripped} food A TeaBiscuit object, stripped of private members and all functions.
     */
    this.load = function (food) {

    };
}

// END res\js\items\consumables\food\TeaBiscuit.js

// res\js\items\consumables\potions\gender\FemPlus.js

/**
 * The FemPlus class represents a potion that can be consumed by the kobold,
 * which makes it more feminine.
 */
function FemPlus() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "FemPlus Potion";
    var itemDescription = "A small glass vial shaped like a pair of breasts. Bright pink liquid sloshes within.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents(breasts, dick, cunt, pregender) {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, and without worrying about the smell, you greedily gulp down its contents.
                            To an outsider, it almost looks as though you're [b]suckling[/b] the fluids from the glass breasts,
                            the [i]nipples[/i] of which are in your mouth.
                        [/p]
                        [p]
                            The rather watery liquid tastes like an odd mixture of strawberries and milk.
                            The moment you swallow the first mouthful, you can feel the stuff already beginning to affect your body.
                        [/p]`;

        if (dick > SizeEnum.SMALLEST) {
            var size = "small";
            if (dick > SizeEnum.SMALL) {
                size = "average";
            }
            if (dick > SizeEnum.AVERAGE) {
                size = "sizeable";
            }

            content += `[p]
                            Your groin begins to tingle.
                            You can feel something about yourself [b]changing[/b].
                            An odd feeling courses through your body, 
                            causing you to gasp out loud as your ${size} prick begins to [b]shrink[/b],
                            your cockslit growing just a little more roomy in the process.
                        [/p]`;

            content += `[p]
                            [b]Your member has grown smaller![/b]<br/>`;
        } else if (!cunt) {
            content += `[p]
                            Your groin begins to tingle, before [b]painfully[/b] rearranging itself.
                            You clutch your loins and your lower gut as ovaries grow inside of your body,
                            and a new hole forms in between your cockslit and your tailhole.
                        [/p]
                        [p]
                            [b]You now have a cunt![/b]<br/>`;
        } else if (dick) {
            content += `[p]
                            Your groin begins to tingle, before [b]painfully[/b] rearranging itself.
                            You clutch your loins and your lower gut as your tiny cock shrinks into non-existence, and your cockslit vanishes without a trace.
                        [/p]
                        [p]
                            [b]You no longer have a dick![/b]<br/>`;
        } else if (!breasts) {
            content += `[p]
                            Your chest begin to tingle.
                            In fact, your entire body starts to feel...[i]odd[/i].
                            You gaze downwards, just in time to witness your bosom rapidly expanding,
                            a pair of small tits forming atop your previously flat chest.
                        [/p]
                        [p]
                            [b]You have grown breasts![/b]<br/>`;
        } else {
            var size = "small";
            if (breasts > SizeEnum.SMALL) {
                size = "average";
            }
            if (breasts > SizeEnum.AVERAGE) {
                size = "large";
            }
            content += `[p]
                            Your nipples begin to tingle.
                            In fact, your entire bosom starts to feel...[i]odd[/i].
                            You gaze downwards at your body, just in time to witness your ${size} breasts rapidly expanding,
                            growing ${breasts > SizeEnum.AVERAGE ? "even" : ""} bigger than they were before!
                        [/p]
                        [p]
                            [b]Your breasts have grown larger![/b]<br/>`;
        }

        var newGender = global.kobold.body.gender;
        if (pregender !== newGender) {
            var newGenderName = GenderEnum.properties[newGender].name.toLowerCase();
            var a = "";
            if (newGender === GenderEnum.CUNTBOY || newGender === GenderEnum.HERMAPHRODITE || newGender === GenderEnum.MALEHERM || newGender === GenderEnum.SHEMALE) {
                a = "a";
            }
            content += `[b]You are now ${a} ${newGenderName}![/b]<br/>`;
        }

        content += `    
                            [b]You have become more feminine.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }
    
    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as feminine as you're ever going to get. You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.FEMPLUS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var pregender = global.kobold.body.gender;
        var breasts = global.kobold.body.breasts;
        var dick = global.kobold.body.dick;
        var cunt = global.kobold.body.cunt;
        var maxed = breasts === SizeEnum.HUGE;

        if (dick > SizeEnum.SMALLEST) {
            global.kobold.body.growDick(dick - 1);
        } else if (!cunt) {
            global.kobold.body.growCunt();
        } else if (dick) {
            global.kobold.body.removeDick();
        } else if (!maxed) {
            global.kobold.body.breasts = global.kobold.body.breasts + 1;
        }

        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }

        global.ui.showFullscreenPopup(buildPopupContents(breasts, dick, cunt, pregender));
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\gender\FemPlus.js

// res\js\items\consumables\potions\gender\FemTo.js

/**
 * The FemTo class represents a potion that can be consumed by the kobold,
 * which turns the kobold into a girl (if it's not already a girl).
 */
function FemTo() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "FemTo Potion";
    var itemDescription = "A small glass vial with an orange liquid within. The label says it'll make you female.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents(breasts, dick, cunt) {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            The contents smell like orange.
                            That's pretty good!
                            You like oranges, although you've only ever had one once.
                            You greedily gulp down the decent-tasting mixture.
                            Its effects don't take long to manifest.
                        [/p]
                        [p]
                            ${dick ? `Your cockslit painfully grows shut, your member disappearing into your body, your internal balls vanishing.` : ``}
                            ${!breasts ? `Your flat chest stirs, a pair of nipples forming atop, followed by a two tiny, scaly mounds.
                            You've had insect bites larger than these.` : ``}
                            ${!cunt ? `The flesh in between your legs painfully parts, a new feminine slit forming, clit-and-all.
                            Inside of your body, your organs displace to make room for your newly formed, fertile uterus.` : ``}
                            Well, the potion did what it advertised.
                            ${dick ? `Your dick is gone, replaced by womanly loins${!breasts ? ` and a more feminine body` : ``}.` : `${!cunt && !breasts ? `
                            You have grown a cunt and some small breasts.` : !cunt ? `You have grown a cunt.` : `You have grown some small breasts.`}`}
                        [/p]
                        [p]
                            [c]
                                [b]
                                    You are now a girl!
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already a girl.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.FEMTO;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 208px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var breasts = global.kobold.body.breasts;
        var dick = global.kobold.body.dick;
        var cunt = global.kobold.body.cunt;
        var gender = global.kobold.body.gender;
        var maxed = gender === GenderEnum.FEMALE;

        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }

        global.kobold.body.removeDick();
        if (!breasts) {
            global.kobold.body.growBreasts(SizeEnum.SMALLEST);
        }
        if (!cunt) {
            global.kobold.body.growCunt();
        }
        global.ui.showFullscreenPopup(buildPopupContents(breasts, dick, cunt));
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\gender\FemTo.js

// res\js\items\consumables\potions\gender\mPlus.js

/**
 * The mPlus class represents a potion that can be consumed by the kobold,
 * which reduces its breasts (if present), grows a dick (if not present), removes a cunt (if present) and finally increases dick size (if present).
 */
function mPlus() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "mPlus Potion";
    var itemDescription = "A small glass vial shaped like a humanoid dick. Dark blue liquid swirls within.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents(breasts, dick, cunt, pregender, lactating) {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, and without worrying about the smell, you greedily gulp down its contents.
                            To an outsider, it almost looks as though you're [b]sucking[/b] the fluids out of the glass cock,
                            the tip of which is in your mouth.
                        [/p]
                        [p]
                            The rather watery liquid tastes like blueberries.
                            The moment you swallow the first mouthful, you can feel the stuff already beginning to effect your body.
                        [/p]`;

        if (breasts) {
            var vanish = breasts === SizeEnum.SMALLEST;
            var size = "small";
            if (breasts > SizeEnum.SMALL) {
                size = "average";
            }
            if (breasts > SizeEnum.AVERAGE) {
                size = "large";
            }
            content += `[p]
                            Your nipples begin to tingle.
                            In fact, your entire bosom starts to feel...[i]odd[/i].
                            You gaze downwards at your body, just in time to witness your two ${size} breasts rapidly shrinking,
                            ${vanish ? "vanishing completely, your chest now smooth and flat." : `becoming ${breasts < SizeEnum.AVERAGE ? "even" : ""} smaller than they were before!`}
                        [/p]`;
            if (vanish) {
                content += `[p]
                            [b]You no longer have breasts![/b]<br/>`;
            } else {
                content += `[p]
                            [b]Your breasts have grown smaller![/b]<br/>`;
            }

        } else if (!dick) {
            content += `[p]
                            Your groin begins to tingle, before [b]painfully[/b] rearranging itself.
                            You clutch your loins as your flesh slowly splits open, a new slit growing in between your legs,
                            this one housing a small, flaccid, koboldic member.
                        [/p]
                        [p]
                            [b]You have grown a dick![/b]<br/>`;
        } else if (cunt) {
            content += `[p]
                            Your groin begins to tingle, before [b]painfully[/b] rearranging itself.
                            You clutch your loins and your lower gut as your ovaries rapidly fade into nothing, and your cunt vanishes without a trace.
                        [/p]
                        [p]
                            [b]You no longer have a vagina![/b]<br/>`;
        } else {
            var size = "small";
            if (dick > SizeEnum.SMALL) {
                size = "average";
            }
            if (dick > SizeEnum.AVERAGE) {
                size = "sizeable";
            }
            content += `[p]
                            Your groin begins to tingle.
                            You can feel something about yourself [b]changing[/b].
                            An intense wave of pleasure courses through your body,
                            causing you to moan out loudly as your ${size} prick begins to [b]enlarge[/b],
                            your cockslit growing just a little less roomy in the process.
                        [/p]
                        [p]
                            [b]Your cock has grown bigger![/b]<br/>`;
        }

        if (lactating) {
            content += `[b]You have stopped lactating.[/b]<br/>`;
        }

        var newGender = global.kobold.body.gender;
        if (pregender !== newGender) {
            var newGenderName = GenderEnum.properties[newGender].name.toLowerCase();
            var a = "";
            if (newGender === GenderEnum.CUNTBOY || newGender === GenderEnum.HERMAPHRODITE || newGender === GenderEnum.MALEHERM || newGender === GenderEnum.SHEMALE) {
                a = "a";
            }
            content += `[b]You are now ${a} ${newGenderName}![/b]<br/>`;
        }

        content += `    
                            [b]You have become more masculine.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as masculine as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.MALEPLUS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var lactating = global.kobold.body.lactating;
        var pregender = global.kobold.body.gender;
        var breasts = global.kobold.body.breasts;
        var dick = global.kobold.body.dick;
        var cunt = global.kobold.body.cunt;
        var maxed = dick === SizeEnum.HUGE;
        if (lactating) {
            global.kobold.body.lactating = false;
        }

        if (breasts) {
            global.kobold.body.growBreasts(breasts - 1);
        } else if (!dick) {
            global.kobold.body.growDick(SizeEnum.SMALLEST);
        } else if (cunt) {
            global.kobold.body.removeCunt();
        } else if (!maxed) {
            global.kobold.body.dick = global.kobold.body.dick + 1;
        }

        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }

        global.ui.showFullscreenPopup(buildPopupContents(breasts, dick, cunt, pregender, lactating));
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\gender\mPlus.js

// res\js\items\consumables\potions\gender\mTo.js

/**
 * The mTo class represents a potion that can be consumed by the kobold,
 * which turns the kobold into a boy (if it's not already a boy).
 */
function mTo() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "mTo Potion";
    var itemDescription = "A small glass vial with a red liquid within. The label says it'll make you male.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents(breasts, dick, cunt) {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            The contents smell like strawberries.
                            Yummy!
                            You love strawberries!
                            You greedily gulp down the tasty mixture.
                            Its effects don't take long to manifest.
                        [/p]
                        [p]
                            ${breasts ? `Your breasts rapidly shrink, disappearing into your chest.` : ``}
                            ${cunt ? `You uterus wilts and shrivels, disappearing into your body.
                            Your cunt, too, [b]painfully[/b] grows shut, flesh overtaking the previously empty cavity.` : ``}
                            ${!dick ? `Your loins stir, a new slit growing, internal testicles forming, a tiny dick manifesting at the depts of your new cockslit.` : ``}
                            ${!dick ? `While your member might not be very impressive,
                            the potion [i]did[/i] do what it advertised.` : `Well,
                            the potion did what it advertised.${cunt ? ` Your cunt has vanished.` : ``}${breasts ? ` Your tits are gone.` : ``}`}
                        [/p]
                        [p]
                            [c]
                                [b]
                                    You are now a boy!
                                [/b]
                            [/c]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already a boy.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.MTO;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var breasts = global.kobold.body.breasts;
        var dick = global.kobold.body.dick;
        var cunt = global.kobold.body.cunt;
        var gender = global.kobold.body.gender;
        var maxed = gender === GenderEnum.MALE;

        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }

        global.kobold.body.removeCunt();
        global.kobold.body.removeBreasts();
        global.kobold.body.stopLactation();
        if (!dick) {
            global.kobold.body.growDick(SizeEnum.SMALLEST);
        }
        global.ui.showFullscreenPopup(buildPopupContents(breasts, dick, cunt));
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\gender\mTo.js

// res\js\items\consumables\potions\RejuvenationPotion.js

/**
 * The RejuvinationPotion class represents a potion that can be consumed by the kobold,
 * which increases its statuses.
 */
function RejuvenationPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Rejuvenation Potion";
    var itemDescription = "A medium-sized potion. White liquid sloshes within. The label on the side says that it'll make you feel better, whatever that means.";
    var baseValue = 30;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The white fluid within smells creamy, like milk.
                            Freshly squeezed milk.
                            Emboldened by the fragrance, you lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            The fluid tastes [b]exactly[/b] the way it smells, just like milk!
                            The consistency matches as well.
                            The only thing discerning it from regular cow-produce is the intense,
                            [b]wholesome[/b] feeling flooding your body the moment the stuff disappears down your throat.
                        [/p]
                        [p]
                            You chug down the entire small bottle's worth of the beneficial liquid,
                            savoring the taste and basking in the regenerative properties of the milk-like substance.
                            Your mind clears, various minor scrapes and cuts slowly vanish,
                            you feel [b]energized[/b] and ready to take on the entire world.
                        [/p]
                        [p]
                            As the last few droplets of the rejuvenation potion disappear down your throat,
                            you are left thoroughly revitalized, prepared for whatever challenges may lie before you.
                            As the reinvigorating sensation fades, you are left lusting for [b]more[/b].
                            You get the feeling this altered milk is slightly [i]addictive[/i].
                            Perhaps you should take care [b]not[/b] to consume [i]too much[/i] of it.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.REJUVINATION_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(20, "Rejuvenation potion");
        global.kobold.status.mental.increase(20, "Rejuvenation potion");
        global.kobold.status.stamina.increase(20, "Rejuvenation potion");
        global.kobold.status.exertion.decrease(20, "Rejuvenation potion");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Potion", "Health", +20), 
            new Change("Potion", "Sanity", +20),
            new Change("Potion", "Stamina", +20),
            new Change("Potion", "Sexual exhaustion", -20, true)
        ]);
        global.ui.showFullscreenPopup(contents);
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\RejuvenationPotion.js

// res\js\items\consumables\potions\simplestat\AgilityPotion.js

/**
 * The AgilityPotion class represents a potion that can be consumed by the kobold,
 * which increases its agility stat.
 */
function AgilityPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Agility Potion";
    var itemDescription = "A small glass vial. Green liquid sloshes within. The label on the side says that it'll make you more agile.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The green fluid within smells like the forest.
                            Like a pine tree, to be more precise. 
                            You quite like the scent.
                            Emboldened by the mild fragrance, you lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            The fluid seems quite a bit more viscous than you'd originally anticipated, oozing from the glass container like a thick slime.
                            While the taste isn't all that bad, the snot-like consistency almost makes you gag. 
                            The green goop sticks to your gums, your teeth and the top of your mouth.
                            You have a hard time getting it down your throat.
                        [/p]
                        [p]
                            Still, you force yourself to gulp down as much of the potion as you can.
                            Two big mouthfuls later, most of the green liquid has either made its way to your stomach, or is left lingering in your muzzle.
                            Content with your own efforts, you discard the near-empty vial.
                        [/p]
                        [p]
                            The leftover goop in your mouth rapidly dissolves, mixing with your saliva and becoming far easier to swallow.
                            You immediately feel the effects of the potion beginning to kick in.
                            While no visible changes occur, your body feels lighter, more nimble.
                        [/p]
                        [p]
                            <span class="bold">You have gained 1 agility.</span>
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as agile as you're ever going to get. You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.AGILITY_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var agi = global.kobold.stats.get(StatsEnum.AGILITY);
        var maxed = (agi >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\AgilityPotion.js

// res\js\items\consumables\potions\simplestat\CharismaPotion.js

/**
 * The CharismaPotion class represents a potion that can be consumed by the kobold,
 * which increases its charisma stat.
 */
function CharismaPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Charisma Potion";
    var itemDescription = "A glass flask. Purple liquid sloshes within. The label on the side says that it'll make you more charismatic.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The purple fluid within smells fruity, like a bunch of squished berries.
                            Like grape juice before being turned into wine.
                        [/p]
                        [p]
                            While you enjoy the scent, you doubt the potion is a simple fruit-extract.
                            Emboldened by the natural fragrance, you lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            The fluid tastes [b]exactly[/b] the way it smells, just like grape juice!
                            If it were a [i]little bit[/i] sweeter, or maybe a little fermented, you might even call this stuff a [i]good[/i] beverage.
                            Still, it beats water, your usual refreshment of choice.
                            And it's [b]definitely[/b] better than that one time you were forced to lap someone's spilled drink off of a filthy tavern floor.
                        [/p]
                        [p]
                            You gulp down the contents of the flask, down to the last droplet.
                            The liquid seems to [i]massage[/i] your throat as you swallow it.
                            It feels [i]soothing[/i] and [i]relieving[/i],
                            as though undoing the damage of a lifetime's worth of talking and shouting and screaming and yelling.
                        [/p]
                        [p]
                            Passing your throat and settling in your stomach, the feeling changes.
                            It becomes [i]odd[/i].
                            [i]Different[/i] from anything you've ever felt before.
                            Like it's not just [i]healing[/i] you, but also [i]changing[/i] something about you.
                            Something that you can't quite put your finger on.
                        [/p]
                        [p]
                            While you're certain this change is for the best, you can't quite help but dislike the sensation.
                            You [i]do[/i] feel a bit more confident in your ability to talk to people, but doubts still linger in your mind.
                            The potion has somewhat altered your very [b]personality[/b].
                            At what point will you stop being [i]you[/i], and instead become whatever [i]it[/i] wants you to be?
                        [/p]
                        [p]
                            [b]You have gained 1 charisma.[/b] [sneak]But at what cost?[/sneak]
                        [/p]`;
        return global.parser.parse(content);
    }
    
    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as charismatic as you're ever going to get. You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.CHARISMA_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var cha = global.kobold.stats.get(StatsEnum.CHARISMA);
        var maxed = (cha >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.CHARISMA, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\CharismaPotion.js

// res\js\items\consumables\potions\simplestat\EndurancePotion.js

/**
 * The EndurancePotion class represents a potion that can be consumed by the kobold,
 * which increases its endurance stat.
 */
function EndurancePotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Endurance Potion";
    var itemDescription = "A small, bowl-shaped glass vial. Orange liquid sloshes within. The label on the side says that it'll make you more resillient.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The orange fluid within smells like soup.
                            Tomato soup, to be more precise.
                            It entices an unnatural hunger from within you.
                            Like you [b]need[/b] to eat something [b]right now[/b].
                        [/p]
                        [p]
                            Feeling like you're about to starve, your mind can not fathom the thought of [b]not[/b] devouring the potion before you.
                            You lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            The thick, viscous liquid tastes like broth.
                            Impossibly large chunks form in your mouth.
                            You try to chew them, but they're [b]tough[/b] and [b]stringy[/b], like year-old meat.
                            Your first instinct is to spit the potion out, yet the intense [b]need[/b] to fill your stomach with food surpasses your natural reflexes.
                        [/p]
                        [p]
                            You slowly wrestle lump after lump of the meat-like substance down your throat, one at a time.
                            They barely fit down your esophagus.
                            You nearly choke on a piece, yet the very [b]instant[/b] the thought of spitting the stuff out crosses your mind,
                            the slab lodged down your throat miraculously finds its way to your stomach.
                            Like it knows.
                            Like it's playing with you.
                        [/p]
                        [p]
                            You painstakingly down the thick, chunky soup-like goop until you can take no more.
                            With the bottle emptied, you finally spit out the last few meat-hunks left in your muzzle.
                            To your surprise, the fluid that escapes your lips is [b]not[/b] filled with chunks,
                            instead resembling the orange liquid the bottle initially contained.
                            Were you [i]imagining[/i] those pieces of meat?
                            Was it all in your head?
                        [/p]
                        [p]
                            You decide not to ponder the situation any further.
                            Besides, a potion that can affect your mind is undoubtebly strong enough to improve your body.
                            Indeed, your scales feel just a little more [b]tough[/b], more [b]solid[/b], less [i]sensitive[/i] and [b]weak[/b].
                            In fact, your entire body feels like it could wither a lot more punishment without faltering.
                        [/p]
                        [p]
                            [b]You have gained 1 endurance.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as tough as you're ever going to get. You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.ENDURANCE_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var end = global.kobold.stats.get(StatsEnum.ENDURANCE);
        var maxed = (end >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\EndurancePotion.js

// res\js\items\consumables\potions\simplestat\IntelligencePotion.js

/**
 * The IntelligencePotion class represents a potion that can be consumed by the kobold,
 * which increases its intelligence stat.
 */
function IntelligencePotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Intelligence potion";
    var itemDescription = "A glass vial shaped like an hourglass. Light blue liquid sloshes within. The label on the side says that it'll make you smarter.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The blue fluid within lacks any kind of smell.
                            Is it just colored water?
                        [/p]
                        [p]
                            Unable to discern much about the vial's contents from the scent alone, you decide to chug it down and hope for the best.
                            You lower the glass container to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            Sparkly water.
                            That's the best way you have to describe the liquid flooding your mouth.
                            It has no taste, it has no texture.
                            It's not viscous.
                            It's just water, with a tingly aftereffect.
                            Thankfully, its properties make the fluid easy to down.
                            You swallow two whole mouthfuls, and with that, the small vial is emptied.
                        [/p]
                        [p]
                            Suddenly, your sight blurs.
                            You attempt to blink, yet after closing your eyes, you find your eyelids far too heavy to open once more.
                            Your mind conjures images of a creature.
                            A dark and ancient being.
                            One not from this world.
                            You stare at it, with your eyes closed.
                            It stares back at you.
                        [/p]
                        [p]
                            You snap back to reality, completely [b]terrified[/b].
                            What the [b]hell[/b] was that [b]thing[/b]?!
                            It noticed you.
                            [i]Oh god.[/i]
                            [b]It noticed you![/b]
                        [/p]
                        [p]
                            You take a deep breath.
                            You calm down a little.
                            You were just daydreaming, nothing more.
                            You didn't anger some sort of interdimensional [i]demon[/i] by [b]stealing[/b] knowledge from it.
                            That would be silly.
                            It was all in your (slightly expanded) mind.
                        [/p]
                        [p]
                            [b]You have gained 1 intelligence.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }
    
    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are smart enough not to drink more of this.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.INTELLIGENCE_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var int = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        var maxed = (int >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\IntelligencePotion.js

// res\js\items\consumables\potions\simplestat\LuckPotion.js

/**
 * The LuckPotion class represents a potion that can be consumed by the kobold,
 * which increases its luck stat.
 */
function LuckPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Luck Potion";
    var itemDescription = "A small glass vial shaped like a clover. Neon-green liquid sloshes within. The label on the side says that it'll improve your luck.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The bright green fluid within smells [b]sugary[/b] and [b]sweet[/b].
                            Like honey, but far less viscous.
                        [/p]
                        [p]
                            You fear this stuff's going to [b]rot[/b] your teeth.
                            Still, enticed by the sweet taste, you lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            The liquid tastes just as sweet as it smelled,
                            like sugary water with a curious aftertaste.
                            As you glug down two whole mouthfuls of the sweet fluid,
                            it is this [i]aftertaste[/i] that begins to dominate your mouth.
                            A strong, salty flavor that heavily contrasts with the initial saccharine taste,
                            to the point where it nearly makes you [b]retch[/b].
                        [/p]
                        [p]
                            You can barely keep your stomach's contents as your entire mouth dries up.
                            It is as though an entire mountain of salt is being crammed down your face.
                            Luckily, the sensation passes as quickly as it emerged.
                            Moisture returns to your mouth, and the overly-intense flavor fades.
                        [/p]
                        [p]
                            A single question emerges in your mind: [b]why[/b]?
                            Was the potion a fake?
                            Did you get tricked?
                            Conned?
                            Or is there some deeper meaning to all of this?
                            Was it showing you the huge world of difference between having [i]good[/i] luck and having [i]bad[/i] luck?
                            If so, you really do hope the potion [b]worked[/b].
                        [/p]
                        [p]
                            [b]You have gained 1 luck.[/b] [sneak]Or have you?[/sneak]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as lucky as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LUCK_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var lck = global.kobold.stats.get(StatsEnum.LUCK);
        var maxed = (lck >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.LUCK, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\LuckPotion.js

// res\js\items\consumables\potions\simplestat\StrengthPotion.js

/**
 * The FinessePotion class represents a potion that can be consumed by the kobold,
 * which increases its finesse stat.
 */
function StrengthPotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Strength Potion";
    var itemDescription = "A small glass vial shaped like a heart. Red liquid sloshes within. The label on the side says that it'll make you more powerful.";
    var baseValue = 50;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The red fluid within smells like blood.
                            [b]Fresh[/b] blood.
                            It sickens you.
                        [/p]
                        [p]
                            Disgust visible on your face, you steel your resolve.
                            You lower the vial to your slightly parted lips, before tipping it backwards, 
                            along with your head, letting the contents flood into your mouth.
                        [/p]
                        [p]
                            It tastes exactly the way it looks.
                            Blood.
                            You've had a lick of your own in the past, but you've never quite tasted anyone else's.
                            You gag as you swallow the first load.
                            You almost throw up, but barely manage to keep your stomach's contents.
                        [/p]
                        [p]
                            Through sheer will and determination alone, you down a second, and thankfully final gulp.
                            In a flash of anger, you toss the empty bottle to the ground, upon which it shatters into a thousand tiny little pieces.
                            You are left retching, the taste of blood still lingering in your mouth.
                            You try to wipe your tongue with your palm, but it does little to help.
                        [/p]
                        [p]
                            You feel completely [b]sickened[/b] with yourself.
                            You knew darn well what that stuff was before you drank it, yet you chugged it down anyway.
                            What kind of [b]monster[/b] are you becoming?!
                            You've got to get a hold of yourself before it's too late.
                        [/p]
                        [p]
                            [b]You have gained 1 strength.[/b]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as strong as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.STRENGTH_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 185px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var str = global.kobold.stats.get(StatsEnum.STRENGTH);
        var maxed = (str >= global.kobold.stats.statMax);
        if (maxed) {
            global.ui.showFullscreenPopup(buildMaxContents());
            return false;
        }
        global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
        global.ui.showFullscreenPopup(buildPopupContents());
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\simplestat\StrengthPotion.js

// res\js\items\consumables\potions\statswap\BeefUp.js

/**
 * The BeefUp class represents a potion that can be consumed by the kobold,
 * which increases its strength, but lowers its luck.
 */
function BeefUp() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "BeefUp Potion";
    var itemDescription = `A glass bottle filled to the brim with a shiny-looking grey goop.
                           Promises to make you stronger.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            It smells pretty [i]neutral[/i].
                            A slightly [i]metallic[/i] fragrance.
                            Odd.
                            Bracing for the worst, you bring the vial to your lips, and chug down.
                        [/p]
                        [p]
                            Iron.
                            It's pure, thick, liquid iron.
                            All of your instincts tell you that you [b]shouldn't[/b] be drinking this.
                            That it's [b]bad[/b] for your wellbeing.
                            That you should go [b]work out[/b] instead of [b]poisoning[/b] yourself.
                            But you don't have time for that.
                            You [b]need[/b] to grow stronger, and quick.
                        [/p]
                        [p]
                            You chug down the remainder of the flask,
                            your gut growing [b]heavy[/b] with metallic contents.
                            This is going to make you [b]sick[/b].
                            Barely refraining from throwing up the toxic sludge,
                            you can feel it starting to effect your muscles.
                        [/p]
                        [p]
                            Your arms and legs grow slightly heavier, as though your limbs are being [b]reinforced[/b] by the iron you drank.
                            You'd expect this to make you more [i]sturdy[/i], but instead the drink acts as a pseudo-steroid,
                            empowering only your muscles, making you stronger while leaving the rest of your body every bit as [i]sensitive[/i] as before. 
                            You get the feeling that, with this much metal in your body, you'll be at risk of getting struck during a lightning storm.
                        [/p]
                        [p]
                            [b]You have gained 1 strength.[/b][nl]
                            [warn][b]You have lost 1 luck.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as strong as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are't lucky enough to drink something like this and survive.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BEEFUP;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var str = global.kobold.stats.get(StatsEnum.STRENGTH);
        var lck = global.kobold.stats.get(StatsEnum.LUCK);
        var maxed = (str >= global.kobold.stats.statMax);
        var minned = (lck <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
            return false;
        } else {
            global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
            global.kobold.stats.decreaseStat(StatsEnum.LUCK, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Potion-stripped} potion A Potion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\statswap\BeefUp.js

// res\js\items\consumables\potions\statswap\CouragePotion.js

/**
 * The CouragePotion class represents a potion that can be consumed by the kobold,
 * which increases its charisma, but lowers its intelligence.
 */
function CouragePotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Courage Potion";
    var itemDescription = `A glass bottle filled to the brim with clear liquid.
                           Promises to make you more courageous.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion, bring it to your snout, and cautiously sniff the contents. 
                            The colorless fluid within smells like liquor.
                            Alcohol.
                            Booze, the strong kind.
                            You're starting to think this may just be a bottle of whiskey.
                        [/p]
                        [p]
                            Does it really matter whether this is an alcoholic beverage or a potion?
                            You could use either, really.
                            You bring the bottle to your lips, and chug down.
                        [/p]
                        [p]
                            The liquid immediately proves to be a [b]hard[/b] drink.
                            You can already [b]feel[/b] yourself getting tipsy after having downed just a first mouthful of the stuff.
                            It somewhat burns your throat, but it's a [b]good[/b] kind of burn.
                            A [b]satifying[/b] pain.
                            It tastes pretty darn great.
                            Whatever liquor this is, it clearly has been aging for [b]years[/b].
                            It [b]really[/b] hits the spot.
                        [/p]
                        [p]
                            A couple of gulps later, you are completely [b]hammered[/b].
                            You almost fall down on your ass while trying to shake the final few drops from the bottle,
                            the entire world spinning around you.
                            You toss the bottle aside, and lick your lips.
                            You feel energized, [b]pumped[/b], as though you can take on the entire world.
                            But your hand-eye coordination is [b]way[/b] off.
                            You barely refrain from tripping over your own tail.
                        [/p]
                        [p]
                            Then, suddenly, as quickly as you became drunk, the sensation fades once more.
                            Your sight stabilizes, you regain fine motor control over [i]all[/i] your limbs, and your dizziness fades.
                            You seem to sober up in a matter of [b]seconds[/b].
                            But not [b]completely[/b].
                            Your brain remains a little sluggish, and your inhibitions are lowered just a bit.
                            You have grown more courageous, as promised.
                            But at what cost?
                        [/p]
                        [p]
                            Huh, guess that thing [b]was[/b] a potion, after all.
                            Booze that doesn't make you drunk?
                            You could use some more of that, even though being drunk is half of the fun of drinking.
                            You're not sure if you feel comfortable with these side effects, though.
                            Being able to approach and talk to people more easily will undoubtebly be a blast,
                            but you'll still need to be able to have the mental capacity to actually [b]form words[/b] to say.
                            And keeping at least [b]some[/b] token of hesitancy would be a good thing.
                            Right?
                        [/p]
                        [p]
                            [b]You have gained 1 charisma.[/b][nl]
                            [warn][b]You have lost 1 intelligence.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as courageous as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are too dumb to drink this potion.
                            You spill it all over the floor.
                            Stupid kobold.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.COURAGE_POTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var cha = global.kobold.stats.get(StatsEnum.CHARISMA);
        var int = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        var maxed = (cha >= global.kobold.stats.statMax);
        var minned = (int <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
        } else {
            global.kobold.stats.increaseStat(StatsEnum.CHARISMA, 1);
            global.kobold.stats.decreaseStat(StatsEnum.INTELLIGENCE, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\statswap\CouragePotion.js

// res\js\items\consumables\potions\statswap\EnduGain.js

/**
 * The EnduGain class represents a potion that can be consumed by the kobold,
 * which increases its endurance, but lowers its charisma.
 */
function EnduGain() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "EnduGain Potion";
    var itemDescription = `A glass bottle filled to the brim with a disgusting-looking brownish slop.
                           Promises to make you more enduring.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            It smells as bad as it looks.
                            Hoping this isn't some kind of liquid waste, you bring the vial to your lips, close your eyes and chug down.
                        [/p]
                        [p]
                            It tastes [b]horrible[/b].
                            Definitely some kind of [b]sewage water[/b].
                            Almost throwing up the moment the first bit of the vile liquid splashes atop your tongue,
                            you seriously consider [b]not[/b] finishing the potion.
                            But you want your [b]endurance[/b], damnit.
                            Zoning out your mind as best you can, you chug down the entire thing, ending in a revolting [b]burp[/b] that makes your stomach churn.
                        [/p]
                        [p]
                            Despite going through that [b]torture[/b], you don't feel stronger in any way.
                            If anything, your breath has been sullied by the filthy potion, making people like you less in the future.
                            Was this a bad move?
                        [/p]
                        [p]
                            Then it hits you.
                            You've [i]endured[/i] drinking the potion.
                            It was more of a [b]mental test[/b] than a physical enhancement.
                            Damnit.
                            Can't those dumb alchemists invent something non-horrible for once?
                        [/p]
                        [p]
                            [b]You have gained 1 endurance.[/b][nl]
                            [warn][b]You have lost 1 charisma.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as enduring as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are't charismatic enough to convince yourself to drink something that looks this revolting.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.ENDUGAIN;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var end = global.kobold.stats.get(StatsEnum.ENDURANCE);
        var cha = global.kobold.stats.get(StatsEnum.CHARISMA);
        var maxed = (end >= global.kobold.stats.statMax);
        var minned = (cha <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
            return false;
        } else {
            global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
            global.kobold.stats.decreaseStat(StatsEnum.CHARISMA, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Potion-stripped} potion A Potion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\statswap\EnduGain.js

// res\js\items\consumables\potions\statswap\LiquidLuck.js

/**
 * The LiquidLuck class represents a potion that can be consumed by the kobold,
 * which increases its luck, but lowers its agility.
 */
function LiquidLuck() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "LiquidLuck Potion";
    var itemDescription = `A glass bottle filled to the brim with a clear, water-like fluid.
                           Promises to make you more lucky.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            It smells like water.
                            It looks like water.
                            You're starting to think this might be a scam.
                        [/p]
                        [p]
                            Shrugging, you gulp down the contents of the vial.
                            Water.
                            It's just water.
                            Is this one of those "it makes you believe in yourself" types of deals?
                            That's nonsense!
                            You feel cheated!
                        [/p]
                        [p]
                            Suddenly, an awkwardly [b]sluggish[/b] sensation washes over you.
                            Your mind and your body both feel [b]slower[/b].
                            Was the water [b]spiked[/b] with something?!
                            You take a seat on the ground for a minute, to recover.
                            Your head is spinning.
                            You nearly pass out.
                        [/p]
                        [p]
                            As quickly as the feeling appeared, so too does the worst of it it fade in a matter of moments.
                            As your mind sharpens once more, you are left feeling unexplicably [i]luckier[/i].
                            Not ordinary water after all, you suppose.
                            Rising to your feet, however, it seems as though part of the [i]sluggishness[/i] persists.
                            You shrug it off, thinking you'll feel better in a bit.
                            [sneak]You won't.[/sneak]
                        [/p]
                        [p]
                            [b]You have gained 1 luck.[/b][nl]
                            [warn][b]You have lost 1 agility.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as lucky as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You feel so sluggish, that you can't even lift the potion to your lips.
                            You do not drink it.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LIQUIDLUCK;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var lck = global.kobold.stats.get(StatsEnum.LUCK);
        var agi = global.kobold.stats.get(StatsEnum.AGILITY);
        var maxed = (lck >= global.kobold.stats.statMax);
        var minned = (agi <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
            return false;
        } else {
            global.kobold.stats.increaseStat(StatsEnum.LUCK, 1);
            global.kobold.stats.decreaseStat(StatsEnum.AGILITY, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Potion-stripped} potion A Potion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\statswap\LiquidLuck.js

// res\js\items\consumables\potions\statswap\MassRedu.js

/**
 * The MassRedu class represents a potion that can be consumed by the kobold,
 * which increases its agility, but lowers its endurance.
 */
function MassRedu() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "MassRedu Potion";
    var itemDescription = `A glass bottle filled to the brim with a black-looking gas-like substance.
                           Promises to make you more agile.`;
    var baseValue = 25;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You pop the lid of the potion.
                            It smells like [b]nothing[/b].
                            While the black stuff looks like gas, sniffing it alone doesn't seem to be able to rouse it from its container.
                            Oh well, probably some alchemical trickery going on.
                            You press the bottle to your lips, and gulp down.
                        [/p]
                        [p]
                            It has no taste.
                            You barely even [b]feel[/b] it vanishing in your mouth, and disappearing down your throat.
                            If you didn't see the bottle emptying before your very eyes, you wouldn't even believe you were [i]drinking[/i] something.
                        [/p]
                        [p]
                            As the final bit of the gas travels down into your stomach,
                            you start to feel [b]weird[/b].
                            Your limbs [b]tingle[/b], but on the [b]inside[/b].
                            They feel lighter, somehow.
                            You can move them around with greater ease!
                            You don't even feel any weaker or anything.
                            The potion worked without a hitch!
                        [/p]
                        [p]
                            Or so you believe, until you realize what is actually happening.
                            To reduce your mass, the concotion is not draining your muscles,
                            but the insides of your very bones!
                            Your usually solid skeleton is sapped of strength, growing considerably weaker, and a lot more brittle.
                            A mixed blessing.
                            At least you can run faster now.
                        [/p]
                        [p]
                            [b]You have gained 1 agility.[/b][nl]
                            [warn][b]You have lost 1 endurance.[/b][/warn]
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when maxed out.
     * @returns {string} The consume popup contents when maxed out.
     */
    function buildMaxContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You are already as agile as you're ever going to get.
                            You do not drink the potion.
                        [/p]`;
        return global.parser.parse(content);
    }

    /**
     * Builds and returns the consume popup contents when minned out.
     * @returns {string} The consume popup contents when minned out.
     */
    function buildMinContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            Your bones are so brittle, that lifting the potion to your lips would break your arm.
                            You do not drink it.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.MASSREDU;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     * @returns {boolean} True if the consumable was consumed. False otherwise.
     */
    this.consume = function () {
        var agi = global.kobold.stats.get(StatsEnum.AGILITY);
        var end = global.kobold.stats.get(StatsEnum.ENDURANCE);
        var maxed = (agi >= global.kobold.stats.statMax);
        var minned = (end <= global.kobold.stats.statMin);
        if (maxed) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMaxContents()));
            return false;
        }
        if (minned) {
            global.ui.showFullscreenPopup(global.parser.parse(buildMinContents()));
            return false;
        } else {
            global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
            global.kobold.stats.decreaseStat(StatsEnum.ENDURANCE, 1);
            global.ui.showFullscreenPopup(buildPopupContents());
        }
        return true;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Potion-stripped} potion A Potion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\consumables\potions\statswap\MassRedu.js

// res\js\items\gear\bra\ChestWrap.js

/**
 * The ChestWrap class represents a chestwrap that can be worn by the kobold.
 */
function ChestWrap() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Chestwrap";
    var itemDescription = "A simple linen chestwrap. Torn and stitched back together more than once.";
    var baseValue = 3;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.CHESTWRAP;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.BRA;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 175px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * BRA-required function.
     * Returns true if the bra is skimpy, and false otherwise.
     * @returns {boolean} True if the bra is skimpy, and false otherwise.
     */
    this.skimpy = function () {
        return false;
    };

    /**
     * BRA-required function.
     * Returns true if the bra can be torn, and false otherwise.
     * @returns {boolean} True if the bra can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * BRA-required function.
     * Returns true if the bra is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the bra is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // A simple chest wrap fit all genders.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {ChestWrap-stripped} bra A ChestWrap object, stripped of private members and all functions.
     */
    this.load = function (bra) {
        
    };
}

// END res\js\items\gear\bra\ChestWrap.js

// res\js\items\gear\bra\LinenBra.js

/**
 * The LinenBra class represents a simple linen bra that can be worn by the kobold.
 */
function LinenBra() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Linen Bra";
    var itemDescription = "A simple, yet effective bra.";
    var baseValue = 8;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LINENBRA;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.BRA;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 190px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * BRA-required function.
     * Returns true if the bra is skimpy, and false otherwise.
     * @returns {boolean} True if the bra is skimpy, and false otherwise.
     */
    this.skimpy = function () {
        return false;
    };

    /**
     * BRA-required function.
     * Returns true if the bra can be torn, and false otherwise.
     * @returns {boolean} True if the bra can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * BRA-required function.
     * Returns true if the bra is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the bra is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return global.kobold.body.getBaseGender() !== GenderEnum.FEMALE; // Bras are for girls.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {ChestWrap-stripped} bra A ChestWrap object, stripped of private members and all functions.
     */
    this.load = function (bra) {
        
    };
}

// END res\js\items\gear\bra\LinenBra.js

// res\js\items\gear\helmet\CowboyHat.js

/**
 * The CowboyHat class represents a hat that can be worn by the kobold.
 */
function CowboyHat() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Cowboy Hat";
    var itemDescription = "A large leather hat with a very distinctive shape.";
    var baseValue = 20;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.COWBOY_HAT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.HELMET;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 150px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet hides the kobold's face, false otherwise.
     * @returns {boolean} True if the helmet hides the kobold's face, false otherwise.
     */
    this.hidesFace = function () {
        return false;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet can be torn, false otherwise.
     * @returns {boolean} True if the helmet can be torn, false otherwise.
     */
    this.canBeTorn = function () {
        return false;
    };

    /**
     * HELMET-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {CowboyHat-stripped} helmet A CowboyHat object, stripped of private members and all functions.
     */
    this.load = function (helmet) {
        
    };
}

// END res\js\items\gear\helmet\CowboyHat.js

// res\js\items\gear\helmet\Eyepatch.js

/**
 * A hat.
 */
function Eyepatch() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Eyepatch";
    var itemDescription = "Usually worn by pirates, smugglers and other shady types that need to be able to see in the dark. Near-useless for kobolds.";
    var baseValue = 20;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.EYEPATCH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.HELMET;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 234px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet hides the kobold's face, false otherwise.
     * @returns {boolean} True if the helmet hides the kobold's face, false otherwise.
     */
    this.hidesFace = function () {
        return false;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet can be torn, false otherwise.
     * @returns {boolean} True if the helmet can be torn, false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * HELMET-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Eyepatch-stripped} helmet A Eyepatch object, stripped of private members and all functions.
     */
    this.load = function (helmet) {
        
    };
}

// END res\js\items\gear\helmet\Eyepatch.js

// res\js\items\gear\helmet\HatScarf.js

/**
 * A hat.
 */
function HatScarf() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Green Hat/Scarf Combo";
    var itemDescription = "Clothing belonging to a dead person. The hat has a bloodspatter on it. The scarf is stained with something white. Cream, you hope. It smells faintly of peaches.";
    var baseValue = 15;
    var offensiveValue = 0;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.HATSCARF;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.HELMET;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 220px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet hides the kobold's face, false otherwise.
     * @returns {boolean} True if the helmet hides the kobold's face, false otherwise.
     */
    this.hidesFace = function () {
        return true;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet can be torn, false otherwise.
     * @returns {boolean} True if the helmet can be torn, false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * HELMET-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Helmet-stripped} helmet A Helmet object, stripped of private members and all functions.
     */
    this.load = function (helmet) {
        
    };
}

// END res\js\items\gear\helmet\HatScarf.js

// res\js\items\gear\helmet\KoboldryHood.js

/**
 * The KoboldryHood class represents a falconry hood tailor made for kobolds.
 */
function KoboldryHood() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Koboldry Hood";
    var itemDescription = "A falconry hood tailor made for kobolds. Blocks the eyes, without restricting the muzzle.";
    var baseValue = 15;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.KOBOLDRY_HOOD;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.HELMET;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 150px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet hides the kobold's face, false otherwise.
     * @returns {boolean} True if the helmet hides the kobold's face, false otherwise.
     */
    this.hidesFace = function () {
        return true;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet can be torn, false otherwise.
     * @returns {boolean} True if the helmet can be torn, false otherwise.
     */
    this.canBeTorn = function () {
        return false;
    };

    /**
     * HELMET-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {CowboyHat-stripped} helmet A CowboyHat object, stripped of private members and all functions.
     */
    this.load = function (helmet) {
        
    };
}

// END res\js\items\gear\helmet\KoboldryHood.js

// res\js\items\gear\helmet\UnderwearGag.js

/**
 * The UnderwearGag class represents a gag Fenrir made out of his underwear in the enslavement route.
 */
function UnderwearGag() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Underwear Gag";
    var itemDescription = "A speech-impeding gag made from a wolf's worn briefs and a length of rope. Absolutely disgusting.";
    var baseValue = -1;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.UNDERWEAR_GAG;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.HELMET;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 250px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet hides the kobold's face, false otherwise.
     * @returns {boolean} True if the helmet hides the kobold's face, false otherwise.
     */
    this.hidesFace = function () {
        return true;
    };

    /**
     * HELMET-required function.
     * Returns true if the helmet can be torn, false otherwise.
     * @returns {boolean} True if the helmet can be torn, false otherwise.
     */
    this.canBeTorn = function () {
        return false;
    };

    /**
     * HELMET-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Hemlet-stripped} helmet A Helmet, stripped of private members and all functions.
     */
    this.load = function (helmet) {

    };
}

// END res\js\items\gear\helmet\UnderwearGag.js

// res\js\items\gear\pants\LeatherPants.js

/**
 * The LeatherPants class represents a pair of leather pants that can be worn by the kobold.
 */
function LeatherPants() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Leather Pants";
    var itemDescription = "A pair of pants made out of sturdy leather, both flexible and durable.";
    var baseValue = 40;
    var offensiveValue = 0;
    var defensiveValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LEATHERPANTS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.PANTS;

    // What this pair of pants is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " They reek of " + that.isStainedWith : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 3 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 175px;";
        } else {
            return "width: 200px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * PANTS-required function.
     * Stains the pants.
     * @param {string} what What the pants get stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are stained, and false otherwise.
     * @returns {boolean} True if the pants are stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * PANTS-required function.
     * Returns what the pants are stained with.
     * @returns {string} What the pants are stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * PANTS-required function.
     * Cleans the pants, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the groin, and false otherwise.
     * @returns {boolean} True if the pants cover the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the rear, and false otherwise.
     * @returns {boolean} True if the pants cover the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants can be torn, and false otherwise.
     * @returns {boolean} True if the pants can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the pants are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // Linen pants fit all genders.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {LinenPants-stripped} pants A LinenPants object, stripped of private members and all functions.
     */
    this.load = function (pants) {
        that.isStainedWith = pants.isStainedWith;
    };
}

// END res\js\items\gear\pants\LeatherPants.js

// res\js\items\gear\pants\LinenPants.js

/**
 * The LinenPants class represents a pair of linen pants that can be worn by the kobold.
 */
function LinenPants() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Linen Pants";
    var itemDescription = "Simple brown linen pants. A little baggy.";
    var baseValue = 5;
    var offensiveValue = 0;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LINENPANTS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.PANTS;

    // What this pair of pants is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " They reek of " + that.isStainedWith.toLowerCase() + "." : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 3 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 150px;";
        } else {
            return "width: 215px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * PANTS-required function.
     * Stains the pants.
     * @param {string} what What the pants get stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are stained, and false otherwise.
     * @returns {boolean} True if the pants are stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * PANTS-required function.
     * Returns what the pants are stained with.
     * @returns {string} What the pants are stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * PANTS-required function.
     * Cleans the pants, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the groin, and false otherwise.
     * @returns {boolean} True if the pants cover the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the rear, and false otherwise.
     * @returns {boolean} True if the pants cover the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants can be torn, and false otherwise.
     * @returns {boolean} True if the pants can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the pants are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // Linen pants fit all genders.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {LinenPants-stripped} pants A LinenPants object, stripped of private members and all functions.
     */
    this.load = function (pants) {
        that.isStainedWith = pants.isStainedWith;
    };
}

// END res\js\items\gear\pants\LinenPants.js

// res\js\items\gear\shirt\LinenShirt.js

/**
 * A shirt.
 */
function LinenShirt() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Linen Shirt";
    var itemDescription = `A simple, cheap, brown shirt. Covers your chest, but doesn't do more than that.`;
    var baseValue = 5;
    var offensiveValue = 0;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LINENSHIRT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;

    // What this shirt is stained with.
    this.isStainedWith = "";


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " Reeks of " + that.isStainedWith : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 30 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 220px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHIRT-required function.
     * Stains the shirt.
     * @param {string} what What the shirt gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is stained, and false otherwise.
     * @returns {boolean} True if the shirt is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * SHIRT-required function.
     * Returns what the shirt is stained with.
     * @returns {string} What the shirt is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * SHIRT-required function.
     * Cleans the shirt, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {PaddedLeatherShirt-stripped} shirt A PaddedLeatherShirt object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        if (shirt.isStainedWith) {
            that.isStainedWith = shirt.isStainedWith;
        }
    };
}

// END res\js\items\gear\shirt\LinenShirt.js

// res\js\items\gear\shirt\PaddedLeatherShirt.js

/**
 * The PaddedLeatherShirt class represents a leather shirt that can be worn by the kobold.
 */
function PaddedLeatherShirt() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Padded Leather Shirt";
    var itemDescription = `A basic piece of armor, often worn by archers and other backline soldiers. A favorite amongst brigands and bandits.`;
    var baseValue = 60;
    var offensiveValue = 0;
    var defensiveValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.PADDEDLEATHERSHIRT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;

    // What this shirt is stained with.
    this.isStainedWith = "";


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " Reeks of " + that.isStainedWith : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 30 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 220px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHIRT-required function.
     * Stains the shirt.
     * @param {string} what What the shirt gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is stained, and false otherwise.
     * @returns {boolean} True if the shirt is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * SHIRT-required function.
     * Returns what the shirt is stained with.
     * @returns {string} What the shirt is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * SHIRT-required function.
     * Cleans the shirt, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return global.kobold.body.getBaseGender() !== GenderEnum.MALE; // Leather armor is for boys.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {PaddedLeatherShirt-stripped} shirt A PaddedLeatherShirt object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        if (shirt.isStainedWith) {
            that.isStainedWith = shirt.isStainedWith;
        }
    };
}

// END res\js\items\gear\shirt\PaddedLeatherShirt.js

// res\js\items\gear\shirt\RedDress.js

/**
 * The RedDress class represents a red dress that can be worn by the kobold.
 */
function RedDress() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Red Dress";
    var itemDescription = `A pretty red dress. Worn by merchants and nobility alike. Fancy, but not <span class="italic">too</span> fancy.`;
    var baseValue = 40;
    var offensiveValue = 0;
    var defensiveValue = 2;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.REDDRESS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHIRT-required function.
     * Stains the shirt.
     * @param {string} what What the shirt gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {

    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is stained, and false otherwise.
     * @returns {boolean} True if the shirt is stained, and false otherwise.
     */
    this.stained = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns what the shirt is stained with.
     * @returns {string} What the shirt is stained with.
     */
    this.stainedWith = function () {
        return "";
    };

    /**
     * SHIRT-required function.
     * Cleans the shirt, removing any stains.
     */
    this.clean = function () {

    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return global.kobold.body.getBaseGender() !== GenderEnum.FEMALE; // Dresses are for girls.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {RedDress-stripped} shirt A RedDress object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        if (shirt.isStainedWith) {
            that.isStainedWith = shirt.isStainedWith;
        }
    };
}

// END res\js\items\gear\shirt\RedDress.js

// res\js\items\gear\shirt\StuddedLeatherChestPiece.js

/**
 * The StuddedLeatherChestPiece class represents a piece of armor that can be worn by the kobold.
 */
function StuddedLeatherChestPiece() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Studded Leather Chest Piece";
    var itemDescription = `The armor of choice for militiamen and rowdy thugs alike.`;
    var baseValue = 100;
    var offensiveValue = 1;
    var defensiveValue = 8;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.STUDDEDLEATHERCHESTPIECE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 230px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHIRT-required function.
     * Stains the shirt.
     * @param {string} what What the shirt gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {

    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is stained, and false otherwise.
     * @returns {boolean} True if the shirt is stained, and false otherwise.
     */
    this.stained = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns what the shirt is stained with.
     * @returns {string} What the shirt is stained with.
     */
    this.stainedWith = function () {
        return "";
    };

    /**
     * SHIRT-required function.
     * Cleans the shirt, removing any stains.
     */
    this.clean = function () {

    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return global.kobold.body.getBaseGender() !== GenderEnum.MALE; // Leather armor is for boys.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {PaddedLeatherShirt-stripped} shirt A PaddedLeatherShirt object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        if (shirt.isStainedWith) {
            that.isStainedWith = shirt.isStainedWith;
        }
    };
}

// END res\js\items\gear\shirt\StuddedLeatherChestPiece.js

// res\js\items\gear\shirt\WoolSweater.js

/**
 * The WoolSweater class represents a wool sweater that can be worn by the kobold.
 */
function WoolSweater() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Wool Sweater";
    var itemDescription = `A warm, comfortable, snug woolen sweater. Keeps you warm at night, and is thick enough to offer a small bit of cushioning protection.`;

    var rippedName = `Ripped Wool Sweater`;
    var rippedDescription = `Once a warm sweater, now nothing more than a bunch of torn rags. 
                             You can still wear it, but it won't keep you nearly as warm as when it wasn't ripped in half`;

    var baseValue = 25;
    var offensiveValue = 0;
    var defensiveValue = 3;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.WOOLSWEATER;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHIRT;

    // Whether or not the sweater is ripped.
    this.ripped = false;

    // What this shirt is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var name = that.ripped ? rippedName : itemName;
        if (that.stained()) {
            name = that.isStainedWith + "-stained " + name;
        }
        var toReturn = caps ? name : name.toLocaleLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var description = that.ripped ? rippedDescription : itemDescription;
        return description;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var price = that.ripped ? baseValue - 20 : baseValue;
        if (that.stained()) {
            price -= 5;
        }
        return price;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        var style = that.ripped ? "width: 230px;" : "width: 205px;";
        return style;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        var defence = that.ripped ? defensiveValue - 2 : defensiveValue;
        return defence;
    };

    /**
     * SHIRT-required function.
     * Stains the shirt.
     * @param {string} what What the shirt gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is stained, and false otherwise.
     * @returns {boolean} True if the shirt is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * SHIRT-required function.
     * Returns what the shirt is stained with.
     * @returns {string} What the shirt is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * SHIRT-required function.
     * Cleans the shirt, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the groin, and false otherwise.
     * @returns {boolean} True if the shirt covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the rear, and false otherwise.
     * @returns {boolean} True if the shirt covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt covers the chest, and false otherwise.
     * @returns {boolean} True if the shirt covers the chest, and false otherwise.
     */
    this.coversChest = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt can be torn, and false otherwise.
     * @returns {boolean} True if the shirt can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * SHIRT-required function.
     * Returns true if the shirt is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shirt is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // Wool sweater is for everyone.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {PaddedLeatherShirt-stripped} shirt A PaddedLeatherShirt object, stripped of private members and all functions.
     */
    this.load = function (shirt) {
        if (shirt.ripped) {
            that.ripped = shirt.ripped;
        }
        if (shirt.isStainedWith) {
            that.isStainedWith = shirt.isStainedWith;
        }
    };
}

function RippedWoolSweater() {
    var sweater = new WoolSweater();
    sweater.ripped = true;
    return sweater;
}

// END res\js\items\gear\shirt\WoolSweater.js

// res\js\items\gear\shoes\KoboldSocks.js

/**
 * A pair of socks that can be worn by the kobold.
 */
function KoboldSocks() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Kobold Socks";
    var itemDescription = "A pair of socks, tailor-made for digitigrade anthros. Someone -- you presume a kobold -- tore off part of the fabric to make it more suitable for people of your [i]size[/i] to wear.";
    var baseValue = 15;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.KOBOLDSOCKS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHOES;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return global.parser.parse(itemDescription);
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 208px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's sole exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's sole exposed, false otherwise.
     */
    this.soleExposed = function () {
        return false;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's toes exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's toes exposed, false otherwise.
     */
    this.toesExposed = function () {
        return false;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {SpurredBoots-stripped} shoes A SpurredBoots object, stripped of private members and all functions.
     */
    this.load = function (shoes) {
        
    };
}

// END res\js\items\gear\shoes\KoboldSocks.js

// res\js\items\gear\shoes\SpurredBoots.js

/**
 * The SpurredBoots class represents a pair of shoes that can be worn by the kobold.
 */
function SpurredBoots() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Spurred Boots";
    var itemDescription = "A pair of brown leather boots, with a sharpened, pointy, rotating star-shaped metal bit attached to the rear.";
    var baseValue = 30;
    var offensiveValue = 2;
    var defensiveValue = 2;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SPURRED_BOOTS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHOES;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's sole exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's sole exposed, false otherwise.
     */
    this.soleExposed = function () {
        return false;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's toes exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's toes exposed, false otherwise.
     */
    this.toesExposed = function () {
        return false;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {SpurredBoots-stripped} shoes A SpurredBoots object, stripped of private members and all functions.
     */
    this.load = function (shoes) {
        
    };
}

// END res\js\items\gear\shoes\SpurredBoots.js

// res\js\items\gear\shoes\WoodenSandals.js

/**
 * A pair of sandals that can be worn by the kobold.
 */
function WoodenSandals() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Wooden Sandals";
    var itemDescription = "A piece of wood with a strap hammered into it. Looks janky enough to have been made by a kobold. Might protect your soles from minor harm.";
    var baseValue = 15;
    var offensiveValue = 0;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.WOODENSANDALS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.SHOES;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 205px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's sole exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's sole exposed, false otherwise.
     */
    this.soleExposed = function () {
        return false;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes leave the kobold's toes exposed, false otherwise.
     * @returns {boolean} True if the shoes leave the kobold's toes exposed, false otherwise.
     */
    this.toesExposed = function () {
        return true;
    };

    /**
     * SHOES-required function.
     * Returns true if the shoes are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the shoes are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {SpurredBoots-stripped} shoes A SpurredBoots object, stripped of private members and all functions.
     */
    this.load = function (shoes) {
        
    };
}

// END res\js\items\gear\shoes\WoodenSandals.js

// res\js\items\gear\underwear\KoboldUnderwear.js

/**
 * The KoboldUnderwear class represents a piece of underwear tailor-made for kobolds that can be worn by the protagonist.
 */
function KoboldUnderwear() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Kobold Underwear";
    var itemDescription = "Proper underwear, made to fit a kobold. Unisex, but looks better on boys. Covers both ends.";
    var baseValue = 6;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.KOBOLDUNDERWEAR;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.UNDERWEAR;

    // What this underwear is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " Stinks like " + that.isStainedWith + "." : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 1 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 220px;";
        } else {
            return "width: 250px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * UNDERWEAR-required function.
     * Stains the underwear.
     * @param {string} what What the underwear gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is stained, and false otherwise.
     * @returns {boolean} True if the underwear is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns what the underwear is stained with.
     * @returns {string} What the underwear is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * UNDERWEAR-required function.
     * Cleans the underwear, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the groin, and false otherwise.
     * @returns {boolean} True if the underwear covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the rear, and false otherwise.
     * @returns {boolean} True if the underwear covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };
    
    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is skimpy, and false otherwise.
     * @returns {boolean} True if the underwear is skimpy, and false otherwise.
     */
    this.skimpy = function () {
        return false;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear can be torn, and false otherwise.
     * @returns {boolean} True if the underwear can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the underwear is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // Unisex.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {KoboldUnderwear-stripped} underwear A KoboldUnderwear object, stripped of private members and all functions.
     */
    this.load = function (underwear) {
        that.isStainedWith = underwear.isStainedWith;
    };
}

// END res\js\items\gear\underwear\KoboldUnderwear.js

// res\js\items\gear\underwear\LoinCloth.js

/**
 * The LoinCloth class represents a loincloth that can be worn by the kobold.
 */
function LoinCloth() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Loincloth";
    var itemDescription = "A small piece of cloth that barely covers your groin. Tied to your body via a waistband.";
    var baseValue = 2;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LOINCLOTH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.UNDERWEAR;

    // What this underwear is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " It reeks of " + that.isStainedWith.toLowerCase() + "." : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 1 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 200px;";
        } else {
            return "width: 250px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * UNDERWEAR-required function.
     * Stains the underwear.
     * @param {string} what What the underwear gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is stained, and false otherwise.
     * @returns {boolean} True if the underwear is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns what the underwear is stained with.
     * @returns {string} What the underwear is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * UNDERWEAR-required function.
     * Cleans the underwear, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the groin, and false otherwise.
     * @returns {boolean} True if the underwear covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the rear, and false otherwise.
     * @returns {boolean} True if the underwear covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return false;
    };
    
    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is skimpy, and false otherwise.
     * @returns {boolean} True if the underwear is skimpy, and false otherwise.
     */
    this.skimpy = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear can be torn, and false otherwise.
     * @returns {boolean} True if the underwear can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the underwear is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return !global.kobold.body.hasDick(); // Anything that doesn't have a dick is a mismatch.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {LoinCloth-stripped} underwear A LoinCloth object, stripped of private members and all functions.
     */
    this.load = function (underwear) {
        that.isStainedWith = underwear.isStainedWith;
    };
}

// END res\js\items\gear\underwear\LoinCloth.js

// res\js\items\gear\underwear\SilkPanties.js

/**
 * The SilkPanties class represents a piece of expensive underwear that can be worn by the protagonist.
 */
function SilkPanties() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Silk Panties";
    var itemDescription = "An expensive, luxurious pair of panties.";
    var baseValue = 50;
    var offensiveValue = 0;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SILKPANTIES;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.UNDERWEAR;

    // What this underwear is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " Stained with a considerable amount of " + that.isStainedWith.toLocaleLowerCase() + "." : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 30 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 165px;";
        } else {
            return "width: 250px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * UNDERWEAR-required function.
     * Stains the underwear.
     * @param {string} what What the underwear gets stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is stained, and false otherwise.
     * @returns {boolean} True if the underwear is stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns what the underwear is stained with.
     * @returns {string} What the underwear is stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * UNDERWEAR-required function.
     * Cleans the underwear, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the groin, and false otherwise.
     * @returns {boolean} True if the underwear covers the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear covers the rear, and false otherwise.
     * @returns {boolean} True if the underwear covers the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };
    
    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is skimpy, and false otherwise.
     * @returns {boolean} True if the underwear is skimpy, and false otherwise.
     */
    this.skimpy = function () {
        return false;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear can be torn, and false otherwise.
     * @returns {boolean} True if the underwear can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * UNDERWEAR-required function.
     * Returns true if the underwear is a mismatch for the kobold's current gender.
     * @returns {boolean} True if the underwear is a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        if(global.kobold.body.gender === GenderEnum.CUNTBOY) {
            return true;
        }
        return global.kobold.body.getBaseGender() !== GenderEnum.FEMALE;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {KoboldUnderwear-stripped} underwear A KoboldUnderwear object, stripped of private members and all functions.
     */
    this.load = function (underwear) {
        that.isStainedWith = underwear.isStainedWith;
    };
}

// END res\js\items\gear\underwear\SilkPanties.js

// res\js\items\gear\weapon\Branch.js

/**
 * The Branch class represents a branch that can be used as a weapon by the kobold.
 */
function Branch() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Tree Branch";
    var itemDescription = "A thick branch. Can be used as a melee weapon.";
    var baseValue = 1;
    var offensiveValue = 2;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BRANCH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {

    };
}

// END res\js\items\gear\weapon\Branch.js

// res\js\items\gear\weapon\Club.js

/**
 * The Club class represents a club that can be used as a weapon by the kobold.
 */
function Club() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Club";
    var itemDescription = "A heavy piece of rounded-cone-shaped wood used to beat people with.";
    var baseValue = 10;
    var offensiveValue = 3;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.CLUB;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {

    };
}

// END res\js\items\gear\weapon\Club.js

// res\js\items\gear\weapon\Dagger.js

/**
 * The Dagger class represents a dagger that can be used as a weapon by the kobold.
 */
function Dagger() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Dagger";
    var rustedItemName = "Rusted dagger";
    var itemDescription = "A dagger. More pointy than a knife. Great at stabbing, but barely able to cut a piece of bread.";
    var rustedItemDescription = "A dagger. More pointy than a knife. Used to be great at stabbing, until it rusted.";
    var baseValue = 30;
    var offensiveValue = 5;
    var defensiveValue = 2;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.DAGGER;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;
    
    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if(that.isRusted) {
            var toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
            return toReturn;
        }
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if(that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if(that.isRusted) {
            return baseValue - 10;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 240px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if(that.isRusted) {
            return offensiveValue - 4;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if(that.isRusted) {
            return defensiveValue - 1;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\Dagger.js

// res\js\items\gear\weapon\GreatSword.js

/**
 * This class represents a weapon that can be used by the kobold.
 */
function GreatSword() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Zweihander";
    var rustedItemName = "Rusted Greatsword";
    var itemDescription = "An imposing weapon taller than you are, often used by dismounted knights in heavy armor or strong men at arms intent on beheading their foes.";
    var rustedItemDescription = "What used to be an enormous weapon, now reduced to a rusty piece of scrap.";
    var baseValue = 100;
    var offensiveValue = 18;
    var defensiveValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.GREATSWORD;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        if (that.isRusted) {
            toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
        }
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if (that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if (that.isRusted) {
            return baseValue - 15;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 212px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if (that.isRusted) {
            return offensiveValue - 16;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if (that.isRusted) {
            return defensiveValue - 5;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 99;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Weapon-stripped} weapon A Weapon, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\GreatSword.js

// res\js\items\gear\weapon\HalfScythe.js

/**
 * This class represents a weapon that can be used by the kobold.
 */
function HalfScythe() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Halfscythe";
    var rustedItemName = "Rusted Scythe";
    var itemDescription = "Once a tool used to tend to crops. Now that the handle has broken, it could be used as a makeshift weapon by someone of your size.";
    var rustedItemDescription = "A rusted blade and a broken shaft. Completely useless.";
    var baseValue = 20;
    var offensiveValue = 4;
    var defensiveValue = 2;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.HALFSCYTHE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        if (that.isRusted) {
            toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
        }
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if (that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if (that.isRusted) {
            return baseValue - 15;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if (that.isRusted) {
            return offensiveValue - 3;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if (that.isRusted) {
            return defensiveValue - 1;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 99;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Weapon-stripped} weapon A Weapon, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\HalfScythe.js

// res\js\items\gear\weapon\KitchenKnife.js

/**
 * The KitchenKnife class represents a knife that can be used as a weapon by the kobold.
 */
function KitchenKnife() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Kitchen Knife";
    var itemDescription = "A steel blade used to cut vegetables, meat, sandwiches, and pretty much anything else present in a kitchen.";
    var baseValue = 35;
    var offensiveValue = 5;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.KITCHENKNIFE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        return;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {KitchenKnife-stripped} weapon A KitchenKnife object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        
    };
}

// END res\js\items\gear\weapon\KitchenKnife.js

// res\js\items\gear\weapon\KoboldTrident.js

/**
 * The KoboldTrident class represents a pitchfork that can be used as a weapon by the kobold.
 */
function KoboldTrident() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Kobold Trident";
    var rustedItemName = "Rusted Kobold Trident";
    var itemDescription = "Shortening pitchforks to use as weapons is a technique often employed by kobold militias and other less well equipped groups.";
    var rustedItemDescription = "Formerly a pitchfork. Now a stick with rusted metal ontop.";
    var baseValue = 20;
    var offensiveValue = 6;
    var defensiveValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.KOBOLD_TRIDENT;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;
    
    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        if(that.isRusted) {
            toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
        }
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if(that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if(that.isRusted) {
            return baseValue - 15;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if(that.isRusted) {
            return offensiveValue - 4;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if(that.isRusted) {
            return defensiveValue - 3;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\KoboldTrident.js

// res\js\items\gear\weapon\Mace.js

/**
 * This class represents a weapon that can be used by the kobold.
 */
function Mace() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Mace";
    var rustedItemName = "Rusted Mace";
    var itemDescription = "A blunt weapon. Undersized and slightly bent out of shape. You think something went wrong while forging this tool of war.";
    var rustedItemDescription = "A completely rusted mace. Useless.";
    var baseValue = 30;
    var offensiveValue = 5;
    var defensiveValue = 3;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.MACE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        if (that.isRusted) {
            toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
        }
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if (that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if (that.isRusted) {
            return baseValue - 25;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 225px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if (that.isRusted) {
            return 1;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if (that.isRusted) {
            return 1;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 99;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Weapon-stripped} weapon A Weapon, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\Mace.js

// res\js\items\gear\weapon\PetrifiedBranch.js

/**
 * A weapon.
 */
function PetrifiedBranch() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Petrified Tree Branch";
    var itemDescription = "A thick branch turned to stone. Rather heavy.";
    var baseValue = 10;
    var offensiveValue = 4;
    var defensiveValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BRANCH;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {

    };
}

// END res\js\items\gear\weapon\PetrifiedBranch.js

// res\js\items\gear\weapon\Rock.js

/**
 * The Rock class represents a rock that can be used as a weapon by the kobold.
 */
function Rock() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Rock";
    var itemDescription = "A heavy rock. Can be used as a blunt melee weapon, or as a one-time ranged weapon.";
    var baseValue = 1;
    var offensiveValue = 1;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.ROCK;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;
    // Whether or not the rock has been thrown.
    this.used = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function() {
        return true;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function() {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return true;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function() {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {
        
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {
        that.used = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return that.used ? 0 : 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.used = weapon.used;
    };
}

// END res\js\items\gear\weapon\Rock.js

// res\js\items\gear\weapon\RottenEgg.js

/**
 * The RottenEgg class represents an egg that can be used as a weapon by the kobold.
 */
function RottenEgg() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Rotten Egg";
    var itemDescription = "A rotten egg. Disgusting. Can be thrown at someone.";
    var baseValue = 1;
    var offensiveValue = 2;
    var defensiveValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.ROCK;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;
    // Whether or not the rock has been thrown.
    this.used = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 160px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function() {
        return true;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function() {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function() {
        return false;
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        
    };
    
    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {
        
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {
        that.used = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return that.used ? 0 : 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.used = weapon.used;
    };
}

// END res\js\items\gear\weapon\RottenEgg.js

// res\js\items\gear\weapon\ShortSword.js

/**
 * This class represents a weapon that can be used by the kobold.
 */
function ShortSword() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Shortsword";
    var rustedItemName = "Rusted Shortsword";
    var itemDescription = "A small, slightly rusty blade with a leather-clad hilt. Good for starting adventurers. Small enough to be easily wielded by someone of your size.";
    var rustedItemDescription = "A completely rusted blade. Useless.";
    var baseValue = 40;
    var offensiveValue = 6;
    var defensiveValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SHORTSWORD;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;

    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        if (that.isRusted) {
            toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
        }
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if (that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if (that.isRusted) {
            return baseValue - 30;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if (that.isRusted) {
            return offensiveValue - 5;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if (that.isRusted) {
            return defensiveValue - 3;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 99;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Weapon-stripped} weapon A Weapon, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\ShortSword.js

// res\js\items\gear\weapon\SmallKnife.js

/**
 * The SmallKnife class represents a knife that can be used as a weapon by the kobold.
 */
function SmallKnife() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Small Knife";
    var rustedItemName = "Rusted small knife";
    var itemDescription = "A plain knife. Probably once belonged to a knight, who used it as a backup weapon.";
    var rustedItemDescription = "A plain knife. Probably once belonged to a knight, who used it as a backup weapon. Now completely rusted, to the point of uselessness.";
    var baseValue = 40;
    var offensiveValue = 6;
    var defensiveValue = 2;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SMALLKNIFE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.WEAPON;
    
    // Whether or not the weapon is rusted.
    this.isRusted = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if(that.isRusted) {
            var toReturn = caps ? rustedItemName : rustedItemName.toLowerCase();
            return toReturn;
        }
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        if(that.isRusted) {
            return rustedItemDescription;
        }
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        if(that.isRusted) {
            return baseValue - 10;
        }
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 180px;";
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        if(that.isRusted) {
            return offensiveValue - 4;
        }
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        if(that.isRusted) {
            return defensiveValue - 1;
        }
        return defensiveValue;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is ranged.
     * @returns {boolean} Whether or not this weapon is ranged.
     */
    this.ranged = function () {
        return that.throw() || that.shoot();
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be thrown.
     * @returns {boolean} Whether or not this weapon can be thrown.
     */
    this.throw = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon can be shot.
     * @returns {boolean} Whether or not this weapon can be shot.
     */
    this.shoot = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a blunt melee weapon.
     * @returns {boolean} Whether or not this weapon is a blunt melee weapon.
     */
    this.blunt = function () {
        return false;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a slashing melee weapon.
     * @returns {boolean} Whether or not this weapon is a slashing melee weapon.
     */
    this.slash = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is a stabbing melee weapon.
     * @returns {boolean} Whether or not this weapon is a stabbing melee weapon.
     */
    this.stab = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns true if this weapon can rust, false otherwise.
     * @returns {boolean} True if this weapon can rust, false otherwise.
     */
    this.canRust = function () {
        return true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Rusts this weapon.
     */
    this.rust = function () {
        that.isRusted = true;
    };

    /**
     * WEAPON-REQUIRED function.
     * Returns whether or not this weapon is rusted.
     * @returns {boolean} True if this weapon is rusted, false otherwise.
     */
    this.rusted = function () {
        return that.isRusted;
    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for melee combat.
     */
    this.useMelee = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Uses this weapon for ranged combat.
     */
    this.useRanged = function () {

    };

    /**
     * WEAPON-REQUIRED function.
     * Returns how many uses this weapon has left.
     * @returns {int} How many uses this weapon has left.
     */
    this.usesLeft = function () {
        return 1;
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Branch-stripped} weapon A Branch object, stripped of private members and all functions.
     */
    this.load = function (weapon) {
        that.isRusted = weapon.isRusted;
    };
}

// END res\js\items\gear\weapon\SmallKnife.js

// res\js\items\ItemEnum.js

/**
 * Enum representing all kinds of items. Essential for the creation thereof.
 */
var ItemEnum = {
    LINENPANTS: 1,
    CHESTWRAP: 2,
    LOINCLOTH: 3,
    AGILITY_POTION: 4,
    ROPE: 5,
    STICK: 6,
    BRANCH: 7,
    ROCK: 8,
    SMALLKNIFE: 9,
    KOBOLDUNDERWEAR: 10,
    DAGGER: 11,
    SILKPANTIES: 12,
    REDDRESS: 13,
    PADDEDLEATHERSHIRT: 14,
    LINENBRA: 15,
    BANDITKEY: 16,
    GOLDENNEEDLE: 17,
    STUDDEDLEATHERCHESTPIECE: 18,
    LEATHERPANTS: 19,
    WOOLSWEATER: 20,
    CHARISMA_POTION: 21,
    ENDURANCE_POTION: 22,
    INTELLIGENCE_POTION: 23,
    LUCK_POTION: 24,
    STRENGTH_POTION: 25,
    REJUVINATION_POTION: 26,
    FEMPLUS: 27,
    MALEPLUS: 28,
    APPLE: 29,
    HEARTY_MEAL: 30,
    LOCKPICK: 31,
    SPURRED_BOOTS: 32,
    COWBOY_HAT: 33,
    CASSIDY_HOUSE_KEY: 34,
    KOBOLDRY_HOOD: 35,
    TEA_BISCUIT: 36,
    SILVERWARE: 37,
    APLUSLOTION: 38,
    KOBOLD_TRIDENT: 39,
    COURAGE_POTION: 40,
    COPPER_RING: 41,
    SANDWICH: 42,
    FANCYGLASSES: 43,
    KITCHENKNIFE: 44,
    BREAD: 45,
    BUTTER: 46,
    HAM: 47,
    ROTTEN_EGG: 48,
    CLUB: 49,
    EYEPATCH: 50,
    PETRIFIED_BRANCH: 51,
    BASILISK_SCALES: 52,
    SMALL_MIRROR: 53,
    UNDERWEAR_GAG: 54,
    GOLDRING: 55,
    GREATSWORD: 56,
    HALFSCYTHE: 57,
    HATSCARF: 58,
    PLUM: 59,
    CARROT: 60,
    FISH: 61,
    KOBOLDSOCKS: 62,
    WOODENSANDALS: 63,
    SHORTSWORD: 64,
    MACE: 65,
    LINENSHIRT: 66,
    ENDUGAIN: 67,
    BEEFUP: 68,
    LIQUIDLUCK: 69,
    MASSREDU: 70,
    MTO: 71,
    FEMTO: 72
};

ItemEnum.properties = {
    1: {create: function(){return new LinenPants();}},
    2: {create: function(){return new ChestWrap();}},
    3: {create: function(){return new LoinCloth();}},
    4: {create: function(){return new AgilityPotion();}},
    5: {create: function(){return new Rope();}},
    6: {create: function(){return new Stick();}},
    7: {create: function(){return new Branch();}},
    8: {create: function(){return new Rock();}},
    9: {create: function(){return new SmallKnife();}},
    10: {create: function(){return new KoboldUnderwear();}},
    11: {create: function(){return new Dagger();}},
    12: {create: function(){return new SilkPanties();}},
    13: {create: function(){return new RedDress();}},
    14: {create: function(){return new PaddedLeatherShirt();}},
    15: {create: function(){return new LinenBra();}},
    16: {create: function(){return new BanditKey();}},
    17: {create: function(){return new GoldenNeedle();}},
    18: {create: function(){return new StuddedLeatherChestPiece();}},
    19: {create: function(){return new LeatherPants();}},
    20: {create: function(){return new WoolSweater();}},
    21: {create: function(){return new CharismaPotion();}},
    22: {create: function(){return new EndurancePotion();}},
    23: {create: function(){return new IntelligencePotion();}},
    24: {create: function(){return new LuckPotion();}},
    25: {create: function(){return new StrengthPotion();}},
    26: {create: function(){return new RejuvenationPotion();}},
    27: {create: function(){return new FemPlus();}},
    28: {create: function(){return new mPlus();}},
    29: {create: function(){return new Apple();}},
    30: {create: function(){return new HeartyMeal();}},
    31: {create: function(){return new Lockpick();}},
    32: {create: function(){return new SpurredBoots();}},
    33: {create: function(){return new CowboyHat();}},
    34: {create: function(){return new CassidyHouseKey();}},
    35: {create: function(){return new KoboldryHood();}},
    36: {create: function(){return new TeaBiscuit();}},
    37: {create: function(){return new Silverware();}},
    38: {create: function(){return new aPlusLotion();}},
    39: {create: function(){return new KoboldTrident();}},
    40: {create: function(){return new CouragePotion();}},
    41: {create: function(){return new CopperRing();}},
    42: {create: function(){return new Sandwich();}},
    43: {create: function(){return new FancyGlasses();}},
    44: {create: function(){return new KitchenKnife();}},
    45: {create: function(){return new Bread();}},
    46: {create: function(){return new Butter();}},
    47: {create: function(){return new Ham();}},
    48: {create: function(){return new RottenEgg();}},
    49: {create: function(){return new Club();}},
    50: {create: function(){return new Eyepatch();}},
    51: {create: function(){return new PetrifiedBranch();}},
    52: {create: function(){return new BasiliskScales();}},
    53: {create: function(){return new SmallMirror();}},
    54: {create: function(){return new UnderwearGag();}},
    55: {create: function(){return new GoldRing();}},
    56: {create: function(){return new GreatSword();}},
    57: {create: function(){return new HalfScythe();}},
    58: {create: function(){return new HatScarf();}},
    59: {create: function(){return new Plum();}},
    60: {create: function(){return new Carrot();}},
    61: {create: function(){return new Fish();}},
    62: {create: function(){return new KoboldSocks();}},
    63: {create: function(){return new WoodenSandals();}},
    64: {create: function(){return new ShortSword();}},
    65: {create: function(){return new Mace();}},
    66: {create: function(){return new LinenShirt();}},
    67: {create: function(){return new EnduGain();}},
    68: {create: function(){return new BeefUp();}},
    69: {create: function(){return new LiquidLuck();}},
    70: {create: function(){return new MassRedu();}},
    71: {create: function(){return new mTo();}},
    72: {create: function(){return new FemTo();}}
};

// END res\js\items\ItemEnum.js

// res\js\items\ItemTypeEnum.js

/**
 * Enum representing the three different kinds of items.
 */
var ItemTypeEnum = {
    CONSUMABLE: 1,
    GEAR: 2,
    MISC: 3
};

// END res\js\items\ItemTypeEnum.js

// res\js\items\misc\aPlusLotion.js

/**
 * The aPlusLotion class represents a vial of oil the kobold should never be able to get into his/her inventory.
 */
function aPlusLotion() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "aPlus Lotion";
    var itemDescription = "A glass vial containing a clear, viscous liquid, meant to be rubbed onto someone's skin. Increases sensitivity, and acts as a potent aphrodisiac.";
    var baseValue = 35;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.APLUSLOTION;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}

// END res\js\items\misc\aPlusLotion.js

// res\js\items\misc\BasiliskScales.js

/**
 * Looted in the cave.
 */
function BasiliskScales() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Basilisk Scales";
    var itemDescription = "The rock-like scales of a mythical basilisk. Sell them, or find a blacksmith to forge them into a set of armor.";
    var baseValue = 60;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BASILISK_SCALES;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {BasiliskScales-stripped} item A BasiliskScales object, stripped of private members and all functions.
     */
    this.load = function (item) {

    };
}

// END res\js\items\misc\BasiliskScales.js

// res\js\items\misc\CopperRing.js

/**
 * The CopperRing class represents a copper ring the kobold can steal.
 */
function CopperRing() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Copper Ring";
    var itemDescription = "A small copper ring. Doesn't fit around your fingers. Not worth a lot. Sell it anyway.";
    var baseValue = 20;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.COPPER_RING;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {CopperRing-stripped} copperring A CopperRing object, stripped of private members and all functions.
     */
    this.load = function (copperring) {

    };
}

// END res\js\items\misc\CopperRing.js

// res\js\items\misc\FancyGlasses.js

/**
 * The FancyGlasses class represents some glasses the kobold can steal.
 */
function FancyGlasses() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Gold-plated Spectacles";
    var itemDescription = "Fancy, expensive, sight-enhancing reading apparatus. Hand-made, and far too fragile for what it's worth.";
    var baseValue = 120;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.FANCYGLASSES;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {FancyGlasses-stripped} misc A FancyGlasses object, stripped of private members and all functions.
     */
    this.load = function (misc) {

    };
}

// END res\js\items\misc\FancyGlasses.js

// res\js\items\misc\GoldenNeedle.js

/**
 * The GoldenNeedle class represents a golden needle the kobold can get as a reward for saving the dragoness.
 */
function GoldenNeedle() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Golden Needle";
    var itemDescription = "A needle made out of solid gold. Used to stitch the most delicate of textiles.";
    var baseValue = 30;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.GOLDENNEEDLE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 190px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Rope-stripped} rope A Rope object, stripped of private members and all functions.
     */
    this.load = function (rope) {

    };
}

// END res\js\items\misc\GoldenNeedle.js

// res\js\items\misc\GoldRing.js

/**
 * The GoldRing class represents a golden ring the kobold can steal.
 */
function GoldRing() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Gold Ring";
    var itemDescription = "A small, finely engraved ring. Doesn't fit around your fingers. Worth quite a bit. Sell it!";
    var baseValue = 150;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.GOLDRING;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 210px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Misc-stripped} misc A CopperRing object, stripped of private members and all functions.
     */
    this.load = function (misc) {

    };
}

// END res\js\items\misc\GoldRing.js

// res\js\items\misc\key\BanditKey.js

/**
 * The BanditKey class represents a key stolen from a group of kobolds in the woods.
 */
function BanditKey() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Kobold Bandit Key";
    var itemDescription = "A key stolen from a group of kobold bandits in the woods. You have no idea where you can use this.";
    var baseValue = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.BANDITKEY;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Rope-stripped} rope A Rope object, stripped of private members and all functions.
     */
    this.load = function (rope) {

    };
}

// END res\js\items\misc\key\BanditKey.js

// res\js\items\misc\key\CassidyHouseKey.js

/**
 * The CassidyHouseKey class represents a key looted from Cassidy.
 */
function CassidyHouseKey() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Cassidy's House Key";
    var itemDescription = "The key to Cassidy's luxurious home in the hamlet.";
    var baseValue = 200;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.CASSIDY_HOUSE_KEY;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var lowerCase = itemName.toLowerCase();
        lowerCase = lowerCase.charAt(0).toUpperCase() + lowerCase.substr(1);
        var toReturn = caps ? itemName : lowerCase;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 200px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {CassidyHouseKey-stripped} key A CassidyHouseKey object, stripped of private members and all functions.
     */
    this.load = function (key) {

    };
}

// END res\js\items\misc\key\CassidyHouseKey.js

// res\js\items\misc\Lockpick.js

/**
 * The Lockpick class represents lockpicks the kobold can find, buy and use.
 */
function Lockpick() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Lockpick";
    var itemDescription = "A small, fragile, delicate piece of metal used to manipulate the tumblers of locks. Tools of the trade, for a thief like yourself.";
    var baseValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LOCKPICK;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 220px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Rope-stripped} rope A Rope object, stripped of private members and all functions.
     */
    this.load = function (rope) {

    };
}

// END res\js\items\misc\Lockpick.js

// res\js\items\misc\Rope.js

/**
 * The Rope class represents a piece of rope that the kobold can find or buy.
 */
function Rope() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Rope";
    var itemDescription = "A small length of rope. Useful in many situations.";
    var baseValue = 6;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.ROPE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 170px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Rope-stripped} rope A Rope object, stripped of private members and all functions.
     */
    this.load = function (rope) {

    };
}

// END res\js\items\misc\Rope.js

// res\js\items\misc\Silverware.js

/**
 * The Silverware class represents some silverware the kobold can steal.
 */
function Silverware() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Silverware";
    var itemDescription = "An incomplete set of fancy, engraved tableware. Sell it.";
    var baseValue = 80;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SILVERWARE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 150px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Silverware-stripped} silverware A Silverware object, stripped of private members and all functions.
     */
    this.load = function (silverware) {

    };
}

// END res\js\items\misc\Silverware.js

// res\js\items\misc\SmallMirror.js

/**
 * The SmallMirror class represents a small mirror the kobold can buy.
 */
function SmallMirror() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Small Mirror";
    var itemDescription = "A hand-crafted reflective surface. Looking at yourself makes you realize you've seen better days. Despite everything, it's still you.";
    var baseValue = 18;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.SMALL_MIRROR;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 230px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Misc-stripped} misc A Misc object, stripped of private members and all functions.
     */
    this.load = function (misc) {

    };
}

// END res\js\items\misc\SmallMirror.js

// res\js\items\misc\Stick.js

/**
 * The Stick class represents a small wooden stick.
 */
function Stick() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Stick";
    var itemDescription = "A wooden stick. Dogs like these.";
    var baseValue = 1;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.STICK;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.MISC;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 150px;";
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {Stick-stripped} stick A Stick object, stripped of private members and all functions.
     */
    this.load = function (stick) {

    };
}

// END res\js\items\misc\Stick.js

// res\js\lib\jquery-3.2.1.min.js

/*! jQuery v3.2.1 | (c) JS Foundation and other contributors | jquery.org/license */
!function(a,b){"use strict";"object"==typeof module&&"object"==typeof module.exports?module.exports=a.document?b(a,!0):function(a){if(!a.document)throw new Error("jQuery requires a window with a document");return b(a)}:b(a)}("undefined"!=typeof window?window:this,function(a,b){"use strict";var c=[],d=a.document,e=Object.getPrototypeOf,f=c.slice,g=c.concat,h=c.push,i=c.indexOf,j={},k=j.toString,l=j.hasOwnProperty,m=l.toString,n=m.call(Object),o={};function p(a,b){b=b||d;var c=b.createElement("script");c.text=a,b.head.appendChild(c).parentNode.removeChild(c)}var q="3.2.1",r=function(a,b){return new r.fn.init(a,b)},s=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,t=/^-ms-/,u=/-([a-z])/g,v=function(a,b){return b.toUpperCase()};r.fn=r.prototype={jquery:q,constructor:r,length:0,toArray:function(){return f.call(this)},get:function(a){return null==a?f.call(this):a<0?this[a+this.length]:this[a]},pushStack:function(a){var b=r.merge(this.constructor(),a);return b.prevObject=this,b},each:function(a){return r.each(this,a)},map:function(a){return this.pushStack(r.map(this,function(b,c){return a.call(b,c,b)}))},slice:function(){return this.pushStack(f.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(a){var b=this.length,c=+a+(a<0?b:0);return this.pushStack(c>=0&&c<b?[this[c]]:[])},end:function(){return this.prevObject||this.constructor()},push:h,sort:c.sort,splice:c.splice},r.extend=r.fn.extend=function(){var a,b,c,d,e,f,g=arguments[0]||{},h=1,i=arguments.length,j=!1;for("boolean"==typeof g&&(j=g,g=arguments[h]||{},h++),"object"==typeof g||r.isFunction(g)||(g={}),h===i&&(g=this,h--);h<i;h++)if(null!=(a=arguments[h]))for(b in a)c=g[b],d=a[b],g!==d&&(j&&d&&(r.isPlainObject(d)||(e=Array.isArray(d)))?(e?(e=!1,f=c&&Array.isArray(c)?c:[]):f=c&&r.isPlainObject(c)?c:{},g[b]=r.extend(j,f,d)):void 0!==d&&(g[b]=d));return g},r.extend({expando:"jQuery"+(q+Math.random()).replace(/\D/g,""),isReady:!0,error:function(a){throw new Error(a)},noop:function(){},isFunction:function(a){return"function"===r.type(a)},isWindow:function(a){return null!=a&&a===a.window},isNumeric:function(a){var b=r.type(a);return("number"===b||"string"===b)&&!isNaN(a-parseFloat(a))},isPlainObject:function(a){var b,c;return!(!a||"[object Object]"!==k.call(a))&&(!(b=e(a))||(c=l.call(b,"constructor")&&b.constructor,"function"==typeof c&&m.call(c)===n))},isEmptyObject:function(a){var b;for(b in a)return!1;return!0},type:function(a){return null==a?a+"":"object"==typeof a||"function"==typeof a?j[k.call(a)]||"object":typeof a},globalEval:function(a){p(a)},camelCase:function(a){return a.replace(t,"ms-").replace(u,v)},each:function(a,b){var c,d=0;if(w(a)){for(c=a.length;d<c;d++)if(b.call(a[d],d,a[d])===!1)break}else for(d in a)if(b.call(a[d],d,a[d])===!1)break;return a},trim:function(a){return null==a?"":(a+"").replace(s,"")},makeArray:function(a,b){var c=b||[];return null!=a&&(w(Object(a))?r.merge(c,"string"==typeof a?[a]:a):h.call(c,a)),c},inArray:function(a,b,c){return null==b?-1:i.call(b,a,c)},merge:function(a,b){for(var c=+b.length,d=0,e=a.length;d<c;d++)a[e++]=b[d];return a.length=e,a},grep:function(a,b,c){for(var d,e=[],f=0,g=a.length,h=!c;f<g;f++)d=!b(a[f],f),d!==h&&e.push(a[f]);return e},map:function(a,b,c){var d,e,f=0,h=[];if(w(a))for(d=a.length;f<d;f++)e=b(a[f],f,c),null!=e&&h.push(e);else for(f in a)e=b(a[f],f,c),null!=e&&h.push(e);return g.apply([],h)},guid:1,proxy:function(a,b){var c,d,e;if("string"==typeof b&&(c=a[b],b=a,a=c),r.isFunction(a))return d=f.call(arguments,2),e=function(){return a.apply(b||this,d.concat(f.call(arguments)))},e.guid=a.guid=a.guid||r.guid++,e},now:Date.now,support:o}),"function"==typeof Symbol&&(r.fn[Symbol.iterator]=c[Symbol.iterator]),r.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(a,b){j["[object "+b+"]"]=b.toLowerCase()});function w(a){var b=!!a&&"length"in a&&a.length,c=r.type(a);return"function"!==c&&!r.isWindow(a)&&("array"===c||0===b||"number"==typeof b&&b>0&&b-1 in a)}var x=function(a){var b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u="sizzle"+1*new Date,v=a.document,w=0,x=0,y=ha(),z=ha(),A=ha(),B=function(a,b){return a===b&&(l=!0),0},C={}.hasOwnProperty,D=[],E=D.pop,F=D.push,G=D.push,H=D.slice,I=function(a,b){for(var c=0,d=a.length;c<d;c++)if(a[c]===b)return c;return-1},J="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",K="[\\x20\\t\\r\\n\\f]",L="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",M="\\["+K+"*("+L+")(?:"+K+"*([*^$|!~]?=)"+K+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+L+"))|)"+K+"*\\]",N=":("+L+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+M+")*)|.*)\\)|)",O=new RegExp(K+"+","g"),P=new RegExp("^"+K+"+|((?:^|[^\\\\])(?:\\\\.)*)"+K+"+$","g"),Q=new RegExp("^"+K+"*,"+K+"*"),R=new RegExp("^"+K+"*([>+~]|"+K+")"+K+"*"),S=new RegExp("="+K+"*([^\\]'\"]*?)"+K+"*\\]","g"),T=new RegExp(N),U=new RegExp("^"+L+"$"),V={ID:new RegExp("^#("+L+")"),CLASS:new RegExp("^\\.("+L+")"),TAG:new RegExp("^("+L+"|[*])"),ATTR:new RegExp("^"+M),PSEUDO:new RegExp("^"+N),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+K+"*(even|odd|(([+-]|)(\\d*)n|)"+K+"*(?:([+-]|)"+K+"*(\\d+)|))"+K+"*\\)|)","i"),bool:new RegExp("^(?:"+J+")$","i"),needsContext:new RegExp("^"+K+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+K+"*((?:-\\d)?\\d*)"+K+"*\\)|)(?=[^-]|$)","i")},W=/^(?:input|select|textarea|button)$/i,X=/^h\d$/i,Y=/^[^{]+\{\s*\[native \w/,Z=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,$=/[+~]/,_=new RegExp("\\\\([\\da-f]{1,6}"+K+"?|("+K+")|.)","ig"),aa=function(a,b,c){var d="0x"+b-65536;return d!==d||c?b:d<0?String.fromCharCode(d+65536):String.fromCharCode(d>>10|55296,1023&d|56320)},ba=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,ca=function(a,b){return b?"\0"===a?"\ufffd":a.slice(0,-1)+"\\"+a.charCodeAt(a.length-1).toString(16)+" ":"\\"+a},da=function(){m()},ea=ta(function(a){return a.disabled===!0&&("form"in a||"label"in a)},{dir:"parentNode",next:"legend"});try{G.apply(D=H.call(v.childNodes),v.childNodes),D[v.childNodes.length].nodeType}catch(fa){G={apply:D.length?function(a,b){F.apply(a,H.call(b))}:function(a,b){var c=a.length,d=0;while(a[c++]=b[d++]);a.length=c-1}}}function ga(a,b,d,e){var f,h,j,k,l,o,r,s=b&&b.ownerDocument,w=b?b.nodeType:9;if(d=d||[],"string"!=typeof a||!a||1!==w&&9!==w&&11!==w)return d;if(!e&&((b?b.ownerDocument||b:v)!==n&&m(b),b=b||n,p)){if(11!==w&&(l=Z.exec(a)))if(f=l[1]){if(9===w){if(!(j=b.getElementById(f)))return d;if(j.id===f)return d.push(j),d}else if(s&&(j=s.getElementById(f))&&t(b,j)&&j.id===f)return d.push(j),d}else{if(l[2])return G.apply(d,b.getElementsByTagName(a)),d;if((f=l[3])&&c.getElementsByClassName&&b.getElementsByClassName)return G.apply(d,b.getElementsByClassName(f)),d}if(c.qsa&&!A[a+" "]&&(!q||!q.test(a))){if(1!==w)s=b,r=a;else if("object"!==b.nodeName.toLowerCase()){(k=b.getAttribute("id"))?k=k.replace(ba,ca):b.setAttribute("id",k=u),o=g(a),h=o.length;while(h--)o[h]="#"+k+" "+sa(o[h]);r=o.join(","),s=$.test(a)&&qa(b.parentNode)||b}if(r)try{return G.apply(d,s.querySelectorAll(r)),d}catch(x){}finally{k===u&&b.removeAttribute("id")}}}return i(a.replace(P,"$1"),b,d,e)}function ha(){var a=[];function b(c,e){return a.push(c+" ")>d.cacheLength&&delete b[a.shift()],b[c+" "]=e}return b}function ia(a){return a[u]=!0,a}function ja(a){var b=n.createElement("fieldset");try{return!!a(b)}catch(c){return!1}finally{b.parentNode&&b.parentNode.removeChild(b),b=null}}function ka(a,b){var c=a.split("|"),e=c.length;while(e--)d.attrHandle[c[e]]=b}function la(a,b){var c=b&&a,d=c&&1===a.nodeType&&1===b.nodeType&&a.sourceIndex-b.sourceIndex;if(d)return d;if(c)while(c=c.nextSibling)if(c===b)return-1;return a?1:-1}function ma(a){return function(b){var c=b.nodeName.toLowerCase();return"input"===c&&b.type===a}}function na(a){return function(b){var c=b.nodeName.toLowerCase();return("input"===c||"button"===c)&&b.type===a}}function oa(a){return function(b){return"form"in b?b.parentNode&&b.disabled===!1?"label"in b?"label"in b.parentNode?b.parentNode.disabled===a:b.disabled===a:b.isDisabled===a||b.isDisabled!==!a&&ea(b)===a:b.disabled===a:"label"in b&&b.disabled===a}}function pa(a){return ia(function(b){return b=+b,ia(function(c,d){var e,f=a([],c.length,b),g=f.length;while(g--)c[e=f[g]]&&(c[e]=!(d[e]=c[e]))})})}function qa(a){return a&&"undefined"!=typeof a.getElementsByTagName&&a}c=ga.support={},f=ga.isXML=function(a){var b=a&&(a.ownerDocument||a).documentElement;return!!b&&"HTML"!==b.nodeName},m=ga.setDocument=function(a){var b,e,g=a?a.ownerDocument||a:v;return g!==n&&9===g.nodeType&&g.documentElement?(n=g,o=n.documentElement,p=!f(n),v!==n&&(e=n.defaultView)&&e.top!==e&&(e.addEventListener?e.addEventListener("unload",da,!1):e.attachEvent&&e.attachEvent("onunload",da)),c.attributes=ja(function(a){return a.className="i",!a.getAttribute("className")}),c.getElementsByTagName=ja(function(a){return a.appendChild(n.createComment("")),!a.getElementsByTagName("*").length}),c.getElementsByClassName=Y.test(n.getElementsByClassName),c.getById=ja(function(a){return o.appendChild(a).id=u,!n.getElementsByName||!n.getElementsByName(u).length}),c.getById?(d.filter.ID=function(a){var b=a.replace(_,aa);return function(a){return a.getAttribute("id")===b}},d.find.ID=function(a,b){if("undefined"!=typeof b.getElementById&&p){var c=b.getElementById(a);return c?[c]:[]}}):(d.filter.ID=function(a){var b=a.replace(_,aa);return function(a){var c="undefined"!=typeof a.getAttributeNode&&a.getAttributeNode("id");return c&&c.value===b}},d.find.ID=function(a,b){if("undefined"!=typeof b.getElementById&&p){var c,d,e,f=b.getElementById(a);if(f){if(c=f.getAttributeNode("id"),c&&c.value===a)return[f];e=b.getElementsByName(a),d=0;while(f=e[d++])if(c=f.getAttributeNode("id"),c&&c.value===a)return[f]}return[]}}),d.find.TAG=c.getElementsByTagName?function(a,b){return"undefined"!=typeof b.getElementsByTagName?b.getElementsByTagName(a):c.qsa?b.querySelectorAll(a):void 0}:function(a,b){var c,d=[],e=0,f=b.getElementsByTagName(a);if("*"===a){while(c=f[e++])1===c.nodeType&&d.push(c);return d}return f},d.find.CLASS=c.getElementsByClassName&&function(a,b){if("undefined"!=typeof b.getElementsByClassName&&p)return b.getElementsByClassName(a)},r=[],q=[],(c.qsa=Y.test(n.querySelectorAll))&&(ja(function(a){o.appendChild(a).innerHTML="<a id='"+u+"'></a><select id='"+u+"-\r\\' msallowcapture=''><option selected=''></option></select>",a.querySelectorAll("[msallowcapture^='']").length&&q.push("[*^$]="+K+"*(?:''|\"\")"),a.querySelectorAll("[selected]").length||q.push("\\["+K+"*(?:value|"+J+")"),a.querySelectorAll("[id~="+u+"-]").length||q.push("~="),a.querySelectorAll(":checked").length||q.push(":checked"),a.querySelectorAll("a#"+u+"+*").length||q.push(".#.+[+~]")}),ja(function(a){a.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var b=n.createElement("input");b.setAttribute("type","hidden"),a.appendChild(b).setAttribute("name","D"),a.querySelectorAll("[name=d]").length&&q.push("name"+K+"*[*^$|!~]?="),2!==a.querySelectorAll(":enabled").length&&q.push(":enabled",":disabled"),o.appendChild(a).disabled=!0,2!==a.querySelectorAll(":disabled").length&&q.push(":enabled",":disabled"),a.querySelectorAll("*,:x"),q.push(",.*:")})),(c.matchesSelector=Y.test(s=o.matches||o.webkitMatchesSelector||o.mozMatchesSelector||o.oMatchesSelector||o.msMatchesSelector))&&ja(function(a){c.disconnectedMatch=s.call(a,"*"),s.call(a,"[s!='']:x"),r.push("!=",N)}),q=q.length&&new RegExp(q.join("|")),r=r.length&&new RegExp(r.join("|")),b=Y.test(o.compareDocumentPosition),t=b||Y.test(o.contains)?function(a,b){var c=9===a.nodeType?a.documentElement:a,d=b&&b.parentNode;return a===d||!(!d||1!==d.nodeType||!(c.contains?c.contains(d):a.compareDocumentPosition&&16&a.compareDocumentPosition(d)))}:function(a,b){if(b)while(b=b.parentNode)if(b===a)return!0;return!1},B=b?function(a,b){if(a===b)return l=!0,0;var d=!a.compareDocumentPosition-!b.compareDocumentPosition;return d?d:(d=(a.ownerDocument||a)===(b.ownerDocument||b)?a.compareDocumentPosition(b):1,1&d||!c.sortDetached&&b.compareDocumentPosition(a)===d?a===n||a.ownerDocument===v&&t(v,a)?-1:b===n||b.ownerDocument===v&&t(v,b)?1:k?I(k,a)-I(k,b):0:4&d?-1:1)}:function(a,b){if(a===b)return l=!0,0;var c,d=0,e=a.parentNode,f=b.parentNode,g=[a],h=[b];if(!e||!f)return a===n?-1:b===n?1:e?-1:f?1:k?I(k,a)-I(k,b):0;if(e===f)return la(a,b);c=a;while(c=c.parentNode)g.unshift(c);c=b;while(c=c.parentNode)h.unshift(c);while(g[d]===h[d])d++;return d?la(g[d],h[d]):g[d]===v?-1:h[d]===v?1:0},n):n},ga.matches=function(a,b){return ga(a,null,null,b)},ga.matchesSelector=function(a,b){if((a.ownerDocument||a)!==n&&m(a),b=b.replace(S,"='$1']"),c.matchesSelector&&p&&!A[b+" "]&&(!r||!r.test(b))&&(!q||!q.test(b)))try{var d=s.call(a,b);if(d||c.disconnectedMatch||a.document&&11!==a.document.nodeType)return d}catch(e){}return ga(b,n,null,[a]).length>0},ga.contains=function(a,b){return(a.ownerDocument||a)!==n&&m(a),t(a,b)},ga.attr=function(a,b){(a.ownerDocument||a)!==n&&m(a);var e=d.attrHandle[b.toLowerCase()],f=e&&C.call(d.attrHandle,b.toLowerCase())?e(a,b,!p):void 0;return void 0!==f?f:c.attributes||!p?a.getAttribute(b):(f=a.getAttributeNode(b))&&f.specified?f.value:null},ga.escape=function(a){return(a+"").replace(ba,ca)},ga.error=function(a){throw new Error("Syntax error, unrecognized expression: "+a)},ga.uniqueSort=function(a){var b,d=[],e=0,f=0;if(l=!c.detectDuplicates,k=!c.sortStable&&a.slice(0),a.sort(B),l){while(b=a[f++])b===a[f]&&(e=d.push(f));while(e--)a.splice(d[e],1)}return k=null,a},e=ga.getText=function(a){var b,c="",d=0,f=a.nodeType;if(f){if(1===f||9===f||11===f){if("string"==typeof a.textContent)return a.textContent;for(a=a.firstChild;a;a=a.nextSibling)c+=e(a)}else if(3===f||4===f)return a.nodeValue}else while(b=a[d++])c+=e(b);return c},d=ga.selectors={cacheLength:50,createPseudo:ia,match:V,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(a){return a[1]=a[1].replace(_,aa),a[3]=(a[3]||a[4]||a[5]||"").replace(_,aa),"~="===a[2]&&(a[3]=" "+a[3]+" "),a.slice(0,4)},CHILD:function(a){return a[1]=a[1].toLowerCase(),"nth"===a[1].slice(0,3)?(a[3]||ga.error(a[0]),a[4]=+(a[4]?a[5]+(a[6]||1):2*("even"===a[3]||"odd"===a[3])),a[5]=+(a[7]+a[8]||"odd"===a[3])):a[3]&&ga.error(a[0]),a},PSEUDO:function(a){var b,c=!a[6]&&a[2];return V.CHILD.test(a[0])?null:(a[3]?a[2]=a[4]||a[5]||"":c&&T.test(c)&&(b=g(c,!0))&&(b=c.indexOf(")",c.length-b)-c.length)&&(a[0]=a[0].slice(0,b),a[2]=c.slice(0,b)),a.slice(0,3))}},filter:{TAG:function(a){var b=a.replace(_,aa).toLowerCase();return"*"===a?function(){return!0}:function(a){return a.nodeName&&a.nodeName.toLowerCase()===b}},CLASS:function(a){var b=y[a+" "];return b||(b=new RegExp("(^|"+K+")"+a+"("+K+"|$)"))&&y(a,function(a){return b.test("string"==typeof a.className&&a.className||"undefined"!=typeof a.getAttribute&&a.getAttribute("class")||"")})},ATTR:function(a,b,c){return function(d){var e=ga.attr(d,a);return null==e?"!="===b:!b||(e+="","="===b?e===c:"!="===b?e!==c:"^="===b?c&&0===e.indexOf(c):"*="===b?c&&e.indexOf(c)>-1:"$="===b?c&&e.slice(-c.length)===c:"~="===b?(" "+e.replace(O," ")+" ").indexOf(c)>-1:"|="===b&&(e===c||e.slice(0,c.length+1)===c+"-"))}},CHILD:function(a,b,c,d,e){var f="nth"!==a.slice(0,3),g="last"!==a.slice(-4),h="of-type"===b;return 1===d&&0===e?function(a){return!!a.parentNode}:function(b,c,i){var j,k,l,m,n,o,p=f!==g?"nextSibling":"previousSibling",q=b.parentNode,r=h&&b.nodeName.toLowerCase(),s=!i&&!h,t=!1;if(q){if(f){while(p){m=b;while(m=m[p])if(h?m.nodeName.toLowerCase()===r:1===m.nodeType)return!1;o=p="only"===a&&!o&&"nextSibling"}return!0}if(o=[g?q.firstChild:q.lastChild],g&&s){m=q,l=m[u]||(m[u]={}),k=l[m.uniqueID]||(l[m.uniqueID]={}),j=k[a]||[],n=j[0]===w&&j[1],t=n&&j[2],m=n&&q.childNodes[n];while(m=++n&&m&&m[p]||(t=n=0)||o.pop())if(1===m.nodeType&&++t&&m===b){k[a]=[w,n,t];break}}else if(s&&(m=b,l=m[u]||(m[u]={}),k=l[m.uniqueID]||(l[m.uniqueID]={}),j=k[a]||[],n=j[0]===w&&j[1],t=n),t===!1)while(m=++n&&m&&m[p]||(t=n=0)||o.pop())if((h?m.nodeName.toLowerCase()===r:1===m.nodeType)&&++t&&(s&&(l=m[u]||(m[u]={}),k=l[m.uniqueID]||(l[m.uniqueID]={}),k[a]=[w,t]),m===b))break;return t-=e,t===d||t%d===0&&t/d>=0}}},PSEUDO:function(a,b){var c,e=d.pseudos[a]||d.setFilters[a.toLowerCase()]||ga.error("unsupported pseudo: "+a);return e[u]?e(b):e.length>1?(c=[a,a,"",b],d.setFilters.hasOwnProperty(a.toLowerCase())?ia(function(a,c){var d,f=e(a,b),g=f.length;while(g--)d=I(a,f[g]),a[d]=!(c[d]=f[g])}):function(a){return e(a,0,c)}):e}},pseudos:{not:ia(function(a){var b=[],c=[],d=h(a.replace(P,"$1"));return d[u]?ia(function(a,b,c,e){var f,g=d(a,null,e,[]),h=a.length;while(h--)(f=g[h])&&(a[h]=!(b[h]=f))}):function(a,e,f){return b[0]=a,d(b,null,f,c),b[0]=null,!c.pop()}}),has:ia(function(a){return function(b){return ga(a,b).length>0}}),contains:ia(function(a){return a=a.replace(_,aa),function(b){return(b.textContent||b.innerText||e(b)).indexOf(a)>-1}}),lang:ia(function(a){return U.test(a||"")||ga.error("unsupported lang: "+a),a=a.replace(_,aa).toLowerCase(),function(b){var c;do if(c=p?b.lang:b.getAttribute("xml:lang")||b.getAttribute("lang"))return c=c.toLowerCase(),c===a||0===c.indexOf(a+"-");while((b=b.parentNode)&&1===b.nodeType);return!1}}),target:function(b){var c=a.location&&a.location.hash;return c&&c.slice(1)===b.id},root:function(a){return a===o},focus:function(a){return a===n.activeElement&&(!n.hasFocus||n.hasFocus())&&!!(a.type||a.href||~a.tabIndex)},enabled:oa(!1),disabled:oa(!0),checked:function(a){var b=a.nodeName.toLowerCase();return"input"===b&&!!a.checked||"option"===b&&!!a.selected},selected:function(a){return a.parentNode&&a.parentNode.selectedIndex,a.selected===!0},empty:function(a){for(a=a.firstChild;a;a=a.nextSibling)if(a.nodeType<6)return!1;return!0},parent:function(a){return!d.pseudos.empty(a)},header:function(a){return X.test(a.nodeName)},input:function(a){return W.test(a.nodeName)},button:function(a){var b=a.nodeName.toLowerCase();return"input"===b&&"button"===a.type||"button"===b},text:function(a){var b;return"input"===a.nodeName.toLowerCase()&&"text"===a.type&&(null==(b=a.getAttribute("type"))||"text"===b.toLowerCase())},first:pa(function(){return[0]}),last:pa(function(a,b){return[b-1]}),eq:pa(function(a,b,c){return[c<0?c+b:c]}),even:pa(function(a,b){for(var c=0;c<b;c+=2)a.push(c);return a}),odd:pa(function(a,b){for(var c=1;c<b;c+=2)a.push(c);return a}),lt:pa(function(a,b,c){for(var d=c<0?c+b:c;--d>=0;)a.push(d);return a}),gt:pa(function(a,b,c){for(var d=c<0?c+b:c;++d<b;)a.push(d);return a})}},d.pseudos.nth=d.pseudos.eq;for(b in{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})d.pseudos[b]=ma(b);for(b in{submit:!0,reset:!0})d.pseudos[b]=na(b);function ra(){}ra.prototype=d.filters=d.pseudos,d.setFilters=new ra,g=ga.tokenize=function(a,b){var c,e,f,g,h,i,j,k=z[a+" "];if(k)return b?0:k.slice(0);h=a,i=[],j=d.preFilter;while(h){c&&!(e=Q.exec(h))||(e&&(h=h.slice(e[0].length)||h),i.push(f=[])),c=!1,(e=R.exec(h))&&(c=e.shift(),f.push({value:c,type:e[0].replace(P," ")}),h=h.slice(c.length));for(g in d.filter)!(e=V[g].exec(h))||j[g]&&!(e=j[g](e))||(c=e.shift(),f.push({value:c,type:g,matches:e}),h=h.slice(c.length));if(!c)break}return b?h.length:h?ga.error(a):z(a,i).slice(0)};function sa(a){for(var b=0,c=a.length,d="";b<c;b++)d+=a[b].value;return d}function ta(a,b,c){var d=b.dir,e=b.next,f=e||d,g=c&&"parentNode"===f,h=x++;return b.first?function(b,c,e){while(b=b[d])if(1===b.nodeType||g)return a(b,c,e);return!1}:function(b,c,i){var j,k,l,m=[w,h];if(i){while(b=b[d])if((1===b.nodeType||g)&&a(b,c,i))return!0}else while(b=b[d])if(1===b.nodeType||g)if(l=b[u]||(b[u]={}),k=l[b.uniqueID]||(l[b.uniqueID]={}),e&&e===b.nodeName.toLowerCase())b=b[d]||b;else{if((j=k[f])&&j[0]===w&&j[1]===h)return m[2]=j[2];if(k[f]=m,m[2]=a(b,c,i))return!0}return!1}}function ua(a){return a.length>1?function(b,c,d){var e=a.length;while(e--)if(!a[e](b,c,d))return!1;return!0}:a[0]}function va(a,b,c){for(var d=0,e=b.length;d<e;d++)ga(a,b[d],c);return c}function wa(a,b,c,d,e){for(var f,g=[],h=0,i=a.length,j=null!=b;h<i;h++)(f=a[h])&&(c&&!c(f,d,e)||(g.push(f),j&&b.push(h)));return g}function xa(a,b,c,d,e,f){return d&&!d[u]&&(d=xa(d)),e&&!e[u]&&(e=xa(e,f)),ia(function(f,g,h,i){var j,k,l,m=[],n=[],o=g.length,p=f||va(b||"*",h.nodeType?[h]:h,[]),q=!a||!f&&b?p:wa(p,m,a,h,i),r=c?e||(f?a:o||d)?[]:g:q;if(c&&c(q,r,h,i),d){j=wa(r,n),d(j,[],h,i),k=j.length;while(k--)(l=j[k])&&(r[n[k]]=!(q[n[k]]=l))}if(f){if(e||a){if(e){j=[],k=r.length;while(k--)(l=r[k])&&j.push(q[k]=l);e(null,r=[],j,i)}k=r.length;while(k--)(l=r[k])&&(j=e?I(f,l):m[k])>-1&&(f[j]=!(g[j]=l))}}else r=wa(r===g?r.splice(o,r.length):r),e?e(null,g,r,i):G.apply(g,r)})}function ya(a){for(var b,c,e,f=a.length,g=d.relative[a[0].type],h=g||d.relative[" "],i=g?1:0,k=ta(function(a){return a===b},h,!0),l=ta(function(a){return I(b,a)>-1},h,!0),m=[function(a,c,d){var e=!g&&(d||c!==j)||((b=c).nodeType?k(a,c,d):l(a,c,d));return b=null,e}];i<f;i++)if(c=d.relative[a[i].type])m=[ta(ua(m),c)];else{if(c=d.filter[a[i].type].apply(null,a[i].matches),c[u]){for(e=++i;e<f;e++)if(d.relative[a[e].type])break;return xa(i>1&&ua(m),i>1&&sa(a.slice(0,i-1).concat({value:" "===a[i-2].type?"*":""})).replace(P,"$1"),c,i<e&&ya(a.slice(i,e)),e<f&&ya(a=a.slice(e)),e<f&&sa(a))}m.push(c)}return ua(m)}function za(a,b){var c=b.length>0,e=a.length>0,f=function(f,g,h,i,k){var l,o,q,r=0,s="0",t=f&&[],u=[],v=j,x=f||e&&d.find.TAG("*",k),y=w+=null==v?1:Math.random()||.1,z=x.length;for(k&&(j=g===n||g||k);s!==z&&null!=(l=x[s]);s++){if(e&&l){o=0,g||l.ownerDocument===n||(m(l),h=!p);while(q=a[o++])if(q(l,g||n,h)){i.push(l);break}k&&(w=y)}c&&((l=!q&&l)&&r--,f&&t.push(l))}if(r+=s,c&&s!==r){o=0;while(q=b[o++])q(t,u,g,h);if(f){if(r>0)while(s--)t[s]||u[s]||(u[s]=E.call(i));u=wa(u)}G.apply(i,u),k&&!f&&u.length>0&&r+b.length>1&&ga.uniqueSort(i)}return k&&(w=y,j=v),t};return c?ia(f):f}return h=ga.compile=function(a,b){var c,d=[],e=[],f=A[a+" "];if(!f){b||(b=g(a)),c=b.length;while(c--)f=ya(b[c]),f[u]?d.push(f):e.push(f);f=A(a,za(e,d)),f.selector=a}return f},i=ga.select=function(a,b,c,e){var f,i,j,k,l,m="function"==typeof a&&a,n=!e&&g(a=m.selector||a);if(c=c||[],1===n.length){if(i=n[0]=n[0].slice(0),i.length>2&&"ID"===(j=i[0]).type&&9===b.nodeType&&p&&d.relative[i[1].type]){if(b=(d.find.ID(j.matches[0].replace(_,aa),b)||[])[0],!b)return c;m&&(b=b.parentNode),a=a.slice(i.shift().value.length)}f=V.needsContext.test(a)?0:i.length;while(f--){if(j=i[f],d.relative[k=j.type])break;if((l=d.find[k])&&(e=l(j.matches[0].replace(_,aa),$.test(i[0].type)&&qa(b.parentNode)||b))){if(i.splice(f,1),a=e.length&&sa(i),!a)return G.apply(c,e),c;break}}}return(m||h(a,n))(e,b,!p,c,!b||$.test(a)&&qa(b.parentNode)||b),c},c.sortStable=u.split("").sort(B).join("")===u,c.detectDuplicates=!!l,m(),c.sortDetached=ja(function(a){return 1&a.compareDocumentPosition(n.createElement("fieldset"))}),ja(function(a){return a.innerHTML="<a href='#'></a>","#"===a.firstChild.getAttribute("href")})||ka("type|href|height|width",function(a,b,c){if(!c)return a.getAttribute(b,"type"===b.toLowerCase()?1:2)}),c.attributes&&ja(function(a){return a.innerHTML="<input/>",a.firstChild.setAttribute("value",""),""===a.firstChild.getAttribute("value")})||ka("value",function(a,b,c){if(!c&&"input"===a.nodeName.toLowerCase())return a.defaultValue}),ja(function(a){return null==a.getAttribute("disabled")})||ka(J,function(a,b,c){var d;if(!c)return a[b]===!0?b.toLowerCase():(d=a.getAttributeNode(b))&&d.specified?d.value:null}),ga}(a);r.find=x,r.expr=x.selectors,r.expr[":"]=r.expr.pseudos,r.uniqueSort=r.unique=x.uniqueSort,r.text=x.getText,r.isXMLDoc=x.isXML,r.contains=x.contains,r.escapeSelector=x.escape;var y=function(a,b,c){var d=[],e=void 0!==c;while((a=a[b])&&9!==a.nodeType)if(1===a.nodeType){if(e&&r(a).is(c))break;d.push(a)}return d},z=function(a,b){for(var c=[];a;a=a.nextSibling)1===a.nodeType&&a!==b&&c.push(a);return c},A=r.expr.match.needsContext;function B(a,b){return a.nodeName&&a.nodeName.toLowerCase()===b.toLowerCase()}var C=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i,D=/^.[^:#\[\.,]*$/;function E(a,b,c){return r.isFunction(b)?r.grep(a,function(a,d){return!!b.call(a,d,a)!==c}):b.nodeType?r.grep(a,function(a){return a===b!==c}):"string"!=typeof b?r.grep(a,function(a){return i.call(b,a)>-1!==c}):D.test(b)?r.filter(b,a,c):(b=r.filter(b,a),r.grep(a,function(a){return i.call(b,a)>-1!==c&&1===a.nodeType}))}r.filter=function(a,b,c){var d=b[0];return c&&(a=":not("+a+")"),1===b.length&&1===d.nodeType?r.find.matchesSelector(d,a)?[d]:[]:r.find.matches(a,r.grep(b,function(a){return 1===a.nodeType}))},r.fn.extend({find:function(a){var b,c,d=this.length,e=this;if("string"!=typeof a)return this.pushStack(r(a).filter(function(){for(b=0;b<d;b++)if(r.contains(e[b],this))return!0}));for(c=this.pushStack([]),b=0;b<d;b++)r.find(a,e[b],c);return d>1?r.uniqueSort(c):c},filter:function(a){return this.pushStack(E(this,a||[],!1))},not:function(a){return this.pushStack(E(this,a||[],!0))},is:function(a){return!!E(this,"string"==typeof a&&A.test(a)?r(a):a||[],!1).length}});var F,G=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,H=r.fn.init=function(a,b,c){var e,f;if(!a)return this;if(c=c||F,"string"==typeof a){if(e="<"===a[0]&&">"===a[a.length-1]&&a.length>=3?[null,a,null]:G.exec(a),!e||!e[1]&&b)return!b||b.jquery?(b||c).find(a):this.constructor(b).find(a);if(e[1]){if(b=b instanceof r?b[0]:b,r.merge(this,r.parseHTML(e[1],b&&b.nodeType?b.ownerDocument||b:d,!0)),C.test(e[1])&&r.isPlainObject(b))for(e in b)r.isFunction(this[e])?this[e](b[e]):this.attr(e,b[e]);return this}return f=d.getElementById(e[2]),f&&(this[0]=f,this.length=1),this}return a.nodeType?(this[0]=a,this.length=1,this):r.isFunction(a)?void 0!==c.ready?c.ready(a):a(r):r.makeArray(a,this)};H.prototype=r.fn,F=r(d);var I=/^(?:parents|prev(?:Until|All))/,J={children:!0,contents:!0,next:!0,prev:!0};r.fn.extend({has:function(a){var b=r(a,this),c=b.length;return this.filter(function(){for(var a=0;a<c;a++)if(r.contains(this,b[a]))return!0})},closest:function(a,b){var c,d=0,e=this.length,f=[],g="string"!=typeof a&&r(a);if(!A.test(a))for(;d<e;d++)for(c=this[d];c&&c!==b;c=c.parentNode)if(c.nodeType<11&&(g?g.index(c)>-1:1===c.nodeType&&r.find.matchesSelector(c,a))){f.push(c);break}return this.pushStack(f.length>1?r.uniqueSort(f):f)},index:function(a){return a?"string"==typeof a?i.call(r(a),this[0]):i.call(this,a.jquery?a[0]:a):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(a,b){return this.pushStack(r.uniqueSort(r.merge(this.get(),r(a,b))))},addBack:function(a){return this.add(null==a?this.prevObject:this.prevObject.filter(a))}});function K(a,b){while((a=a[b])&&1!==a.nodeType);return a}r.each({parent:function(a){var b=a.parentNode;return b&&11!==b.nodeType?b:null},parents:function(a){return y(a,"parentNode")},parentsUntil:function(a,b,c){return y(a,"parentNode",c)},next:function(a){return K(a,"nextSibling")},prev:function(a){return K(a,"previousSibling")},nextAll:function(a){return y(a,"nextSibling")},prevAll:function(a){return y(a,"previousSibling")},nextUntil:function(a,b,c){return y(a,"nextSibling",c)},prevUntil:function(a,b,c){return y(a,"previousSibling",c)},siblings:function(a){return z((a.parentNode||{}).firstChild,a)},children:function(a){return z(a.firstChild)},contents:function(a){return B(a,"iframe")?a.contentDocument:(B(a,"template")&&(a=a.content||a),r.merge([],a.childNodes))}},function(a,b){r.fn[a]=function(c,d){var e=r.map(this,b,c);return"Until"!==a.slice(-5)&&(d=c),d&&"string"==typeof d&&(e=r.filter(d,e)),this.length>1&&(J[a]||r.uniqueSort(e),I.test(a)&&e.reverse()),this.pushStack(e)}});var L=/[^\x20\t\r\n\f]+/g;function M(a){var b={};return r.each(a.match(L)||[],function(a,c){b[c]=!0}),b}r.Callbacks=function(a){a="string"==typeof a?M(a):r.extend({},a);var b,c,d,e,f=[],g=[],h=-1,i=function(){for(e=e||a.once,d=b=!0;g.length;h=-1){c=g.shift();while(++h<f.length)f[h].apply(c[0],c[1])===!1&&a.stopOnFalse&&(h=f.length,c=!1)}a.memory||(c=!1),b=!1,e&&(f=c?[]:"")},j={add:function(){return f&&(c&&!b&&(h=f.length-1,g.push(c)),function d(b){r.each(b,function(b,c){r.isFunction(c)?a.unique&&j.has(c)||f.push(c):c&&c.length&&"string"!==r.type(c)&&d(c)})}(arguments),c&&!b&&i()),this},remove:function(){return r.each(arguments,function(a,b){var c;while((c=r.inArray(b,f,c))>-1)f.splice(c,1),c<=h&&h--}),this},has:function(a){return a?r.inArray(a,f)>-1:f.length>0},empty:function(){return f&&(f=[]),this},disable:function(){return e=g=[],f=c="",this},disabled:function(){return!f},lock:function(){return e=g=[],c||b||(f=c=""),this},locked:function(){return!!e},fireWith:function(a,c){return e||(c=c||[],c=[a,c.slice?c.slice():c],g.push(c),b||i()),this},fire:function(){return j.fireWith(this,arguments),this},fired:function(){return!!d}};return j};function N(a){return a}function O(a){throw a}function P(a,b,c,d){var e;try{a&&r.isFunction(e=a.promise)?e.call(a).done(b).fail(c):a&&r.isFunction(e=a.then)?e.call(a,b,c):b.apply(void 0,[a].slice(d))}catch(a){c.apply(void 0,[a])}}r.extend({Deferred:function(b){var c=[["notify","progress",r.Callbacks("memory"),r.Callbacks("memory"),2],["resolve","done",r.Callbacks("once memory"),r.Callbacks("once memory"),0,"resolved"],["reject","fail",r.Callbacks("once memory"),r.Callbacks("once memory"),1,"rejected"]],d="pending",e={state:function(){return d},always:function(){return f.done(arguments).fail(arguments),this},"catch":function(a){return e.then(null,a)},pipe:function(){var a=arguments;return r.Deferred(function(b){r.each(c,function(c,d){var e=r.isFunction(a[d[4]])&&a[d[4]];f[d[1]](function(){var a=e&&e.apply(this,arguments);a&&r.isFunction(a.promise)?a.promise().progress(b.notify).done(b.resolve).fail(b.reject):b[d[0]+"With"](this,e?[a]:arguments)})}),a=null}).promise()},then:function(b,d,e){var f=0;function g(b,c,d,e){return function(){var h=this,i=arguments,j=function(){var a,j;if(!(b<f)){if(a=d.apply(h,i),a===c.promise())throw new TypeError("Thenable self-resolution");j=a&&("object"==typeof a||"function"==typeof a)&&a.then,r.isFunction(j)?e?j.call(a,g(f,c,N,e),g(f,c,O,e)):(f++,j.call(a,g(f,c,N,e),g(f,c,O,e),g(f,c,N,c.notifyWith))):(d!==N&&(h=void 0,i=[a]),(e||c.resolveWith)(h,i))}},k=e?j:function(){try{j()}catch(a){r.Deferred.exceptionHook&&r.Deferred.exceptionHook(a,k.stackTrace),b+1>=f&&(d!==O&&(h=void 0,i=[a]),c.rejectWith(h,i))}};b?k():(r.Deferred.getStackHook&&(k.stackTrace=r.Deferred.getStackHook()),a.setTimeout(k))}}return r.Deferred(function(a){c[0][3].add(g(0,a,r.isFunction(e)?e:N,a.notifyWith)),c[1][3].add(g(0,a,r.isFunction(b)?b:N)),c[2][3].add(g(0,a,r.isFunction(d)?d:O))}).promise()},promise:function(a){return null!=a?r.extend(a,e):e}},f={};return r.each(c,function(a,b){var g=b[2],h=b[5];e[b[1]]=g.add,h&&g.add(function(){d=h},c[3-a][2].disable,c[0][2].lock),g.add(b[3].fire),f[b[0]]=function(){return f[b[0]+"With"](this===f?void 0:this,arguments),this},f[b[0]+"With"]=g.fireWith}),e.promise(f),b&&b.call(f,f),f},when:function(a){var b=arguments.length,c=b,d=Array(c),e=f.call(arguments),g=r.Deferred(),h=function(a){return function(c){d[a]=this,e[a]=arguments.length>1?f.call(arguments):c,--b||g.resolveWith(d,e)}};if(b<=1&&(P(a,g.done(h(c)).resolve,g.reject,!b),"pending"===g.state()||r.isFunction(e[c]&&e[c].then)))return g.then();while(c--)P(e[c],h(c),g.reject);return g.promise()}});var Q=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;r.Deferred.exceptionHook=function(b,c){a.console&&a.console.warn&&b&&Q.test(b.name)&&a.console.warn("jQuery.Deferred exception: "+b.message,b.stack,c)},r.readyException=function(b){a.setTimeout(function(){throw b})};var R=r.Deferred();r.fn.ready=function(a){return R.then(a)["catch"](function(a){r.readyException(a)}),this},r.extend({isReady:!1,readyWait:1,ready:function(a){(a===!0?--r.readyWait:r.isReady)||(r.isReady=!0,a!==!0&&--r.readyWait>0||R.resolveWith(d,[r]))}}),r.ready.then=R.then;function S(){d.removeEventListener("DOMContentLoaded",S),
a.removeEventListener("load",S),r.ready()}"complete"===d.readyState||"loading"!==d.readyState&&!d.documentElement.doScroll?a.setTimeout(r.ready):(d.addEventListener("DOMContentLoaded",S),a.addEventListener("load",S));var T=function(a,b,c,d,e,f,g){var h=0,i=a.length,j=null==c;if("object"===r.type(c)){e=!0;for(h in c)T(a,b,h,c[h],!0,f,g)}else if(void 0!==d&&(e=!0,r.isFunction(d)||(g=!0),j&&(g?(b.call(a,d),b=null):(j=b,b=function(a,b,c){return j.call(r(a),c)})),b))for(;h<i;h++)b(a[h],c,g?d:d.call(a[h],h,b(a[h],c)));return e?a:j?b.call(a):i?b(a[0],c):f},U=function(a){return 1===a.nodeType||9===a.nodeType||!+a.nodeType};function V(){this.expando=r.expando+V.uid++}V.uid=1,V.prototype={cache:function(a){var b=a[this.expando];return b||(b={},U(a)&&(a.nodeType?a[this.expando]=b:Object.defineProperty(a,this.expando,{value:b,configurable:!0}))),b},set:function(a,b,c){var d,e=this.cache(a);if("string"==typeof b)e[r.camelCase(b)]=c;else for(d in b)e[r.camelCase(d)]=b[d];return e},get:function(a,b){return void 0===b?this.cache(a):a[this.expando]&&a[this.expando][r.camelCase(b)]},access:function(a,b,c){return void 0===b||b&&"string"==typeof b&&void 0===c?this.get(a,b):(this.set(a,b,c),void 0!==c?c:b)},remove:function(a,b){var c,d=a[this.expando];if(void 0!==d){if(void 0!==b){Array.isArray(b)?b=b.map(r.camelCase):(b=r.camelCase(b),b=b in d?[b]:b.match(L)||[]),c=b.length;while(c--)delete d[b[c]]}(void 0===b||r.isEmptyObject(d))&&(a.nodeType?a[this.expando]=void 0:delete a[this.expando])}},hasData:function(a){var b=a[this.expando];return void 0!==b&&!r.isEmptyObject(b)}};var W=new V,X=new V,Y=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,Z=/[A-Z]/g;function $(a){return"true"===a||"false"!==a&&("null"===a?null:a===+a+""?+a:Y.test(a)?JSON.parse(a):a)}function _(a,b,c){var d;if(void 0===c&&1===a.nodeType)if(d="data-"+b.replace(Z,"-$&").toLowerCase(),c=a.getAttribute(d),"string"==typeof c){try{c=$(c)}catch(e){}X.set(a,b,c)}else c=void 0;return c}r.extend({hasData:function(a){return X.hasData(a)||W.hasData(a)},data:function(a,b,c){return X.access(a,b,c)},removeData:function(a,b){X.remove(a,b)},_data:function(a,b,c){return W.access(a,b,c)},_removeData:function(a,b){W.remove(a,b)}}),r.fn.extend({data:function(a,b){var c,d,e,f=this[0],g=f&&f.attributes;if(void 0===a){if(this.length&&(e=X.get(f),1===f.nodeType&&!W.get(f,"hasDataAttrs"))){c=g.length;while(c--)g[c]&&(d=g[c].name,0===d.indexOf("data-")&&(d=r.camelCase(d.slice(5)),_(f,d,e[d])));W.set(f,"hasDataAttrs",!0)}return e}return"object"==typeof a?this.each(function(){X.set(this,a)}):T(this,function(b){var c;if(f&&void 0===b){if(c=X.get(f,a),void 0!==c)return c;if(c=_(f,a),void 0!==c)return c}else this.each(function(){X.set(this,a,b)})},null,b,arguments.length>1,null,!0)},removeData:function(a){return this.each(function(){X.remove(this,a)})}}),r.extend({queue:function(a,b,c){var d;if(a)return b=(b||"fx")+"queue",d=W.get(a,b),c&&(!d||Array.isArray(c)?d=W.access(a,b,r.makeArray(c)):d.push(c)),d||[]},dequeue:function(a,b){b=b||"fx";var c=r.queue(a,b),d=c.length,e=c.shift(),f=r._queueHooks(a,b),g=function(){r.dequeue(a,b)};"inprogress"===e&&(e=c.shift(),d--),e&&("fx"===b&&c.unshift("inprogress"),delete f.stop,e.call(a,g,f)),!d&&f&&f.empty.fire()},_queueHooks:function(a,b){var c=b+"queueHooks";return W.get(a,c)||W.access(a,c,{empty:r.Callbacks("once memory").add(function(){W.remove(a,[b+"queue",c])})})}}),r.fn.extend({queue:function(a,b){var c=2;return"string"!=typeof a&&(b=a,a="fx",c--),arguments.length<c?r.queue(this[0],a):void 0===b?this:this.each(function(){var c=r.queue(this,a,b);r._queueHooks(this,a),"fx"===a&&"inprogress"!==c[0]&&r.dequeue(this,a)})},dequeue:function(a){return this.each(function(){r.dequeue(this,a)})},clearQueue:function(a){return this.queue(a||"fx",[])},promise:function(a,b){var c,d=1,e=r.Deferred(),f=this,g=this.length,h=function(){--d||e.resolveWith(f,[f])};"string"!=typeof a&&(b=a,a=void 0),a=a||"fx";while(g--)c=W.get(f[g],a+"queueHooks"),c&&c.empty&&(d++,c.empty.add(h));return h(),e.promise(b)}});var aa=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,ba=new RegExp("^(?:([+-])=|)("+aa+")([a-z%]*)$","i"),ca=["Top","Right","Bottom","Left"],da=function(a,b){return a=b||a,"none"===a.style.display||""===a.style.display&&r.contains(a.ownerDocument,a)&&"none"===r.css(a,"display")},ea=function(a,b,c,d){var e,f,g={};for(f in b)g[f]=a.style[f],a.style[f]=b[f];e=c.apply(a,d||[]);for(f in b)a.style[f]=g[f];return e};function fa(a,b,c,d){var e,f=1,g=20,h=d?function(){return d.cur()}:function(){return r.css(a,b,"")},i=h(),j=c&&c[3]||(r.cssNumber[b]?"":"px"),k=(r.cssNumber[b]||"px"!==j&&+i)&&ba.exec(r.css(a,b));if(k&&k[3]!==j){j=j||k[3],c=c||[],k=+i||1;do f=f||".5",k/=f,r.style(a,b,k+j);while(f!==(f=h()/i)&&1!==f&&--g)}return c&&(k=+k||+i||0,e=c[1]?k+(c[1]+1)*c[2]:+c[2],d&&(d.unit=j,d.start=k,d.end=e)),e}var ga={};function ha(a){var b,c=a.ownerDocument,d=a.nodeName,e=ga[d];return e?e:(b=c.body.appendChild(c.createElement(d)),e=r.css(b,"display"),b.parentNode.removeChild(b),"none"===e&&(e="block"),ga[d]=e,e)}function ia(a,b){for(var c,d,e=[],f=0,g=a.length;f<g;f++)d=a[f],d.style&&(c=d.style.display,b?("none"===c&&(e[f]=W.get(d,"display")||null,e[f]||(d.style.display="")),""===d.style.display&&da(d)&&(e[f]=ha(d))):"none"!==c&&(e[f]="none",W.set(d,"display",c)));for(f=0;f<g;f++)null!=e[f]&&(a[f].style.display=e[f]);return a}r.fn.extend({show:function(){return ia(this,!0)},hide:function(){return ia(this)},toggle:function(a){return"boolean"==typeof a?a?this.show():this.hide():this.each(function(){da(this)?r(this).show():r(this).hide()})}});var ja=/^(?:checkbox|radio)$/i,ka=/<([a-z][^\/\0>\x20\t\r\n\f]+)/i,la=/^$|\/(?:java|ecma)script/i,ma={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};ma.optgroup=ma.option,ma.tbody=ma.tfoot=ma.colgroup=ma.caption=ma.thead,ma.th=ma.td;function na(a,b){var c;return c="undefined"!=typeof a.getElementsByTagName?a.getElementsByTagName(b||"*"):"undefined"!=typeof a.querySelectorAll?a.querySelectorAll(b||"*"):[],void 0===b||b&&B(a,b)?r.merge([a],c):c}function oa(a,b){for(var c=0,d=a.length;c<d;c++)W.set(a[c],"globalEval",!b||W.get(b[c],"globalEval"))}var pa=/<|&#?\w+;/;function qa(a,b,c,d,e){for(var f,g,h,i,j,k,l=b.createDocumentFragment(),m=[],n=0,o=a.length;n<o;n++)if(f=a[n],f||0===f)if("object"===r.type(f))r.merge(m,f.nodeType?[f]:f);else if(pa.test(f)){g=g||l.appendChild(b.createElement("div")),h=(ka.exec(f)||["",""])[1].toLowerCase(),i=ma[h]||ma._default,g.innerHTML=i[1]+r.htmlPrefilter(f)+i[2],k=i[0];while(k--)g=g.lastChild;r.merge(m,g.childNodes),g=l.firstChild,g.textContent=""}else m.push(b.createTextNode(f));l.textContent="",n=0;while(f=m[n++])if(d&&r.inArray(f,d)>-1)e&&e.push(f);else if(j=r.contains(f.ownerDocument,f),g=na(l.appendChild(f),"script"),j&&oa(g),c){k=0;while(f=g[k++])la.test(f.type||"")&&c.push(f)}return l}!function(){var a=d.createDocumentFragment(),b=a.appendChild(d.createElement("div")),c=d.createElement("input");c.setAttribute("type","radio"),c.setAttribute("checked","checked"),c.setAttribute("name","t"),b.appendChild(c),o.checkClone=b.cloneNode(!0).cloneNode(!0).lastChild.checked,b.innerHTML="<textarea>x</textarea>",o.noCloneChecked=!!b.cloneNode(!0).lastChild.defaultValue}();var ra=d.documentElement,sa=/^key/,ta=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,ua=/^([^.]*)(?:\.(.+)|)/;function va(){return!0}function wa(){return!1}function xa(){try{return d.activeElement}catch(a){}}function ya(a,b,c,d,e,f){var g,h;if("object"==typeof b){"string"!=typeof c&&(d=d||c,c=void 0);for(h in b)ya(a,h,c,d,b[h],f);return a}if(null==d&&null==e?(e=c,d=c=void 0):null==e&&("string"==typeof c?(e=d,d=void 0):(e=d,d=c,c=void 0)),e===!1)e=wa;else if(!e)return a;return 1===f&&(g=e,e=function(a){return r().off(a),g.apply(this,arguments)},e.guid=g.guid||(g.guid=r.guid++)),a.each(function(){r.event.add(this,b,e,d,c)})}r.event={global:{},add:function(a,b,c,d,e){var f,g,h,i,j,k,l,m,n,o,p,q=W.get(a);if(q){c.handler&&(f=c,c=f.handler,e=f.selector),e&&r.find.matchesSelector(ra,e),c.guid||(c.guid=r.guid++),(i=q.events)||(i=q.events={}),(g=q.handle)||(g=q.handle=function(b){return"undefined"!=typeof r&&r.event.triggered!==b.type?r.event.dispatch.apply(a,arguments):void 0}),b=(b||"").match(L)||[""],j=b.length;while(j--)h=ua.exec(b[j])||[],n=p=h[1],o=(h[2]||"").split(".").sort(),n&&(l=r.event.special[n]||{},n=(e?l.delegateType:l.bindType)||n,l=r.event.special[n]||{},k=r.extend({type:n,origType:p,data:d,handler:c,guid:c.guid,selector:e,needsContext:e&&r.expr.match.needsContext.test(e),namespace:o.join(".")},f),(m=i[n])||(m=i[n]=[],m.delegateCount=0,l.setup&&l.setup.call(a,d,o,g)!==!1||a.addEventListener&&a.addEventListener(n,g)),l.add&&(l.add.call(a,k),k.handler.guid||(k.handler.guid=c.guid)),e?m.splice(m.delegateCount++,0,k):m.push(k),r.event.global[n]=!0)}},remove:function(a,b,c,d,e){var f,g,h,i,j,k,l,m,n,o,p,q=W.hasData(a)&&W.get(a);if(q&&(i=q.events)){b=(b||"").match(L)||[""],j=b.length;while(j--)if(h=ua.exec(b[j])||[],n=p=h[1],o=(h[2]||"").split(".").sort(),n){l=r.event.special[n]||{},n=(d?l.delegateType:l.bindType)||n,m=i[n]||[],h=h[2]&&new RegExp("(^|\\.)"+o.join("\\.(?:.*\\.|)")+"(\\.|$)"),g=f=m.length;while(f--)k=m[f],!e&&p!==k.origType||c&&c.guid!==k.guid||h&&!h.test(k.namespace)||d&&d!==k.selector&&("**"!==d||!k.selector)||(m.splice(f,1),k.selector&&m.delegateCount--,l.remove&&l.remove.call(a,k));g&&!m.length&&(l.teardown&&l.teardown.call(a,o,q.handle)!==!1||r.removeEvent(a,n,q.handle),delete i[n])}else for(n in i)r.event.remove(a,n+b[j],c,d,!0);r.isEmptyObject(i)&&W.remove(a,"handle events")}},dispatch:function(a){var b=r.event.fix(a),c,d,e,f,g,h,i=new Array(arguments.length),j=(W.get(this,"events")||{})[b.type]||[],k=r.event.special[b.type]||{};for(i[0]=b,c=1;c<arguments.length;c++)i[c]=arguments[c];if(b.delegateTarget=this,!k.preDispatch||k.preDispatch.call(this,b)!==!1){h=r.event.handlers.call(this,b,j),c=0;while((f=h[c++])&&!b.isPropagationStopped()){b.currentTarget=f.elem,d=0;while((g=f.handlers[d++])&&!b.isImmediatePropagationStopped())b.rnamespace&&!b.rnamespace.test(g.namespace)||(b.handleObj=g,b.data=g.data,e=((r.event.special[g.origType]||{}).handle||g.handler).apply(f.elem,i),void 0!==e&&(b.result=e)===!1&&(b.preventDefault(),b.stopPropagation()))}return k.postDispatch&&k.postDispatch.call(this,b),b.result}},handlers:function(a,b){var c,d,e,f,g,h=[],i=b.delegateCount,j=a.target;if(i&&j.nodeType&&!("click"===a.type&&a.button>=1))for(;j!==this;j=j.parentNode||this)if(1===j.nodeType&&("click"!==a.type||j.disabled!==!0)){for(f=[],g={},c=0;c<i;c++)d=b[c],e=d.selector+" ",void 0===g[e]&&(g[e]=d.needsContext?r(e,this).index(j)>-1:r.find(e,this,null,[j]).length),g[e]&&f.push(d);f.length&&h.push({elem:j,handlers:f})}return j=this,i<b.length&&h.push({elem:j,handlers:b.slice(i)}),h},addProp:function(a,b){Object.defineProperty(r.Event.prototype,a,{enumerable:!0,configurable:!0,get:r.isFunction(b)?function(){if(this.originalEvent)return b(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[a]},set:function(b){Object.defineProperty(this,a,{enumerable:!0,configurable:!0,writable:!0,value:b})}})},fix:function(a){return a[r.expando]?a:new r.Event(a)},special:{load:{noBubble:!0},focus:{trigger:function(){if(this!==xa()&&this.focus)return this.focus(),!1},delegateType:"focusin"},blur:{trigger:function(){if(this===xa()&&this.blur)return this.blur(),!1},delegateType:"focusout"},click:{trigger:function(){if("checkbox"===this.type&&this.click&&B(this,"input"))return this.click(),!1},_default:function(a){return B(a.target,"a")}},beforeunload:{postDispatch:function(a){void 0!==a.result&&a.originalEvent&&(a.originalEvent.returnValue=a.result)}}}},r.removeEvent=function(a,b,c){a.removeEventListener&&a.removeEventListener(b,c)},r.Event=function(a,b){return this instanceof r.Event?(a&&a.type?(this.originalEvent=a,this.type=a.type,this.isDefaultPrevented=a.defaultPrevented||void 0===a.defaultPrevented&&a.returnValue===!1?va:wa,this.target=a.target&&3===a.target.nodeType?a.target.parentNode:a.target,this.currentTarget=a.currentTarget,this.relatedTarget=a.relatedTarget):this.type=a,b&&r.extend(this,b),this.timeStamp=a&&a.timeStamp||r.now(),void(this[r.expando]=!0)):new r.Event(a,b)},r.Event.prototype={constructor:r.Event,isDefaultPrevented:wa,isPropagationStopped:wa,isImmediatePropagationStopped:wa,isSimulated:!1,preventDefault:function(){var a=this.originalEvent;this.isDefaultPrevented=va,a&&!this.isSimulated&&a.preventDefault()},stopPropagation:function(){var a=this.originalEvent;this.isPropagationStopped=va,a&&!this.isSimulated&&a.stopPropagation()},stopImmediatePropagation:function(){var a=this.originalEvent;this.isImmediatePropagationStopped=va,a&&!this.isSimulated&&a.stopImmediatePropagation(),this.stopPropagation()}},r.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,"char":!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(a){var b=a.button;return null==a.which&&sa.test(a.type)?null!=a.charCode?a.charCode:a.keyCode:!a.which&&void 0!==b&&ta.test(a.type)?1&b?1:2&b?3:4&b?2:0:a.which}},r.event.addProp),r.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(a,b){r.event.special[a]={delegateType:b,bindType:b,handle:function(a){var c,d=this,e=a.relatedTarget,f=a.handleObj;return e&&(e===d||r.contains(d,e))||(a.type=f.origType,c=f.handler.apply(this,arguments),a.type=b),c}}}),r.fn.extend({on:function(a,b,c,d){return ya(this,a,b,c,d)},one:function(a,b,c,d){return ya(this,a,b,c,d,1)},off:function(a,b,c){var d,e;if(a&&a.preventDefault&&a.handleObj)return d=a.handleObj,r(a.delegateTarget).off(d.namespace?d.origType+"."+d.namespace:d.origType,d.selector,d.handler),this;if("object"==typeof a){for(e in a)this.off(e,b,a[e]);return this}return b!==!1&&"function"!=typeof b||(c=b,b=void 0),c===!1&&(c=wa),this.each(function(){r.event.remove(this,a,c,b)})}});var za=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,Aa=/<script|<style|<link/i,Ba=/checked\s*(?:[^=]|=\s*.checked.)/i,Ca=/^true\/(.*)/,Da=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Ea(a,b){return B(a,"table")&&B(11!==b.nodeType?b:b.firstChild,"tr")?r(">tbody",a)[0]||a:a}function Fa(a){return a.type=(null!==a.getAttribute("type"))+"/"+a.type,a}function Ga(a){var b=Ca.exec(a.type);return b?a.type=b[1]:a.removeAttribute("type"),a}function Ha(a,b){var c,d,e,f,g,h,i,j;if(1===b.nodeType){if(W.hasData(a)&&(f=W.access(a),g=W.set(b,f),j=f.events)){delete g.handle,g.events={};for(e in j)for(c=0,d=j[e].length;c<d;c++)r.event.add(b,e,j[e][c])}X.hasData(a)&&(h=X.access(a),i=r.extend({},h),X.set(b,i))}}function Ia(a,b){var c=b.nodeName.toLowerCase();"input"===c&&ja.test(a.type)?b.checked=a.checked:"input"!==c&&"textarea"!==c||(b.defaultValue=a.defaultValue)}function Ja(a,b,c,d){b=g.apply([],b);var e,f,h,i,j,k,l=0,m=a.length,n=m-1,q=b[0],s=r.isFunction(q);if(s||m>1&&"string"==typeof q&&!o.checkClone&&Ba.test(q))return a.each(function(e){var f=a.eq(e);s&&(b[0]=q.call(this,e,f.html())),Ja(f,b,c,d)});if(m&&(e=qa(b,a[0].ownerDocument,!1,a,d),f=e.firstChild,1===e.childNodes.length&&(e=f),f||d)){for(h=r.map(na(e,"script"),Fa),i=h.length;l<m;l++)j=e,l!==n&&(j=r.clone(j,!0,!0),i&&r.merge(h,na(j,"script"))),c.call(a[l],j,l);if(i)for(k=h[h.length-1].ownerDocument,r.map(h,Ga),l=0;l<i;l++)j=h[l],la.test(j.type||"")&&!W.access(j,"globalEval")&&r.contains(k,j)&&(j.src?r._evalUrl&&r._evalUrl(j.src):p(j.textContent.replace(Da,""),k))}return a}function Ka(a,b,c){for(var d,e=b?r.filter(b,a):a,f=0;null!=(d=e[f]);f++)c||1!==d.nodeType||r.cleanData(na(d)),d.parentNode&&(c&&r.contains(d.ownerDocument,d)&&oa(na(d,"script")),d.parentNode.removeChild(d));return a}r.extend({htmlPrefilter:function(a){return a.replace(za,"<$1></$2>")},clone:function(a,b,c){var d,e,f,g,h=a.cloneNode(!0),i=r.contains(a.ownerDocument,a);if(!(o.noCloneChecked||1!==a.nodeType&&11!==a.nodeType||r.isXMLDoc(a)))for(g=na(h),f=na(a),d=0,e=f.length;d<e;d++)Ia(f[d],g[d]);if(b)if(c)for(f=f||na(a),g=g||na(h),d=0,e=f.length;d<e;d++)Ha(f[d],g[d]);else Ha(a,h);return g=na(h,"script"),g.length>0&&oa(g,!i&&na(a,"script")),h},cleanData:function(a){for(var b,c,d,e=r.event.special,f=0;void 0!==(c=a[f]);f++)if(U(c)){if(b=c[W.expando]){if(b.events)for(d in b.events)e[d]?r.event.remove(c,d):r.removeEvent(c,d,b.handle);c[W.expando]=void 0}c[X.expando]&&(c[X.expando]=void 0)}}}),r.fn.extend({detach:function(a){return Ka(this,a,!0)},remove:function(a){return Ka(this,a)},text:function(a){return T(this,function(a){return void 0===a?r.text(this):this.empty().each(function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=a)})},null,a,arguments.length)},append:function(){return Ja(this,arguments,function(a){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var b=Ea(this,a);b.appendChild(a)}})},prepend:function(){return Ja(this,arguments,function(a){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var b=Ea(this,a);b.insertBefore(a,b.firstChild)}})},before:function(){return Ja(this,arguments,function(a){this.parentNode&&this.parentNode.insertBefore(a,this)})},after:function(){return Ja(this,arguments,function(a){this.parentNode&&this.parentNode.insertBefore(a,this.nextSibling)})},empty:function(){for(var a,b=0;null!=(a=this[b]);b++)1===a.nodeType&&(r.cleanData(na(a,!1)),a.textContent="");return this},clone:function(a,b){return a=null!=a&&a,b=null==b?a:b,this.map(function(){return r.clone(this,a,b)})},html:function(a){return T(this,function(a){var b=this[0]||{},c=0,d=this.length;if(void 0===a&&1===b.nodeType)return b.innerHTML;if("string"==typeof a&&!Aa.test(a)&&!ma[(ka.exec(a)||["",""])[1].toLowerCase()]){a=r.htmlPrefilter(a);try{for(;c<d;c++)b=this[c]||{},1===b.nodeType&&(r.cleanData(na(b,!1)),b.innerHTML=a);b=0}catch(e){}}b&&this.empty().append(a)},null,a,arguments.length)},replaceWith:function(){var a=[];return Ja(this,arguments,function(b){var c=this.parentNode;r.inArray(this,a)<0&&(r.cleanData(na(this)),c&&c.replaceChild(b,this))},a)}}),r.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(a,b){r.fn[a]=function(a){for(var c,d=[],e=r(a),f=e.length-1,g=0;g<=f;g++)c=g===f?this:this.clone(!0),r(e[g])[b](c),h.apply(d,c.get());return this.pushStack(d)}});var La=/^margin/,Ma=new RegExp("^("+aa+")(?!px)[a-z%]+$","i"),Na=function(b){var c=b.ownerDocument.defaultView;return c&&c.opener||(c=a),c.getComputedStyle(b)};!function(){function b(){if(i){i.style.cssText="box-sizing:border-box;position:relative;display:block;margin:auto;border:1px;padding:1px;top:1%;width:50%",i.innerHTML="",ra.appendChild(h);var b=a.getComputedStyle(i);c="1%"!==b.top,g="2px"===b.marginLeft,e="4px"===b.width,i.style.marginRight="50%",f="4px"===b.marginRight,ra.removeChild(h),i=null}}var c,e,f,g,h=d.createElement("div"),i=d.createElement("div");i.style&&(i.style.backgroundClip="content-box",i.cloneNode(!0).style.backgroundClip="",o.clearCloneStyle="content-box"===i.style.backgroundClip,h.style.cssText="border:0;width:8px;height:0;top:0;left:-9999px;padding:0;margin-top:1px;position:absolute",h.appendChild(i),r.extend(o,{pixelPosition:function(){return b(),c},boxSizingReliable:function(){return b(),e},pixelMarginRight:function(){return b(),f},reliableMarginLeft:function(){return b(),g}}))}();function Oa(a,b,c){var d,e,f,g,h=a.style;return c=c||Na(a),c&&(g=c.getPropertyValue(b)||c[b],""!==g||r.contains(a.ownerDocument,a)||(g=r.style(a,b)),!o.pixelMarginRight()&&Ma.test(g)&&La.test(b)&&(d=h.width,e=h.minWidth,f=h.maxWidth,h.minWidth=h.maxWidth=h.width=g,g=c.width,h.width=d,h.minWidth=e,h.maxWidth=f)),void 0!==g?g+"":g}function Pa(a,b){return{get:function(){return a()?void delete this.get:(this.get=b).apply(this,arguments)}}}var Qa=/^(none|table(?!-c[ea]).+)/,Ra=/^--/,Sa={position:"absolute",visibility:"hidden",display:"block"},Ta={letterSpacing:"0",fontWeight:"400"},Ua=["Webkit","Moz","ms"],Va=d.createElement("div").style;function Wa(a){if(a in Va)return a;var b=a[0].toUpperCase()+a.slice(1),c=Ua.length;while(c--)if(a=Ua[c]+b,a in Va)return a}function Xa(a){var b=r.cssProps[a];return b||(b=r.cssProps[a]=Wa(a)||a),b}function Ya(a,b,c){var d=ba.exec(b);return d?Math.max(0,d[2]-(c||0))+(d[3]||"px"):b}function Za(a,b,c,d,e){var f,g=0;for(f=c===(d?"border":"content")?4:"width"===b?1:0;f<4;f+=2)"margin"===c&&(g+=r.css(a,c+ca[f],!0,e)),d?("content"===c&&(g-=r.css(a,"padding"+ca[f],!0,e)),"margin"!==c&&(g-=r.css(a,"border"+ca[f]+"Width",!0,e))):(g+=r.css(a,"padding"+ca[f],!0,e),"padding"!==c&&(g+=r.css(a,"border"+ca[f]+"Width",!0,e)));return g}function $a(a,b,c){var d,e=Na(a),f=Oa(a,b,e),g="border-box"===r.css(a,"boxSizing",!1,e);return Ma.test(f)?f:(d=g&&(o.boxSizingReliable()||f===a.style[b]),"auto"===f&&(f=a["offset"+b[0].toUpperCase()+b.slice(1)]),f=parseFloat(f)||0,f+Za(a,b,c||(g?"border":"content"),d,e)+"px")}r.extend({cssHooks:{opacity:{get:function(a,b){if(b){var c=Oa(a,"opacity");return""===c?"1":c}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{"float":"cssFloat"},style:function(a,b,c,d){if(a&&3!==a.nodeType&&8!==a.nodeType&&a.style){var e,f,g,h=r.camelCase(b),i=Ra.test(b),j=a.style;return i||(b=Xa(h)),g=r.cssHooks[b]||r.cssHooks[h],void 0===c?g&&"get"in g&&void 0!==(e=g.get(a,!1,d))?e:j[b]:(f=typeof c,"string"===f&&(e=ba.exec(c))&&e[1]&&(c=fa(a,b,e),f="number"),null!=c&&c===c&&("number"===f&&(c+=e&&e[3]||(r.cssNumber[h]?"":"px")),o.clearCloneStyle||""!==c||0!==b.indexOf("background")||(j[b]="inherit"),g&&"set"in g&&void 0===(c=g.set(a,c,d))||(i?j.setProperty(b,c):j[b]=c)),void 0)}},css:function(a,b,c,d){var e,f,g,h=r.camelCase(b),i=Ra.test(b);return i||(b=Xa(h)),g=r.cssHooks[b]||r.cssHooks[h],g&&"get"in g&&(e=g.get(a,!0,c)),void 0===e&&(e=Oa(a,b,d)),"normal"===e&&b in Ta&&(e=Ta[b]),""===c||c?(f=parseFloat(e),c===!0||isFinite(f)?f||0:e):e}}),r.each(["height","width"],function(a,b){r.cssHooks[b]={get:function(a,c,d){if(c)return!Qa.test(r.css(a,"display"))||a.getClientRects().length&&a.getBoundingClientRect().width?$a(a,b,d):ea(a,Sa,function(){return $a(a,b,d)})},set:function(a,c,d){var e,f=d&&Na(a),g=d&&Za(a,b,d,"border-box"===r.css(a,"boxSizing",!1,f),f);return g&&(e=ba.exec(c))&&"px"!==(e[3]||"px")&&(a.style[b]=c,c=r.css(a,b)),Ya(a,c,g)}}}),r.cssHooks.marginLeft=Pa(o.reliableMarginLeft,function(a,b){if(b)return(parseFloat(Oa(a,"marginLeft"))||a.getBoundingClientRect().left-ea(a,{marginLeft:0},function(){return a.getBoundingClientRect().left}))+"px"}),r.each({margin:"",padding:"",border:"Width"},function(a,b){r.cssHooks[a+b]={expand:function(c){for(var d=0,e={},f="string"==typeof c?c.split(" "):[c];d<4;d++)e[a+ca[d]+b]=f[d]||f[d-2]||f[0];return e}},La.test(a)||(r.cssHooks[a+b].set=Ya)}),r.fn.extend({css:function(a,b){return T(this,function(a,b,c){var d,e,f={},g=0;if(Array.isArray(b)){for(d=Na(a),e=b.length;g<e;g++)f[b[g]]=r.css(a,b[g],!1,d);return f}return void 0!==c?r.style(a,b,c):r.css(a,b)},a,b,arguments.length>1)}});function _a(a,b,c,d,e){return new _a.prototype.init(a,b,c,d,e)}r.Tween=_a,_a.prototype={constructor:_a,init:function(a,b,c,d,e,f){this.elem=a,this.prop=c,this.easing=e||r.easing._default,this.options=b,this.start=this.now=this.cur(),this.end=d,this.unit=f||(r.cssNumber[c]?"":"px")},cur:function(){var a=_a.propHooks[this.prop];return a&&a.get?a.get(this):_a.propHooks._default.get(this)},run:function(a){var b,c=_a.propHooks[this.prop];return this.options.duration?this.pos=b=r.easing[this.easing](a,this.options.duration*a,0,1,this.options.duration):this.pos=b=a,this.now=(this.end-this.start)*b+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),c&&c.set?c.set(this):_a.propHooks._default.set(this),this}},_a.prototype.init.prototype=_a.prototype,_a.propHooks={_default:{get:function(a){var b;return 1!==a.elem.nodeType||null!=a.elem[a.prop]&&null==a.elem.style[a.prop]?a.elem[a.prop]:(b=r.css(a.elem,a.prop,""),b&&"auto"!==b?b:0)},set:function(a){r.fx.step[a.prop]?r.fx.step[a.prop](a):1!==a.elem.nodeType||null==a.elem.style[r.cssProps[a.prop]]&&!r.cssHooks[a.prop]?a.elem[a.prop]=a.now:r.style(a.elem,a.prop,a.now+a.unit)}}},_a.propHooks.scrollTop=_a.propHooks.scrollLeft={set:function(a){a.elem.nodeType&&a.elem.parentNode&&(a.elem[a.prop]=a.now)}},r.easing={linear:function(a){return a},swing:function(a){return.5-Math.cos(a*Math.PI)/2},_default:"swing"},r.fx=_a.prototype.init,r.fx.step={};var ab,bb,cb=/^(?:toggle|show|hide)$/,db=/queueHooks$/;function eb(){bb&&(d.hidden===!1&&a.requestAnimationFrame?a.requestAnimationFrame(eb):a.setTimeout(eb,r.fx.interval),r.fx.tick())}function fb(){return a.setTimeout(function(){ab=void 0}),ab=r.now()}function gb(a,b){var c,d=0,e={height:a};for(b=b?1:0;d<4;d+=2-b)c=ca[d],e["margin"+c]=e["padding"+c]=a;return b&&(e.opacity=e.width=a),e}function hb(a,b,c){for(var d,e=(kb.tweeners[b]||[]).concat(kb.tweeners["*"]),f=0,g=e.length;f<g;f++)if(d=e[f].call(c,b,a))return d}function ib(a,b,c){var d,e,f,g,h,i,j,k,l="width"in b||"height"in b,m=this,n={},o=a.style,p=a.nodeType&&da(a),q=W.get(a,"fxshow");c.queue||(g=r._queueHooks(a,"fx"),null==g.unqueued&&(g.unqueued=0,h=g.empty.fire,g.empty.fire=function(){g.unqueued||h()}),g.unqueued++,m.always(function(){m.always(function(){g.unqueued--,r.queue(a,"fx").length||g.empty.fire()})}));for(d in b)if(e=b[d],cb.test(e)){if(delete b[d],f=f||"toggle"===e,e===(p?"hide":"show")){if("show"!==e||!q||void 0===q[d])continue;p=!0}n[d]=q&&q[d]||r.style(a,d)}if(i=!r.isEmptyObject(b),i||!r.isEmptyObject(n)){l&&1===a.nodeType&&(c.overflow=[o.overflow,o.overflowX,o.overflowY],j=q&&q.display,null==j&&(j=W.get(a,"display")),k=r.css(a,"display"),"none"===k&&(j?k=j:(ia([a],!0),j=a.style.display||j,k=r.css(a,"display"),ia([a]))),("inline"===k||"inline-block"===k&&null!=j)&&"none"===r.css(a,"float")&&(i||(m.done(function(){o.display=j}),null==j&&(k=o.display,j="none"===k?"":k)),o.display="inline-block")),c.overflow&&(o.overflow="hidden",m.always(function(){o.overflow=c.overflow[0],o.overflowX=c.overflow[1],o.overflowY=c.overflow[2]})),i=!1;for(d in n)i||(q?"hidden"in q&&(p=q.hidden):q=W.access(a,"fxshow",{display:j}),f&&(q.hidden=!p),p&&ia([a],!0),m.done(function(){p||ia([a]),W.remove(a,"fxshow");for(d in n)r.style(a,d,n[d])})),i=hb(p?q[d]:0,d,m),d in q||(q[d]=i.start,p&&(i.end=i.start,i.start=0))}}function jb(a,b){var c,d,e,f,g;for(c in a)if(d=r.camelCase(c),e=b[d],f=a[c],Array.isArray(f)&&(e=f[1],f=a[c]=f[0]),c!==d&&(a[d]=f,delete a[c]),g=r.cssHooks[d],g&&"expand"in g){f=g.expand(f),delete a[d];for(c in f)c in a||(a[c]=f[c],b[c]=e)}else b[d]=e}function kb(a,b,c){var d,e,f=0,g=kb.prefilters.length,h=r.Deferred().always(function(){delete i.elem}),i=function(){if(e)return!1;for(var b=ab||fb(),c=Math.max(0,j.startTime+j.duration-b),d=c/j.duration||0,f=1-d,g=0,i=j.tweens.length;g<i;g++)j.tweens[g].run(f);return h.notifyWith(a,[j,f,c]),f<1&&i?c:(i||h.notifyWith(a,[j,1,0]),h.resolveWith(a,[j]),!1)},j=h.promise({elem:a,props:r.extend({},b),opts:r.extend(!0,{specialEasing:{},easing:r.easing._default},c),originalProperties:b,originalOptions:c,startTime:ab||fb(),duration:c.duration,tweens:[],createTween:function(b,c){var d=r.Tween(a,j.opts,b,c,j.opts.specialEasing[b]||j.opts.easing);return j.tweens.push(d),d},stop:function(b){var c=0,d=b?j.tweens.length:0;if(e)return this;for(e=!0;c<d;c++)j.tweens[c].run(1);return b?(h.notifyWith(a,[j,1,0]),h.resolveWith(a,[j,b])):h.rejectWith(a,[j,b]),this}}),k=j.props;for(jb(k,j.opts.specialEasing);f<g;f++)if(d=kb.prefilters[f].call(j,a,k,j.opts))return r.isFunction(d.stop)&&(r._queueHooks(j.elem,j.opts.queue).stop=r.proxy(d.stop,d)),d;return r.map(k,hb,j),r.isFunction(j.opts.start)&&j.opts.start.call(a,j),j.progress(j.opts.progress).done(j.opts.done,j.opts.complete).fail(j.opts.fail).always(j.opts.always),r.fx.timer(r.extend(i,{elem:a,anim:j,queue:j.opts.queue})),j}r.Animation=r.extend(kb,{tweeners:{"*":[function(a,b){var c=this.createTween(a,b);return fa(c.elem,a,ba.exec(b),c),c}]},tweener:function(a,b){r.isFunction(a)?(b=a,a=["*"]):a=a.match(L);for(var c,d=0,e=a.length;d<e;d++)c=a[d],kb.tweeners[c]=kb.tweeners[c]||[],kb.tweeners[c].unshift(b)},prefilters:[ib],prefilter:function(a,b){b?kb.prefilters.unshift(a):kb.prefilters.push(a)}}),r.speed=function(a,b,c){var d=a&&"object"==typeof a?r.extend({},a):{complete:c||!c&&b||r.isFunction(a)&&a,duration:a,easing:c&&b||b&&!r.isFunction(b)&&b};return r.fx.off?d.duration=0:"number"!=typeof d.duration&&(d.duration in r.fx.speeds?d.duration=r.fx.speeds[d.duration]:d.duration=r.fx.speeds._default),null!=d.queue&&d.queue!==!0||(d.queue="fx"),d.old=d.complete,d.complete=function(){r.isFunction(d.old)&&d.old.call(this),d.queue&&r.dequeue(this,d.queue)},d},r.fn.extend({fadeTo:function(a,b,c,d){return this.filter(da).css("opacity",0).show().end().animate({opacity:b},a,c,d)},animate:function(a,b,c,d){var e=r.isEmptyObject(a),f=r.speed(b,c,d),g=function(){var b=kb(this,r.extend({},a),f);(e||W.get(this,"finish"))&&b.stop(!0)};return g.finish=g,e||f.queue===!1?this.each(g):this.queue(f.queue,g)},stop:function(a,b,c){var d=function(a){var b=a.stop;delete a.stop,b(c)};return"string"!=typeof a&&(c=b,b=a,a=void 0),b&&a!==!1&&this.queue(a||"fx",[]),this.each(function(){var b=!0,e=null!=a&&a+"queueHooks",f=r.timers,g=W.get(this);if(e)g[e]&&g[e].stop&&d(g[e]);else for(e in g)g[e]&&g[e].stop&&db.test(e)&&d(g[e]);for(e=f.length;e--;)f[e].elem!==this||null!=a&&f[e].queue!==a||(f[e].anim.stop(c),b=!1,f.splice(e,1));!b&&c||r.dequeue(this,a)})},finish:function(a){return a!==!1&&(a=a||"fx"),this.each(function(){var b,c=W.get(this),d=c[a+"queue"],e=c[a+"queueHooks"],f=r.timers,g=d?d.length:0;for(c.finish=!0,r.queue(this,a,[]),e&&e.stop&&e.stop.call(this,!0),b=f.length;b--;)f[b].elem===this&&f[b].queue===a&&(f[b].anim.stop(!0),f.splice(b,1));for(b=0;b<g;b++)d[b]&&d[b].finish&&d[b].finish.call(this);delete c.finish})}}),r.each(["toggle","show","hide"],function(a,b){var c=r.fn[b];r.fn[b]=function(a,d,e){return null==a||"boolean"==typeof a?c.apply(this,arguments):this.animate(gb(b,!0),a,d,e)}}),r.each({slideDown:gb("show"),slideUp:gb("hide"),slideToggle:gb("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(a,b){r.fn[a]=function(a,c,d){return this.animate(b,a,c,d)}}),r.timers=[],r.fx.tick=function(){var a,b=0,c=r.timers;for(ab=r.now();b<c.length;b++)a=c[b],a()||c[b]!==a||c.splice(b--,1);c.length||r.fx.stop(),ab=void 0},r.fx.timer=function(a){r.timers.push(a),r.fx.start()},r.fx.interval=13,r.fx.start=function(){bb||(bb=!0,eb())},r.fx.stop=function(){bb=null},r.fx.speeds={slow:600,fast:200,_default:400},r.fn.delay=function(b,c){return b=r.fx?r.fx.speeds[b]||b:b,c=c||"fx",this.queue(c,function(c,d){var e=a.setTimeout(c,b);d.stop=function(){a.clearTimeout(e)}})},function(){var a=d.createElement("input"),b=d.createElement("select"),c=b.appendChild(d.createElement("option"));a.type="checkbox",o.checkOn=""!==a.value,o.optSelected=c.selected,a=d.createElement("input"),a.value="t",a.type="radio",o.radioValue="t"===a.value}();var lb,mb=r.expr.attrHandle;r.fn.extend({attr:function(a,b){return T(this,r.attr,a,b,arguments.length>1)},removeAttr:function(a){return this.each(function(){r.removeAttr(this,a)})}}),r.extend({attr:function(a,b,c){var d,e,f=a.nodeType;if(3!==f&&8!==f&&2!==f)return"undefined"==typeof a.getAttribute?r.prop(a,b,c):(1===f&&r.isXMLDoc(a)||(e=r.attrHooks[b.toLowerCase()]||(r.expr.match.bool.test(b)?lb:void 0)),void 0!==c?null===c?void r.removeAttr(a,b):e&&"set"in e&&void 0!==(d=e.set(a,c,b))?d:(a.setAttribute(b,c+""),c):e&&"get"in e&&null!==(d=e.get(a,b))?d:(d=r.find.attr(a,b),
null==d?void 0:d))},attrHooks:{type:{set:function(a,b){if(!o.radioValue&&"radio"===b&&B(a,"input")){var c=a.value;return a.setAttribute("type",b),c&&(a.value=c),b}}}},removeAttr:function(a,b){var c,d=0,e=b&&b.match(L);if(e&&1===a.nodeType)while(c=e[d++])a.removeAttribute(c)}}),lb={set:function(a,b,c){return b===!1?r.removeAttr(a,c):a.setAttribute(c,c),c}},r.each(r.expr.match.bool.source.match(/\w+/g),function(a,b){var c=mb[b]||r.find.attr;mb[b]=function(a,b,d){var e,f,g=b.toLowerCase();return d||(f=mb[g],mb[g]=e,e=null!=c(a,b,d)?g:null,mb[g]=f),e}});var nb=/^(?:input|select|textarea|button)$/i,ob=/^(?:a|area)$/i;r.fn.extend({prop:function(a,b){return T(this,r.prop,a,b,arguments.length>1)},removeProp:function(a){return this.each(function(){delete this[r.propFix[a]||a]})}}),r.extend({prop:function(a,b,c){var d,e,f=a.nodeType;if(3!==f&&8!==f&&2!==f)return 1===f&&r.isXMLDoc(a)||(b=r.propFix[b]||b,e=r.propHooks[b]),void 0!==c?e&&"set"in e&&void 0!==(d=e.set(a,c,b))?d:a[b]=c:e&&"get"in e&&null!==(d=e.get(a,b))?d:a[b]},propHooks:{tabIndex:{get:function(a){var b=r.find.attr(a,"tabindex");return b?parseInt(b,10):nb.test(a.nodeName)||ob.test(a.nodeName)&&a.href?0:-1}}},propFix:{"for":"htmlFor","class":"className"}}),o.optSelected||(r.propHooks.selected={get:function(a){var b=a.parentNode;return b&&b.parentNode&&b.parentNode.selectedIndex,null},set:function(a){var b=a.parentNode;b&&(b.selectedIndex,b.parentNode&&b.parentNode.selectedIndex)}}),r.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){r.propFix[this.toLowerCase()]=this});function pb(a){var b=a.match(L)||[];return b.join(" ")}function qb(a){return a.getAttribute&&a.getAttribute("class")||""}r.fn.extend({addClass:function(a){var b,c,d,e,f,g,h,i=0;if(r.isFunction(a))return this.each(function(b){r(this).addClass(a.call(this,b,qb(this)))});if("string"==typeof a&&a){b=a.match(L)||[];while(c=this[i++])if(e=qb(c),d=1===c.nodeType&&" "+pb(e)+" "){g=0;while(f=b[g++])d.indexOf(" "+f+" ")<0&&(d+=f+" ");h=pb(d),e!==h&&c.setAttribute("class",h)}}return this},removeClass:function(a){var b,c,d,e,f,g,h,i=0;if(r.isFunction(a))return this.each(function(b){r(this).removeClass(a.call(this,b,qb(this)))});if(!arguments.length)return this.attr("class","");if("string"==typeof a&&a){b=a.match(L)||[];while(c=this[i++])if(e=qb(c),d=1===c.nodeType&&" "+pb(e)+" "){g=0;while(f=b[g++])while(d.indexOf(" "+f+" ")>-1)d=d.replace(" "+f+" "," ");h=pb(d),e!==h&&c.setAttribute("class",h)}}return this},toggleClass:function(a,b){var c=typeof a;return"boolean"==typeof b&&"string"===c?b?this.addClass(a):this.removeClass(a):r.isFunction(a)?this.each(function(c){r(this).toggleClass(a.call(this,c,qb(this),b),b)}):this.each(function(){var b,d,e,f;if("string"===c){d=0,e=r(this),f=a.match(L)||[];while(b=f[d++])e.hasClass(b)?e.removeClass(b):e.addClass(b)}else void 0!==a&&"boolean"!==c||(b=qb(this),b&&W.set(this,"__className__",b),this.setAttribute&&this.setAttribute("class",b||a===!1?"":W.get(this,"__className__")||""))})},hasClass:function(a){var b,c,d=0;b=" "+a+" ";while(c=this[d++])if(1===c.nodeType&&(" "+pb(qb(c))+" ").indexOf(b)>-1)return!0;return!1}});var rb=/\r/g;r.fn.extend({val:function(a){var b,c,d,e=this[0];{if(arguments.length)return d=r.isFunction(a),this.each(function(c){var e;1===this.nodeType&&(e=d?a.call(this,c,r(this).val()):a,null==e?e="":"number"==typeof e?e+="":Array.isArray(e)&&(e=r.map(e,function(a){return null==a?"":a+""})),b=r.valHooks[this.type]||r.valHooks[this.nodeName.toLowerCase()],b&&"set"in b&&void 0!==b.set(this,e,"value")||(this.value=e))});if(e)return b=r.valHooks[e.type]||r.valHooks[e.nodeName.toLowerCase()],b&&"get"in b&&void 0!==(c=b.get(e,"value"))?c:(c=e.value,"string"==typeof c?c.replace(rb,""):null==c?"":c)}}}),r.extend({valHooks:{option:{get:function(a){var b=r.find.attr(a,"value");return null!=b?b:pb(r.text(a))}},select:{get:function(a){var b,c,d,e=a.options,f=a.selectedIndex,g="select-one"===a.type,h=g?null:[],i=g?f+1:e.length;for(d=f<0?i:g?f:0;d<i;d++)if(c=e[d],(c.selected||d===f)&&!c.disabled&&(!c.parentNode.disabled||!B(c.parentNode,"optgroup"))){if(b=r(c).val(),g)return b;h.push(b)}return h},set:function(a,b){var c,d,e=a.options,f=r.makeArray(b),g=e.length;while(g--)d=e[g],(d.selected=r.inArray(r.valHooks.option.get(d),f)>-1)&&(c=!0);return c||(a.selectedIndex=-1),f}}}}),r.each(["radio","checkbox"],function(){r.valHooks[this]={set:function(a,b){if(Array.isArray(b))return a.checked=r.inArray(r(a).val(),b)>-1}},o.checkOn||(r.valHooks[this].get=function(a){return null===a.getAttribute("value")?"on":a.value})});var sb=/^(?:focusinfocus|focusoutblur)$/;r.extend(r.event,{trigger:function(b,c,e,f){var g,h,i,j,k,m,n,o=[e||d],p=l.call(b,"type")?b.type:b,q=l.call(b,"namespace")?b.namespace.split("."):[];if(h=i=e=e||d,3!==e.nodeType&&8!==e.nodeType&&!sb.test(p+r.event.triggered)&&(p.indexOf(".")>-1&&(q=p.split("."),p=q.shift(),q.sort()),k=p.indexOf(":")<0&&"on"+p,b=b[r.expando]?b:new r.Event(p,"object"==typeof b&&b),b.isTrigger=f?2:3,b.namespace=q.join("."),b.rnamespace=b.namespace?new RegExp("(^|\\.)"+q.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,b.result=void 0,b.target||(b.target=e),c=null==c?[b]:r.makeArray(c,[b]),n=r.event.special[p]||{},f||!n.trigger||n.trigger.apply(e,c)!==!1)){if(!f&&!n.noBubble&&!r.isWindow(e)){for(j=n.delegateType||p,sb.test(j+p)||(h=h.parentNode);h;h=h.parentNode)o.push(h),i=h;i===(e.ownerDocument||d)&&o.push(i.defaultView||i.parentWindow||a)}g=0;while((h=o[g++])&&!b.isPropagationStopped())b.type=g>1?j:n.bindType||p,m=(W.get(h,"events")||{})[b.type]&&W.get(h,"handle"),m&&m.apply(h,c),m=k&&h[k],m&&m.apply&&U(h)&&(b.result=m.apply(h,c),b.result===!1&&b.preventDefault());return b.type=p,f||b.isDefaultPrevented()||n._default&&n._default.apply(o.pop(),c)!==!1||!U(e)||k&&r.isFunction(e[p])&&!r.isWindow(e)&&(i=e[k],i&&(e[k]=null),r.event.triggered=p,e[p](),r.event.triggered=void 0,i&&(e[k]=i)),b.result}},simulate:function(a,b,c){var d=r.extend(new r.Event,c,{type:a,isSimulated:!0});r.event.trigger(d,null,b)}}),r.fn.extend({trigger:function(a,b){return this.each(function(){r.event.trigger(a,b,this)})},triggerHandler:function(a,b){var c=this[0];if(c)return r.event.trigger(a,b,c,!0)}}),r.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),function(a,b){r.fn[b]=function(a,c){return arguments.length>0?this.on(b,null,a,c):this.trigger(b)}}),r.fn.extend({hover:function(a,b){return this.mouseenter(a).mouseleave(b||a)}}),o.focusin="onfocusin"in a,o.focusin||r.each({focus:"focusin",blur:"focusout"},function(a,b){var c=function(a){r.event.simulate(b,a.target,r.event.fix(a))};r.event.special[b]={setup:function(){var d=this.ownerDocument||this,e=W.access(d,b);e||d.addEventListener(a,c,!0),W.access(d,b,(e||0)+1)},teardown:function(){var d=this.ownerDocument||this,e=W.access(d,b)-1;e?W.access(d,b,e):(d.removeEventListener(a,c,!0),W.remove(d,b))}}});var tb=a.location,ub=r.now(),vb=/\?/;r.parseXML=function(b){var c;if(!b||"string"!=typeof b)return null;try{c=(new a.DOMParser).parseFromString(b,"text/xml")}catch(d){c=void 0}return c&&!c.getElementsByTagName("parsererror").length||r.error("Invalid XML: "+b),c};var wb=/\[\]$/,xb=/\r?\n/g,yb=/^(?:submit|button|image|reset|file)$/i,zb=/^(?:input|select|textarea|keygen)/i;function Ab(a,b,c,d){var e;if(Array.isArray(b))r.each(b,function(b,e){c||wb.test(a)?d(a,e):Ab(a+"["+("object"==typeof e&&null!=e?b:"")+"]",e,c,d)});else if(c||"object"!==r.type(b))d(a,b);else for(e in b)Ab(a+"["+e+"]",b[e],c,d)}r.param=function(a,b){var c,d=[],e=function(a,b){var c=r.isFunction(b)?b():b;d[d.length]=encodeURIComponent(a)+"="+encodeURIComponent(null==c?"":c)};if(Array.isArray(a)||a.jquery&&!r.isPlainObject(a))r.each(a,function(){e(this.name,this.value)});else for(c in a)Ab(c,a[c],b,e);return d.join("&")},r.fn.extend({serialize:function(){return r.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var a=r.prop(this,"elements");return a?r.makeArray(a):this}).filter(function(){var a=this.type;return this.name&&!r(this).is(":disabled")&&zb.test(this.nodeName)&&!yb.test(a)&&(this.checked||!ja.test(a))}).map(function(a,b){var c=r(this).val();return null==c?null:Array.isArray(c)?r.map(c,function(a){return{name:b.name,value:a.replace(xb,"\r\n")}}):{name:b.name,value:c.replace(xb,"\r\n")}}).get()}});var Bb=/%20/g,Cb=/#.*$/,Db=/([?&])_=[^&]*/,Eb=/^(.*?):[ \t]*([^\r\n]*)$/gm,Fb=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,Gb=/^(?:GET|HEAD)$/,Hb=/^\/\//,Ib={},Jb={},Kb="*/".concat("*"),Lb=d.createElement("a");Lb.href=tb.href;function Mb(a){return function(b,c){"string"!=typeof b&&(c=b,b="*");var d,e=0,f=b.toLowerCase().match(L)||[];if(r.isFunction(c))while(d=f[e++])"+"===d[0]?(d=d.slice(1)||"*",(a[d]=a[d]||[]).unshift(c)):(a[d]=a[d]||[]).push(c)}}function Nb(a,b,c,d){var e={},f=a===Jb;function g(h){var i;return e[h]=!0,r.each(a[h]||[],function(a,h){var j=h(b,c,d);return"string"!=typeof j||f||e[j]?f?!(i=j):void 0:(b.dataTypes.unshift(j),g(j),!1)}),i}return g(b.dataTypes[0])||!e["*"]&&g("*")}function Ob(a,b){var c,d,e=r.ajaxSettings.flatOptions||{};for(c in b)void 0!==b[c]&&((e[c]?a:d||(d={}))[c]=b[c]);return d&&r.extend(!0,a,d),a}function Pb(a,b,c){var d,e,f,g,h=a.contents,i=a.dataTypes;while("*"===i[0])i.shift(),void 0===d&&(d=a.mimeType||b.getResponseHeader("Content-Type"));if(d)for(e in h)if(h[e]&&h[e].test(d)){i.unshift(e);break}if(i[0]in c)f=i[0];else{for(e in c){if(!i[0]||a.converters[e+" "+i[0]]){f=e;break}g||(g=e)}f=f||g}if(f)return f!==i[0]&&i.unshift(f),c[f]}function Qb(a,b,c,d){var e,f,g,h,i,j={},k=a.dataTypes.slice();if(k[1])for(g in a.converters)j[g.toLowerCase()]=a.converters[g];f=k.shift();while(f)if(a.responseFields[f]&&(c[a.responseFields[f]]=b),!i&&d&&a.dataFilter&&(b=a.dataFilter(b,a.dataType)),i=f,f=k.shift())if("*"===f)f=i;else if("*"!==i&&i!==f){if(g=j[i+" "+f]||j["* "+f],!g)for(e in j)if(h=e.split(" "),h[1]===f&&(g=j[i+" "+h[0]]||j["* "+h[0]])){g===!0?g=j[e]:j[e]!==!0&&(f=h[0],k.unshift(h[1]));break}if(g!==!0)if(g&&a["throws"])b=g(b);else try{b=g(b)}catch(l){return{state:"parsererror",error:g?l:"No conversion from "+i+" to "+f}}}return{state:"success",data:b}}r.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:tb.href,type:"GET",isLocal:Fb.test(tb.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Kb,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":r.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(a,b){return b?Ob(Ob(a,r.ajaxSettings),b):Ob(r.ajaxSettings,a)},ajaxPrefilter:Mb(Ib),ajaxTransport:Mb(Jb),ajax:function(b,c){"object"==typeof b&&(c=b,b=void 0),c=c||{};var e,f,g,h,i,j,k,l,m,n,o=r.ajaxSetup({},c),p=o.context||o,q=o.context&&(p.nodeType||p.jquery)?r(p):r.event,s=r.Deferred(),t=r.Callbacks("once memory"),u=o.statusCode||{},v={},w={},x="canceled",y={readyState:0,getResponseHeader:function(a){var b;if(k){if(!h){h={};while(b=Eb.exec(g))h[b[1].toLowerCase()]=b[2]}b=h[a.toLowerCase()]}return null==b?null:b},getAllResponseHeaders:function(){return k?g:null},setRequestHeader:function(a,b){return null==k&&(a=w[a.toLowerCase()]=w[a.toLowerCase()]||a,v[a]=b),this},overrideMimeType:function(a){return null==k&&(o.mimeType=a),this},statusCode:function(a){var b;if(a)if(k)y.always(a[y.status]);else for(b in a)u[b]=[u[b],a[b]];return this},abort:function(a){var b=a||x;return e&&e.abort(b),A(0,b),this}};if(s.promise(y),o.url=((b||o.url||tb.href)+"").replace(Hb,tb.protocol+"//"),o.type=c.method||c.type||o.method||o.type,o.dataTypes=(o.dataType||"*").toLowerCase().match(L)||[""],null==o.crossDomain){j=d.createElement("a");try{j.href=o.url,j.href=j.href,o.crossDomain=Lb.protocol+"//"+Lb.host!=j.protocol+"//"+j.host}catch(z){o.crossDomain=!0}}if(o.data&&o.processData&&"string"!=typeof o.data&&(o.data=r.param(o.data,o.traditional)),Nb(Ib,o,c,y),k)return y;l=r.event&&o.global,l&&0===r.active++&&r.event.trigger("ajaxStart"),o.type=o.type.toUpperCase(),o.hasContent=!Gb.test(o.type),f=o.url.replace(Cb,""),o.hasContent?o.data&&o.processData&&0===(o.contentType||"").indexOf("application/x-www-form-urlencoded")&&(o.data=o.data.replace(Bb,"+")):(n=o.url.slice(f.length),o.data&&(f+=(vb.test(f)?"&":"?")+o.data,delete o.data),o.cache===!1&&(f=f.replace(Db,"$1"),n=(vb.test(f)?"&":"?")+"_="+ub++ +n),o.url=f+n),o.ifModified&&(r.lastModified[f]&&y.setRequestHeader("If-Modified-Since",r.lastModified[f]),r.etag[f]&&y.setRequestHeader("If-None-Match",r.etag[f])),(o.data&&o.hasContent&&o.contentType!==!1||c.contentType)&&y.setRequestHeader("Content-Type",o.contentType),y.setRequestHeader("Accept",o.dataTypes[0]&&o.accepts[o.dataTypes[0]]?o.accepts[o.dataTypes[0]]+("*"!==o.dataTypes[0]?", "+Kb+"; q=0.01":""):o.accepts["*"]);for(m in o.headers)y.setRequestHeader(m,o.headers[m]);if(o.beforeSend&&(o.beforeSend.call(p,y,o)===!1||k))return y.abort();if(x="abort",t.add(o.complete),y.done(o.success),y.fail(o.error),e=Nb(Jb,o,c,y)){if(y.readyState=1,l&&q.trigger("ajaxSend",[y,o]),k)return y;o.async&&o.timeout>0&&(i=a.setTimeout(function(){y.abort("timeout")},o.timeout));try{k=!1,e.send(v,A)}catch(z){if(k)throw z;A(-1,z)}}else A(-1,"No Transport");function A(b,c,d,h){var j,m,n,v,w,x=c;k||(k=!0,i&&a.clearTimeout(i),e=void 0,g=h||"",y.readyState=b>0?4:0,j=b>=200&&b<300||304===b,d&&(v=Pb(o,y,d)),v=Qb(o,v,y,j),j?(o.ifModified&&(w=y.getResponseHeader("Last-Modified"),w&&(r.lastModified[f]=w),w=y.getResponseHeader("etag"),w&&(r.etag[f]=w)),204===b||"HEAD"===o.type?x="nocontent":304===b?x="notmodified":(x=v.state,m=v.data,n=v.error,j=!n)):(n=x,!b&&x||(x="error",b<0&&(b=0))),y.status=b,y.statusText=(c||x)+"",j?s.resolveWith(p,[m,x,y]):s.rejectWith(p,[y,x,n]),y.statusCode(u),u=void 0,l&&q.trigger(j?"ajaxSuccess":"ajaxError",[y,o,j?m:n]),t.fireWith(p,[y,x]),l&&(q.trigger("ajaxComplete",[y,o]),--r.active||r.event.trigger("ajaxStop")))}return y},getJSON:function(a,b,c){return r.get(a,b,c,"json")},getScript:function(a,b){return r.get(a,void 0,b,"script")}}),r.each(["get","post"],function(a,b){r[b]=function(a,c,d,e){return r.isFunction(c)&&(e=e||d,d=c,c=void 0),r.ajax(r.extend({url:a,type:b,dataType:e,data:c,success:d},r.isPlainObject(a)&&a))}}),r._evalUrl=function(a){return r.ajax({url:a,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,"throws":!0})},r.fn.extend({wrapAll:function(a){var b;return this[0]&&(r.isFunction(a)&&(a=a.call(this[0])),b=r(a,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&b.insertBefore(this[0]),b.map(function(){var a=this;while(a.firstElementChild)a=a.firstElementChild;return a}).append(this)),this},wrapInner:function(a){return r.isFunction(a)?this.each(function(b){r(this).wrapInner(a.call(this,b))}):this.each(function(){var b=r(this),c=b.contents();c.length?c.wrapAll(a):b.append(a)})},wrap:function(a){var b=r.isFunction(a);return this.each(function(c){r(this).wrapAll(b?a.call(this,c):a)})},unwrap:function(a){return this.parent(a).not("body").each(function(){r(this).replaceWith(this.childNodes)}),this}}),r.expr.pseudos.hidden=function(a){return!r.expr.pseudos.visible(a)},r.expr.pseudos.visible=function(a){return!!(a.offsetWidth||a.offsetHeight||a.getClientRects().length)},r.ajaxSettings.xhr=function(){try{return new a.XMLHttpRequest}catch(b){}};var Rb={0:200,1223:204},Sb=r.ajaxSettings.xhr();o.cors=!!Sb&&"withCredentials"in Sb,o.ajax=Sb=!!Sb,r.ajaxTransport(function(b){var c,d;if(o.cors||Sb&&!b.crossDomain)return{send:function(e,f){var g,h=b.xhr();if(h.open(b.type,b.url,b.async,b.username,b.password),b.xhrFields)for(g in b.xhrFields)h[g]=b.xhrFields[g];b.mimeType&&h.overrideMimeType&&h.overrideMimeType(b.mimeType),b.crossDomain||e["X-Requested-With"]||(e["X-Requested-With"]="XMLHttpRequest");for(g in e)h.setRequestHeader(g,e[g]);c=function(a){return function(){c&&(c=d=h.onload=h.onerror=h.onabort=h.onreadystatechange=null,"abort"===a?h.abort():"error"===a?"number"!=typeof h.status?f(0,"error"):f(h.status,h.statusText):f(Rb[h.status]||h.status,h.statusText,"text"!==(h.responseType||"text")||"string"!=typeof h.responseText?{binary:h.response}:{text:h.responseText},h.getAllResponseHeaders()))}},h.onload=c(),d=h.onerror=c("error"),void 0!==h.onabort?h.onabort=d:h.onreadystatechange=function(){4===h.readyState&&a.setTimeout(function(){c&&d()})},c=c("abort");try{h.send(b.hasContent&&b.data||null)}catch(i){if(c)throw i}},abort:function(){c&&c()}}}),r.ajaxPrefilter(function(a){a.crossDomain&&(a.contents.script=!1)}),r.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(a){return r.globalEval(a),a}}}),r.ajaxPrefilter("script",function(a){void 0===a.cache&&(a.cache=!1),a.crossDomain&&(a.type="GET")}),r.ajaxTransport("script",function(a){if(a.crossDomain){var b,c;return{send:function(e,f){b=r("<script>").prop({charset:a.scriptCharset,src:a.url}).on("load error",c=function(a){b.remove(),c=null,a&&f("error"===a.type?404:200,a.type)}),d.head.appendChild(b[0])},abort:function(){c&&c()}}}});var Tb=[],Ub=/(=)\?(?=&|$)|\?\?/;r.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var a=Tb.pop()||r.expando+"_"+ub++;return this[a]=!0,a}}),r.ajaxPrefilter("json jsonp",function(b,c,d){var e,f,g,h=b.jsonp!==!1&&(Ub.test(b.url)?"url":"string"==typeof b.data&&0===(b.contentType||"").indexOf("application/x-www-form-urlencoded")&&Ub.test(b.data)&&"data");if(h||"jsonp"===b.dataTypes[0])return e=b.jsonpCallback=r.isFunction(b.jsonpCallback)?b.jsonpCallback():b.jsonpCallback,h?b[h]=b[h].replace(Ub,"$1"+e):b.jsonp!==!1&&(b.url+=(vb.test(b.url)?"&":"?")+b.jsonp+"="+e),b.converters["script json"]=function(){return g||r.error(e+" was not called"),g[0]},b.dataTypes[0]="json",f=a[e],a[e]=function(){g=arguments},d.always(function(){void 0===f?r(a).removeProp(e):a[e]=f,b[e]&&(b.jsonpCallback=c.jsonpCallback,Tb.push(e)),g&&r.isFunction(f)&&f(g[0]),g=f=void 0}),"script"}),o.createHTMLDocument=function(){var a=d.implementation.createHTMLDocument("").body;return a.innerHTML="<form></form><form></form>",2===a.childNodes.length}(),r.parseHTML=function(a,b,c){if("string"!=typeof a)return[];"boolean"==typeof b&&(c=b,b=!1);var e,f,g;return b||(o.createHTMLDocument?(b=d.implementation.createHTMLDocument(""),e=b.createElement("base"),e.href=d.location.href,b.head.appendChild(e)):b=d),f=C.exec(a),g=!c&&[],f?[b.createElement(f[1])]:(f=qa([a],b,g),g&&g.length&&r(g).remove(),r.merge([],f.childNodes))},r.fn.load=function(a,b,c){var d,e,f,g=this,h=a.indexOf(" ");return h>-1&&(d=pb(a.slice(h)),a=a.slice(0,h)),r.isFunction(b)?(c=b,b=void 0):b&&"object"==typeof b&&(e="POST"),g.length>0&&r.ajax({url:a,type:e||"GET",dataType:"html",data:b}).done(function(a){f=arguments,g.html(d?r("<div>").append(r.parseHTML(a)).find(d):a)}).always(c&&function(a,b){g.each(function(){c.apply(this,f||[a.responseText,b,a])})}),this},r.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(a,b){r.fn[b]=function(a){return this.on(b,a)}}),r.expr.pseudos.animated=function(a){return r.grep(r.timers,function(b){return a===b.elem}).length},r.offset={setOffset:function(a,b,c){var d,e,f,g,h,i,j,k=r.css(a,"position"),l=r(a),m={};"static"===k&&(a.style.position="relative"),h=l.offset(),f=r.css(a,"top"),i=r.css(a,"left"),j=("absolute"===k||"fixed"===k)&&(f+i).indexOf("auto")>-1,j?(d=l.position(),g=d.top,e=d.left):(g=parseFloat(f)||0,e=parseFloat(i)||0),r.isFunction(b)&&(b=b.call(a,c,r.extend({},h))),null!=b.top&&(m.top=b.top-h.top+g),null!=b.left&&(m.left=b.left-h.left+e),"using"in b?b.using.call(a,m):l.css(m)}},r.fn.extend({offset:function(a){if(arguments.length)return void 0===a?this:this.each(function(b){r.offset.setOffset(this,a,b)});var b,c,d,e,f=this[0];if(f)return f.getClientRects().length?(d=f.getBoundingClientRect(),b=f.ownerDocument,c=b.documentElement,e=b.defaultView,{top:d.top+e.pageYOffset-c.clientTop,left:d.left+e.pageXOffset-c.clientLeft}):{top:0,left:0}},position:function(){if(this[0]){var a,b,c=this[0],d={top:0,left:0};return"fixed"===r.css(c,"position")?b=c.getBoundingClientRect():(a=this.offsetParent(),b=this.offset(),B(a[0],"html")||(d=a.offset()),d={top:d.top+r.css(a[0],"borderTopWidth",!0),left:d.left+r.css(a[0],"borderLeftWidth",!0)}),{top:b.top-d.top-r.css(c,"marginTop",!0),left:b.left-d.left-r.css(c,"marginLeft",!0)}}},offsetParent:function(){return this.map(function(){var a=this.offsetParent;while(a&&"static"===r.css(a,"position"))a=a.offsetParent;return a||ra})}}),r.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(a,b){var c="pageYOffset"===b;r.fn[a]=function(d){return T(this,function(a,d,e){var f;return r.isWindow(a)?f=a:9===a.nodeType&&(f=a.defaultView),void 0===e?f?f[b]:a[d]:void(f?f.scrollTo(c?f.pageXOffset:e,c?e:f.pageYOffset):a[d]=e)},a,d,arguments.length)}}),r.each(["top","left"],function(a,b){r.cssHooks[b]=Pa(o.pixelPosition,function(a,c){if(c)return c=Oa(a,b),Ma.test(c)?r(a).position()[b]+"px":c})}),r.each({Height:"height",Width:"width"},function(a,b){r.each({padding:"inner"+a,content:b,"":"outer"+a},function(c,d){r.fn[d]=function(e,f){var g=arguments.length&&(c||"boolean"!=typeof e),h=c||(e===!0||f===!0?"margin":"border");return T(this,function(b,c,e){var f;return r.isWindow(b)?0===d.indexOf("outer")?b["inner"+a]:b.document.documentElement["client"+a]:9===b.nodeType?(f=b.documentElement,Math.max(b.body["scroll"+a],f["scroll"+a],b.body["offset"+a],f["offset"+a],f["client"+a])):void 0===e?r.css(b,c,h):r.style(b,c,e,h)},b,g?e:void 0,g)}})}),r.fn.extend({bind:function(a,b,c){return this.on(a,null,b,c)},unbind:function(a,b){return this.off(a,null,b)},delegate:function(a,b,c,d){return this.on(b,a,c,d)},undelegate:function(a,b,c){return 1===arguments.length?this.off(a,"**"):this.off(b,a||"**",c)}}),r.holdReady=function(a){a?r.readyWait++:r.ready(!0)},r.isArray=Array.isArray,r.parseJSON=JSON.parse,r.nodeName=B,"function"==typeof define&&define.amd&&define("jquery",[],function(){return r});var Vb=a.jQuery,Wb=a.$;return r.noConflict=function(b){return a.$===r&&(a.$=Wb),b&&a.jQuery===r&&(a.jQuery=Vb),r},b||(a.jQuery=a.$=r),r});

// END res\js\lib\jquery-3.2.1.min.js

// res\js\loaders\HotLoader.js

/**
 * The Hotloader is capable of loading *all* scenes at once,
 * favoring a single, optimized loading period over several smaller ones at scene-transition time.
 * Note that the Titlescreen scene is not loaded by this class.
 */
function HotLoader() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Resolve function for loading promise.
    var loadResolve;
    // Reject function for loading promise.
    var loadReject;

    // ID of the div containing the hot load in progress warning.
    var hotLoadWarningId = "hotloadwarning";
    var hotLoadProgressId = "hotloadprogress";
    var hotLoadProgressNumberId = "hotloadprogressnumber";
    var compiledFileLocation = "scenes/compiled/AllScenes_compiled.js";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Stores the resolve and reject functions for use later.
     * @param {function} resolve Load resolve function.
     * @param {function} reject Load reject function.
     */
    function storeLoadResolve(resolve, reject) {
        loadResolve = function () {
            $(`#${hotLoadWarningId}`).hide(400);
            $(`#${hotLoadProgressId}`).hide(400);
            resolve();
        };
        loadReject = function (reason) {
            $(`#${hotLoadWarningId}`).hide(400);
            $(`#${hotLoadProgressId}`).hide(400);
            reject(reason);
        };
    }

    /**
     * Updates the UI when download progress happens.
     * @param {Event} evt The event triggered by the XHTML request.
     */
    function compiledHotLoadingProgressUpdate(evt) {
        if (evt.lengthComputable) {
            var percentComplete = Math.round((evt.loaded / evt.total) * 100);
            $(`#${hotLoadProgressNumberId}`).text(percentComplete);
        }
    }

    /**
     * Creates an XMLHttpRequest with a registered listener that will listen to changes in download progress and update the UI accordingly.
     */
    function compiledHotLoadingProgressListen() {
        var xhr = new window.XMLHttpRequest();
        xhr.addEventListener("progress", compiledHotLoadingProgressUpdate);
        return xhr;
    }

    /**
     * Signals something fucked up.
     * Should be called when loading fails.
     */
    function hotLoadFailure() {
        loadReject(`Failed to load scenes. Perhaps there's something wrong with your internet connection? Clear your browser cache and try again.`);
    }

    /**
     * Hotloads the compiled version of all scenes.
     * @param {string} compiledFile A path to the compiled file.
     */
    function compiledHotLoading(compiledFile) {
        $.ajax({
            url: compiledFile,
            dataType: "script",
            xhr: compiledHotLoadingProgressListen
        }).done(loadResolve).fail(hotLoadFailure);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.loaded = new Promise(storeLoadResolve);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Hotloads all the required game files asynchronously by adding references to them to the DOM.
     */
    this.hotLoad = function () {
        $(`#${hotLoadWarningId}`).show(400);
        $(`#${hotLoadProgressId}`).show(400);
        compiledHotLoading(compiledFileLocation);
    };

    /**
     * Performs regular loading.
     */
    this.regularLoad = function () {
        loadResolve();
    };

    /**
     * Performs regular or hotloading depending on the settings.
     */
    this.hotLoadIfRequired = function () {
        if (global.settings.getHotLoad()) {
            that.hotLoad();
        } else {
            that.regularLoad();
        }
    };
}

// END res\js\loaders\HotLoader.js

// res\js\loaders\SceneLoader.js

/**
 * The SceneLoader is in charge of loading scenes.
 * If need be, it may use a HTTP request to fetch the scene first.
 * If the scene is already cached, it will simply be loaded from memory.
 */
function SceneLoader() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // All currently loaded and cached scenes.
    var cachedScenes = {};

    // Error message shown when loading fails.
    var sceneLoadErrorMessage = "Scene loading failed! " +
            "This probably broke something. " +
            "Please reload the page and try again. " +
            "If that doesn't work, try clearing your browser cache (ctrl+F5 on Chrome)." +
            "If the problem persists, please do notify the creator of Kobold Adventure. " +
            "Keep in mind that it may just be your internet connection acting up!";

    // Queue for registration functions.
    var registrationQueue = [];

    // Whether or not registration processing has started.
    var registerStarted = false;

    // How many files advanced loading is currently processing.
    var loadingFiles = 0;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets the scroll level of the main scene, if required.
     * @param {int} scroll What to set the scroll level to.
     *                     If undefined or settings say not to scroll, will reset.
     *                     If not undefined, and settings say to scroll to save state, will scroll to the passed parameter.
     *                     If not undefined, and settings say to scroll to bottom, will scroll to bottom of the scene.
     */
    function setScroll(scroll) {
        var saveScroll = global.settings.getSaveScroll();
        var scrollBottom = global.settings.getScrollBottom();
        if (!saveScroll || typeof scroll === 'undefined') {
            global.ui.resetScroll();
        } else {
            if (scrollBottom) {
                global.ui.scrollBottom();
            } else {
                global.ui.sceneScroll = scroll;
                global.ui.setScroll();
            }
        }
    }

    /**
     * Waits ten milliseconds, then sets the scroll level.
     * Ugly fix for even uglier bugs.
     * @param {int} scroll What to set the scroll level to.
     *                     If undefined or settings say not to scroll, will reset.
     *                     If not undefined, and settings say to scroll to save state, will scroll to the passed parameter.
     *                     If not undefined, and settings say to scroll to bottom, will scroll to bottom of the scene.
     */
    function setScrollLater(scroll) {
        setTimeout(function () {
            setScroll(scroll);
        }, 10);
    }

    /**
     * Builds and returns the path to the root of the scene.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene is contained in. Optional parameter.
     * @returns {string} The path to the root of the scene.
     */
    function buildSceneRoot(sceneName, subfolder) {
        var root = "scenes" + "/";
        if (subfolder) {
            root += subfolder + "/";
        }
        root += sceneName + "/";
        return root;
    }

    /**
     * Builds and returns a path to the main JSON file of a scene.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene is contained in. Optional parameter.
     * @returns {string} The path to the main JSON file of the scene.
     */
    function buildJsonFilePath(sceneName, subfolder) {
        return buildSceneRoot(sceneName, subfolder) + sceneName + ".json";
    }

    /**
     * Builds and returns a path to the compiled file of a scene.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene is contained in. Optional parameter.
     * @returns {string} The path to the compiled file of the scene.
     */
    function buildCompiledFilePath(sceneName, subfolder) {
        return "scenes/compiled/" + sceneName + "_compiled.js";
    }

    /**
     * Builds and returns a path to a JavaScript file of a scene.
     * @param {string} sceneName The name of the scene.
     * @param {string} fileName The name of the JavaScript file.
     * @param {string} subfolder The subfolder the scene is contained in. Optional parameter.
     * @returns {string} The path to the JavaScript file.
     */
    function buildJSFilePath(sceneName, fileName, subfolder) {
        return buildSceneRoot(sceneName, subfolder) + fileName + ".js";
    }

    /**
     * Function to be called when something fails to load.
     * Shows an error-alert and hides the loading spinner.
     */
    function loadError() {
        global.error(sceneLoadErrorMessage);
    }

    /**
     * Loads the scene of which the name is passed into the game proper.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    function loadFinished(sceneName, subfolder, scrollLevel) {
        setTimeout(function () {
            that.registerStart();
            cachedScenes[sceneName].load();
            global.ui.loadbar.hide();
            setScrollLater(scrollLevel);
        }, 50);
    }

    /**
     * Loads a single JavaScript file using the advanced loading method.
     * @param {type} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {string} fileName The name of the file to load.
     */
    function loadJavaScript(sceneName, subfolder, fileName) {
        return new Promise(function (resolve, reject) {
            var script = document.createElement("script");
            script.type = `text/javascript`;
            script.onerror = reject;
            script.src = buildJSFilePath(sceneName, fileName, subfolder);
            script.onload = function () {
                loadingFiles--;
                global.ui.loadbar.update(loadingFiles);
                resolve();
            };
            document.body.appendChild(script); // Don't use jQuery for this.
        });
    }

    /**
     * Simultaneous, asynchronous JavaScript loader.
     * Also known as "advanced loading".
     * Loads all required files simultaneously, letting the browser manage download multiplicity.
     * @param {type} sceneName The name of the scene.
     * @param {array} queue The queue of files to load.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    function loadJavaScriptSimultaneously(sceneName, queue, subfolder, scrollLevel) {
        loadingFiles = queue.length;
        var promises = [];
        for (var i = 0; i < queue.length; i++) {
            promises.push(loadJavaScript(sceneName, subfolder, queue[i]));
        }

        Promise.all(promises).then(function () {
            loadFinished(sceneName, subfolder, scrollLevel);
        }, loadError);
    }

    /**
     * The developer way of loading scenes.
     * First loads the JSON, then loads all files by adding them to the DOM.
     * "Old" loading.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    function devLoad(sceneName, subfolder, scrollLevel) {
        $.getJSON(buildJsonFilePath(sceneName, subfolder))
                .done(function (data) {
                    global.ui.loadbar.show(data.length);
                    loadJavaScriptSimultaneously(sceneName, data, subfolder, scrollLevel);
                })
                .fail(loadError);
    }

    /**
     * Updates the UI when download progress happens.
     * @param {Event} evt The event triggered by the XHTML request.
     */
    function compiledLoadingProgressUpdate(evt) {
        if (evt.lengthComputable) {
            var percentComplete = Math.round((evt.loaded / evt.total) * 100);
            global.ui.loadbar.update(100 - percentComplete);
        }
    }

    /**
     * Creates an XMLHttpRequest with a registered listener that will listen to changes in download progress and update the UI accordingly.
     */
    function compiledLoadingProgressListen() {
        var xhr = new window.XMLHttpRequest();
        xhr.addEventListener("progress", compiledLoadingProgressUpdate);
        return xhr;
    }

    /**
     * Loads the compiled version of the requested scene.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    function compiledLoading(sceneName, subfolder, scrollLevel) {
        global.ui.loadbar.show(100);
        $.ajax({
            url: buildCompiledFilePath(sceneName, subfolder),
            dataType: "script",
            xhr: compiledLoadingProgressListen
        }).done(function () {
            loadFinished(sceneName, subfolder, scrollLevel);
        }).fail(loadError);
    }

    /**
     * Loads a scene via http.
     * Calls the simultaneous loading JavaScript loading function,
     * or the newer compiled loading depending on the settings.
     * Shows the loading indicator at the start.
     * @param {string} sceneName The name of the scene.
     * @param {string} subfolder The subfolder the scene can be found in. Optional parameter.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    function httpLoad(sceneName, subfolder, scrollLevel) {
        registerStarted = false;
        if (global.settings.getOldLoad()) {
            devLoad(sceneName, subfolder, scrollLevel);
        } else {
            compiledLoading(sceneName, subfolder, scrollLevel);
        }
    }

    /**
     * Processes the registration queue, emptying it one entry at a time, calling each registration function.
     * Does nothing if the registration process has not yet started.
     */
    function processRegistrationQueue() {
        if (!registerStarted) {
            return;
        }
        while (registrationQueue.length) {
            registrationQueue.shift()();
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads a scene.
     * Disables all currently playing audio, if any.
     * If cached, will just call the scene's load method.
     * If not cached, will dynamically load the scene using a http call.
     * @param {string} sceneName The name of the scene to load.
     * @param {string} subfolder An optional parameter declaring the subfolder the scene is in. 
     *                           The folder "scenes" and the folder with the name of the scene itself are always implied.
     * @param {int} scrollLevel How far down to scroll the scene. Optional parameter.
     */
    this.loadScene = function (sceneName, subfolder, scrollLevel) {
        global.audio.fadeAllAudio();
        if (cachedScenes[sceneName]) {
            cachedScenes[sceneName].load();
            setScrollLater(scrollLevel);
        } else {
            httpLoad(sceneName, subfolder, scrollLevel);
        }
    };

    /**
     * Loads a tab scene.
     * Does not disable any audio.
     * Assumes the tab scene is cached.
     * @param {string} sceneName The name of the tab scene to load.
     * @param {string} subfolder An optional parameter declaring the subfolder the tab scene is in. 
     *                           The folder "scenes" and the folder with the name of the scene itself are always implied.
     */
    this.loadTabScene = function (sceneName, subfolder) {
        if (cachedScenes[sceneName]) {
            cachedScenes[sceneName].load();
        } else {
            httpLoad(sceneName, subfolder);
        }
    };

    /**
     * Registers a scene with this SceneLoader.
     * Should ideally be immediately called from the scene's JavaScript file.
     * @param {string} name The name of the loaded scene. Will be the key to which the scene is registered.
     * @param {Scene} scene An instance of the scene-object, which will be registered with the name as its key.
     */
    this.registerScene = function (name, scene) {
        cachedScenes[name] = scene;
    };


    /**
     * Queues a scene registration, and immediately processes all pending registrations if registration processessing has been started.
     * @param {function} register Registration function.
     */
    this.queueRegister = function (register) {
        registrationQueue.push(register);
        processRegistrationQueue();
    };

    /**
     * Must be called when registration of scenes can begin.
     * The scene registration queue will remain filled until this function has been called.
     */
    this.registerStart = function () {
        registerStarted = true;
        processRegistrationQueue();
    };
}

// END res\js\loaders\SceneLoader.js

// res\js\minigames\lockpicking\components\background\LockPickingBackground.js

/**
 * Background for the lockpicking minigame.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingBackground(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;
    
    var image = new LockPickingPreloadingBackground();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML required for the minigame.
     */
    this.build = function () {
        var element = `<img class="background" src=${image.url} id="${elementId}">`;
        return element;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };
}

// END res\js\minigames\lockpicking\components\background\LockPickingBackground.js

// res\js\minigames\lockpicking\components\helpscreen\LockPickingHelpScreen.js

/**
 * Help screen for the lockpicking minigame with assorted button.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingHelpScreen(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;
    var buttonId = id + "_" + key + "_" + uniqueIdentifier + "Button";

    var image = new LockPickingPreloadingHelpScreen();

    var ctrl = new ChoiceController(id, elementId);
    var invisible = ctrl.has() && ctrl.get();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var screen = `<img class="help" ${invisible ? `style="display:none;"` : ``} src=${image.url} id="${elementId}">`;
        var button = `<button class="helpbutton" id="${buttonId}">?</button>`;
        return screen + button;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        $("#" + elementId).click(that.fade);
        $("#" + buttonId).click(that.show);
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Causes the element to fade out.
     */
    this.fade = function () {
        ctrl.set(true);
        $("#" + elementId).fadeOut(400);
    };

    /**
     * Causes the element to show.
     */
    this.show = function () {
        ctrl.set(false);
        $("#" + elementId).fadeIn(400);
    };
}

// END res\js\minigames\lockpicking\components\helpscreen\LockPickingHelpScreen.js

// res\js\minigames\lockpicking\components\loadingscreen\LockPickingLoadingScreen.js

/**
 * Loading screen for the lockpicking minigame.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingLoadingScreen(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    
    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;

    var visible = true;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var element = `<div class="loadingscreen" ${!visible ? `style="display:none;"` : ``} id="${elementId}">Loading</div>`;
        return element;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    };

    /**
     * Causes the element to fade out.
     */
    this.fade = function () {
        visible = false;
        $("#" + elementId).fadeOut(400);
    };
    
    /**
     * Causes the element to show.
     */
    this.show = function () {
        visible = true;
        $("#" + elementId).fadeIn(400);
    };
}

// END res\js\minigames\lockpicking\components\loadingscreen\LockPickingLoadingScreen.js

// res\js\minigames\lockpicking\components\lockpick\LockPickingLockPick.js

/**
 * Help screen for the lockpicking minigame with assorted button.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 * @param {LockPickingMinigame} master The master that created this component. 
 */
function LockPickingLockPick(id, key, master) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;
    var ctrl = new ChoiceController(id, elementId);
    var image = new LockPickingPreloadingLockPick();

    var amountOfClickBoxes = 5;
    var inTransition = false;
    var tapDelay = 300;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Called when a CSS transition ends.
     */
    function transitionEnd() {
        inTransition = false;
        $("#" + elementId).removeClass("lockpicktap");
    }

    /**
     * Plays the tap animation.
     */
    function tapAnimation() {
        inTransition = true;
        $("#" + elementId).addClass("lockpicktap");
    }

    /**
     * Turns the lockpick's position into a CSS class.
     * @returns {string} The CSS class.
     */
    function positionToClass() {
        return "lockpickpos" + that.position;
    }

    /**
     * Swaps position classes.
     * @param {string} oldClass Old position class.
     * @param {string} newClass New position class.
     */
    function swapClasses(oldClass, newClass) {
        inTransition = true;
        $("#" + elementId).removeClass(oldClass);
        $("#" + elementId).addClass(newClass);
    }

    /**
     * Attaches a listener to the box with the given rank number.
     * @param {int} number A number from 0 to 4 denoting the box.
     */
    function listenerFor(number) {
        $("#" + elementId + number).click(function () {
            if (that.position === number) {
                that.tap();
            } else {
                that.moveToPos(number);
            }
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    this.position = (ctrl.has() && ctrl.get()) || 0;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var lockpick = `<img src=${image.url} id="${elementId}" class="lockpick ${positionToClass()}">`;
        var boxes = ``;
        for (var i = 0; i < amountOfClickBoxes; i++) {
            boxes += `<div class="clickbox clickbox${i}" id="${elementId + i}"></div>`;
        }
        return lockpick + boxes;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        $("#" + elementId).on("transitionend MSTransitionEnd msTransitionEnd webkitTransitionEnd oTransitionEnd animationend oAnimationEnd webkitAnimationEnd", transitionEnd);
        for (var i = 0; i < amountOfClickBoxes; i++) {
            listenerFor(i);
        }
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Taps the lockpick.
     */
    this.tap = function () {
        if (inTransition) {
            return;
        }
        tapAnimation();
        setTimeout(function () {
            master.springsAndTumblers.up(that.position);
        }, tapDelay);
    };

    /**
     * Makes the lockpick break. Animation only.
     */
    this.break = function () {
        inTransition = true;
        $("#" + elementId).addClass("lockpickpos-1");
        setTimeout(function() {
            $("#" + elementId).removeClass("lockpickpos-1");
        }, 600);
    };

    /**
     * Moves the lockpick to a new position.
     * @param {int} newPos The new position to move to. A number from 0 to 4.
     */
    this.moveToPos = function (newPos) {
        ctrl.set(newPos);
        var currentClass = positionToClass();
        that.position = newPos;
        var newClass = positionToClass();
        swapClasses(currentClass, newClass);
    };
}

// END res\js\minigames\lockpicking\components\lockpick\LockPickingLockPick.js

// res\js\minigames\lockpicking\components\lockpickcount\LockPickingLockPickCount.js

/**
 * Lockpick count for the lockpicking minigame with assorted button.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingLockPickCount(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;

    var ctrl = new ChoiceController(id, elementId);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets the controller's value to how many lockpicks the kobold currently has.
     */
    function setController() {
        ctrl.set(global.kobold.inventory.bag.hasMisc(new Lockpick()));
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var lockpick = `<div class="text lockpickcount" id="${elementId}">Lockpicks left: ${ctrl.get()}</div>`;
        return lockpick;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return new Promise(function (resolve, reject) {
            resolve();
        });
    };

    /**
     * Updates the number of lockpicks.
     */
    this.update = function (){
        setController();
        $("#" + elementId).html(`Lockpicks left: ${ctrl.get()}`);
    };

    if (!ctrl.has()) {
        setController();
    }
}

// END res\js\minigames\lockpicking\components\lockpickcount\LockPickingLockPickCount.js

// res\js\minigames\lockpicking\components\losescreen\LockPickingLoseScreen.js

/**
 * Loss screen for the lockpicking minigame.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingLoseScreen(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;

    var image = new LockPickingPreloadingLossScreen();

    var ctrl = new ChoiceController(id, elementId);
    var visible = ctrl.has() && ctrl.get();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var screen = `<img ${visible ? `` : `style="display:none;"`} class="lose" src=${image.url} id="${elementId}">`;
        return screen;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Causes the element to fade out.
     */
    this.fade = function () {
        ctrl.set(false);
        $("#" + elementId).fadeOut(400);
    };

    /**
     * Causes the element to show.
     */
    this.show = function () {
        ctrl.set(true);
        $("#" + elementId).fadeIn(400);
    };
}

// END res\js\minigames\lockpicking\components\losescreen\LockPickingLoseScreen.js

// res\js\minigames\lockpicking\components\springsandtumblers\LockPickingSpringsAndTumblers.js

/**
 * Collects springs and tumblers.
 */
function LockPickingSpringsAndTumblers(id, key, master, num, diff) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;

    var elementId = id + "_" + key + "_" + uniqueIdentifier;
    var ctrl = new ChoiceController(id, elementId);

    if (!ctrl.has()) {
        ctrl.set(num);
    }

    var originalSeed = +new Date();
    var calculatedSeed = originalSeed;
    var graceDivider = 5;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Generates a random number.
     * @returns {number} A random number between 0 and 1.
     */
    function rng() {
        calculatedSeed = (calculatedSeed * 9301 + 49297) % 233280;
        return calculatedSeed / 233280;
    }

    /**
     * Generates a decimal number between minimum and maximum value.
     * Will never generate the maximum number itself.
     * @param {number} min The minimum number. Default 0.
     * @param {number} max The maximum number. Will never be generated. Default 1.
     * @returns {number} A random decimal between the minimum and maximum number.
     */
    function generateDecimal(min, max) {
        min = min || 0;
        max = max || 1;
        return min + rng() * (max - min);
    }

    /**
     * Generates an integral number between minimum and maximum value.
     * Will never generate the maximum number itself.
     * @param {number} min The minimum number. Default 0.
     * @param {number} max The maximum number. Will never be generated. Default 100.
     * @returns {number} A random integral number between the minimum and maximum number.
     */
    function generateIntegral(min, max) {
        min = min || 0;
        max = max || 100;
        return Math.floor(generateDecimal(min, max));
    }

    /**
     * Generates how long it should take for a tumbler to travel upwards.
     * @returns {int} How long it should take for a tumbler to travel upwards, in milliseconds.
     */
    function genTime() {
        var z = 200 - diff;
        var x = Math.floor(generateIntegral(0, z));
        var x2 = x * x;
        var base = -x2 + (z * x);
        var timescaled = Math.round(base / 10);
        var withmin = timescaled + 200;
        return withmin;
    }

    /**
     * Calculates a grace period from the given time.
     * @param {int} time How long it will take for the tumbler to travel upwards.
     * @returns {int} How much of a grace period the user will have to click it at the top.
     */
    function timeToGrace(time) {
        return Math.round(time / graceDivider);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.springs = new LockPickingSprings(id, key, master, num);
    this.tumblers = new LockPickingTumblers(id, key, master, num);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        return that.springs.build() + that.tumblers.build();
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        that.tumblers.registerListeners();
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return Promise.all([that.springs.preload(), that.tumblers.preload()]);
    };

    /**
     * Causes the specified tumbler to go up.
     * @param {int} which The number of the tumbler (from 0 to 4).
     */
    this.up = function (which) {
        if (that.tumblers.inAnimation(which)) {
            return;
        }
        // Make tumbler go up and spring contract
        var time = genTime();
        that.springs.up(which, time);
        var grace = timeToGrace(time);
        that.tumblers.up(which, time, grace);
    };

    /**
     * Records the completion of a tumbler.
     * Returns whether the game has been won or not.
     */
    this.done = function (position) {
        that.springs.stopSpring(position);
        ctrl.set(ctrl.get()-1);
        return !ctrl.get();
    };
    
    this.reset = function() {
        ctrl.set(num);
        that.tumblers.reset();
        that.springs.reset();
    };
}

// END res\js\minigames\lockpicking\components\springsandtumblers\LockPickingSpringsAndTumblers.js

// res\js\minigames\lockpicking\components\springsandtumblers\springs\LockPickingSpring.js

/**
 * A single spring.
 */
function LockPickingSpring(id, key, master, position, active) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier + position;

    var ctrl = new ChoiceController(id, elementId);

    if (!active) {
        ctrl.set(true);
    }

    var image = new LockPickingPreloadingSpring();
    
    var upHeight = "120px";
    var downHeight = "300px";
    var width = "150px";
    
    var downTime = 750;
    var fastDownTime = 300;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Animates the tumbler going up.
     */
    function animateUp(time) {
        $("#" + elementId).animate({height: upHeight}, time, animateDownSlow);
    }
    
    /**
     * Animates the tumbler going down slowly.
     */
    function animateDownSlow() {
        $("#" + elementId).animate({height: downHeight}, downTime);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML required for the minigame.
     */
    this.build = function () {
        var spring = `<img class="spring spring${position}" src=${image.url} id="${elementId}" style="width: ${width}; height: ${ctrl.get() ? upHeight : downHeight};">`;
        return spring;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Causes this spring to go up.
     * @param {int} time How quickly it should go up, in milliseconds
     */
    this.up = function (time) {
        if(ctrl.get()) {
            return;
        }
        animateUp(time);
    };
    
    this.stop = function() {
        ctrl.set(true);
        $("#" + elementId).stop(true, false);
    };
    
    /**
     * Causes this spring to go down.
     */
    this.reset = function() {
        if (!active) {
            return;
        }

        ctrl.set(false);
        $("#" + elementId).stop(true, false);
        $("#" + elementId).animate({height: downHeight}, fastDownTime);
    };
}

// END res\js\minigames\lockpicking\components\springsandtumblers\springs\LockPickingSpring.js

// res\js\minigames\lockpicking\components\springsandtumblers\springs\LockPickingSprings.js

/**
 * All of the springs.
 */
function LockPickingSprings(id, key, master, springsActive) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var amountOfSprings = 5;

    var image = new LockPickingPreloadingSpring();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the springs.
     */
    function populateSprings() {
        for (var i = 0; i < springsActive; i++) {
            that.springs.push(new LockPickingSpring(id, key, master, i, true));
        }
        for (var i = springsActive; i < amountOfSprings; i++) {
            that.springs.push(new LockPickingSpring(id, key, master, i, false));
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.springs = [];


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var output = "";
        for (var i = 0; i < that.springs.length; i++) {
            output += that.springs[i].build();
        }
        return output;
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Causes the specified spring to go up.
     * @param {int} which Which tumbler needs to go up.
     * @param {int} time How quickly it should go up, in milliseconds.
     */
    this.up = function (which, time) {
        that.springs[which].up(time);
    };

    this.stopSpring = function (position) {
        that.springs[position].stop();
    };

    this.reset = function () {
        for (var i = 0; i < springsActive; i++) {
            that.springs[i].reset();
        }
    };

    populateSprings();
}

// END res\js\minigames\lockpicking\components\springsandtumblers\springs\LockPickingSprings.js

// res\js\minigames\lockpicking\components\springsandtumblers\tumblers\LockPickingTumbler.js

/**
 * Information about the class.
 */
function LockPickingTumbler(id, key, master, position, active) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var upTop = "90px";
    var downTop = "250px";

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier + position;

    var ctrl = new ChoiceController(id, elementId);

    if (!active) {
        ctrl.set(true);
    }

    var image = new LockPickingPreloadingTumbler();

    var upStart;
    var upTime;
    var upGrace;

    var downTime = 750;
    var fastDownTime = 300;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Animates the tumbler going up.
     */
    function animateUp() {
        that.inAnimation = true;
        $("#" + elementId).animate({top: upTop}, upTime, animateDownSlow);
    }

    /**
     * Animates the tumbler going down slowly.
     */
    function animateDownSlow() {
        $("#" + elementId).animate({top: downTop}, downTime, animateDone);
    }

    /**
     * Called when the animation is over.
     */
    function animateDone() {
        that.inAnimation = false;
    }

    /**
     * Tries to set the tumbler in place.
     */
    function inPlace() {
        if (!that.inAnimation) {
            return;
        }
        if(clickCheck()) {
            ctrl.set(true);
            $("#" + elementId).stop(true, false);
            that.inAnimation = false;
            master.done(position);
        } else {
            master.fail(position);
        }
    }

    /**
     * Checks whether the tumbler was successfully set in place.
     * @returns {boolean} True if the setting was successful, false otherwise.
     */
    function clickCheck() {
        var timeElapsed = +new Date() - upStart;
        var absoluteDifference = Math.abs(upTime - timeElapsed);
        return absoluteDifference < Math.ceil(upGrace / 2);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.inAnimation = false;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    this.build = function () {
        var top = ctrl.get() ? upTop : downTop;
        var tumbler = `<img class="tumbler tumbler${position}" src=${image.url} id="${elementId}" style="top: ${top};">`;
        //var clickbox = `<div class="tumblerclickbox tumblerclickbox${position}"></div>`;
        var clickbox = "";
        return tumbler + clickbox;
    };

    this.registerListeners = function () {
        $("#" + elementId).click(inPlace);
    };

    /**
     * Causes this tumbler to go up.
     * @param {int} time How quickly it should go up, in milliseconds.
     * @param {int} grace Distributed around the peak of the tumbler's trajectory, how long in milliseconds the user has to click it to succeed.
     */
    this.up = function (time, grace) {
        if (ctrl.get()) {
            return;
        }
        upStart = +new Date();
        upTime = time;
        upGrace = grace;
        animateUp();
    };
    
    /**
     * Causes this tumbler to go down.
     */
    this.reset = function() {
        if (!active) {
            return;
        }

        ctrl.set(false);
        $("#" + elementId).stop(true, false);
        that.inAnimation = true;
        $("#" + elementId).animate({top: downTop}, fastDownTime, animateDone);
    };
}

// END res\js\minigames\lockpicking\components\springsandtumblers\tumblers\LockPickingTumbler.js

// res\js\minigames\lockpicking\components\springsandtumblers\tumblers\LockPickingTumblers.js

/**
 * All of the tumblers.
 */
function LockPickingTumblers(id, key, master, tumblersActive) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var amountOfTumblers = 5;

    var image = new LockPickingPreloadingTumbler();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the tumblers.
     */
    function populateTumblers() {
        for (var i = 0; i < tumblersActive; i++) {
            that.tumblers.push(new LockPickingTumbler(id, key, master, i, true));
        }
        for (var i = tumblersActive; i < amountOfTumblers; i++) {
            that.tumblers.push(new LockPickingTumbler(id, key, master, i, false));
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.tumblers = [];


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var output = "";
        for (var i = 0; i < that.tumblers.length; i++) {
            output += that.tumblers[i].build();
        }
        return output;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        for (var i = 0; i < that.tumblers.length; i++) {
            that.tumblers[i].registerListeners();
        }
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Causes the specified tumbler to go up.
     * @param {int} which Which tumbler needs to go up.
     * @param {int} time How quickly it should go up, in milliseconds.
     * @param {int} grace Distributed around the peak of the tumbler's trajectory, how long in milliseconds the user has to click it to succeed.
     */
    this.up = function (which, time, grace) {
        that.tumblers[which].up(time, grace);
    };

    /**
     * Returns whether or not the specified tumbler is mid-animation.
     * @param {int} which Which tumbler.
     * @return {boolean} Whether or not the specified tumbler is mid-animation.
     */
    this.inAnimation = function (which) {
        return that.tumblers[which].inAnimation;
    };

    this.reset = function () {
        for (var i = 0; i < tumblersActive; i++) {
            that.tumblers[i].reset();
        }
    };

    populateTumblers();
}

// END res\js\minigames\lockpicking\components\springsandtumblers\tumblers\LockPickingTumblers.js

// res\js\minigames\lockpicking\components\winscreen\LockPickingWinScreen.js

/**
 * Loss screen for the lockpicking minigame.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 */
function LockPickingWinScreen(id, key) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;

    var image = new LockPickingPreloadingWinScreen();

    var ctrl = new ChoiceController(id, elementId);
    var visible = ctrl.has() && ctrl.get();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var screen = `<img ${visible ? `` : `style="display:none;"`} class="win" src=${image.url} id="${elementId}">`;
        return screen;
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {

    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        return image.preload();
    };

    /**
     * Causes the element to fade out.
     */
    this.fade = function () {
        ctrl.set(false);
        $("#" + elementId).fadeOut(400);
    };

    /**
     * Causes the element to show.
     */
    this.show = function () {
        ctrl.set(true);
        $("#" + elementId).fadeIn(400);
    };
}

// END res\js\minigames\lockpicking\components\winscreen\LockPickingWinScreen.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingBackground.js

/**
 * Preloading image.
 */
function LockPickingPreloadingBackground() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/background.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingBackground.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingHelpScreen.js

/**
 * Preloading image.
 */
function LockPickingPreloadingHelpScreen() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/help.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingHelpScreen.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingLockPick.js

/**
 * Preloading image.
 */
function LockPickingPreloadingLockPick() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/lockpick.png";

    PreloadingImage.call(this, this.cacheId, this.url);


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingLockPick.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingLossScreen.js

/**
 * Preloading image.
 */
function LockPickingPreloadingLossScreen() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/loss.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingLossScreen.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingSpring.js

/**
 * Preloading image.
 */
function LockPickingPreloadingSpring() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/spring.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingSpring.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingTumbler.js

/**
 * Preloading image.
 */
function LockPickingPreloadingTumbler() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/tumbler.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingTumbler.js

// res\js\minigames\lockpicking\images\LockPickingPreloadingWinScreen.js

/**
 * Preloading image.
 */
function LockPickingPreloadingWinScreen() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.cacheId = arguments.callee.name;
    this.url = "res/img/minigames/lockpicking/win.png";

    PreloadingImage.call(this, this.cacheId, this.url);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\minigames\lockpicking\images\LockPickingPreloadingWinScreen.js

// res\js\minigames\lockpicking\LockPickingMinigame.js

/**
 * Lockpicking minigame.
 * @param {string} id Scene id. Used to create controllers.
 * @param {string} key Unique key-space for the lockpicking minigame.
 * @param {int} difficulty How hard the lockpicking minigame should be. Alters tumbler travel time and allowed locking times.
 * @param {int} numTumblers  How many tumblers to use. A number from one to five.
 * @param {function} onLoss Function to be called when the player loses the minigame.
 * @param {function} onWin Function to be called when the player wins the minigame.
 */
function LockPickingMinigame(id, key, difficulty, numTumblers, onLoss, onWin) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var uniqueIdentifier = arguments.callee.name;
    var elementId = id + "_" + key + "_" + uniqueIdentifier;

    var ctrl = new ChoiceController(id, elementId);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Called when all components have finished preloading.
     */
    function preloadDone() {
        that.loadingScreen.fade();
    }

    /**
     * Build the minigame itself.
     * @param {string} components All of the built components.
     */
    function buildSelf(components) {
        var part = `<div id=${elementId} class="lockpicking">`;
        part += components;
        part += `</div>`;
        return part;
    }

    /**
     * Register the minigame's own listeners.
     */
    function ownListeners() {

    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.loadingScreen = new LockPickingLoadingScreen(id, key);
    this.helpScreen = new LockPickingHelpScreen(id, key);
    this.winScreen = new LockPickingWinScreen(id, key);
    this.loseScreen = new LockPickingLoseScreen(id, key);
    this.background = new LockPickingBackground(id, key);
    this.lockpick = new LockPickingLockPick(id, key, this);
    this.lockpickCount = new LockPickingLockPickCount(id, key);
    this.springsAndTumblers = new LockPickingSpringsAndTumblers(id, key, this, numTumblers, difficulty);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the HTML.
     */
    this.build = function () {
        var components = that.loadingScreen.build();
        components += that.helpScreen.build();
        components += that.winScreen.build();
        components += that.loseScreen.build();
        components += that.background.build();
        components += that.lockpick.build();
        components += that.lockpickCount.build();
        components += that.springsAndTumblers.build();
        return buildSelf(components);
    };

    /**
     * Registers all the required listeners.
     */
    this.registerListeners = function () {
        that.loadingScreen.registerListeners();
        that.helpScreen.registerListeners();
        that.winScreen.registerListeners();
        that.loseScreen.registerListeners();
        that.background.registerListeners();
        that.lockpick.registerListeners();
        that.lockpickCount.registerListeners();
        that.springsAndTumblers.registerListeners();
        ownListeners();
    };

    /**
     * Preloads all assets required, if they have not yet already been loaded.
     */
    this.preload = function () {
        Promise.all([
            that.loadingScreen.preload(),
            that.helpScreen.preload(),
            that.winScreen.preload(),
            that.loseScreen.preload(),
            that.background.preload(),
            that.lockpick.preload(),
            that.lockpickCount.preload(),
            that.springsAndTumblers.preload()
        ]).then(preloadDone);
    };

    /**
     * Called when a tumbler has successfully finished.
     * @param {int} position Which tumbler.
     */
    this.done = function (position) {
        if (that.springsAndTumblers.done(position)) {
            that.winScreen.show();
            onWin && onWin();
        }
    };

    /**
     * Called when the user fails to set a tumbler in place.
     * @param {int} position Which tumbler.
     */
    this.fail = function () {
        global.kobold.inventory.bag.removeMisc(new Lockpick());
        that.springsAndTumblers.reset();
        setTimeout(function () {
            that.lockpick.break();
            that.lockpickCount.update();
            setTimeout(function () {
                if (!global.kobold.inventory.bag.hasMisc(new Lockpick())) {
                    that.loseScreen.show();
                    onLoss && onLoss();
                }
            }, 100);
        }, 300);
    };
}

// END res\js\minigames\lockpicking\LockPickingMinigame.js

// res\js\minigames\preload\PreloadingImage.js

/**
 * Image capable of preloading into an invisible element.
 * @param {string} id A unique identifier for the preloading image.
 * @param {string} url URL to the image resource.
 */
function PreloadingImage(id, url) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var invisibleClass = "invisible";

    var promiseResolve;
    var promiseReject;
    var promise = new Promise(function (resolve, reject) {
        promiseResolve = resolve;
        promiseReject = reject;
    });

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether or not the image already exists.
     * @returns {boolean} Truthy if the image already exists, false otherwise.
     */
    function exists() {
        return $("#" + id).length;
    }

    /**
     * Checks whether or not the image is currently loading.
     * @returns {boolean} Truthy if the image exists and is already loading, false otherwise.
     */
    function isLoading() {
        return exists() && !$("#" + id)[0].complete;
    }

    /**
     * Adds the image to the DOM, with a registered load listener.
     */
    function addImage() {
        var img = document.createElement("img");
        img.onload = promiseResolve;
        img.className = invisibleClass;
        img.src = url;
        img.id = id;
        $("body").append(img);
    }

    /**
     * Adds the onload listener to an image already loading.
     */
    function addListenerToExistingImage() {
        var img = $("#" + id)[0];
        if (img.onload) {
            var original = img.onload;
            img.onload = function () {
                original();
                promiseResolve();
            };
        } else {
            img.onload = promiseResolve;
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the image load itself if it has not yet loaded.
     * @returns {Promise} A promise which will resolve when the image has finished loading, or immediately if it already has loaded.
     */
    this.preload = function () {
        if (!exists()) {
            addImage();
        } else if (isLoading()) {
            addListenerToExistingImage();
        } else {
            promiseResolve();
        }
        return promise;
    };
}

// END res\js\minigames\preload\PreloadingImage.js

// res\js\parser\KoboldParser.js

/**
 * Parses scenes from BBcode to HTML.
 */
function KoboldParser() {
    // PRIVATE MEMBERS \\       var private = "test";

    var colorClasses = {
        red: "red",
        bred: "brightred",
        dred: "darkred",
        pred: "pinkred",
        blue: "blue",
        cyan: "cyan",
        gblue: "greyishblue",
        pink: "pink",
        ppink: "palepink",
        purple: "purple",
        yellow: "yellow",
        green: "green",
        mgreen: "milgreen",
        orange: "orange",
        white: "white",
        black: "black",
        brown: "brown"
    };

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Replaces fail bbcode with its HTML representation.
     * Must be called before big, center, paragraph and color.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function fail(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[fail\]/g, '[p][c][big][color=pred]&gt; FAILED: ');
        toReturn = toReturn.replace(/\[\/fail\]/g, '[/color][/big][/c][/p]');
        return toReturn;
    }

    /**
     * Replaces succ bbcode with its HTML representation.
     * Must be called before big, center and color.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function succ(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[succ\]/g, '[p][c][big][color=mgreen]&gt; SUCCESS: ');
        toReturn = toReturn.replace(/\[\/succ\]/g, '[/color][/big][/c][/p]');
        return toReturn;
    }

    /**
     * Replaces paragraph bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function title(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[t\]/g, '<div class="title fancy">');
        toReturn = toReturn.replace(/\[\/t\]/g, "</div>");
        return toReturn;
    }

    /**
     * Replaces paragraph bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function subtitle(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[st\]/g, '<div class="subtitle">');
        toReturn = toReturn.replace(/\[\/st\]/g, "</div>");
        return toReturn;
    }

    /**
     * Replaces paragraph bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function paragraph(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[p\]/g, '<div class="text">');
        toReturn = toReturn.replace(/\[\/p\]/g, "</div>");
        return toReturn;
    }

    /**
     * Replaces center bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function center(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[c\]/g, '<div class="center">');
        toReturn = toReturn.replace(/\[\/c\]/g, "</div>");
        return toReturn;
    }

    /**
     * Replaces bold bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function bold(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[b\]/g, '<span class="bold">');
        toReturn = toReturn.replace(/\[\/b\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces italic bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function italic(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[i\]/g, '<span class="italic">');
        toReturn = toReturn.replace(/\[\/i\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces underline bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function strikethrough(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[s\]/g, '<span class="strikethrough">');
        toReturn = toReturn.replace(/\[\/s\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces underline bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function underline(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[u\]/g, '<span class="underline">');
        toReturn = toReturn.replace(/\[\/u\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces small bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function small(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[small\]/g, '<span class="small">');
        toReturn = toReturn.replace(/\[\/small\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces sub bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function sub(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[sub\]/g, '<span class="small">');
        toReturn = toReturn.replace(/\[\/sub\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces big bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function big(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[big\]/g, '<span class="big">');
        toReturn = toReturn.replace(/\[\/big\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces sneaky bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function sneaky(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[sneak\]/g, '<span class="sneakytext">');
        toReturn = toReturn.replace(/\[\/sneak\]/g, "</span>");
        return toReturn;
    }

    /**
     * Returns the class corresponding with the color,
     * or null if no such class exists.
     * @param {string} color The color to get a class for.
     * @returns {string} The class of the color, or null if it is a bad color.
     */
    function colorClass(color) {
        if (colorClasses.hasOwnProperty(color)) {
            return colorClasses[color];
        } else {
            return null;
        }
    }

    /**
     * Replaces the first occurrence of needle with an appropriate color span,
     * or removes the occurrence altogether if it does not match a predefined color,
     * replacing it with a <span>.
     * @param {string} string The bbcode string in which to replace.
     * @param {array} needle The color-needle.
     * @returns {string} The replaced string.
     */
    function replaceColor(string, needle) {
        var toReturn = string;
        var color = needle;
        color = color.substring(7); // Remove "[color="
        color = color.slice(0, -1); // remove "]"
        var css = colorClass(color);
        if (css) {
            toReturn = toReturn.replace(needle, `<span class="${css}">`);
        } else {
            toReturn = toReturn.replace(needle, `<span>`);
        }
        return toReturn;
    }

    /**
     * Replaces each of the needles with its appropriate color, if any.
     * @param {string} string The bbcode string in which to replace.
     * @param {array} needles The color-needles.
     * @returns {string} The replaced string.
     */
    function replaceColors(string, needles) {
        var toReturn = string;
        if (!needles) {
            return toReturn;
        }
        for (var i = 0; i < needles.length; i++) {
            toReturn = replaceColor(toReturn, needles[i]);
        }
        return toReturn;
    }

    /**
     * Replaces color bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function color(string) {
        var toReturn = string;
        var needles = toReturn.match(/\[color=([^\]]{3,6})\]/g);
        toReturn = replaceColors(toReturn, needles);
        toReturn = toReturn.replace(/\[\/color\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces comment bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function comment(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[com\]/g, '<span class="invisible">');
        toReturn = toReturn.replace(/\[\/com\]/g, "</span>");
        return toReturn;
    }

    /**
     * Replaces newline bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function newline(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[nl\]/g, '<br/>');
        toReturn = toReturn.replace(/\[\/nl\]/g, "");
        return toReturn;
    }

    /**
     * Replaces hline bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function hline(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[hl\]/g, '<hr class="smallseparator">');
        toReturn = toReturn.replace(/\[\/hl\]/g, "");
        return toReturn;
    }

    /**
     * Replaces todo bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function todo(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[todo\]/g, '<span class="todo">');
        toReturn = toReturn.replace(/\[\/todo\]/g, "</span>");
        return toReturn;
    }
    
    /**
     * Replaces warn bbcode with its HTML representation.
     * @param {string} string The bbcode string in which to replace.
     * @returns {string} The replaced string.
     */
    function warn(string) {
        var toReturn = string;
        toReturn = toReturn.replace(/\[warn\]/g, '<span class="warningtext">');
        toReturn = toReturn.replace(/\[\/warn\]/g, "</span>");
        return toReturn;
    }

    /**
     * Transforms all BBcode tags into their appropriate HTML counterparts.
     * @param {string} string The message to transform.
     * @returns {string} The transformed message.
     */
    function tags(string) {
        var toReturn = string;
        toReturn = fail(toReturn);
        toReturn = succ(toReturn);
        toReturn = title(toReturn);
        toReturn = subtitle(toReturn);
        toReturn = paragraph(toReturn);
        toReturn = center(toReturn);
        toReturn = bold(toReturn);
        toReturn = italic(toReturn);
        toReturn = strikethrough(toReturn);
        toReturn = underline(toReturn);
        toReturn = small(toReturn);
        toReturn = sub(toReturn);
        toReturn = big(toReturn);
        toReturn = sneaky(toReturn);
        toReturn = color(toReturn);
        toReturn = comment(toReturn);
        toReturn = newline(toReturn);
        toReturn = hline(toReturn);
        toReturn = todo(toReturn);
        toReturn = warn(toReturn);
        return toReturn;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Parses the BBcode string to HTML.
     * Returns the BBcode string if the
     * @param {string} bbcode The BBcode.
     * @returns {string} HTML representing the BBcode.
     */
    this.parse = function (bbcode) {
        return tags(bbcode);
    };
}

// END res\js\parser\KoboldParser.js

// res\js\saver\export\SaveExporter.js

/**
 * Class capable of exporting single saves or entire save lists to file.
 */
function SaveExporter() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;


    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Constructs an appropriate save name.
     * @param {boolean} all Whether this concerns all saves or not.
     * @param {int} slot The specific slot of the save, in case this does not concern all saves.
     * @returns {string} The save name.
     */
    function constructSaveName(all, slot) {
        if (all) {
            return `savelist.kas`;
        }
        slot = slot || 0;
        slot++;
        return `save${slot}.kas`;
    }

    /**
     * Downloads the passed text as a file with the given filename.
     * @param {string} filename The default filename.
     * @param {string} text The file's contents.
     */
    function download(filename, text) {
        var element = document.createElement('a');
        element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(text));
        element.setAttribute('download', filename);
        element.style.display = 'none';
        document.body.appendChild(element);
        element.click();
        document.body.removeChild(element);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Exports a single save slot to a file and prompts download thereof.
     * @param {int} slot The save slot to export.
     */
    this.slot = function (slot) {
        var json = global.saveManager.slotToJson(slot);
        var jsonArray = "[" + json + "]";
        var fileName = constructSaveName(false, slot);
        download(fileName, jsonArray);
    };

    /**
     * Exports all the save slots to a file and prompts download thereof.
     */
    this.all = function () {
        var saveList = global.saveManager.saveList();
        var jsonArray = "[";
        for (var i in saveList) {
            var slot = saveList[i].slot;
            var json = global.saveManager.slotToJson(slot);
            jsonArray += json + ",";
        }
        if (saveList.length) { // Remove trailing comma if necessary.
            jsonArray = jsonArray.substring(0, jsonArray.length - 1);
        }
        jsonArray += "]";
        var fileName = constructSaveName(true, false);
        download(fileName, jsonArray);
    };
}

// END res\js\saver\export\SaveExporter.js

// res\js\saver\import\SaveImporter.js

/**
 * Class capable of importing single saves or entire save lists from file.
 * Does not distinguish between lists and single saves.
 */
function SaveImporter() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var fileErrorCached = false;
    var saveLimitErrorCached = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Caches a file error.
     */
    function fileError() {
        fileErrorCached = true;
    }

    /**
     * Caches a save limit error.
     */
    function saveLimitError() {
        saveLimitErrorCached = true;
    }

    /**
     * Resolves any cached file reading errors.
     */
    function resolveFileError() {
        if (fileErrorCached) {
            fileErrorCached = false;
            alert("One or more of the selected files failed to parse. They may be corrupted.");
        }
    }

    /**
     * Resolves any cached save limit errors.
     */
    function resolveSaveLimitError() {
        if (saveLimitErrorCached) {
            saveLimitErrorCached = false;
            alert("The limit of total saves has been reached. Some saves may not have been added!");
        }
    }

    /**
     * Resolves all error caches, telling the user what went wrong (if anything).
     */
    function resolveErrorCaches() {
        resolveFileError();
        resolveSaveLimitError();
    }

    /**
     * Attempts to turn the passed text into a JSON-parsed array.
     * Shows an error message if this fails.
     * @param {string} text File-read text.
     * @returns {Array} The parsed array. Empty if an error occured.
     */
    function textToArray(text) {
        var array;
        try {
            array = JSON.parse(text);
        } catch (e) {
            fileError();
            return [];
        }
        if (!array || !Array.isArray(array)) {
            fileError();
            return [];
        }
        return array;
    }

    /**
     * Adds the passed object representing a save to the game's list of saves.
     * @param {object} object The object representing a save. Will be parsed to JSON.
     */
    function save(object) {
        var slot = global.saveManager.getFirstOpenSaveSlot();
        if (slot < 0) {
            saveLimitError();
            return;
        }
        var json = JSON.stringify(object);
        global.saveManager.jsonToSlot(slot, json);
    }

    /**
     * Extracts the saves from the text, which is assumed to be a JSON-array.
     * Shows an alert when the save is malformed.
     * @param {string} text The text to extract saves from. Assumed to be a JSON-array.
     */
    function saves(text) {
        var array = textToArray(text);
        for (var i in array) {
            save(array[i]);
        }
    }

    /**
     * Called when the filereader is done reading the file.
     * Will extract the text read from the file and pass it to the save creation function.
     * @param {event} event The done reading event.
     */
    function doneReading(event) {
        var text = event.target.result;
        saves(text);
    }

    /**
     * Imports a save file.
     * Shows a file dialogue to pick the file.
     * @param {File} toRead A single file taken from a FileList produced by an input event.
     * @returns {Promise} A promise that will resolve when the file has been read and parsed.
     */
    function file(toRead) {
        var reader = new FileReader();
        var promiseResolve;
        var promise = new Promise(function (resolve, reject) {
            promiseResolve = resolve;
        });
        reader.onload = function (event) {
            doneReading(event);
            promiseResolve();
        };
        reader.readAsText(toRead);
        return promise;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Imports multiple save files.
     * Shows a file dialogue to pick the file.
     * @param {FileList} fileList The FileList produced by the input event.
     * @param {function} fun Callback function. Gets called after every file has been read.
     */
    this.files = function (fileList, fun) {
        if (!window.File || !window.FileReader || !window.FileList || !window.Blob) {
            alert("Browser does not support file importing.");
            return;
        }
        var loadingPromises = [];
        for (var i = 0; i < fileList.length; i++) {
            loadingPromises.push(file(fileList[i]));
        }
        Promise.all(loadingPromises).then(function () {
            resolveErrorCaches();
            fun && fun();
        });
    };

    this.test = file;

}

// END res\js\saver\import\SaveImporter.js

// res\js\saver\KoboldAdventureSave.js

/**
 * A class representing a single save.
 * Helper class for the SaveGameManager.
 * Should not be used outside of that class.
 * Only works one way (TO json).
 * @param {Character} character The kobold to save.
 * @param {string} currentScene The current scene. 
 * @param {string} currentSceneName The name of the current scene. 
 * @param {string} currentSceneFolder The subfolder of the current scene. 
 * @param {string} saveComment A comment to add to the save.
 * @param {int} scroll The scroll level of the main scene.
 * @param {string} timeStamp The time at which the save was made.
 */
function KoboldAdventureSave(character, currentScene, currentSceneName, currentSceneFolder, saveComment, scroll, timeStamp) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.kobold = character;
    this.scene = currentScene;
    this.sceneName = currentSceneName;
    this.folder = currentSceneFolder;
    this.comment = saveComment;
    this.sceneScroll = scroll || 0;
    this.time = timeStamp;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Returns a JSON representation of this save.
     * @returns A JSON representation of this save.
     */
    this.toJson = function(){
        return JSON.stringify(that);
    };
}

// END res\js\saver\KoboldAdventureSave.js

// res\js\saver\SaveManager.js

/**
 * The SaveGameManager allows for the saving and loading of Kobold Adventure.
 * It also allows for easy enumeration over the various saves.
 */
function SaveManager() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var autoSaveSlot = "autosave";
    var newGameSlot = "new";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Fetches data from localstorage at the specified slot,
     * and returns it as a raw string.
     * @param {int} slot The slot to fetch data from.
     * @returns {string} The slot data, as a string.
     */
    function getSlotRaw(slot) {
        return localStorage[slot];
    }

    /**
     * Fetches data from localstorage at the specified slot,
     * converts the data into an object,
     * and returns that object.
     * @param {int} slot The slot to fetch data from.
     * @returns {object} The JSON data contained in that slot, parsed to an object.
     */
    function readSlot(slot) {
        var json = getSlotRaw(slot);
        return JSON.parse(json);
    }

    /**
     * Checks whether the slot is occupied or not.
     * @param {int} slot The slot to check.
     * @returns {boolean} True if the slot is occupied, false otherwise.
     */
    function slotOccupied(slot) {
        return localStorage.getItem(slot) !== null;
    }

    /**
     * Builds a SaveMetaData object from the save in the passed slot.
     * @param {string} slot The slot to read from.
     * @returns {SaveMetaData} The constructed SaveMetaData object.
     */
    function metaDataFromSlot(slot) {
        var obj = readSlot(slot);
        return new SaveMetaData(slot, obj.scene, obj.sceneName, obj.comment, obj.kobold.name, obj.kobold.body.gender, obj.time);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.maxAmountOfSaves = 99;
    this.import = new SaveImporter();
    this.export = new SaveExporter();

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Autosaves the game.
     * @param {bool} noThrobber If thruthy, no saving throbber will be shown. May be omitted.
     */
    this.autoSave = function (noThrobber) {
        that.save(autoSaveSlot, "Autosave", noThrobber);
    };

    /**
     * Saves the current state to the new game slot.
     */
    this.newGameSave = function () {
        that.save(newGameSlot, "New game", true);
    };

    /**
     * Checks if an autosave exists.
     * @returns {boolean} True if it does, false otherwise.
     */
    this.hasAutoSave = function () {
        return slotOccupied(autoSaveSlot);
    };

    /**
     * Checks if a new game save exists.
     * @returns {boolean} True if it does, false otherwise.
     */
    this.hasNewGameSave = function () {
        return slotOccupied(newGameSlot);
    };

    /**
     * Gets the meta information of the save in the autosave slot.
     */
    this.getAutoSaveMeta = function () {
        return metaDataFromSlot(autoSaveSlot);
    };

    /**
     * Gets the meta information of the save in the new game slot.
     */
    this.getNewGameSaveMeta = function () {
        return metaDataFromSlot(newGameSlot);
    };

    /**
     * Loads the game from the auto save slot.
     */
    this.loadAutoSave = function () {
        that.load(autoSaveSlot);
    };

    /**
     * Loads the game from the start save slot.
     */
    this.loadNewGame = function () {
        that.load(newGameSlot);
    };

    /**
     * Saves the game. 
     * @param {int} slot The slot to save to.
     * @param {string} comment A comment to store alongside the save. May be empty.
     * @param {bool} noThrobber If thruthy, no saving throbber will be shown. May be omitted.
     */
    this.save = function (slot, comment, noThrobber) {
        if (!noThrobber) {
            global.ui.throbbers.save.showAndHide();
        }
        if (!comment) {
            comment = "";
        }
        var toSave = new KoboldAdventureSave(global.kobold, global.currentScene, global.currentSceneName, global.currentSceneFolder, comment, global.ui.sceneScroll, Date.now());
        localStorage.setItem(slot, toSave.toJson());
    };

    /**
     * Loads the game from a saveslot.
     * Sets the global kobold object, 
     * fades out the current content 
     * and then loads the new scene.
     * @param {int} slot The slot to load from.
     */
    this.load = function (slot) {
        var obj = readSlot(slot);
        var kobold = new Character();
        kobold.load(obj.kobold);
        global.kobold = kobold;
        var scroll = obj.sceneScroll || 0;
        global.ui.fadeOutContent(function () {
            global.ui.tabs.scene.activate();
            global.sceneLoader.loadScene(obj.scene, obj.folder, scroll);
        });
    };

    /**
     * Deletes the save at the specified slot.
     * @param {int} slot The slot to delete.
     */
    this.delete = function (slot) {
        localStorage.removeItem(slot);
    };

    /**
     * Fetches and returns the Meta object for the save at the passed slot.
     * @param {string} slot The slot for which to fetch meta data.
     * @returns {SaveMetaData} The fetched SaveMetaData object.
     */
    this.getMeta = function (slot) {
        return metaDataFromSlot(slot);
    };

    /**
     * Looks up and returns the id of the first open save slot.
     * Returns -1 if no save slot is available.
     * @returns {int} The id of the first open save slot. -1 if none are available.
     */
    this.getFirstOpenSaveSlot = function () {
        var i = 0;

        while (slotOccupied(i)) {
            i++;
        }

        if (i >= that.maxAmountOfSaves) {
            return -1;
        }

        return i;
    };

    /**
     * Fetches a summation list of save slots.
     * The list will contain one SaveMetaData object entry per save in use.
     * The list may be empty.
     * Does not include information about the autosave or the new game save.
     * @returns {array} An array of SaveMetaData objects.
     */
    this.saveList = function () {
        var toReturn = [];

        for (var i = 0; i < that.maxAmountOfSaves; i++) {
            if (slotOccupied(i)) {
                toReturn.push(metaDataFromSlot(i));
            }
        }

        return toReturn;
    };

    /**
     * Returns how many saves there are currently present, apart from the
     * autosave and the starting save. The same as the length of the save list.
     * @returns {int} How many saves there currently are.
     */
    this.getAmountOfSaves = function () {
        var toReturn = 0;

        for (var i = 0; i < that.maxAmountOfSaves; i++) {
            if (slotOccupied(i)) {
                toReturn++;
            }
        }

        return toReturn;
    };

    /**
     * Turns the specified saveslot into a JSON representation thereof.
     * @param {int} slot The slot to turn into JSON.
     * @returns {string} The slot, in JSON-string format.
     */
    this.slotToJson = function (slot) {
        return getSlotRaw(slot);
    };

    /**
     * Turns the passed JSON into a saveslot.
     * @param {int} slot The number of the slot to use.
     * @param {string} json The raw save data JSON.
     */
    this.jsonToSlot = function (slot, json) {
        localStorage[slot] = json;
    };
}

// END res\js\saver\SaveManager.js

// res\js\saver\SaveMetaData.js

/**
 * A class representing the metadata of a single save.
 * @param {int} id The slot the save was read from.
 * @param {string} currentScene The save's current scene.
 * @param {string} currentSceneName The name of the save's current scene.
 * @param {string} saveComment The save's comment.
 * @param {string} saveName The name of the kobold.
 * @param {GenderEnum} saveGender The gender of the kobold.
 * @param {string} timeStamp The time at which the save was made.
 */
function SaveMetaData(id, currentScene, currentSceneName, saveComment, saveName, saveGender, timeStamp) {
    // PRIVATE MEMBERS \\       var private = "test";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.slot = id;
    this.scene = currentScene;
    this.sceneName = currentSceneName;
    this.comment = saveComment;
    this.name = saveName;
    this.gender = saveGender;
    this.time = timeStamp;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\saver\SaveMetaData.js

// res\js\SceneMap.js

/**
 * Contains a mapping from scene name to scene folder.
 */
function SceneMap() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var folderMap = {
        "Introduction": "",
        "Forest": "",
        "Home": "",
        "SekiWorkshop": "HamletDay2",
        "BreedingFarm": "HamletDay2",
        "FarmerHouse": "HamletDay2",
        "HamletWell": "HamletDay2",
        "UndergroundCave": "HamletDay2",
        "NightTwo": "Varanar",
        "MorningThree": "Varanar/DayThree",
        "NoonThreeStore": "Varanar/DayThree",
        "AfternoonThree": "Varanar/DayThree",
        "VaranarSoupKitchen": "Varanar/DayThree",
        "CaravanConfinement": "Enslavement",
        "SlaveNoonTwo": "Enslavement/DayTwo",
        "RiversideMarsh": "Deepwoods"
    };

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Returns the foldermap. You probably shouldn't alter anything in it, though.
     * @returns {object} The foldermap.
     */
    this.get = function () {
        return folderMap;
    };

    /**
     * Returns all of the scenes in the foldermap, without their folders.
     * @returns {array} An array containing all the scene names.
     */
    this.getAll = function () {
        return Object.keys(folderMap);
    };

    /**
     * Returns true if the passed scene exists, false otherwise.
     * @param {string} scene The scene to check.
     * @returns {boolean} True if the passed scene exists, false otherwise.
     */
    this.exists = function (scene) {
        return scene in folderMap;
    };

    /**
     * Constructs the path to the base folder of the scene with the given id.
     * @param {string} id The id of the scene for which to construct the folder path.
     * @returns {string} The base folder path for the scene with the given id.
     */
    this.constructFolderPath = function (id) {
        var folder = folderMap[id] || "";
        if (folder) {
            return `scenes/${folder}/${id}/`;
        } else {
            return `scenes/${id}/`;
        }
    };

    /**
     * Constructs the path to the JSON file of the scene with the given id.
     * @param {string} id The id of the scene for which to construct the JSON file path.
     * @returns {string} The JSON file path for the scene with the given id.
     */
    this.constructJsonPath = function (id) {
        return that.constructFolderPath(id) + `${id}.json`;
    };

    /**
     * For each entry in the folder map, constructs a path to the JSON file and adds it to an array.
     * @returns {array} The JSON path array.
     */
    this.constructJsonPathArray = function () {
        var jsonPathArray = {};
        for (var id in folderMap) {
            jsonPathArray[id] = that.constructJsonPath(id);
        }
        return jsonPathArray;
    };

}

// END res\js\SceneMap.js

// res\js\settings\SettingMasterListener.js

/**
 * Listens to changes in the localstorage settings.
 * Converts the localstorage from an object to JSON, and then passes it to all registered sublisteners.
 * @param {string} settingsKey The storage-key for settings.
 */
function SettingMasterListener(settingsKey) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Array of listener functions.
    var listeners = [];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    /**
     * Registers a listener.
     * It will be passed both the old value and the new value of the settings-object (in that order).
     * @param {function} fun The function to unregister.
     */
    this.register = function (fun) {
        listeners.push(fun);
    };

    /**
     * Unregisters a listener.
     * @param {function} fun The function to unregister.
     */
    this.unregister = function (fun) {
        var index = listeners.indexOf(fun);
        if (index < 0 || index >= listeners.length) {
            return;
        }
        listeners.splice(index, 1);
    };

    /**
     * Add the storage-event listener to the window.
     * Will listen only to changes in settings.
     */
    window.addEventListener('storage', function (event) {
        if (event.key !== settingsKey) {
            return;
        }

        var oldValue = JSON.parse(event.oldValue);
        var newValue = JSON.parse(event.newValue);

        for (var i = 0; i < listeners.length; i++) {
            listeners[i](oldValue, newValue);
        }
    });
}

// END res\js\settings\SettingMasterListener.js

// res\js\settings\Settings.js

/**
 * Stores, reads and alters Kobold Adventure settings.
 */
function Settings() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var settingsSlot = "settings";

    var flatChest = "flatchest";
    var hotLoad = "hotload";
    var oldLoad = "oldLoad";
    var hideNsfw = "hidensfw";
    var conditionsInvisible = "conditionsInvisible";
    var textSize = "textSize";
    var saveScroll = "saveScroll";
    var scrollBottom = "scrollBottom";
    var theming = "themesEnabled";
    var darkTheme = "darkThemeEnabled";
    var darkThemeModifier = "darkThemeModifier";
    var customThemeEnabled = "customThemeEnabled";
    var customTheme = "customTheme";
    var audio = "audioEnabled";
    var audioInMenu = "audioInMenu";
    var masterVolume = "masterVolume";
    var effectsVolume = "effectsVolume";
    var backgroundVolume = "backgroundVolume";

    // Master listener for setting changes.
    var masterListener = new SettingMasterListener(settingsSlot);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Gets the setting with the passed key.
     * @param {string} key The key of the setting.
     * @returns {?} The setting.
     */
    function getSetting(key) {
        var json = localStorage.getItem(settingsSlot) || "{}";
        var obj = JSON.parse(json);
        return obj[key];
    }

    /**
     * Sets the setting with the passed key to the passed value.
     * @param {string} key The key of the setting.
     * @param {?} value The value to set the setting to.
     */
    function setSetting(key, value) {
        var json = localStorage.getItem(settingsSlot) || "{}";
        var obj = JSON.parse(json);
        obj[key] = value;
        localStorage.setItem(settingsSlot, JSON.stringify(obj));
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Gets the flat chested female setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getFlatChestedFemale = function () {
        return getSetting(flatChest);
    };

    /**
     * Sets the flat chested female setting.
     * @param {boolean} value The value to set it to.
     */
    this.setFlatChestedFemale = function (value) {
        setSetting(flatChest, value);
    };

    /**
     * Gets the hotload setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getHotLoad = function () {
        return getSetting(hotLoad);
    };

    /**
     * Sets the hotload setting.
     * @param {boolean} value The value to set it to.
     */
    this.setHotLoad = function (value) {
        setSetting(hotLoad, value);
    };

    /**
     * Gets the old load setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getOldLoad = function () {
        return getSetting(oldLoad);
    };

    /**
     * Sets the old load setting.
     * @param {boolean} value The value to set it to.
     */
    this.setOldLoad = function (value) {
        setSetting(oldLoad, value);
    };

    /**
     * Gets the hide NSFW setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getHideNsfw = function () {
        return getSetting(hideNsfw);
    };

    /**
     * Sets the hide NSFW setting.
     * @param {boolean} value The value to set it to.
     */
    this.setHideNsfw = function (value) {
        setSetting(hideNsfw, value);
    };
    
    /**
     * Setting listener for hiding nsfw images.
     */
    this.hideNsfwListener = new SettingSubListener(masterListener, hideNsfw);

    /**
     * Gets the conditions invisible setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getConditionsInvisible = function () {
        return getSetting(conditionsInvisible);
    };

    /**
     * Sets the conditions invisible setting.
     * @param {boolean} value The value to set it to.
     */
    this.setConditionsInvisible = function (value) {
        setSetting(conditionsInvisible, value);
    };

    /**
     * Gets the text size setting.
     * @returns {undefined | string} Returns the text size or 16 if not yet set.
     */
    this.getTextSize = function () {
        return getSetting(textSize) || "16";
    };

    /**
     * Sets the text size setting.
     * @param {string} value The value to set it to.
     */
    this.setTextSize = function (value) {
        setSetting(textSize, value);
    };
    
    /**
     * Setting listener for the text size.
     */
    this.textSizeListener = new SettingSubListener(masterListener, textSize);

    /**
     * Gets the save scroll setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getSaveScroll = function () {
        return getSetting(saveScroll);
    };

    /**
     * Sets the save scroll setting.
     * @param {boolean} value The value to set it to.
     */
    this.setSaveScroll = function (value) {
        setSetting(saveScroll, value);
    };

    /**
     * Gets the scroll bottom setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getScrollBottom = function () {
        return getSetting(scrollBottom);
    };

    /**
     * Sets the scroll bottom setting.
     * @param {boolean} value The value to set it to.
     */
    this.setScrollBottom = function (value) {
        setSetting(scrollBottom, value);
    };

    /**
     * Gets the theme setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getThemesDisabled = function () {
        return getSetting(theming);
    };

    /**
     * Sets the theme setting.
     * @param {boolean} value The value to set it to.
     */
    this.setThemesDisabled = function (value) {
        setSetting(theming, value);
    };

    /**
     * Gets the dark theme setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getDarkThemeEnabled = function () {
        return getSetting(darkTheme);
    };

    /**
     * Sets the dark theme setting.
     * @param {boolean} value The value to set it to.
     */
    this.setDarkThemeEnabled = function (value) {
        setSetting(darkTheme, value);
    };

    /**
     * Gets the dark theme modifier setting.
     * The modifier is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The dark theme modifier. May be any number from 0 up to and including 100. Default 50.
     */
    this.getDarkThemeModifier = function () {
        var setting = getSetting(darkThemeModifier);
        return volumeTransform(setting);
    };

    /**
     * Sets the dark theme modifier setting.
     * The modifier will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setDarkThemeModifier = function (value) {
        setSetting(darkThemeModifier, +value + 1);
    };

    /**
     * Gets the custom theme enabled setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getCustomThemeEnabled = function () {
        return getSetting(customThemeEnabled);
    };

    /**
     * Sets the custom theme enabled setting.
     * @param {boolean} value The value to set it to.
     */
    this.setCustomThemeEnabled = function (value) {
        setSetting(customThemeEnabled, value);
    };

    /**
     * Transforms the setting from its raw JSON-fetched value to a usable value.
     * @param {string} setting The setting before conversion.
     * @returns {object} The setting after conversion.
     */
    function customThemeTransform(setting) {
        if (setting) {
            return JSON.parse(setting);
        } else {
            return {};
        }
    }

    /**
     * Gets the custom theme setting as an object converted from JSON.
     * @returns {object} May return an empty object if this setting has not yet been set, or a Theme object stripped of object functionality.
     */
    this.getCustomTheme = function () {
        var setting = getSetting(customTheme);
        return customThemeTransform(setting);
    };

    /**
     * Sets the custom theme setting. Will automatically be converted to JSON.
     * @param {object} value The object to set it to.
     */
    this.setCustomTheme = function (value) {
        setSetting(customTheme, JSON.stringify(value));
    };

    /**
     * Setting listener for the custom theme.
     */
    this.customThemeListener = new SettingSubListener(masterListener, customTheme, customThemeTransform);

    /**
     * Gets the audio setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getAudioEnabled = function () {
        return getSetting(audio);
    };

    /**
     * Sets the audio setting.
     * @param {boolean} value The value to set it to.
     */
    this.setAudioEnabled = function (value) {
        setSetting(audio, value);
    };

    /**
     * Gets the audio in menu setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getAudioInMenu = function () {
        return getSetting(audioInMenu);
    };

    /**
     * Sets the audio in menu setting.
     * @param {boolean} value The value to set it to.
     */
    this.setAudioInMenu = function (value) {
        setSetting(audioInMenu, value);
    };

    /**
     * Transforms the setting from its raw JSON-converted value to a usable value.
     * @param {int} setting The setting before conversion.
     * @returns {int} The setting after conversion.
     */
    function volumeTransform(setting) {
        return setting ? setting - 1 : 50;
    }

    /**
     * Gets the master volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getMasterVolume = function () {
        var setting = getSetting(masterVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the master volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setMasterVolume = function (value) {
        setSetting(masterVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.masterVolumeListener = new SettingSubListener(masterListener, masterVolume, volumeTransform);

    /**
     * Gets the effects volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getEffectsVolume = function () {
        var setting = getSetting(effectsVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the effects volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setEffectsVolume = function (value) {
        setSetting(effectsVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.effectsVolumeListener = new SettingSubListener(masterListener, effectsVolume, volumeTransform);

    /**
     * Gets the background volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getBackgroundVolume = function () {
        var setting = getSetting(backgroundVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the background volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setBackgroundVolume = function (value) {
        setSetting(backgroundVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.backgroundVolumeListener = new SettingSubListener(masterListener, backgroundVolume, volumeTransform);
}

// END res\js\settings\Settings.js

// res\js\settings\SettingSubListener.js

/**
 * Listens to changes in a single setting, and passes the new value on to all registered listeners.
 * @param {SettingChangeMasterListener} masterListener The MasterListener providing this SubListener with changed settings objects.
 * @param {string} setting The key for the setting itself.
 * @param {function} transform Transforms the changed setting before passing it to the listeners.
 *                             Gets the value as a parameter. Must return the transformed value. May be omitted.
 */
function SettingSubListener(masterListener, setting, transform) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Array of listener functions.
    var listeners = [];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Gets registered to the masterListener.
     * Listen to changes in this setting, and notifies the listeners if there are any changes.
     * @param {object} oldValue The old settings object.
     * @param {object} newValue The new settings object.
     */
    function listen(oldValue, newValue) {
        oldValue = oldValue || {};
        var oldSetting = oldValue[setting];
        var newSetting = newValue[setting];
        if (oldSetting === newSetting) {
            return;
        }
        if (transform) {
            newSetting = transform(newSetting);
        }
        for (var i = 0; i < listeners.length; i++) {
            listeners[i](newSetting);
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    /**
     * Registers a listener.
     * The listener will be passed the changed value, potentially altered by the transformation function (if there is one).
     * @param {function} fun The function to unregister.
     */
    this.register = function (fun) {
        listeners.push(fun);
    };

    /**
     * Unregisters a listener.
     * @param {function} fun The function to unregister.
     */
    this.unregister = function (fun) {
        var index = listeners.indexOf(fun);
        if (index < 0 || index >= listeners.length) {
            return;
        }
        listeners.splice(index, 1);
    };

    // Register this listener.
    masterListener.register(listen);
}

// END res\js\settings\SettingSubListener.js

// res\js\ui\change\Change.js

/**
 * This class represents a single change for use with the ChangeNotificationBuilder.
 * @param {string} why Why did the change occur?
 * @param {string} what What changed? Stamina, agility, health?
 * @param {int} amount By how much did it change?
 * @param {boolean} positiveBad Inverts positive and negative value display if set to true.
 */
function Change(why, what, amount, positiveBad) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.reason = why;
    this.stat = what;
    this.value = amount;
    this.invert = positiveBad;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Transforms this change into valid HTML.
     * @returns {string} HTML describing this change.
     */
    this.toHtml = function () {
        var abs = Math.abs(that.value);
        var span;
        if (!that.invert) {
            span = that.value < 0 ? `<span class="warningtext bold">` : `<span>`;
        } else {
            span = that.value >= 0 ? `<span class="warningtext bold">` : `<span>`;

        }
        var number = that.value < 0 ? ` -${abs}` : ` +${abs}`;

        return `<div class="nomargin center">${that.reason}: ${span}${that.stat}${number}</span></div>`;
    };
}

// END res\js\ui\change\Change.js

// res\js\ui\change\ChangeNotificationBuilder.js

/**
 * This class can build change notifications for display in scenes.
 */
function ChangeNotificationBuilder() {
    // PRIVATE MEMBERS \\       var private = "test";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds the body for the tooltip of the changes.
     * @param {array} changes The Changes-array.
     * @returns {string} HTML describing the tooltip body.
     */
    function buildBody(changes) {
        var body = "";
        for (var i in changes) {
            body += changes[i].toHtml();
        }
        return body;
    }

    // PUBLIC MEMBERS \\        this.public = "test";


    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds a notification of a bunch of changes to display in a scene.
     * @param {array} changes An array of Change objects describing what changes occured.
     * @returns {string} HTML describing the changes.
     */
    this.buildNotification = function (changes) {
        var change = buildBody(changes);
        return `<div class="text center">` + global.ui.makeStaticToolTip(change, false, false, "width: auto; display: inline-block; background-color: transparent;") + `</div>`;
    };
}

// END res\js\ui\change\ChangeNotificationBuilder.js

// res\js\ui\icons\Icons.js

/**
 * This class can create icons.
 */
function Icons() {
    // PRIVATE MEMBERS \\       var private = "test";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    
    /**
     * Creates and returns a sword icon.
     * @returns {string} Sword icon.
     */
    this.sword = function() {
        return `<img src="res/img/icons/sword.png" alt="sword" class="icon">`;
    };
    
    /**
     * Creates and returns a sword icon.
     * @returns {string} Sword icon.
     */
    this.shield = function() {
        return `<img src="res/img/icons/shield.png" alt="shield" class="icon">`;
    };
    
    /**
     * Creates and returns a sword icon.
     * @returns {string} Sword icon.
     */
    this.coin = function() {
        return `<img src="res/img/icons/coin.png" alt="coin" class="icon">`;
    };
}

// END res\js\ui\icons\Icons.js

// res\js\ui\img\ImageBackground.js

/**
 * Information about the class.
 */
function ImageBackground() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "imagebackground";
    var cls = "imagebackground";
    var src = "";
    var alt = "image background";
    var dur = 1000;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    /**
     * Hides and deletes any currently present background images, then calls the passed function.
     * Calls the function immediately if there is no currently present background image.
     * @param {function} fun Function to call after hiding. May be called immediately.
     */
    function hideAndThen(fun) {
        if (src) {
            var elem = $("#" + id);
            elem.removeAttr("id");
            elem.stop(true);
            src = "";
            elem.fadeOut(dur, function () {
                elem.remove();
                fun();
            });
        } else {
            fun();
        }
    }

    /**
     * Loads the image with the given url, and then shows it once it's fully loaded.
     * @param {string} url URL of the image relative to the site root.
     */
    function loadAndShowImage(url) {
        var img = new Image();
        img.id = id;
        img.className = cls;
        img.alt = alt;
        img.style = "display: none;";
        img.onload = function () {
            $(this).fadeIn(dur);
        };
        img.src = url;
        src = url;
        $("body").prepend(img);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Shows the given url as background image, fading it in properly.
     * Hides any currently present background images first.
     * @param {string} url URL of the image relative to the site root.
     */
    this.show = function (url) {
        if (src === url) {
            return;
        }
        hideAndThen(function () {
            loadAndShowImage(url);
        });
    };

    /**
     * Hides and deletes any present background images.
     */
    this.hide = function () {
        hideAndThen(function () {});
    };
}

//

// END res\js\ui\img\ImageBackground.js

// res\js\ui\img\ImageInText.js

/**
 * Can create the HTML necessary to add images mid-text.
 */
function ImageInText() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var sfwClass = "sfw";
    var nsfwClass = "nsfw";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Creates the HTML required for an inline image with link to the full image.
     * @param {type} url The URL of the image. Should contain "m" or "f" or some other gender-specific variable if applicable. Should not contain the "-full" suffix.
     * @param {string} ext The extension of the image. Should not start with a period. Ex.: "png"
     * @param {ImageWidthEnum} width A member of the ImageWidthEnum. CUSTOM means no width will be added, and the customCSS must contain a width.
     * @param {boolean} sfw True if safe for work, false otherwise.
     * @param {string} artist The name of the artist.
     * @param {string} customCSS Custom CSS added to the image. May be omitted.
     * @returns {string} The HTML describing the inline image.
     */
    this.create = function (url, ext, width, sfw, artist, customCSS) {
        var link = `<a href="${url}-full.${ext}" target="_blank">`;
        var image = `<img class="${sfw ? sfwClass : nsfwClass} ${ImageWidthEnum.properties[width]}"`;
        image += `src="${url}.${ext}" title="Made by ${artist}" ${customCSS ? `style="${customCSS}"` : ``}>`;
        return link + image + `</a>`;
    };
}

var ImageWidthEnum = {
    SMALLER: 1,
    SMALL: 2,
    NORMAL: 3,
    MEDIUM: 4,
    WIDE: 5,
    CUSTOM: 99
};

ImageWidthEnum.properties = {
    1: "smallerimage",
    2: "smallimage",
    3: "standardimage",
    4: "mediumimage",
    5: "wideimage",
    99: ""
};

// END res\js\ui\img\ImageInText.js

// res\js\ui\loadbar\LoadBar.js

/**
 * Controls a loading bar which may be hidden and shown at will.
 */
function LoadBar() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var curr = 0;
    var max = 100;
    var id = "loadbar";
    var innerId = "loadbarinner";
    var absoluteId = "loadbarabsolute";
    var relativeId = "loadbarrelative";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Updates the relative part of the load bar.
     */
    function updateRelative() {
        var percent = Math.round((curr / max) * 100);
        $("#" + relativeId).html(percent + "%");
    }

    /**
     * Updates the absolute part of the load bar.
     */
    function updateAbsolute() {
        $("#" + absoluteId).html(curr + "/" + max);
    }

    /**
     * Updates the bar of the load bar.
     */
    function updateBar() {
        var percent = Math.round((curr / max) * 100);
        $("#" + innerId).attr('style', `width: ${percent}%`);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Shows the loadbar.
     * @param {int} total How many things need to be loaded.
     */
    this.show = function (total) {
        max = total;
        that.update(total);
        $("#" + id).fadeIn(400);
    };

    /**
     * Initializes the load bar with the total amount of things to load.
     * @param {int} remaining How many things still need to be loaded.
     */
    this.update = function (remaining) {
        curr = max - remaining;
        updateRelative();
        updateAbsolute();
        updateBar();
    };

    /**
     * Hides the loadbar.
     */
    this.hide = function () {
        that.update(0);
        $("#" + id).stop();
        $("#" + id).fadeOut(400);
    };
}

// END res\js\ui\loadbar\LoadBar.js

// res\js\ui\menu\buttons\AllMenuButtons.js

/**
 * This class contains all the necessary functions to deal with the tabs section of the UI.
 * Its init() function should be called immediately upon successful project load.
 */
function AllMenuButtons() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var gen = new GenericMenuButton();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    this.map = new MapMenuButton(gen);
    this.preview = new PreviewMenuButton(gen);
    this.settings = new SettingsMenuButton(gen);
    this.story = new StoryMenuButton(gen);
    this.fanart = new FanArtMenuButton(gen);
    this.contribute = new ContributeMenuButton(gen);

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes all tabs.
     */
    this.init = function () {
        that.map.init();
        that.preview.init();
        that.settings.init();
        that.story.init();
        that.fanart.init();
        that.contribute.init();
    };
    
    /**
     * Unsets all buttons.
     */
    this.unset = function () {
        that.map.unset();
        that.preview.unset();
        that.settings.unset();
        that.story.unset();
        that.fanart.unset();
        that.contribute.unset();
    };
}

// END res\js\ui\menu\buttons\AllMenuButtons.js

// res\js\ui\menu\buttons\ContributeMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function ContributeMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "contributemenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "Contribute";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` Feel like writing your own addition to Kobold Adventure?
                        Want to draw art, or make audio to be featured in the game?
                        Are you talented and confident in your abilities?
                        Or are you just interested in reading the strict guidelines contributions must adhere to?
                        If so, you're one click on the Contribute button away from being taken to the contribution guidelines page, which will open in a new tab!`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "contribute/index.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\ContributeMenuButton.js

// res\js\ui\menu\buttons\FanArtMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function FanArtMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "fanartmenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "Fan art";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` A collection of all Kobold Adventure fan art made so far.
                        You can find all the artists listed on the credits page.
                        If you want to contribute your own fan art, feel free to message me on any of the platforms listed on the title screen of the game.
                        Click the Fan Art button again to open the separate page in a new tab.`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "fanart/index.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\FanArtMenuButton.js

// res\js\ui\menu\buttons\GenericMenuButton.js

/**
 * A generic menu button, containing functionality used by all of them.
 */
function GenericMenuButton() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var selectedClass = "menubuttonselected";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /*
     * Opens a url in a new tab.
     * @param {string} url The url to open. May be relative if it starts with a slash.
     */
    this.open = function (url) {
        var win = window.open(url, '_blank');
        win.focus();
    };

    /**
     * Unsets the button with the given id.
     * @param {string} id The id of the button.
     */
    this.unset = function (id) {
        $("#" + id).removeClass(selectedClass);
        global.ui.menu.resetSubtitle();
        global.ui.menu.resetContent();
    };

    /**
     * Sets the button with the given id.
     * @param {string} id The id of the button.
     * @param {string} subtitle Subtitle to set.
     * @param {string} content Content to set.
     */
    this.set = function (id, subtitle, content) {
        global.ui.menu.buttons.unset();
        $("#" + id).addClass(selectedClass);
        global.ui.menu.setSubtitle(subtitle);
        global.ui.menu.setContent(content);
    };
}

// END res\js\ui\menu\buttons\GenericMenuButton.js

// res\js\ui\menu\buttons\MapMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function MapMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "amapmenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "aMap";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` aMap is a handy feature that allows you to view scene trees, which are overviews of all the choices you can make in the game.
                        aMap contains a lot of spoilers.
                        Usage of aMap is considered cheating.
                        Use it only if you want to make sure you've found every single path available.
                        Note that certain secret paths may be omitted from aMap.
                        Click the aMap button again to open aMap in a new tab.`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "map/index.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\MapMenuButton.js

// res\js\ui\menu\buttons\PreviewMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function PreviewMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "apreviewmenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "aPreview";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` aPreview is a development tool for Kobold Adventure.
                        It allows for developers and contributors to write and test koBBold, the BBcode-alternative employed by the game.
                        aPreview offers live translation between koBBold and end results.
                        Click the aPreview button again to open aPreview in a new tab.`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "./preview/index.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\PreviewMenuButton.js

// res\js\ui\menu\buttons\SettingsMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function SettingsMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "settingsmenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "Settings";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` The settings page contains a bunch of options which may or may not alter your Kobold Adventure experience.
                        It is highly recommended to check out the settings page at least once.
                        If you want to make sure you're up to date with all the latest fancy knobs and buttons,
                        it might pay off to check the settings once every update.
                        Click the settings button again to open the settings in a new tab.`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "./settings/index.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\SettingsMenuButton.js

// res\js\ui\menu\buttons\StoryMenuButton.js

/**
 * A menu button.
 * @param {GenericMenuButton} gen Generic map button for base functionality. 
 */
function StoryMenuButton(gen) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "astorymenubutton";
    var selected = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Returns the subtitle of this button.
     * @returns {string} The subtitle of this button.
     */
    function getSubtitle() {
        return "aStory";
    }

    /**
     * Returns the instructional content of this button.
     * @returns {string} The instructional content of this button.
     */
    function getContent() {
        var content = ` aStory is a way to access the entire history of your save, the entire story of your character up until now, as a single continuous page.
                        aStory may take a while to load.
                        For best performance: download the game and check out aStory locally.
                        Click the aStory button again to open aStory in a new tab.`;
        return global.parser.parse(content);
    }

    /**
     * Returns the URL to route to when this button is clicked twice.
     * @returns {string} The URL to route to when this button is clicked twice.
     */
    function getUrl() {
        return "./aStory.html";
    }

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Opens a new tab with the button's specified URL.
     * @param {event} event The click event.
     */
    function clickedTwice(event) {
        that.unset();
        gen.open(getUrl());
    }

    /**
     * Onclick function.
     * @param {event} event The click event.
     */
    function clicked(event) {
        if (selected) {
            clickedTwice(event);
        } else {
            gen.set(id, getSubtitle(), getContent());
            selected = true;
        }
        stopProp(event);
    }

    /*
     * Registers listeners for this button.
     */
    function registerListeners() {
        $("#" + id).click(clicked);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the button.
     */
    this.init = function () {
        registerListeners();
    };

    /**
     * Unsets the button.
     */
    this.unset = function () {
        if (!selected) {
            return;
        }
        selected = false;
        gen.unset(id);
    };
}

// END res\js\ui\menu\buttons\StoryMenuButton.js

// res\js\ui\menu\PopupMenu.js

/**
 * This class represents the popup menu.
 * Its init() function should be called immediately upon successful project load.
 */
function PopupMenu() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var popupId = "menupopup";
    var menuId = "menu";
    var openButtonId = "menuopenbutton";
    var subtitleId = "menusubtitle";
    var contentId = "menucontent";
    var closeButtonId = "menuclosebutton";
    var invisibleClass = "popupmenuinvisible";

    var subtitleDefault = false;
    var contentDefault = false;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Stops event propagation.
     * @param {event} event The event to stop propagation of.
     */
    function stopProp(event) {
        event.stopPropagation();
    }

    /**
     * Function called when no button is clicked.
     * @param {event} event The click event.
     */
    function noButton(event) {
        if (!subtitleDefault || !contentDefault) {
            that.buttons.unset();
        }
        stopProp(event);
    }

    /**
     * Registers the button listener for the menu button.
     */
    function registerListeners() {
        $("#" + popupId).click(that.hide);
        $("#" + closeButtonId).click(that.hide);
        $("#" + menuId).click(noButton);
        $("#" + openButtonId).click(that.show);
    }

    /**
     * Resets the menu's text and makes it invisible.
     */
    function reset() {
        $("#" + menuId).addClass(invisibleClass);
        that.resetContent();
        that.resetSubtitle();
    }

    /**
     * Makes the menu visible after it being reset.
     */
    function expand() {
        $("#" + menuId).removeClass(invisibleClass);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // Buttons contained in this menu.
    this.buttons = new AllMenuButtons();
    
    this.audio;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the UI controller.
     */
    this.init = function () {
        that.audio = global.audio.load("res/audio/ui/menu.mp3", 80);
        that.buttons.init();
        registerListeners();
        reset();
    };

    /**
     * Shows the menu.
     */
    this.show = function () {
        global.audio.play(that.audio);
        that.buttons.unset();
        $("#" + popupId).fadeIn(400);
        expand();
    };

    /**
     * Fades out the menu.
     */
    this.hide = function () {
        $("#" + popupId).fadeOut(400, reset);
    };

    /**
     * Sets the subtitle of the popup menu.
     * @param {string} subtitle The new subtitle.
     */
    this.setSubtitle = function (subtitle) {
        subtitleDefault = false;
        $("#" + subtitleId).html(subtitle);
    };

    /**
     * Sets the content of the popup menu.
     * @param {string} content The new content.
     */
    this.setContent = function (content) {
        contentDefault = false;
        $("#" + contentId).html(content);
    };

    /**
     * Resets the subtitle of the popup menu.
     */
    this.resetSubtitle = function () {
        if (subtitleDefault) {
            return;
        }
        subtitleDefault = true;
        that.setSubtitle("Instructions");
    };

    /*
     * Resets the content of the popup menu.
     */
    this.resetContent = function () {
        if (contentDefault) {
            return;
        }
        contentDefault = true;
        that.setContent(`Welcome to the Kobold Adventure menu!
                    In here, you can find a bunch of links to interesting features you may want to check out.
                    Click on one of the buttons below to get more information on what it entails.
                    Clicking on a button after it's been selected will open the specified page in a new tab.
                    Click anywhere outside of this window, or on the big X at the top right corner, to close this window.`);
    };
}

// END res\js\ui\menu\PopupMenu.js

// res\js\ui\merchant\MerchantInventoryGenerator.js

/**
 * Class capable of generating a merchant's inventory screen and registering listeners for it.
 * @param {string} uniqueId Unique id of this shop, to allow for multiple shops in a single scene.
 * @param {array} items The items the merchant has for sale as an array of ItemAmount objects.
 * @param {float} sellmod The value-modifier at which the kobold sells items.
 * @param {float} buymod The value-modifier at which the kobold buys items.
 * @param {int} coins How many coins the kobold has at scene-load time.
 * @param {boolean} disabled Whether or not this shop is still active.
 * @param {function} sellListener A listener for when the kobold sells something from its inventory. Gets passed the item which was sold. Has no return value.
 * @param {function} buyListener A listener for when the kobold buys something from the merchant. 
 *                               Gets passed the index in the item-array of the item which was bought.
 */
function MerchantInventoryGenerator(uniqueId, items, coins, sellmod, buymod, disabled, sellListener, buyListener) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var coinSpanId = uniqueId + "Coins";
    var buyButtonClass = "itemrowuse";
    
    var coinSound = global.audio.load("res/audio/ui/coin.mp3", 80);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    function playSoundAndCallSellListener(item) {
        global.audio.play(coinSound);
        sellListener(item);
    }

    /**
     * Sets all required global variables.
     */
    function setGlobals() {
        global.kobold.inventory.enterSafeZone();
        global.kobold.inventory.enterSellZone();
        global.kobold.inventory.sellMod = sellmod;
        global.kobold.inventory.sellListener = playSoundAndCallSellListener;
    }

    /**
     * Resets all required global variables.
     */
    function resetGlobals() {
        global.kobold.inventory.exitSafeZone();
        global.kobold.inventory.exitSellZone();
        global.kobold.inventory.sellMod = 1.0;
        global.kobold.inventory.sellListener = null;
    }

    /**
     * Updates amount and fades bought item away if required, 
     * then updates all of the buttons and the amount of coins.
     * @param {int} id Index of the bought item in the items array.
     */
    function updateEverything(id) {
        var itemAmount = items[id];
        $("#" + uniqueId + id).find(".itemrownumber").html(itemAmount.amount);
        if (itemAmount.amount <= 0) {
            $("#" + uniqueId + id).hide(400);
        }
        $("#" + coinSpanId).html(coins);
        for (var i = 0; i < items.length; i++) {
            var item = items[i].item;
            if (Math.ceil(item.price() * buymod) > coins) {
                $("#" + uniqueId + i).find("button").prop("disabled", true);
            }
        }
    }

    function bought(id) {
        items[id].amount--;
        var item = items[id].item;
        coins -= Math.ceil(item.price() * buymod);
        global.kobold.inventory.coins -= Math.ceil(item.price() * buymod);
        global.kobold.inventory.addItem(item);
        updateEverything(id);
    }

    /**
     * Builds and returns the number section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The number section of an item row.
     */
    function buildItemRowNumber(itemAmount) {
        return `<div class="itemrownumber">${itemAmount.amount}</div>`;
    }

    /**
     * Builds and returns the X section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The X section of an item row.
     */
    function buildItemRowX(itemAmount) {
        return `<div class="itemrowx">x</div>`;
    }

    /**
     * Builds and returns the name section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The name section of an item row.
     */
    function buildItemRowName(itemAmount) {
        return `<div class="itemrowname">${global.kobold.inventory.createItemWithTooltip(itemAmount.item, true)}</div>`;
    }

    /**
     * Builds and returns the value section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The value section of an item row.
     */
    function buildItemRowValue(itemAmount) {
        return `<div class="itemrowvalue">${Math.ceil(itemAmount.item.price() * buymod)} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Builds and returns the sell section of an item row.
     * This is just an empty container to preserve proper spacing.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The sell section of an item row.
     */
    function buildItemRowSell(itemAmount) {
        var toReturn = `<div class="itemrowsellcontainer">`;
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the buy section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The use section of an item row.
     */
    function buildItemRowBuy(itemAmount) {
        var canBuy = coins >= Math.ceil(itemAmount.item.price() * buymod);
        var toReturn = `<div class="itemrowusecontainer">`;
        toReturn += `<button class="button ${buyButtonClass}" ${disabled || !canBuy ? "disabled" : ""}>Buy</button>`;
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the innards of a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The innards of a row for a single item.
     */
    function buildItemRowInnards(itemAmount) {
        var innards = buildItemRowNumber(itemAmount);
        innards += buildItemRowX(itemAmount);
        innards += buildItemRowName(itemAmount);
        innards += buildItemRowValue(itemAmount);
        innards += buildItemRowSell(itemAmount);
        innards += buildItemRowBuy(itemAmount);
        return innards;
    }

    /**
     * Builds and returns a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @argument {int} i The 0-based row number of the item.
     * @returns {string} A row for a single item.
     */
    function buildItemRow(itemAmount, i) {
        var id = uniqueId + i;
        var rowClass = "itemroweven";
        if (i % 2) {
            rowClass = "itemrowodd";
        }
        var row = `<div id="${id}" class="itemrow ${rowClass}">`;
        row += buildItemRowInnards(itemAmount);
        return row + `</div>`;
    }

    /**
     * Disables all of the buy buttons of this store.
     */
    function disableButtons() {
        $("#" + uniqueId).find("button").prop("disabled", true);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the merchant's inventory.
     * @returns {string} The bag section of the inventory, as HTML string.
     */
    this.build = function () {
        if (!disabled) {
            setGlobals();
        }
        var bag = `<div id="${uniqueId}" class="shop">`;
        bag += `<div class="text">You have <span id="${coinSpanId}">${coins}</span> coin${coins !== 1 ? "s" : ""}.</div>`;
        // Check if all zero, then also no items
        if (items.length < 1) {
            bag += `<div class="text">The merchant has no items.</div>`;
        } else {
            for (var i = 0; i < items.length; i++) {
                if (items[i].amount <= 0) {
                    continue;
                }
                bag += buildItemRow(items[i], i);
            }
        }
        bag += `</div>`;
        return bag;
    };

    /**
     * Registers listeners for the buy buttons of this store.
     */
    this.registerListeners = function () {
        if (disabled) {
            return;
        }
        $("#" + uniqueId).find("." + buyButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var messyId = row.attr("id");
            var id = messyId.slice(uniqueId.length);
            var itemAmount = items[id];            
            global.audio.play(coinSound);
            bought(id, itemAmount);
            buyListener(id);
        });
    };

    /**
     * Disables this store.
     */
    this.disable = function () {
        disabled = true;
        resetGlobals();
        disableButtons();
    };
}

// END res\js\ui\merchant\MerchantInventoryGenerator.js

// res\js\ui\tabs\AllTabs.js

/**
 * This class contains all the necessary functions to deal with the tabs section of the UI.
 * Its init() function should be called immediately upon successful project load.
 */
function AllTabs() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var gen = new GenericTab();
    
    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    this.scene = new SceneTab(gen);
    this.stats = new StatsTab(gen);
    this.status = new StatusTab(gen);
    this.inventory = new InventoryTab(gen);
    this.save = new SaveTab(gen);
    this.load = new LoadTab(gen);

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes all tabs.
     */
    this.init = function () {
        gen.init();
        that.scene.init();
        that.stats.init();
        that.status.init();
        that.inventory.init();
        that.save.init();
        that.load.init();
    };
}

// END res\js\ui\tabs\AllTabs.js

// res\js\ui\tabs\GenericTab.js

/**
 * This class contains generic functions shared across all tabs.
 */
function GenericTab() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var activeClass = "tabactive";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Removes the active tab class from all elements that currently possess it.
     */
    function removeActiveClass() {
        $("." + activeClass).removeClass(activeClass);
    }

    /**
     * Adds the active tab class to the element with the passed id.
     * @param {string} id The id of the new active tab button.
     */
    function addActiveClass(id) {
        $("#" + id).addClass(activeClass);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.saveSlotClass = "saveslot";
    this.smallSaveSlotClass = "smallsaveslot";
    this.saveSlotSelectedClass = "saveslotselected";

    this.audio;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Transitions the UI from one tab button to another.
     * @param {string} id The id of the new active tab button.
     */
    this.tabButtonTransition = function (id) {
        //global.ui.cancelThemeChangeTransition(); //Instantly completes any theme-changes in progress.
        removeActiveClass();
        addActiveClass(id);
    };

    /**
     * Returns true if the tab with the passed id is active. False otherwise.
     * @param {string} id The id of the tab to check.
     * @returns {boolean} True if the tab with the passed id is active. False otherwise.
     */
    this.isActive = function (id) {
        return $("#" + id).hasClass(activeClass);
    };

    /*
     * Builds and returns the top right maximum save indicator.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The maximum save indicator.
     */
    this.buildMaxSaveIndicator = function (saveMeta) {
        var amount = saveMeta.length;
        var topRight = `<div class="savelimit">${amount} / ${global.saveManager.maxAmountOfSaves}</div>`;
        return topRight;
    };

    /**
     * Updates the maximum save indicator to new values.
     */
    this.updateMaxSaveIndicator = function () {
        var amount = global.saveManager.getAmountOfSaves();
        $(".savelimit").html(`${amount} / ${global.saveManager.maxAmountOfSaves}`);
    };

    /**
     * Converts a timestamp to a string-representation of how long ago it was.
     * @param {Date} time The timestamp.
     * @returns {string} A string-representation of how long ago the timestamp was.
     */
    this.timeToTimePassed = function (time) {
        var seconds = Math.floor((new Date() - time) / 1000);
        var interval = Math.floor(seconds / 31536000);
        if (interval > 1) {
            return interval + " years ago";
        }
        interval = Math.floor(seconds / 2592000);
        if (interval > 1) {
            return interval + " months ago";
        }
        interval = Math.floor(seconds / 86400);
        if (interval > 1) {
            return interval + " days ago";
        }
        interval = Math.floor(seconds / 3600);
        if (interval > 1) {
            return interval + " hours ago";
        }
        interval = Math.floor(seconds / 60);
        if (interval > 1) {
            return interval + " minutes ago";
        }
        interval = seconds;
        if (interval > 10) {
            return interval + " seconds ago";
        }
        return "Just now";
    };

    /**
     * Removes the saveslotselected class from all elements.
     */
    this.removeSaveSlotSelected = function () {
        $("." + that.saveSlotSelectedClass).removeClass(that.saveSlotSelectedClass);
    };

    /**
     * Enables the element with the passed id.
     * @param {string} elementId The id of the element.
     */
    this.enable = function (elementId) {
        $("#" + elementId).prop("disabled", false);
    };

    /**
     * Disables the element with the passed id.
     * @param {string} elementId The id of the element.
     */
    this.disable = function (elementId) {
        $("#" + elementId).prop("disabled", true);
    };

    /**
     * Builds a generic header to be used for every tab apart from scene.
     * @param {string} title The title of the tab.
     * @returns {string} A generic tab header.
     */
    this.buildTabHeader = function (title) {
        var start = global.ui.buildGenericHeader();
        return start + "<div class='title'>" + title + "</div>";
    };

    /**
     * Builds a generic footer to be used for every tab apart from scene.
     * @returns {string} A generic tab footer.
     */
    this.buildTabFooter = function () {
        return global.ui.buildGenericFooter();
    };

    /**
     * Resets the scroll position.
     */
    this.resetScroll = function () {
        $("#main").scrollTop(0);
    };

    /**
     * Generic initialization function that should be called once.
     */
    this.init = function () {
        that.audio = global.audio.load("res/audio/ui/click.mp3", 80);
    };

    /**
     * Generic load function for all tabs.
     */
    this.load = function () {
        global.ui.imageBackground.hide();
        if (!global.settings.getAudioInMenu()) {
            global.audio.fadeAllAudio();
        }
        global.audio.play(that.audio);
        that.resetScroll();
    };
}

// END res\js\ui\tabs\GenericTab.js

// res\js\ui\tabs\InventoryTab.js

/**
 * This class contains functions specific to the Inventory tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function InventoryTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "inventorytab";
    var title = "Inventory";
    var sellButtonClass = "itemrowsell";
    var useButtonClass = "itemrowuse";
    var coinSpanId = "coins";
    var blinking = false;
    var blinkClass = "blinktab";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds and returns a HTML-subtitle.
     * @param {string} subtitle The actual subtitle.
     * @returns {string} The subtitle as a HTML-subtitle.
     */
    function buildSubtitle(subtitle) {
        return `<h2 class="subtitle">${subtitle}</h2>`;
    }

    /**
     * Builds a bold-faced item name which will reveal a tooltip when moused over.
     * @param {object} item The item for which to build a tooltip.
     * @param {boolean} caps Whether or not the item's name should be capitalized. May be omitted.
     * @returns {String} The constructed item name with tooltip.
     */
    function buildItemWithTooltip(item, caps) {
        return global.kobold.inventory.createItemWithTooltip(item, caps);
    }

    /**
     * Builds and returns the bra section of the equipment part of the inventory.
     * @returns {string} The bra section of the equipment part of the inventory as a HTML string.
     */
    function buildBra() {
        var isWearingBra = global.kobold.inventory.equipment.isWearing(SlotEnum.BRA);
        var genderHasBreasts = global.kobold.body.hasBreasts();
        // First authority = bra, second authority = gender.
        var shouldBeWearingBra = isWearingBra ? !global.kobold.inventory.equipment.bra.wrongGender() : genderHasBreasts;
        var bra = `<div class="text nomargin">`;

        if (isWearingBra && shouldBeWearingBra) {
            bra += `You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.bra)}.`;
            if (global.kobold.inventory.equipment.bra.skimpy()) {
                bra += ` <span class="warningtext">It barely covers your breasts.</span>`;
            } else {
                bra += ` It covers your chest nicely.`;
            }
        }

        if (isWearingBra && !shouldBeWearingBra) {
            bra += `You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.bra)}. <span class="sneakytext">Why are you wearing this?`;
            if (global.kobold.inventory.equipment.bra.skimpy()) {
                bra += " You look like a slut.";
            } else {
                bra += " You look silly.";
            }
            bra += "</span>";
        }

        if (!isWearingBra && shouldBeWearingBra) {
            bra += `You aren't wearing a bra.`;
            if (global.kobold.inventory.equipment.chestExposed()) {
                bra += ` <span class="warningtext">Your chest is completely exposed.</span>`;
            }
        }

        if (!isWearingBra && !shouldBeWearingBra) {
            bra += `You aren't wearing a bra. <span class="sneakytext">Why would you?</span>`;
        }

        return bra + "</div>";
    }

    /**
     * Builds and returns the helmet section of the equipment part of the inventory.
     * @returns {string} The helmet section of the equipment part of the inventory as a HTML string.
     */
    function buildHelmet() {
        var isWearingHelmet = global.kobold.inventory.equipment.isWearing(SlotEnum.HELMET);

        if (!isWearingHelmet) {
            return `<div class="text nomargin">You aren't wearing a helmet.</div>`;
        }

        var helmetConceals = global.kobold.inventory.equipment.helmet.hidesFace();
        var helmet = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.helmet)}.`;
        if (helmetConceals) {
            helmet += ` <span class="sneakytext">It conceals your face, allowing you to blend in more easily.</span>`;
        }
        return helmet + "</div>";
    }

    /**
     * Builds and returns the pants section of the equipment part of the inventory.
     * @returns {string} The pants section of the equipment part of the inventory as a HTML string.
     */
    function buildPants() {
        var isWearingPants = global.kobold.inventory.equipment.isWearing(SlotEnum.PANTS);
        var humiliate = global.kobold.body.hasCunt();

        if (isWearingPants) {
            return `<div class="text nomargin">You are wearing ${buildItemWithTooltip(global.kobold.inventory.equipment.pants)}.</div>`;
        }

        var pants = `<div class="text nomargin">You aren't wearing any pants.`;
        if (humiliate) {
            pants += ` <span class="warningtext">You slut.</span>`;
        }
        return pants + "</div>";
    }

    /**
     * Builds and returns the shirt section of the equipment part of the inventory.
     * @returns {string} The shirt section of the equipment part of the inventory as a HTML string.
     */
    function buildShirt() {
        var isWearingShirt = global.kobold.inventory.equipment.isWearing(SlotEnum.SHIRT);
        var coversGroin = global.kobold.inventory.equipment.shirt && global.kobold.inventory.equipment.shirt.coversGroin();
        var coversRear = global.kobold.inventory.equipment.shirt && global.kobold.inventory.equipment.shirt.coversRear();

        if (isWearingShirt) {
            var toReturn = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.shirt)}.`;
            if (coversGroin && coversRear) {
                toReturn += ` It's long enough to cover your waist.`;
            } else if (coversGroin) {
                toReturn += ` It's long enough to cover your groin.`;
            } else if (coversRear) {
                toReturn += ` It's long enough to cover your ass.`;
            }
            toReturn += `</div>`;
            return toReturn;
        }

        return `<div class="text nomargin">You aren't wearing a shirt.</div>`;
    }

    /**
     * Builds and returns the shoes section of the equipment part of the inventory.
     * @returns {string} The shoes section of the equipment part of the inventory as a HTML string.
     */
    function buildShoes() {
        var isWearingShoes = global.kobold.inventory.equipment.isWearing(SlotEnum.SHOES);

        if (!isWearingShoes) {
            return `<div class="text nomargin">You aren't wearing shoes. You're used to walking on your bare feet.</div>`;
        }

        var soleExposed = global.kobold.inventory.equipment.shoes.soleExposed();
        var shoe = `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.shoes)}.`;
        if (soleExposed) {
            shoe += ` They do little to nothing to protect your soles.`;
        } else {
            shoe += ` Not walking on your bare feet for once is rather enjoyable.`;
        }
        return shoe + "</div>";
    }

    /**
     * Builds and returns the underwear section of the equipment part of the inventory.
     * @returns {string} The underwear section of the equipment part of the inventory as a HTML string.
     */
    function buildUnderwear() {
        var isWearingUnderwear = global.kobold.inventory.equipment.isWearing(SlotEnum.UNDERWEAR);
        var groinExposed = global.kobold.inventory.equipment.groinExposed();
        var rearExposed = global.kobold.inventory.equipment.rearExposed();

        var pantsu = "";
        if (!isWearingUnderwear) {
            pantsu += `<div class="text nomargin">You aren't wearing any underwear.`;
            if (groinExposed) {
                pantsu += ` <span class="warningtext">Your groin is completely naked.</span>`;
            }
            if (rearExposed) {
                pantsu += ` <span class="warningtext">Your butt is exposed.</span>`;
            }
            if (!groinExposed && !rearExposed) {
                pantsu += ` <span class="sneakytext">Luckily, another piece of clothing is covering your waist, hiding your indecency.</span>`;
            }
        }
        if (isWearingUnderwear) {
            pantsu += `<div class="text nomargin">You are wearing a ${buildItemWithTooltip(global.kobold.inventory.equipment.underwear)}.`;
            if (groinExposed) {
                pantsu += ` <span class="warningtext">Your groin is still completely visible.</span>`;
            }
            if (global.kobold.inventory.equipment.underwear.coversGroin()) {
                pantsu += ` It hides your groin from prying eyes.`;
            }
            if (rearExposed) {
                pantsu += ` <span class="warningtext">Your rear is left uncovered.</span>`;
            }
            if (groinExposed && global.kobold.inventory.equipment.underwear.coversRear()) {
                pantsu += ` <span class="sneakytext">At least it covers your ass.</span>`;
            }
        }

        return pantsu + "</div>";
    }

    /**
     * Builds and returns the weapon section of the equipment part of the inventory.
     * @returns {string} The weapon section of the equipment part of the inventory as a HTML string.
     */
    function buildWeapon() {
        var hasWeapon = global.kobold.inventory.equipment.isWearing(SlotEnum.WEAPON);
        if (!hasWeapon) {
            return `<div class="text nomargin">You are unarmed.</div>`;
        } else {
            return `<div class="text nomargin">You are wielding a ${buildItemWithTooltip(global.kobold.inventory.equipment.weapon)}.</div>`;
        }
    }

    /**
     * Builds a comment based on how well the kobold is clothed.
     * @returns {string} The comment, as a HTML string.
     */
    function buildComment() {
        if (global.kobold.inventory.equipment.completelyNaked()) {
            return `<div class="text nomargin"><span class="warningtext">You aren't wearing anything at all.</span></div>`;
        }

        if (global.kobold.inventory.equipment.importantNaked()) {
            var shoes = global.kobold.inventory.equipment.shoes;
            var helmet = global.kobold.inventory.equipment.helmet;
            if (shoes && helmet) {
                return `<div class="text nomargin"><span class="sneakytext">Shoes and a ${helmet.name()} do little to hide your shame.</span></div>`;
            }
            if (helmet) {
                return `<div class="text nomargin"><span class="sneakytext">Your ${helmet.name()} offers little solace in face of your nudity.</span></div>`;
            }
            if (shoes) {
                return `<div class="text nomargin"><span class="sneakytext">At least you've got shoes, right?</span></div>`;
            }
        }

        return "";
    }

    /**
     * Builds and returns the equipment section of the inventory.
     * @returns {string} The equipment section of the inventory, as HTML string.
     */
    function buildEquipment() {
        var equipment = buildSubtitle("Equipment");
        equipment += buildBra();
        equipment += buildHelmet();
        equipment += buildPants();
        equipment += buildShirt();
        equipment += buildShoes();
        equipment += buildUnderwear();
        equipment += buildWeapon();
        equipment += buildComment();
        return equipment;
    }

    /**
     * Sorts the passed array of ItemAmounts by their uncapitalized names.
     * @param {array} itemAmounts The array of ItemAmounts to sort.
     * @returns {array} The sorted array.
     */
    function sortItemArray(itemAmounts) {
        itemAmounts.sort(function (a, b) {
            var aName = a.item.name();
            var bName = b.item.name();
            if (aName < bName) {
                return -1;
            }
            if (aName > bName) {
                return 1;
            }
            return 0;
        });
        return itemAmounts;
    }

    /**
     * Builds and returns the number section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The number section of an item row.
     */
    function buildItemRowNumber(itemAmount) {
        return `<div class="itemrownumber">${itemAmount.amount}</div>`;
    }

    /**
     * Builds and returns the X section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The X section of an item row.
     */
    function buildItemRowX(itemAmount) {
        return `<div class="itemrowx">x</div>`;
    }

    /**
     * Builds and returns the name section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The name section of an item row.
     */
    function buildItemRowName(itemAmount) {
        return `<div class="itemrowname">${buildItemWithTooltip(itemAmount.item, true)}</div>`;
    }

    /**
     * Builds and returns the value section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The value section of an item row.
     */
    function buildItemRowValue(itemAmount) {
        return `<div class="itemrowvalue">${Math.floor(itemAmount.item.price() * global.kobold.inventory.sellMod)} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Builds and returns the sell section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The sell section of an item row.
     */
    function buildItemRowSell(itemAmount) {
        var price = itemAmount.item.price();
        var sellZone = global.kobold.inventory.sellZone;
        var priceRight = (price > 0);

        var toReturn = `<div class="itemrowsellcontainer">`;
        if (priceRight && sellZone) {
            toReturn += `<button class="button ${sellButtonClass}">Sell</button>`;
        }
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the use section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The use section of an item row.
     */
    function buildItemRowUse(itemAmount) {
        var type = itemAmount.item.type;
        var safeZone = global.kobold.inventory.safeZone;
        var rightType = (type == ItemTypeEnum.CONSUMABLE);

        var toReturn = `<div class="itemrowusecontainer">`;
        if (rightType && safeZone) {
            toReturn += `<button class="button ${useButtonClass}">Use</button>`;
        }
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the innards of a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The innards of a row for a single item.
     */
    function buildItemRowInnards(itemAmount) {
        var innards = buildItemRowNumber(itemAmount);
        innards += buildItemRowX(itemAmount);
        innards += buildItemRowName(itemAmount);
        innards += buildItemRowValue(itemAmount);
        innards += buildItemRowSell(itemAmount);
        innards += buildItemRowUse(itemAmount);
        return innards;
    }

    /**
     * Builds and returns a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @argument {int} i The 0-based row number of the item.
     * @returns {string} A row for a single item.
     */
    function buildItemRow(itemAmount, i) {
        var id = itemAmount.item.id;
        var rowClass = "itemroweven";
        if (i % 2) {
            rowClass = "itemrowodd";
        }
        var row = `<div id="${id}" class="itemrow ${rowClass}">`;
        row += buildItemRowInnards(itemAmount);
        return row + `</div>`;
    }

    /**
     * Builds and returns the bag section of the inventory.
     * @returns {string} The bag section of the inventory, as HTML string.
     */
    function buildBag() {
        var items = global.kobold.inventory.getItemList();
        var sortedItems = sortItemArray(items);

        var bag = buildSubtitle("Backpack");
        var coins = global.kobold.inventory.coins;
        var safeZone = global.kobold.inventory.safeZone;
        var sellZone = global.kobold.inventory.sellZone;
        bag += `<div class="text">You have <span id="${coinSpanId}">${coins}</span> coin${coins !== 1 ? "s" : ""}.`;

        if (sortedItems.length > 0) {
            if (safeZone) {
                bag += ` You are in a safe zone. You may use items from the list below.`;
            } else {
                bag += ` Find a safe zone to interact with items.`;
            }
            if (sellZone) {
                bag += ` You are in a sell zone. You may sell items from the list below.`;
            }
        } else {
            if (safeZone) {
                bag += ` You are in a safe zone, but have no items to use.`;
            }
            if (sellZone) {
                bag += ` You are in a sell zone, but have no items to sell.`;
            }
        }

        bag += `</div>`;

        if (sortedItems.length < 1) {
            bag += `<div class="text">You have no items.</div>`;
        } else {
            for (var i = 0; i < sortedItems.length; i++) {
                bag += buildItemRow(sortedItems[i], i);
            }
        }
        return bag;
    }

    /**
     * Builds and returns the inventory.
     * @returns {string} The inventory, as a HTML string.
     */
    function buildInventory() {
        return buildEquipment() + buildBag();
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = gen.buildTabHeader(title);
        tab += buildInventory();
        tab += gen.buildTabFooter();
        return tab;
    }

    /**
     * Updates the kobold's coincount.
     */
    function updateCoins() {
        var coins = global.kobold.inventory.coins;
        $("#" + coinSpanId).html(coins);
    }

    /**
     * Disables the row's buttons and then hides the row.
     * @param {DOM element} row The row to hide.
     */
    function hideRow(row) {
        row.find("button").attr("disabled", true);
        row.hide(400);
    }

    /**
     * Updates the amount of the row.
     * @param {DOM element} row The row to update.
     * @param {int} newAmount The new amount of items.
     */
    function updateAmount(row, newAmount) {
        row.children(".itemrownumber").html(newAmount);
    }

    /**
     * Updates the amount of an item and removes the item if necessary.
     * @param {object} item The item of which to update the row.
     */
    function itemUpdate(item) {
        var row = $("#" + item.id);
        var amount;
        if (item.type === ItemTypeEnum.CONSUMABLE) {
            amount = global.kobold.inventory.bag.hasConsumable(item);
        } else {
            amount = global.kobold.inventory.bag.hasMisc(item);
        }
        updateAmount(row, amount);
        if (amount === 0) {
            hideRow(row);
        }
    }


    /**
     * Sells an item with the passed id.
     * @param {string | int} id The id of the item.
     */
    function sellItem(id) {
        var item = global.kobold.inventory.constructItem(id);
        global.kobold.inventory.coins += Math.floor(item.price() * global.kobold.inventory.sellMod);
        if (item.type === ItemTypeEnum.CONSUMABLE) {
            global.kobold.inventory.bag.removeConsumable(item);
        } else {
            global.kobold.inventory.bag.removeMisc(item);
        }
        itemUpdate(item);
        updateCoins();
        if (global.kobold.inventory.sellListener) {
            global.kobold.inventory.sellListener(item);
        }
    }

    /**
     * Uses an item with the passed id.
     * @param {string | int} id The id of the item.
     */
    function useItem(id) {
        var item = global.kobold.inventory.constructItem(id);
        if (!item.consume()) {
            return;
        }
        global.kobold.history.consumable.use(id);
        global.kobold.inventory.bag.removeConsumable(item);
        itemUpdate(item);
    }

    /**
     * Registers the listener for sell buttons.
     */
    function registerSellButtonListener() {
        $("." + sellButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var id = row.attr("id");
            sellItem(id);
        });
    }

    /**
     * Registers the listener for use buttons.
     */
    function registerUseButtonListener() {
        $("." + useButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var id = row.attr("id");
            useItem(id);
        });
    }

    /*
     * Registers listeners for sell and use buttons.
     */
    function registerButtonListeners() {
        registerSellButtonListener();
        registerUseButtonListener();
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the tab button start blinking if it is not already blinking.
     */
    this.startBlinking = function () {
        if (blinking) {
            return;
        }
        blinking = true;
        $("#" + id).addClass(blinkClass);
    };

    /**
     * Makes the tab button stop blinking if it is.
     */
    this.stopBlinking = function () {
        if (!blinking) {
            return;
        }
        blinking = false;
        $("#" + id).removeClass(blinkClass);
    };

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerButtonListeners();
    };
}

// END res\js\ui\tabs\InventoryTab.js

// res\js\ui\tabs\LoadTab.js

/**
 * This class contains functions specific to the Load tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function LoadTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "loadtab";
    var title = "Load";
    var deleteButtonId = "deletebutton";
    var deleteAllButtonId = "deleteAllButton";
    var importButtonId = "importButton";
    var uploadInputId = "uploadInput";
    var loadButtonId = "loadbutton";
    var dragAndDropDetectorId = "dragAndDropDetector";
    var dragAndDropOverlayId = "dragAndDropOverlay";

    // The currently selected slot's id.
    var selected = "";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds the drag and drop overlay.
     * @returns {string} The drag and drop overlay.
     */
    function buildDragAndDropOverlay() {
        return `<div id="${dragAndDropOverlayId}" class="fullscreenpopup savedraganddropoverlay" style="display: none;"></div>`;
    }

    /**
     * Builds and returns the new game save slot HTML string.
     * @returns {string} The new game save slot HTML string.
     */
    function buildNewGameSlot() {
        var save = global.saveManager.getNewGameSaveMeta();
        return buildSmallSlotWithTime(save, "New game", false);
    }

    /**
     * Builds and returns the auto save slot HTML string.
     * @returns {string} The auto save slot HTML string.
     */
    function buildAutoSaveSlot() {
        var save = global.saveManager.getAutoSaveMeta();
        return buildSmallSlotWithTime(save, "Autosave", true);
    }

    /**
     * Builds and returns a small save slot, with comment box omitted.
     * @param {object} save The save to create a small slot for. 
     * @param {int} rank The rank (= slot number) of the save.
     * @param {boolean} displayTime If true, adds the time to the small save slot.
     * @returns {string} The save slot.
     */
    function buildSmallSlotWithTime(save, rank, displayTime) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${rank}</div>`;
        if (displayTime) {
            slot += `   <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>`;
        }

        slot += `    <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns a generic save slot for loading purposes.
     * @param {object} save The save to create a slot for.
     * @returns {string} The save slot.
     */
    function buildSaveSlot(save) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.saveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${save.slot + 1}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                        <textarea rows="2" cols="55" maxlength="114" class="savecomment" disabled>${save.comment}</textarea>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the control bar, containing the load and delete buttons.
     */
    function buildControlBar() {
        var disableImport = global.saveManager.getFirstOpenSaveSlot() < 0;
        return `<div class="savecontrolbar">
                    <button class="button saveleftbutton warning" id="${deleteButtonId}" disabled>Delete</button>
                    <button class="button savesmallleftbutton warning" id="${deleteAllButtonId}">Delete ALL</button>
                    <button class="button savesmallrightbutton" id="${importButtonId}" ${disableImport ? "disabled" : ""}
                        title="You can also drag and drop save files onto the list above.">Import</button>
                    <button class="button saverightbutton" id="${loadButtonId}" disabled>Load</button>
                    <input id="${uploadInputId}" type="file" style="visibility:hidden;" accept=".kas" multiple>
                </div>`;
    }

    /**
     * Builds and returns an additional finisher.
     */
    function extraFinisher() {
        return `<div class="finish"></div>`;
    }

    /**
     * Builds and returns the save list.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The save list.
     */
    function buildSaveList(saveMeta) {
        var saveList = buildNewGameSlot();
        saveList += buildAutoSaveSlot();

        for (var i = 0; i < saveMeta.length; i++) {
            saveList += buildSaveSlot(saveMeta[i]);
        }

        return saveList;
    }

    /**
     * Disables the import button if there are no more free save slots.
     */
    function disableImportButtonIfRequired() {
        var disable = global.saveManager.getFirstOpenSaveSlot() < 0;
        if (disable) {
            gen.disable(importButtonId);
        }
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var saveMeta = global.saveManager.saveList();
        var tab = gen.buildMaxSaveIndicator(saveMeta);
        tab += gen.buildTabHeader(title);
        var content = buildSaveList(saveMeta);
        content += extraFinisher();
        content += extraFinisher();
        content = `<div id="${dragAndDropDetectorId}" style="width: 100%; min-height: 80%;">` + content + `</div>`;
        tab += content;
        tab += gen.buildTabFooter();
        tab += buildDragAndDropOverlay();
        tab += buildControlBar();
        return tab;
    }



    /**
     * Registers listeners to the auto and new game slots.
     */
    function registerAutoAndNewListeners() {
        $("." + gen.smallSaveSlotClass).click(function () {
            gen.disable(deleteButtonId);
            gen.enable(loadButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Function registered as click listeners to list elements (save slots).
     * "this" in this context refers to the save slot itself.
     * @returns {undefined}
     */
    function saveListClicked() {
        gen.enable(deleteButtonId);
        gen.enable(loadButtonId);
        selectSlot($(this));
    }

    /**
     * Registers listeners to the list of saves.
     */
    function registerListListeners() {
        $("." + gen.saveSlotClass).click(saveListClicked);
    }

    /**
     * Registers a listener for the delete button.
     */
    function registerDeleteButtonListener() {
        $("#" + deleteButtonId).click(function () {
            if (confirm("Are you sure you want to delete this save?")) {
                gen.disable(deleteButtonId);
                gen.disable(loadButtonId);
                gen.enable(importButtonId);
                global.saveManager.delete(selected);
                gen.updateMaxSaveIndicator();
                var toRemove = $("#" + selected);
                toRemove.hide(400, function () {
                    toRemove.remove();
                });
            }
        });
    }

    /**
     * Registers a listener for the load button.
     */
    function registerLoadButtonListener() {
        $("#" + loadButtonId).click(function () {
            gen.disable(deleteButtonId);
            gen.disable(loadButtonId);
            global.saveManager.load(selected);
        });
    }

    /**
     * Registers a listener for the delete all saves button.
     */
    function registerDeleteAllButtonListener() {
        $("#" + deleteAllButtonId).click(function () {
            if (!confirm("Do you really want to delete ALL your saves?")
                    || !confirm("There's no going back from this.")
                    || !confirm("Really consider the consequences of what you're doing.")
                    || !confirm("ALL your saves will be GONE. FOREVER.")
                    || confirm("Click CANCEL this time if you really want to go through with this.")
                    || !confirm("This is the last time I'm asking you.")
                    || !confirm("If you click OK, ALL your saves will be DELETED. Gone for good.")
                    ) {
                alert("Crisis averted.");
                return;
            }
            alert("So be it. I warned you.");
            gen.disable(deleteButtonId);
            gen.disable(loadButtonId);
            gen.enable(importButtonId);
            var meta = global.saveManager.saveList();
            for (var i = 0; i < meta.length; i++) {
                global.saveManager.delete(meta[i].slot);
            }
            var allElements = $("." + gen.saveSlotClass);
            allElements.hide(400, function () {
                allElements.remove();
            });
            gen.updateMaxSaveIndicator();
        });
    }

    /**
     * Registers a listener for the import button.
     */
    function registerImportButtonListener() {
        $("#" + importButtonId).click(function () {
            $("#" + uploadInputId).click();
        });
    }

    /**
     * Adds a save to the correct location in the list.
     * Animates the adding.
     * Assumes all previous save slots have been added.
     * @param {object} meta The metadata object of the slot to add.
     */
    function addSave(meta) {
        var slot = meta.slot;
        var previous = $("#" + (slot - 1));
        if (!saveExists(slot - 1)) {
            previous = $("#autosave");
        }
        var newSaveSlot = buildSaveSlot(meta);
        previous.after(newSaveSlot);
        var newSaveSlotElement = $("#" + slot);
        newSaveSlotElement.hide(0);
        newSaveSlotElement.show(400);
        newSaveSlotElement.click(saveListClicked);
    }

    /**
     * Checks whether or not the specified save exists in the load list.
     * @param {int} slot The slot of the save to check.
     * @returns {boolean} True if the slot exists, false otherwise.
     */
    function saveExists(slot) {
        var elem = $("#" + slot);
        return elem.length;
    }

    /**
     * Called when file import finishes.
     */
    function postImport() {
        $("#" + uploadInputId).val(""); // Reset to allow the same file to be uploaded twice in a row.
        var meta = global.saveManager.saveList();
        for (var i = 0; i < meta.length; i++) {
            var slot = meta[i].slot;
            if (!saveExists(slot)) {
                addSave(meta[i]);
            }
        }
        gen.enable(importButtonId);
        disableImportButtonIfRequired();
        gen.updateMaxSaveIndicator();
    }

    /**
     * Registers a change listener that will detect file imports.
     */
    function registerUploadChangeListener() {
        $("#" + uploadInputId).change(function (evt) {
            gen.disable(importButtonId);
            var list = evt.target.files;
            global.saveManager.import.files(list, postImport);
        });
    }

    var overlayCurrentlyDraggedOver = false;

    /**
     * Function called when a file gets hovered over the drag and drop area.
     * @param {event} evt The drag event.
     */
    function dropAreaDragOver(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (overlayCurrentlyDraggedOver) {
            return;
        }
        overlayCurrentlyDraggedOver = true;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeIn(100);
    }

    /**
     * Function called when a file stops hovering over the drag and drop area.
     * @param {event} evt The drag leave event.
     */
    function dropAreaDragLeave(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        if (!overlayCurrentlyDraggedOver) {
            return;
        }
        overlayCurrentlyDraggedOver = false;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeOut(100);
    }

    /**
     * Function called when a file gets dropped on the drag and drop area.
     * @param {event} evt The drop event.
     */
    function dropAreaDragDrop(evt) {
        evt.preventDefault();
        evt.stopPropagation();
        gen.disable(importButtonId);
        global.saveManager.import.files(evt.originalEvent.dataTransfer.files, postImport);
        overlayCurrentlyDraggedOver = false;
        var element = $("#" + dragAndDropOverlayId);
        element.stop(true);
        element.fadeOut(100);
    }

    /**
     * Registers a listener for the drop area.
     */
    function registerDropAreaListener() {
        $("#" + dragAndDropDetectorId).on("dragover", dropAreaDragOver);
        $("#" + dragAndDropOverlayId).on("dragover", dropAreaDragOver);
        $("#" + dragAndDropOverlayId).on("dragleave", dropAreaDragLeave);
        $("#" + dragAndDropOverlayId).on("drop", dropAreaDragDrop);
    }

    /**
     * Registers listeners to all load slots.
     */
    function registerLoadListeners() {
        registerAutoAndNewListeners();
        registerListListeners();
        registerDeleteButtonListener();
        registerDeleteAllButtonListener();
        registerLoadButtonListener();
        registerImportButtonListener();
        registerUploadChangeListener();
        registerDropAreaListener();
    }

    /**
     * Selects this element and unselects all others.
     * @param {DOMelement} element The element to select.
     */
    function selectSlot(element) {
        gen.removeSaveSlotSelected();
        element.addClass(gen.saveSlotSelectedClass);
        selected = element.attr('id');
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerLoadListeners();
    };
}

// END res\js\ui\tabs\LoadTab.js

// res\js\ui\tabs\SaveTab.js

/**
 * This class contains functions specific to the Save tab.
 * Its init() function should be called immediately upon successful project load.
 * Not the best implementation. Can be improved in many ways.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function SaveTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "savetab";
    var title = "Save";
    var deleteButtonId = "deletebutton";
    var saveButtonId = "savebutton";
    var exportButtonId = "exportButton";
    var exportAllButtonId = "exportAllbutton";
    var newCommentFieldId = "newcomment";
    var selected = "";
    var newSaveSlotId = "";
    var autoSaveSlotId = "";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds and returns the new save slot HTML string.
     * @param {string} passedId If passed, will overwrite open slot lookup.
     * @returns {string} The new save slot HTML string.
     */
    function buildNewSaveSlot(passedId) {
        var saveId;
        if (passedId) {
            saveId = passedId;
        } else {
            saveId = global.saveManager.getFirstOpenSaveSlot();
        }
        if (saveId < 0) {
            return "";
        }

        newSaveSlotId = saveId;
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${saveId}">
                        <img class="savegender" src="${global.kobold.body.getGenderIcon()}" alt="${global.kobold.body.getGenderName()}">
                        <div class="saverank">New save</div>    
                        <div class="savename"><span class="bold">${global.kobold.name}</span></div>
                        <div class="savescene">${global.currentSceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the auto save slot HTML string.
     * @returns {string} The auto save slot HTML string.
     */
    function buildAutoSaveSlot() {
        var save = global.saveManager.getAutoSaveMeta();
        autoSaveSlotId = save.slot;
        return buildSmallSlotWithTime(save, "Autosave");
    }

    /**
     * Builds and returns a small save slot, with comment box omitted.
     * @param {object} save The save to create a small slot for. 
     * @param {int} rank The rank (= slot number) of the save.
     * @returns {string} The save slot.
     */
    function buildSmallSlotWithTime(save, rank) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${rank}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>        
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns a generic save slot for loading purposes.
     * @param {object} save The save to create a slot for.
     * @returns {string} The save slot.
     */
    function buildSaveSlot(save) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.saveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${+save.slot + 1}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                        <textarea rows="2" cols="55" maxlength="114" class="savecomment" disabled>${save.comment}</textarea>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the save list.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The save list.
     */
    function buildSaveList(saveMeta) {
        var saveList = buildNewSaveSlot();
        saveList += buildAutoSaveSlot();

        for (var i = 0; i < saveMeta.length; i++) {
            saveList += buildSaveSlot(saveMeta[i]);
        }

        return saveList;
    }

    /**
     * Builds and returns an additional finisher.
     */
    function extraFinisher() {
        return `<div class="finish"></div>`;
    }

    /**
     * Builds and returns the control bar, containing the save and delete buttons.
     */
    function buildControlBar() {
        return `<div class="savecontrolbar">
                    <button class="button saveleftbutton warning" id="${deleteButtonId}" disabled>Delete</button>
                    <button class="button savesmallleftbutton" id="${exportAllButtonId}">Export ALL</button>
                    <textarea rows="2" cols="55" maxlength="114" class="savecomment savenewcommentfield" placeholder="Comment. Old comment if left blank." id="${newCommentFieldId}"></textarea>
                    <button class="button saverightbutton" id="${saveButtonId}" disabled>Save</button>
                    <button class="button savesmallrightbutton" id="${exportButtonId}" disabled>Export</button>
                </div>`;
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var saveMeta = global.saveManager.saveList();
        var tab = gen.buildMaxSaveIndicator(saveMeta);
        tab += gen.buildTabHeader(title);
        tab += buildSaveList(saveMeta);
        tab += extraFinisher();
        tab += extraFinisher();
        tab += gen.buildTabFooter();
        tab += buildControlBar();
        return tab;
    }

    /**
     * Registers a listener for the new save slot.
     */
    function registerNewSaveListener() {
        if (newSaveSlotId === "") {
            return;
        }
        $("#" + newSaveSlotId).click(function () {
            gen.disable(deleteButtonId);
            gen.enable(saveButtonId);
            gen.disable(exportButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Registers a listener for the autosave slot.
     */
    function registerAutoSaveListener() {
        $("#" + autoSaveSlotId).click(function () {
            gen.disable(deleteButtonId);
            gen.disable(saveButtonId);
            gen.disable(exportButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Selects this element and unselects all others.
     * @param {DOMelement} element The element to select.
     */
    function selectSlot(element) {
        gen.removeSaveSlotSelected();
        element.addClass(gen.saveSlotSelectedClass);
        selected = element.attr('id');
    }

    /**
     * Registers listeners for the list of saveslots.
     * @param {string} selector The selector pointing to the list of saveslots.
     */
    function registerListListeners(selector) {
        $(selector).click(function () {
            gen.enable(deleteButtonId);
            gen.enable(saveButtonId);
            gen.enable(exportButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Registers a listener for the delete button.
     */
    function registerDeleteButtonListener() {
        $("#" + deleteButtonId).click(function () {
            if (confirm("Are you sure you want to delete this save?")) {
                gen.disable(deleteButtonId);
                gen.disable(saveButtonId);
                gen.disable(exportButtonId);
                global.saveManager.delete(selected);
                gen.updateMaxSaveIndicator();
                var toRemove = $("#" + selected);
                toRemove.hide(400, function () {
                    toRemove.remove();
                });
                showNewSaveSlot(selected);
            }
        });
    }

    /**
     * This function hides the new save slot and disables both save and load
     * buttons (forcing a repick). Should be done when no more save slots are
     * available.
     */
    function hideNewSaveSlotAndDisableStuff() {
        var element = $("#" + newSaveSlotId);
        element.attr("id", "-1");
        element.hide(400);
        gen.disable(deleteButtonId);
        gen.disable(saveButtonId);
        gen.disable(exportButtonId);
        selected = "";
        newSaveSlotId = "-1";
    }

    /**
     * Shows the new save slot and updates its id.
     * Does nothing if the save slot is visible.
     * Adds the save slot if necessary.
     * @param {string} newId The new id.
     */
    function showNewSaveSlot(newId) {
        if (newSaveSlotId === "") {
            var slot = buildNewSaveSlot(newId);
            $("#" + autoSaveSlotId).before(slot);
            $("#" + newId).hide(0);
            $("#" + newId).show(400);
            newSaveSlotId = newId;
            registerNewSaveListener();
        } else if (!($("#" + newSaveSlotId).is(":visible"))) {
            updateNewSaveSlot(newId);
            $("#" + newId).show(400);
        } else if (newId < newSaveSlotId) {
            updateNewSaveSlot(newId);
        }
    }

    /**
     * Updates the new save slot to a new id.
     * @param {string} newId The new id.
     */
    function updateNewSaveSlot(newId) {
        $("#" + newSaveSlotId).attr("id", newId);
        newSaveSlotId = newId;
    }

    /**
     * Inserts the passed HTML-save after the auto save slot.
     * @param {string} save
     * @returns {undefined}
     */
    function insertAfterAutoSaveSlot(save) {
        $("#" + autoSaveSlotId).after(save);
    }

    /**
     * Builds a graphical save slot from the passed meta and inserts it in the
     * correct place in the save list.
     * @param {SaveMetaData} meta The metadata of the new save.
     */
    function buildAndInsertNewSave(meta) {
        var newSave = buildSaveSlot(meta);
        var saves = $("." + gen.saveSlotClass);
        if (saves.length == 0 || $(saves[0]).attr('id') > meta.slot) {
            insertAfterAutoSaveSlot(newSave);
        } else {
            var after = $(saves[0]);
            for (var i = 0; i < saves.length; i++) {
                if (+$(saves[i]).attr('id') < meta.slot) {
                    after = $(saves[i]);
                }
            }
            after.after(newSave);
        }
        var element = $("#" + meta.slot);
        element.hide(0);
        registerListListeners("#" + meta.slot);
        scrollToAndShowNewSave(element);
    }

    /**
     * Scrolls to the newly created save and then shows it.
     * @param {DOM element} element The save.
     */
    function scrollToAndShowNewSave(element) {
        var main = $("#main");
        var currentScroll = main.scrollTop();
        var offset = element.offset().top;

        // Element visible: do nothing.
        if (offset > 75 && offset < 720) {
            element.show(400);
            return;
        }

        var newScroll = currentScroll + offset - 300;
        main.animate({scrollTop: newScroll}, 400, function () {
            element.show(400);
        });
    }

    /**
     * Saves the game, creates and inserts a new save into the UI
     * and updates the save limit, hiding the new save bar if need be.
     * @param {string} comment The textbox-entered comment.
     */
    function newSave(comment) {
        var slot = selected;
        global.saveManager.save(slot, comment);
        var newId = global.saveManager.getFirstOpenSaveSlot();
        if (newId < 0) {
            hideNewSaveSlotAndDisableStuff();
        } else {
            updateNewSaveSlot(newId);
            selected = newId;
        }
        buildAndInsertNewSave(global.saveManager.getMeta(slot));
        gen.updateMaxSaveIndicator();
    }

    /**
     * Rebuilds the slot with the passed id, replacing its contents with the new metadata.
     * @param {string} id The id of the element.
     * @param {SaveMetaData} newMeta The new metadata object.
     */
    function rebuildSlot(id, newMeta) {
        var parent = $("#" + id);
        // .savegender, .savetimestamp, .savename, .savescene, .savecomment
        parent.children().not(".saverank").fadeOut(400, function () {
            var src = GenderEnum.properties[newMeta.gender].icon;
            var element = parent.children(".savegender");
            element.attr("src", src);
            element = parent.children(".savetimestamp");
            element.html(gen.timeToTimePassed(newMeta.time));
            element = parent.children(".savename");
            element.html('<span class="bold">' + newMeta.name + '</span>');
            element = parent.children(".savescene");
            element.html(newMeta.sceneName);
            element = parent.children(".savecomment");
            element.val(newMeta.comment);
            parent.children().fadeIn(400);
        });
    }

    /**
     * Overwrites the selected save after confirming the user's actions.
     * @param {string} comment The textbox-entered comment.
     */
    function overwriteSave(comment) {
        if (!confirm("Do you want to overwrite this save?")) {
            return;
        }
        var oldMeta = global.saveManager.getMeta(selected);
        if (!comment || comment === "") {
            comment = oldMeta.comment;
        }
        global.saveManager.save(selected, comment);
        var newMeta = global.saveManager.getMeta(selected);
        rebuildSlot(selected, newMeta);
    }

    /**
     * Registers a listener for the save button.
     */
    function registerSaveButtonListener() {
        $("#" + saveButtonId).click(function () {
            var comment = $("#" + newCommentFieldId).val();
            if (selected == newSaveSlotId) {
                newSave(comment);
            } else {
                overwriteSave(comment);
            }
        });
    }

    /**
     * Registers a listener for the export all button.
     */
    function registerExportAllButtonListener() {
        $("#" + exportAllButtonId).click(function () {
            global.saveManager.export.all();
        });
    }

    /**
     * Registers a listener for the export button.
     */
    function registerExportButtonListener() {
        $("#" + exportButtonId).click(function () {
            global.saveManager.export.slot(selected);
        });
    }

    /**
     * Registers a listener for the text area.
     */
    function registerTextAreaListener() {
        $("#" + newCommentFieldId).keydown(function (event) {
            if (event.keyCode == 13) {
                $("#" + saveButtonId + ":enabled").click();
                event.preventDefault();
            }
        });
    }

    /**
     * Registers listeners to all save slots.
     */
    function registerSaveListeners() {
        registerNewSaveListener();
        registerAutoSaveListener();
        registerListListeners("." + gen.saveSlotClass);
        registerDeleteButtonListener();
        registerSaveButtonListener();
        registerExportAllButtonListener();
        registerExportButtonListener();
        registerTextAreaListener();
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        newSaveSlotId = "";
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerSaveListeners();
    };
}

// END res\js\ui\tabs\SaveTab.js

// res\js\ui\tabs\SceneTab.js

/**
 * This class contains functions specific to the Scene tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function SceneTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "scenetab";
    var mainId = "main";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * The onscroll listener for the main window.
     * Updates the sceneScroll level of the UiController
     *   only when this tab is active.
     */
    function onScroll() {
        if (!gen.isActive(id)) {
            return;
        }
        var scroll = $("#" + mainId).scrollTop();
        global.ui.sceneScroll = scroll;
    }

    /**
     * Registers the onclick function for this tab.
     * Registers a scroll listener to the main window to update the sceneScroll.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
        $("#" + mainId).scroll(onScroll);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        global.sceneLoader.loadTabScene(global.currentScene);
        global.ui.setScroll();
    };

    /**
     * Activates this tab.
     * Only alters button visuals.
     */
    this.activate = function () {
        gen.tabButtonTransition(id);
    };
}

// END res\js\ui\tabs\SceneTab.js

// res\js\ui\tabs\StatsTab.js

/**
 * This class contains functions specific to the Stats tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function StatsTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "statstab";
    var title = "Stats";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds the comment for the Agility attribute.
     * @returns {string} The comment.
     */
    function buildAgilityComment() {
        var statValue = global.kobold.stats.get(StatsEnum.AGILITY);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Can't reach own toes.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">A little stiff.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Nimble.</div>`;
        }

        return `<div class="text nomargin">Part snake.</div>`;
    }

    /**
     * Builds the header for the Agility attribute.
     * @returns {String} The header.
     */
    function buildAgilityHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.AGILITY);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Agility attribute.
     * @returns {String} The number.
     */
    function buildAgilityNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.AGILITY);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Charisma attribute.
     * @returns {string} The comment.
     */
    function buildCharismaComment() {
        var statValue = global.kobold.stats.get(StatsEnum.CHARISMA);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Pariah.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Unlikable.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Friendly.</div>`;
        }

        return `<div class="text nomargin">Inspirational.</div>`;
    }

    /**
     * Builds the header for the Charisma attribute.
     * @returns {String} The header.
     */
    function buildCharismaHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.CHARISMA);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Charisma attribute.
     * @returns {String} The number.
     */
    function buildCharismaNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.CHARISMA);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Endurance attribute.
     * @returns {string} The comment.
     */
    function buildEnduranceComment() {
        var statValue = global.kobold.stats.get(StatsEnum.ENDURANCE);
        if (statValue < 3) {
            return `<div class="text warningtext">Wimp.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text sneakytext">Soft-skinned.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text">Resillient.</div>`;
        }

        return `<div class="text">Tough.</div>`;
    }

    /**
     * Builds the header for the Endurance attribute.
     * @returns {String} The header.
     */
    function buildEnduranceHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.ENDURANCE);
        return `<div class="text">${statName}</div>`;
    }

    /**
     * Builds the number for the Endurance attribute.
     * @returns {String} The number.
     */
    function buildEnduranceNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.ENDURANCE);
        return `<div class="text">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Intelligence attribute.
     * @returns {string} The comment.
     */
    function buildIntelligenceComment() {
        var statValue = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Dumbass.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Dim-witted.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Smart.</div>`;
        }

        return `<div class="text nomargin">Genius.</div>`;
    }

    /**
     * Builds the header for the Intelligence attribute.
     * @returns {String} The header.
     */
    function buildIntelligenceHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.INTELLIGENCE);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Intelligence attribute.
     * @returns {String} The number.
     */
    function buildIntelligenceNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.INTELLIGENCE);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Luck attribute.
     * @returns {string} The comment.
     */
    function buildLuckComment() {
        var statValue = global.kobold.stats.get(StatsEnum.LUCK);
        if (statValue < 3) {
            return `<div class="text warningtext">Doomed.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text sneakytext">Ill-fated.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text">Lucky.</div>`;
        }

        return `<div class="text">Blessed.</div>`;
    }

    /**
     * Builds the header for the Luck attribute.
     * @returns {String} The header.
     */
    function buildLuckHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.LUCK);
        return `<div class="text">${statName}</div>`;
    }

    /**
     * Builds the number for the Luck attribute.
     * @returns {String} The number.
     */
    function buildLuckNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.LUCK);
        return `<div class="text">${statValue}</div>`;
    }

    /**
     * Builds the comment for the Strength attribute.
     * @returns {string} The comment.
     */
    function buildStrengthComment() {
        var statValue = global.kobold.stats.get(StatsEnum.STRENGTH);
        if (statValue < 3) {
            return `<div class="text nomargin warningtext">Noodle arms.</div>`;
        }
        if (statValue < 5) {
            return `<div class="text nomargin sneakytext">Weak.</div>`;
        }
        if (statValue < 6) {
            return `<div class="text nomargin sneakytext">Average.</div>`;
        }
        if (statValue < 8) {
            return `<div class="text nomargin">Strong.</div>`;
        }

        return `<div class="text nomargin">Koboldcules.</div>`;
    }

    /**
     * Builds the header for the Strength attribute.
     * @returns {String} The header.
     */
    function buildStrengthHeader() {
        var statName = global.kobold.stats.getName(StatsEnum.STRENGTH);
        return `<div class="text nomargin">${statName}</div>`;
    }

    /**
     * Builds the number for the Strength attribute.
     * @returns {String} The number.
     */
    function buildStrengthNumber() {
        var statValue = global.kobold.stats.get(StatsEnum.STRENGTH);
        return `<div class="text nomargin">${statValue}</div>`;
    }

    /**
     * Builds the headers for the left hand side of the attributes section.
     * @returns {string} The headers for the left hand side of the attributes section.
     */
    function buildLeftStatHeaders() {
        var headers = `<div class="statname">`;
        headers += buildAgilityHeader();
        headers += buildStrengthHeader();
        headers += buildEnduranceHeader();
        return headers + `</div>`;
    }

    /**
     * Builds the numbers for the left hand side of the attributes section.
     * @returns {string} The numbers for the left hand side of the attributes section.
     */
    function buildLeftStatNumbers() {
        var numbers = `<div class="statnumber">`;
        numbers += buildAgilityNumber();
        numbers += buildStrengthNumber();
        numbers += buildEnduranceNumber();
        return numbers + `</div>`;
    }

    /**
     * Builds the comments for the left hand side of the attributes section.
     * @returns {string} The comments for the left hand side of the attributes section.
     */
    function buildLeftStatComments() {
        var comments = `<div class="statcomment">`;
        comments += buildAgilityComment();
        comments += buildStrengthComment();
        comments += buildEnduranceComment();
        return comments + `</div>`;
    }

    /**
     * Builds the headers for the right hand side of the attributes section.
     * @returns {string} The headers for the right hand side of the attributes section.
     */
    function buildRightStatHeaders() {
        var headers = `<div class="statname">`;
        headers += buildIntelligenceHeader();
        headers += buildCharismaHeader();
        headers += buildLuckHeader();
        return headers + `</div>`;
    }

    /**
     * Builds the numbers for the right hand side of the attributes section.
     * @returns {string} The numbers for the right hand side of the attributes section.
     */
    function buildRightStatNumbers() {
        var numbers = `<div class="statnumber">`;
        numbers += buildIntelligenceNumber();
        numbers += buildCharismaNumber();
        numbers += buildLuckNumber();
        return numbers + `</div>`;
    }

    /**
     * Builds the comments for the right hand side of the attributes section.
     * @returns {string} The comments for the right hand side of the attributes section.
     */
    function buildRightStatComments() {
        var comments = `<div class="statcomment">`;
        comments += buildIntelligenceComment();
        comments += buildCharismaComment();
        comments += buildLuckComment();
        return comments + `</div>`;
    }

    /**
     * Builds and returns the stats half of the stats tab.
     * @returns {string} The stats half of the stats tab as a HTML string.
     */
    function buildStats() {
        var subtitle = `<h2 class="subtitle">Attributes</h2>`;
        var leftHalf = `<div class="half">`;
        leftHalf += buildLeftStatHeaders();
        leftHalf += buildLeftStatNumbers();
        leftHalf += buildLeftStatComments();
        leftHalf += `</div>`;
        var rightHalf = `<div class="half">`;
        rightHalf += buildRightStatHeaders();
        rightHalf += buildRightStatNumbers();
        rightHalf += buildRightStatComments();
        rightHalf += `</div>`;
        return subtitle + leftHalf + rightHalf;
    }

    /**
     * Builds the gender section of the body part of the stats tab.
     * @returns {string} The gender section of the body part of the stats tab.
     */
    function buildGender() {
        var gender = global.kobold.body.gender;
        var toReturn = `<div class="text nomargin">`;

        if (gender == GenderEnum.NONE) {
            toReturn += "You are amorphic.";
        }

        if (gender == GenderEnum.MALE) {
            toReturn += "You are a boy.";
        }

        if (gender == GenderEnum.FEMALE) {
            toReturn += "You are a girl.";
        }

        if (gender == GenderEnum.SHEMALE) {
            toReturn += "You are a girl with boy parts.";
        }

        if (gender == GenderEnum.HERMAPHRODITE) {
            toReturn += "You are both a boy and a girl.";
        }

        if (gender == GenderEnum.CUNTBOY) {
            toReturn += "You are a boy with girl parts.";
        }

        if (gender == GenderEnum.MALEHERM) {
            toReturn += "You are a boy with a cunt, but not a cuntboy.";
        }

        toReturn += ` Your name is <span class="bold">${global.kobold.name}</span>.`;

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the breasts section of the body part of the stats tab.
     * @returns {string} The breasts section of the body part of the stats tab.
     */
    function buildBreasts() {
        var bodyPart = global.kobold.body.hasBreasts();
        var lactation = global.kobold.body.lactating;
        var toReturn = `<div class="text nomargin">`;

        if (!bodyPart) {
            toReturn += `Your chest is smooth and flat.`;
        }

        if (bodyPart == SizeEnum.SMALLEST) {
            toReturn += `Your breasts are barely large enough to notice.`;
            if (lactation) {
                toReturn += " You produce only the tiniest bit of milk.";
            }
        }

        if (bodyPart == SizeEnum.SMALL) {
            toReturn += `Two small tits adorn your chest.`;
            if (lactation) {
                toReturn += " Your nipples occasionally dampen from the milk produced within.";
            }
        }

        if (bodyPart == SizeEnum.AVERAGE) {
            toReturn += `You have a pair of average-sized breasts. They look nice on you.`;
            if (lactation) {
                toReturn += " Small droplets of milk sometimes form on your nipples, rolling down your scaly body or dampening your clothes.";
            }
        }

        if (bodyPart == SizeEnum.LARGE) {
            toReturn += `Your two sizable boobs attract plenty of attention.`;
            if (lactation) {
                toReturn += " Your breasts squirt milk when squeezed.";
            }
        }

        if (bodyPart == SizeEnum.LARGER) {
            toReturn += `Your pair of oversized tits are weighing you down.`;
            if (lactation) {
                toReturn += " A slow, yet constant stream of milk leaks from your chest.";
            }
        }

        if (bodyPart == SizeEnum.HUGE) {
            toReturn += `<span class="warningtext">With <span class="bold">udders</span> this big, it's a miracle you can still move around.</span>`;
            if (lactation) {
                toReturn += " Your huge tits need to be milked hourly to stop them from constantly muddying the ground.";
            }
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the dick section of the body part of the stats tab.
     * @returns {string} The dick section of the body part of the stats tab.
     */
    function buildDick() {
        var bodyPart = global.kobold.body.hasDick();
        var toReturn = `<div class="text nomargin">`;

        if (!bodyPart) {
            return "";
        }

        if (bodyPart == SizeEnum.SMALLEST) {
            toReturn += `A tiny member resides inside your cockslit, about large enough to pleasure a virginal feral bunny.`;
        }

        if (bodyPart == SizeEnum.SMALL) {
            toReturn += `The small member resting in your cockslit is big enough to satisfy most kobolds.`;
        }

        if (bodyPart == SizeEnum.AVERAGE) {
            toReturn += `Your dick, safely concealed within your cockslit, is big enough to bring humans to orgasm.`;
        }

        if (bodyPart == SizeEnum.LARGE) {
            toReturn += `Canines and felines are typically more gifted than humans, when it comes to dicks. You measure up to them quite nicely. You could give a feral horse a run for its money.`;
        }

        if (bodyPart == SizeEnum.LARGER) {
            toReturn += `The slit between your legs is growing too small for the huge member within. Your length rivals even the most gifted of anthros. You could please <span class="bold">small dragons</span> with it.`;
        }

        if (bodyPart == SizeEnum.HUGE) {
            toReturn += `<span class="warningtext">Your oversized schlong can <span class="bold">barely</span> even fit inside of <span class="bold">fully grown dragons</span>.</span>`;
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the cunt section of the body part of the stats tab.
     * @returns {string} The cunt section of the body part of the stats tab.
     */
    function buildCunt() {
        var cunt = global.kobold.body.hasCunt();
        var dick = global.kobold.body.hasDick();
        var virgin = global.kobold.body.vaginalVirgin;
        var toReturn = `<div class="text nomargin">`;

        if (!cunt && !dick) {
            toReturn += "You have not a single slit between your legs.";
        }

        if (!cunt && dick) {
            toReturn += "Your cockslit is the only slit you have.";
        }

        if (cunt && !dick && !virgin) {
            toReturn += "While no longer a virgin, your cunt continues to allure a multitude of molesters.";
        }

        if (cunt && !dick && virgin) {
            toReturn += "The virginal cunt between your legs tempts your every assailant.";
        }

        if (cunt && dick && !virgin) {
            toReturn += "Behind your cockslit rests a second, slightly more puffy mound.";
        }

        if (cunt && dick && virgin) {
            toReturn += "Past your cockslit, your virginal twat lays ripe for the picking.";
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds the ass section of the body part of the stats tab.
     * @returns {string} The ass section of the body part of the stats tab.
     */
    function buildAss() {
        var virgin = global.kobold.body.analVirgin;
        var toReturn = `<div class="text">`;

        if (!virgin) {
            toReturn += "Your tailhole is positioned in between your buttcheeks. Where else would it be?";
        }

        if (virgin) {
            toReturn += "Your tail covers your virginal butthole.";
        }

        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the Body part of the stats tab.
     * @returns {string} The Body part of the stats tab as a HTML string.
     */
    function buildBody() {
        var body = `<h2 class="subtitle">Body</h2>`;
        body += buildGender();
        body += buildBreasts();
        body += buildDick();
        body += buildCunt();
        body += buildAss();
        return body;
    }

    /**
     * Builds and returns the oral virgin status part of the stats tab.
     * @returns {string} The oral virgin status part of the stats tab.
     */
    function buildOralVirginStatus() {
        if (global.kobold.body.oralVirgin) {
            return `<div class="text nomargin">You haven't been made to suck anyone off or eat anyone out yet.</div>`;
        }
        return `<div class="text nomargin">Your muzzle has been used. You can still taste whatever you were made to service.</div>`;
    }

    /**
     * Builds and returns the anal virgin status part of the stats tab.
     * @returns {string} The anal virgin status part of the stats tab.
     */
    function buildAnalVirginStatus() {
        if (global.kobold.body.analVirgin) {
            return `<div class="text nomargin">Your ass has never been fucked. You intend to keep it that way.</div>`;
        }
        return `<div class="text nomargin">Your tailhole remains a little sore from the intense pounding it received.</div>`;
    }

    /**
     * Builds and returns the cockslit virgin status part of the stats tab.
     * @returns {string} The cockslit virgin status part of the stats tab.
     */
    function buildCockslitVirginStatus() {
        if (!global.kobold.body.hasDick()) {
            return ``;
        }
        if (global.kobold.body.cockslitVirgin) {
            return `<div class="text nomargin">Your cockslit has never been fucked before. Thank goodness!</div>`;
        }
        return `<div class="text nomargin">The tight, warm confines of your cockslit have been defiled.</div>`;
    }

    /**
     * Builds and returns the cock virgin status part of the stats tab.
     * @returns {string} The cock virgin status part of the stats tab.
     */
    function buildCockVirginStatus() {
        if (!global.kobold.body.hasDick()) {
            return ``;
        }
        if (global.kobold.body.dickVirgin) {
            return `<div class="text nomargin">You've never fucked anyone with your member.</div>`;
        }
        return `<div class="text nomargin">You finally got laid!</div>`;
    }

    /**
     * Builds and returns the vaginal virgin status part of the stats tab.
     * @returns {string} The vaginal virgin status part of the stats tab.
     */
    function buildVaginalVirginStatus() {
        if (!global.kobold.body.hasCunt()) {
            return ``;
        }
        if (global.kobold.body.vaginalVirgin) {
            return `<div class="text nomargin">Your puffy mound remains untouched.</div>`;
        }
        return `<div class="text nomargin">You have been robbed of your maidenhood.</div>`;
    }

    /**
     * Builds and returns a summary of virgin status.
     * @returns {string} A summary of virgin status.
     */
    function buildVirginSummary() {
        if (global.kobold.body.virgin()) {
            return `<div class="text nomargin">You remain a pure virgin.</div>`;
        }
        return `<div class="text nomargin">You are no longer a virgin.</div>`;
    }

    /**
     * Builds and returns the virgin status part of the stats tab.
     * @returns {string} The virgin status part of the stats tab as a HTML string.
     */
    function buildVirginStatus() {
        var virginStatus = `<h2 class="subtitle">Virginity</h2>`;
        virginStatus += buildOralVirginStatus();
        virginStatus += buildAnalVirginStatus();
        virginStatus += buildCockVirginStatus();
        virginStatus += buildCockslitVirginStatus();
        virginStatus += buildVaginalVirginStatus();
        virginStatus += buildVirginSummary();
        return virginStatus;
    }

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = gen.buildTabHeader(title);
        tab += buildStats();
        tab += buildBody();
        tab += buildVirginStatus();
        tab += gen.buildTabFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
    };
}

// END res\js\ui\tabs\StatsTab.js

// res\js\ui\tabs\StatusTab.js

/**
 * This class contains functions specific to the Status tab.
 * Its init() function should be called immediately upon successful project load.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function StatusTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "statustab";
    var title = "Status";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds and returns a HTML-subtitle.
     * @param {string} subtitle The actual subtitle.
     * @returns {string} The subtitle as a HTML-subtitle.
     */
    function buildSubtitle(subtitle) {
        return `<h2 class="subtitle">${subtitle}</h2>`;
    }

    /**
     * Builds and returns the healthbar of the Physical Health section of the status tab.
     * @returns {string} The healthbar of the Physical Health section of the status tab.
     */
    function buildPhysicalHealthBar() {
        var percentage = (global.kobold.status.physical.health / global.kobold.status.physical.max) * 100;
        var bar = `<div class="healthbar"><div class="healthbarinner health" style="width: ${percentage}%;"></div></div>`;
        return bar;
    }

    /**
     * Builds and returns the comment of the Physical Health section of the status tab.
     * @returns {string} The comment of the Physical Health section of the status tab.
     */
    function buildPhysicalHealthComment() {
        var percentage = (global.kobold.status.physical.health / global.kobold.status.physical.max) * 100;
        var comment;
        if (percentage <= 5) {
            comment = `<span class="warningtext">On the brink of death.</span>`;
        }
        if (5 < percentage && percentage <= 20) {
            comment = `<span class="warningtext">In need of medical attention.</span>`;
        }
        if (20 < percentage && percentage <= 50) {
            comment = `Seriously hurt.`;
        }
        if (50 < percentage && percentage <= 70) {
            comment = `In pain.`;
        }
        if (70 < percentage && percentage <= 90) {
            comment = `<span class="sneakytext">Cuts and bruises.</span>`;
        }
        if (90 < percentage) {
            comment = `<span class="sneakytext">Feeling fine.</span>`;
        }

        return `<div class="text healhbarcomment">${comment}</div>`;
    }

    /**
     * Builds and returns the sub-bar elements of the Physical Health section of the status tab.
     * @returns {string} The sub-bar elements of the Physical Health section of the status tab.
     */
    function buildPhysicalHealthSubBar() {
        var value = global.kobold.status.physical.health;
        var max = global.kobold.status.physical.max;
        var affected = global.kobold.status.physical.lasthit;
        var comment = buildPhysicalHealthComment();
        var numbers = `<div class="text healthbarnumber bold">${value} / ${max}</div>`;
        var lastHit = `<div class="text healhbarlasthit">Last affected by: ${affected}</div>`;
        return comment + numbers + lastHit;
    }

    /**
     * Builds and returns the Physical Health section of the status tab.
     * @returns {string} The Physical Health section of the status tab.
     */
    function buildPhysicalHealth() {
        return buildSubtitle("Health") + buildPhysicalHealthBar() + buildPhysicalHealthSubBar();
    }

    /**
     * Builds and returns the healthbar of the Mental Health section of the status tab.
     * @returns {string} The healthbar of the Mental Health section of the status tab.
     */
    function buildMentalHealthBar() {
        var percentage = (global.kobold.status.mental.health / global.kobold.status.mental.max) * 100;
        var bar = `<div class="healthbar"><div class="healthbarinner sanity" style="width: ${percentage}%;"></div></div>`;
        return bar;
    }

    /**
     * Builds and returns the comment of the Mental Health section of the status tab.
     * @returns {string} The comment of the Mental Health section of the status tab.
     */
    function buildMentalHealthComment() {
        var percentage = (global.kobold.status.mental.health / global.kobold.status.mental.max) * 100;
        var comment;
        if (percentage <= 5) {
            comment = `<span class="warningtext">Scarred for life.</span>`;
        }
        if (5 < percentage && percentage <= 20) {
            comment = `<span class="warningtext">Losing it.</span>`;
        }
        if (20 < percentage && percentage <= 50) {
            comment = `Distraught.`;
        }
        if (50 < percentage && percentage <= 70) {
            comment = `Confused.`;
        }
        if (70 < percentage && percentage <= 90) {
            comment = `<span class="sneakytext">Slightly irked.</span>`;
        }
        if (90 < percentage) {
            comment = `<span class="sneakytext">Clear of mind.</span>`;
        }

        return `<div class="text healhbarcomment">${comment}</div>`;
    }

    /**
     * Builds and returns the sub-bar elements of the Mental Health section of the status tab.
     * @returns {string} The sub-bar elements of the Mental Health section of the status tab.
     */
    function buildMentalHealthSubBar() {
        var value = global.kobold.status.mental.health;
        var max = global.kobold.status.mental.max;
        var affected = global.kobold.status.mental.lasthit;
        var comment = buildMentalHealthComment();
        var numbers = `<div class="text healthbarnumber bold">${value} / ${max}</div>`;
        var lastHit = `<div class="text healhbarlasthit">Last affected by: ${affected}</div>`;
        return comment + numbers + lastHit;
    }

    /**
     * Builds and returns the Mental Health section of the status tab.
     * @returns {string} The Mental Health section of the status tab.
     */
    function buildMentalHealth() {
        return buildSubtitle("Sanity") + buildMentalHealthBar() + buildMentalHealthSubBar();
    }

    /**
     * Builds and returns the healthbar of the Stamina section of the status tab.
     * @returns {string} The healthbar of the Stamina section of the status tab.
     */
    function buildStaminaBar() {
        var percentage = (global.kobold.status.stamina.health / global.kobold.status.stamina.max) * 100;
        var bar = `<div class="healthbar"><div class="healthbarinner stamina" style="width: ${percentage}%;"></div></div>`;
        return bar;
    }

    /**
     * Builds and returns the comment of the Stamina section of the status tab.
     * @returns {string} The comment of the Stamina section of the status tab.
     */
    function buildStaminaComment() {
        var percentage = (global.kobold.status.stamina.health / global.kobold.status.stamina.max) * 100;
        var comment;
        if (percentage <= 5) {
            comment = `<span class="warningtext">Ready to drop.</span>`;
        }
        if (5 < percentage && percentage <= 20) {
            comment = `<span class="warningtext">Exhausted.</span>`;
        }
        if (20 < percentage && percentage <= 50) {
            comment = `Out of breath.`;
        }
        if (50 < percentage && percentage <= 70) {
            comment = `Tired.`;
        }
        if (70 < percentage && percentage <= 90) {
            comment = `<span class="sneakytext">Active.</span>`;
        }
        if (90 < percentage) {
            comment = `<span class="sneakytext">Fresh.</span>`;
        }

        return `<div class="text healhbarcomment">${comment}</div>`;
    }

    /**
     * Builds and returns the sub-bar elements of the Stamina section of the status tab.
     * @returns {string} The sub-bar elements of the Stamina section of the status tab.
     */
    function buildStaminaSubBar() {
        var value = global.kobold.status.stamina.health;
        var max = global.kobold.status.stamina.max;
        var affected = global.kobold.status.stamina.lasthit;
        var comment = buildStaminaComment();
        var numbers = `<div class="text healthbarnumber bold">${value} / ${max}</div>`;
        var lastHit = `<div class="text healhbarlasthit">Last affected by: ${affected}</div>`;
        return comment + numbers + lastHit;
    }

    /**
     * Builds and returns the Stamina section of the status tab.
     * @returns {string} The Stamina section of the status tab.
     */
    function buildStamina() {
        return buildSubtitle("Stamina") + buildStaminaBar() + buildStaminaSubBar();
    }

    /**
     * Builds and returns the healthbar of the Sexual Exertion section of the status tab.
     * @returns {string} The healthbar of the Sexual Exertion section of the status tab.
     */
    function buildSexualExertionBar() {
        var percentage = (global.kobold.status.exertion.health / global.kobold.status.exertion.max) * 100;
        var bar = `<div class="healthbar"><div class="healthbarinner exertion" style="width: ${percentage}%;"></div></div>`;
        return bar;
    }

    /**
     * Builds and returns the comment of the Sexual Exertion section of the status tab.
     * @returns {string} The comment of the Sexual Exertion section of the status tab.
     */
    function buildSexualExertionComment() {
        var percentage = (global.kobold.status.exertion.health / global.kobold.status.exertion.max) * 100;
        var comment;

        if (percentage < 10) {
            comment = `<span class="sneakytext">Clean and tight.</span>`;
        }
        if (10 <= percentage && percentage < 30) {
            comment = `<span class="sneakytext">Gently fucked.</span>`;
        }
        if (30 <= percentage && percentage < 50) {
            comment = `Spent.`;
        }
        if (50 <= percentage && percentage < 80) {
            comment = `Used and abused.`;
        }
        if (80 <= percentage && percentage < 95) {
            comment = `<span class="warningtext">Thoroughly used fucktoy.</span>`;
        }
        if (95 <= percentage) {
            comment = `<span class="warningtext">Worn out cumdump.</span>`;
        }

        return `<div class="text healhbarcomment">${comment}</div>`;
    }

    /**
     * Builds and returns the sub-bar elements of the Sexual Exertion section of the status tab.
     * @returns {string} The sub-bar elements of the Sexual Exertion section of the status tab.
     */
    function buildSexualExertionSubBar() {
        var value = global.kobold.status.exertion.health;
        var max = global.kobold.status.exertion.max;
        var affected = global.kobold.status.exertion.lasthit;
        var comment = buildSexualExertionComment();
        var numbers = `<div class="text healthbarnumber bold">${value} / ${max}</div>`;
        var lastHit = `<div class="text healhbarlasthit">Last affected by: ${affected}</div>`;
        return comment + numbers + lastHit;
    }

    /**
     * Builds and returns the Sexual Exertion section of the status tab.
     * @returns {string} The Sexual Exertion section of the status tab.
     */
    function buildSexualExertion() {
        return buildSubtitle("Sexual exertion") + buildSexualExertionBar() + buildSexualExertionSubBar();
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = gen.buildTabHeader(title);
        tab += buildPhysicalHealth();
        tab += buildMentalHealth();
        tab += buildStamina();
        tab += buildSexualExertion();
        tab += gen.buildTabFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        var tab = buildTab();
        global.ui.loadToMain(tab);
    };
}

// END res\js\ui\tabs\StatusTab.js

// res\js\ui\themegen\Theme.js

/**
 * Class that represents a single theme.
 * By default, all values will be akin to the main theme of Kobold Adventure.
 * @param {object} colors An object containing the colors to be used in this theme.
 *                        Every color must be either an {r, g, b} object, a ThemeColor object or a hexadecimal string.
 *                        Every color is optional. If a color is not specified, the default from the main theme will be used.
 *                        Available options are:
 *                          -background
 *                          -main
 *                          -detail
 *                          -text
 *                          -rare
 *                          -danger
 *                          -sneaky
 *                          -blood
 *                          -sanity
 *                          -energy
 *                          -exertion
 * @param {string} additionalRules Any additional custom CSS rules to add to this theme in the form of a string.
 */
function Theme(colors, additionalRules) {
    colors = colors || {};
    for (var option in colors) {
        if (!colors[option]) {
            continue;
        } else if (colors[option].constructor === ThemeColor) {
            continue;
        } else if (typeof colors[option] === "string") {
            colors[option] = new ThemeColor(colors[option]);
        } else {
            colors[option] = new ThemeColor(colors[option]);
        }
    }

    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.background = colors.background || new ThemeColor("#191919");
    this.main = colors.main || new ThemeColor("#005AA1");
    this.detail = colors.detail || new ThemeColor("#003C6C");
    this.text = colors.text || new ThemeColor("#FFFFFF");
    this.rare = colors.rare || new ThemeColor("#FF6EC7");
    this.danger = colors.danger || new ThemeColor("#BE2625");
    this.sneaky = colors.sneaky || new ThemeColor("#C0C0C0");
    this.blood = colors.blood || new ThemeColor("#AF111C");
    this.sanity = colors.sanity || new ThemeColor("#4B0082");
    this.energy = colors.energy || new ThemeColor("#FFCC11");
    this.exertion = colors.exertion || new ThemeColor("#FF6EC7");

    this.extraRules = additionalRules || "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {object} theme Type-stripped Theme object.
     */
    this.load = function (theme) {
        var defaults = new Theme();
        if (!theme) {
            that.load(defaults);
            return;
        }
        that.background = theme.background ? new ThemeColor(theme.background) : defaults.background;
        that.main = theme.main ? new ThemeColor(theme.main) : defaults.main;
        that.detail = theme.detail ? new ThemeColor(theme.detail) : defaults.detail;
        that.text = theme.text ? new ThemeColor(theme.text) : defaults.text;
        that.rare = theme.rare ? new ThemeColor(theme.rare) : defaults.rare;
        that.danger = theme.danger ? new ThemeColor(theme.danger) : defaults.danger;
        that.sneaky = theme.sneaky ? new ThemeColor(theme.sneaky) : defaults.sneaky;
        that.blood = theme.blood ? new ThemeColor(theme.blood) : defaults.blood;
        that.sanity = theme.sanity ? new ThemeColor(theme.sanity) : defaults.sanity;
        that.energy = theme.energy ? new ThemeColor(theme.energy) : defaults.energy;
        that.exertion = theme.exertion ? new ThemeColor(theme.exertion) : defaults.exertion;
        that.extraRules = theme.extraRules ? theme.extraRules : defaults.extraRules;
    };
}

// END res\js\ui\themegen\Theme.js

// res\js\ui\themegen\ThemeColor.js

/**
 * A single color to be used in themes.
 * @param {string} color The color either in RGB or in HEX. Hex should be a string (with or without # sign). RGB should be an {r, g, b} object or a 3-value array.
 */
function ThemeColor(color) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var minColor = 0;
    var maxColor = 255;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Forces all colors to remain within color-bounds.
     */
    function boundsForce() {
        that.r = Math.max(Math.min(that.r, maxColor), minColor);
        that.g = Math.max(Math.min(that.g, maxColor), minColor);
        that.b = Math.max(Math.min(that.b, maxColor), minColor);
    }

    /**
     * Converts the passed hexadecimal string to a color within bounds.
     * If the color is just a single digit, adds a second digit that is the same as the first digit.
     * @param {string} hex A hexidecimal string representing a color.
     * @returns {integer} The color, within (min, max) bounds.
     */
    function hexToCappedColor(hex) {
        var chars = hex.length;
        if (chars === 1) {
            return hexToCappedColor(hex + hex);
        }
        var max = Math.pow(16, chars);
        var ratio = (maxColor + 1) / max;
        var number = parseInt(hex, 16);
        if (isNaN(number)) {
            return 0;
        }
        return Math.floor(number * ratio);
    }

    /**
     * Converts the passed hexadecimal strings to an {r, g, b} object within color bounds.
     * @param {string} hexR Red hexadecimal string.
     * @param {string} hexG Green hexadecimal string.
     * @param {string} hexB Blue hexadecimal string.
     * @returns {object} {r, g, b} object.
     */
    function hexToRgb(hexR, hexG, hexB) {
        var output = {};
        output.r = hexToCappedColor(hexR);
        output.g = hexToCappedColor(hexG);
        output.b = hexToCappedColor(hexB);
        return output;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.r = 0;
    this.g = 0;
    this.b = 0;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Returns a textual hexadecimal representation of this color, starting with #.
     * @returns {string} A textual hexadecimal representation of this color, starting with #.
     */
    this.toHex = function () {
        var red = that.r.toString(16);
        var green = that.g.toString(16);
        var blue = that.b.toString(16);
        red = red.length > 1 ? red : "0" + red;
        green = green.length > 1 ? green : "0" + green;
        blue = blue.length > 1 ? blue : "0" + blue;
        return `#${red}${green}${blue}`;
    };

    /**
     * Returns a textual rgb representation of this color, starting with rgb( and ending with ).
     * @returns {string} A textual rgb representation of this color, starting with rgb( and ending with ).
     */
    this.toRgb = function () {
        return `rgb(${that.r}, ${that.g}, ${that.b})`;
    };

    /**
     * Returns a textual rgba representation of this color, starting with rgba( and ending with ).
     * @param {double} a The alpha value, between 0 and 1. 
     * @returns {string} A textual rgba representation of this color, starting with rgba( and ending with ).
     */
    this.toRgba = function (a) {
        return `rgba(${that.r}, ${that.g}, ${that.b}, ${a})`;
    };

    /**
     * Sets this ThemeColor's colors to the values of a hexadecimal string.
     * @param {string} hex A string (with or without # sign)
     */
    this.fromHex = function (hex) {
        var hexValue = hex;
        if (hexValue.startsWith("#")) {
            hexValue = hexValue.substr(1);
        }
        var charsPerColor = Math.floor(hexValue.length / 3);
        if (!charsPerColor) {
            that.reset();
            return;
        }
        var red = hexValue.substr(0, charsPerColor);
        var green = hexValue.substr(charsPerColor, charsPerColor);
        var blue = hexValue.substr(charsPerColor * 2, charsPerColor);
        that.fromRgb(hexToRgb(red, green, blue));
    };

    /**
     * Sets this ThemeColor's colors to the values of an {r, g, b} object.
     * @param {object} rgb {r, g, b} object containing color values (max 255).
     */
    this.fromRgb = function (rgb) {
        that.r = rgb.r || 0;
        that.g = rgb.g || 0;
        that.b = rgb.b || 0;
        boundsForce();
    };

    /**
     * Resets all colors to zero.
     */
    this.reset = function () {
        that.r = 0;
        that.g = 0;
        that.b = 0;
    };

    /**
     * Dampens this color by the specified factor between zero and one.
     * @param {number} factor The factor by which to dampen the color. Should be between zero and one. Above one, will amplify the color instead.
     */
    this.dampen = function (factor) {
        that.fromRgb({r: Math.round(that.r * factor), g: Math.round(that.g * factor), b: Math.round(that.b * factor)});
    };

    // Init
    if (color) {
        if (typeof color === "string") {
            that.fromHex(color);
        } else {
            if (Array.isArray(color)) {
                color = {r: color[0], g: color[1], b: color[2]};
            }
            that.fromRgb(color);
        }
    }
}

// END res\js\ui\themegen\ThemeColor.js

// res\js\ui\themegen\ThemeGenerator.js

/**
 * Class capable of generating themes and adding those themes that have not yet been added to the game as CSS code.
 */
function ThemeGenerator() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Generates the theme as a CSS script and returns that script.
     * @param {string} name The name of the theme.
     * @param {Theme} theme The theme for which to generate a css script.
     * @returns {string} The generated css script.
     */
    this.generate = function (name, theme) {
        var output = `
            /**
             * COLOR SCHEME
             * Background:   ${theme.background.toHex()}    ${theme.background.toRgb()}
             * Main:         ${theme.main.toHex()}    ${theme.main.toRgb()}
             * Detail:       ${theme.detail.toHex()}    ${theme.detail.toRgb()}
             * Text:         ${theme.text.toHex()}    ${theme.text.toRgb()}
             * Rare:         ${theme.rare.toHex()}    ${theme.rare.toRgb()}
             * Danger:       ${theme.danger.toHex()}    ${theme.danger.toRgb()}
             * Sneaky:       ${theme.sneaky.toHex()}    ${theme.sneaky.toRgb()}
             * Blood:        ${theme.blood.toHex()}    ${theme.blood.toRgb()}
             * Sanity:       ${theme.sanity.toHex()}    ${theme.sanity.toRgb()}
             * Energy:       ${theme.energy.toHex()}    ${theme.energy.toRgb()}
             * Exertion:     ${theme.exertion.toHex()}    ${theme.exertion.toRgb()}
            **/

            .${name}, 
            .${name} .uibutton:active, 
            .${name} .saveslot, 
            .${name} .smallsaveslot, 
            .${name} .button:active,
            .${name} .button:disabled,
            .${name} .bigbutton,
            .${name} .textinput:disabled,
            .${name} .menuclosebutton,
            .${name} .menubutton,
            .${name} .loadingscreenwrapper{
                background-color: ${theme.background.toRgb()};  /* Background */
            }

            .${name} .popupcenter,
            .${name} .uibutton:hover,
            .${name} .button:hover,
            .${name} .tooltip,
            .${name} .healthbar,
            .${name} .textinput{
                border-color: ${theme.background.toRgb()};      /* Background */
            }

            .${name} .fullscreenpopup{
                background-color: ${theme.background.toRgba(0.7)};        /* Background */
            }

            .${name} .popupcenter,
            .${name} .popupcontrolbar,
            .${name} .tab,
            .${name} .uibutton,
            .${name} .main,
            .${name} .savecontrolbar,
            .${name} .tooltipheader,
            .${name} .tooltipbottom,
            .${name} .itemrowodd{
                background-color: ${theme.main.toRgb()};  /* Main */
            }

            .${name} .button,
            .${name} .button:disabled{
                border-color: ${theme.main.toRgb()};     /* Main */
            }

            .${name} .koboldadventure,
            .${name} .tabactive,
            .${name} .button,
            .${name} .smallseparator,
            .${name} .tooltip,
            .${name} .itemroweven,
            .${name} .textinput,
            .${name} .menubuttonselected{
                background-color: ${theme.detail.toRgb()};  /* Detail */
            }

            /* Separated to make things not-break on firefox. */
            .${name} .main::-webkit-scrollbar-thumb,
            .${name} .popupcenter::-webkit-scrollbar-thumb{ 
                background-color: ${theme.detail.toRgb()};  /* Detail */
            }

            .${name} .saveslot:hover, 
            .${name} .smallsaveslot:hover{
                border-color: ${theme.detail.toRgb()};    /* Detail */
            }

            .${name} a:link, 
            .${name} a:visited,
            .${name} .popupcontent,
            .${name} .koboldadventure,
            .${name} .tab,
            .${name} .tab:hover,
            .${name} .uibutton,
            .${name} .main,
            .${name} .savecomment,
            .${name} .button,
            .${name} .textinput,
            .${name} .textinput:disabled,
            .${name} .loadbar,
            .${name} .popupmenu{
                color: ${theme.text.toRgb()};               /* Text */
            }

            .${name} .tooltip{
                color: ${theme.text.toRgb()} !important;    /* Text */
            }

            .${name} .inlineseparator{
                background-color: ${theme.text.toRgb()};    /* Text */
            }

            .${name} .saveslotselected{
                border-color: ${theme.text.toRgb()} !important;     /* Text */
            }

            .${name} .selected{
                border-color: ${theme.rare.toRgb()} !important;      /* Rare */
            }

            .${name} .warning{
                background-color: ${theme.danger.toRgb()};  /* Danger */
            }

            .${name} .warningtext{
                color: ${theme.danger.toRgb()};    /* Danger */
            }

            .${name} .savelimit,
            .${name} .sneakytext{
                color: ${theme.sneaky.toRgb()};             /* Sneaky */
            }

            .${name} .health {
                background-color: ${theme.blood.toRgb()};      /* Blood */
            }

            .${name} .sanity {
                background-color: ${theme.sanity.toRgb()};      /* Sanity */
            }

            .${name} .stamina {
                background-color: ${theme.energy.toRgb()};      /* Energy */
            }

            .${name} .exertion {
                background-color: ${theme.exertion.toRgb()};      /* Exertion */
            }`;
        
        if(theme.extraRules) {
            output += theme.extraRules;
        }
        return output;
    };
}

// END res\js\ui\themegen\ThemeGenerator.js

// res\js\ui\themegen\ThemeLoader.js

/**
 * Backwards-compatible theme loader.
 * @param {boolean} careAboutSettings If true, this ThemeLoader will care about settings in use.
 */
function ThemeLoader(careAboutSettings) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // All known themes.
    var themes = new Themes(careAboutSettings);
    // Generator to turn Theme objects into CSS.
    var generator = new ThemeGenerator();
    // What the default theme is called.
    var defaultThemeName = "defaulttheme";

    // Stores loaded themes by name.
    var themesByName = {};

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Given the name of a theme, returns the id for the css tag containing that theme.
     * @param {string} name The name of the scene.
     * @returns {string} The id of the css tag. 
     */
    function nameToId(name) {
        return `ThemeLoader-${name}-Style`;
    }

    /**
     * Adds the passed CSS to the DOM.
     * @param {string} name The name of the scene.
     * @param {string} css The CSS to add.
     */
    function addCss(name, css) {
        var sheet = document.createElement('style');
        sheet.innerHTML = css;
        sheet.id = nameToId(name);
        document.head.appendChild(sheet);
    }

    /**
     * Updates the custom theme.
     */
    function updateCustomTheme() {
        that.unload(defaultThemeName);
        themesByName[defaultThemeName] = true;
        loadCustomTheme();
    }

    /**
     * Registers custom theme settings listener.
     */
    function registerListeners() {
        global.settings.customThemeListener.register(updateCustomTheme);
    }

    /**
     * Loads the custom theme from settings.
     */
    function loadCustomTheme() {
        var defaultTheme = new Theme();
        defaultTheme.load(global.settings.getCustomTheme());
        var css = generator.generate(defaultThemeName, defaultTheme);
        addCss(defaultThemeName, css);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Generates and adds the specified theme if it has not yet been added.
     * @param {string} theme The name of the theme to add.
     */
    this.load = function (theme) {
        if (!theme || themesByName[theme] || !themes[theme]) {
            return;
        }
        themesByName[theme] = true;
        var themeObject = themes[theme];
        var css = generator.generate(theme, themeObject);
        addCss(theme, css);
    };

    /**
     * Unloads the specified theme. Should probably not be used. Ever.
     * @param {string} theme The name of the theme to remove.
     */
    this.unload = function (theme) {
        delete themesByName[theme];
        $("#" + nameToId(theme)).remove();
    };

    /**
     * Initialization function. Only needed if the themeloader should care about settings.
     */
    this.init = function () {
        themes.init();
        themesByName[defaultThemeName] = true; // Disable unnecessary empty theme loading.
        if (careAboutSettings && global.settings.getCustomThemeEnabled()) {
            loadCustomTheme();
            registerListeners();
        }
    };
}

// END res\js\ui\themegen\ThemeLoader.js

// res\js\ui\themegen\Themes.js

/**
 * Repository for all themes.
 * @param {boolean} careAboutSettings If true, the Themes will care about settings in use.
 */
function Themes(careAboutSettings) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Darkens all themes by dampening their background, main, detail and rare color values.
     */
    function darkenThemes() {
        var darkenFactor = global.settings.getDarkThemeModifier() / 100;
        for (var theme in that) {
            if (that[theme].constructor !== Theme) {
                continue;
            }
            that[theme].background.dampen(darkenFactor);
            that[theme].main.dampen(darkenFactor);
            that[theme].detail.dampen(darkenFactor);
            that[theme].rare.dampen(darkenFactor);
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // /forest
    this.forest = new Theme({
        background: "#3B5323",
        main: "#5E2612",
        detail: "#008B00",
        rare: "#FCD116"
    });

    this.deepwoods = new Theme({
        background: {r: 40, g: 61, b: 35},
        main: {r: 70, g: 47, b: 18},
        detail: {r: 49, g: 91, b: 55},
        rare: {r: 255, g: 193, b: 132}
    });

    this.forest_night = new Theme({
        background: {r: 31, g: 13, b: 61},
        main: {r: 28, g: 0, b: 89},
        detail: {r: 41, g: 0, b: 132},
        rare: {r: 127, g: 193, b: 132}
    });

    // /hamlet
    this.hamlet_barn = new Theme({
        background: {r: 51, g: 39, b: 25},
        main: {r: 63, g: 51, b: 45},
        detail: {r: 58, g: 69, b: 78},
        rare: {r: 178, g: 128, b: 59}
    });

    this.hamlet_cave = new Theme({
        background: {r: 10, g: 10, b: 14},
        main: {r: 6, g: 6, b: 12},
        detail: {r: 11, g: 13, b: 23},
        rare: {r: 12, g: 38, b: 50}
    });

    this.hamlet_day = new Theme({
        background: {r: 32, g: 40, b: 3},
        main: {r: 42, g: 37, b: 31},
        detail: {r: 76, g: 34, b: 20},
        rare: {r: 60, g: 65, b: 59}
    });

    this.hamlet_mansion = new Theme({
        background: {r: 61, g: 15, b: 20},
        main: {r: 74, g: 37, b: 0},
        detail: {r: 159, g: 126, b: 40},
        rare: {r: 61, g: 15, b: 61}
    });

    this.hamlet_shop = new Theme({
        background: {r: 20, g: 20, b: 20},
        main: {r: 21, g: 18, b: 15},
        detail: {r: 30, g: 18, b: 12},
        rare: {r: 82, g: 82, b: 82}
    });

    this.hamlet_well = new Theme({
        background: {r: 10, g: 24, b: 23},
        main: {r: 7, g: 13, b: 1},
        detail: {r: 30, g: 44, b: 44},
        rare: {r: 16, g: 82, b: 123}
    });

    // /home
    this.home = new Theme({
        background: {r: 40, g: 61, b: 35},
        main: {r: 70, g: 47, b: 18},
        detail: {r: 49, g: 91, b: 55},
        rare: {r: 255, g: 193, b: 132}
    });

    this.home_dawn = new Theme({
        background: {r: 87, g: 21, b: 53},
        main: {r: 134, g: 12, b: 42},
        detail: {r: 160, g: 48, b: 68},
        rare: {r: 165, g: 49, b: 76}
    });

    this.home_night = new Theme({
        background: {r: 31, g: 13, b: 61},
        main: {r: 28, g: 0, b: 89},
        detail: {r: 41, g: 0, b: 132},
        rare: {r: 127, g: 96, b: 193}
    });

    // /varanar
    this.varanar_alley = new Theme({
        background: {r: 19, g: 19, b: 19},
        main: {r: 23, g: 23, b: 26},
        detail: {r: 35, g: 35, b: 35},
        rare: {r: 69, g: 0, b: 0}
    });

    this.varanar_generic = new Theme({
        background: {r: 49, g: 67, b: 97},
        main: {r: 63, g: 63, b: 77},
        detail: {r: 110, g: 118, b: 133},
        rare: {r: 63, g: 87, b: 121}
    });

    this.varanar_night = new Theme({
        background: {r: 31, g: 24, b: 37},
        main: {r: 42, g: 31, b: 39},
        detail: {r: 51, g: 49, b: 43},
        rare: {r: 95, g: 87, b: 77}
    });

    this.varanar_morning = new Theme({
        background: {r: 105, g: 28, b: 0},
        main: {r: 68, g: 34, b: 34},
        detail: {r: 76, g: 68, b: 46},
        rare: {r: 95, g: 114, b: 68}
    });

    this.varanar_indoor_knockout = new Theme({
        background: {r: 11, g: 7, b: 7},
        main: {r: 21, g: 17, b: 15},
        detail: {r: 27, g: 31, b: 34},
        rare: {r: 63, g: 51, b: 45}
    });

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the themes. Does nothing if settings are not cared about.
     */
    this.init = function () {
        if (careAboutSettings && global.settings.getDarkThemeEnabled()) {
            darkenThemes();
        }
    };
}

// END res\js\ui\themegen\Themes.js

// res\js\ui\throbbers\AllThrobbers.js

/**
 * This class contains all the necessary functions to deal with the throbber-UI elements.
 * Throbbers are, in case you didn't know, loading icons.
 */
function AllThrobbers() {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = new GenericThrobber();

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.save = new SaveThrobber(gen);
    this.load = new LoadThrobber(gen);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END res\js\ui\throbbers\AllThrobbers.js

// res\js\ui\throbbers\GenericThrobber.js

/**
 * This class contains generic functions shared across all throbbers.
 */
function GenericThrobber() {
    // PRIVATE MEMBERS \\       var private = "test";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Uses jQuery to show the element with the passed id.
     * @param {string} id The id of the element to show.
     */
    this.show = function (id) {
        var element = $("#" + id);
        element.stop(true);
        element.show(400);
    };

    /**
     * Uses jQuery to hide the element with the passed id.
     * @param {string} id The id of the element to show.
     */
    this.hide = function (id) {
        var element = $("#" + id);
        element.stop(true);
        element.hide(400);
    };
}

// END res\js\ui\throbbers\GenericThrobber.js

// res\js\ui\throbbers\LoadThrobber.js

/**
 * This class contains functions specific to the Load throbber.
 * @param {GenericTab} generic An optional GenericThrobber to prevent having to create multiple copies.
 */
function LoadThrobber(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericThrobber();
    var id = "loading";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Shows this throbber.
     */
    this.show = function () {
        gen.show(id);
    };

    /**
     * Hides this throbber.
     */
    this.hide = function () {
        gen.hide(id);
    };
}

// END res\js\ui\throbbers\LoadThrobber.js

// res\js\ui\throbbers\SaveThrobber.js

/**
 * This class contains functions specific to the Save throbber.
 * @param {GenericTab} generic An optional GenericThrobber to prevent having to create multiple copies.
 */
function SaveThrobber(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var gen = generic ? generic : new GenericThrobber();
    var id = "saving";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Shows this throbber.
     */
    this.show = function () {
        gen.show(id);
    };

    /**
     * Shows and then hides this throbber.
     */
    this.showAndHide = function () {
        that.show();
        setTimeout(function () {
            that.hide();
        }, 2600);
    };

    /**
     * Hides this throbber.
     */
    this.hide = function () {
        gen.hide(id);
    };
}

// END res\js\ui\throbbers\SaveThrobber.js

// res\js\ui\UiController.js

/**
 * This class contains all the necessary functions to deal with the UI.
 * Its init() function should be called immediately upon successful project load.
 */
function UiController() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var mainId = "main";
    var popupId = "popup";
    var popupContentId = "popupcontent";
    var popupButtonId = "popupbutton";
    var currentTheme = "defaulttheme";
    var themeTimeout = null; // Current timeout-set function for clearing theme changes.

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the header of a tooltip.
     * @param {string} header The header's contents.
     * @returns {string} The header.
     */
    function createToolTipHeader(header) {
        if (header) {
            return `<div class="tooltipheader">${header}</div>`;
        }
        return "";
    }

    /**
     * Creates the body of a tooltip.
     * @param {string} body The body's contents.
     * @returns {string} The vody.
     */
    function createToolTipBody(body) {
        if (body) {
            return `<div class="tooltipbody">${body}</div>`;
        }
        return "";
    }

    /**
     * Creates the footer of a tooltip.
     * @param {string} footer The footer's contents.
     * @returns {string} The footer.
     */
    function createToolTipFooter(footer) {
        if (footer) {
            return `<div class="tooltipbottom">${footer}</div>`;
        }
        return "";
    }

    /**
     *  Creates and returns a tooltip.
     *  @param {string} body The body of the tooltip. Required.
     *  @param {string} header The header of the tooltip. Optional.
     *  @param {string} footer The element that when moused over should show the tooltip. Optional.
     *  @param {string} style Additional style to add to the tooltip. Optional.
     *  @return {string} The tooltip.
     */
    function createToolTip(body, header, footer, style) {
        var div = `<div class="tooltip" style="${style ? style : ""}">`;
        div += createToolTipHeader(header);
        div += createToolTipBody(body);
        div += createToolTipFooter(footer);
        div += `</div>`;
        return div;
    }

    /**
     * Registers all required listeners.
     */
    function registerListeners() {
        $("#" + popupButtonId).click(that.hideFullscreenPopup);
        global.settings.hideNsfwListener.register(hideNsfwIfRequired);
        global.settings.textSizeListener.register(setFontSize);
    }

    /**
     * Clears the temporary class used to facilitate theme changes.
     * Automatically cleared at the end of a theme change, or when cancelThemeChange is called.
     */
    function clearThemeChangeClass() {
        $("body, body *").removeClass("themetransition");
    }

    /**
     * Hides NSFW images if the settings demand it.
     */
    function hideNsfwIfRequired() {
        var cssId = "UiController-HideNsfw";
        $("#" + cssId).remove();
        if (global.settings.getHideNsfw()) {
            var sheet = document.createElement('style');
            sheet.innerHTML = `img.nsfw { display: none; }`;
            sheet.id = cssId;
            document.head.appendChild(sheet);
        }
    }

    /**
     * Sets the font size of body text.
     */
    function setFontSize() {
        var cssId = "UiController-FontSize";
        $("#" + cssId).remove();
        var size = global.settings.getTextSize();
        var sheet = document.createElement('style');
        sheet.innerHTML = `.text { font-size: ${size}px; }`;
        sheet.id = cssId;
        document.head.appendChild(sheet);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.genericHeaderId = "content";
    this.genericFooterId = "finisher";

    // Icon creator.
    this.icons = new Icons();

    // The collection of tabs.
    this.tabs = new AllTabs();

    // The scroll level of the current scene.
    this.sceneScroll = 0;

    // Load bar.
    this.loadbar = new LoadBar();

    // Popup menu.
    this.menu = new PopupMenu();

    // All throbbers (i.e. loading icons).
    this.throbbers = new AllThrobbers();

    // Change notification builder.
    this.change = new ChangeNotificationBuilder();

    // Image background enabler.
    this.imageBackground = new ImageBackground();

    // Image in text enabler.
    this.inlineImage = new ImageInText();

    // Generates themes dynamically to avoid having to include a billion CSS files.
    this.themeLoader = new ThemeLoader(true);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the UI controller.
     */
    this.init = function () {
        hideNsfwIfRequired();
        setFontSize();
        that.tabs.init();
        that.menu.init();
        that.themeLoader.init();
        registerListeners();
    };

    /**
     * Clears the main view, and loads the passed string into it as HTML.
     * @param {string} toLoad The HTML to load.
     */
    this.loadToMain = function (toLoad) {
        var main = $("#" + mainId);
        main.empty();
        main.html(toLoad);
    };

    /**
     * Appends the passed HTML string to the main view.
     * If the main content is wrapped in a container div, pass its id to this function.
     * @param {string} toAppend The HTML to append.
     * @param {string} id Optional: the id of the object to append the HTML to.
     */
    this.appendToMain = function (toAppend, id) {
        var element;
        if (id) {
            element = $("#" + id);
        } else {
            element = $("#" + mainId);
        }
        element.append(toAppend);
    };

    /**
     * Appends the passed HTML string to the main view, before a standard finisher.
     * Usage of this function is preferred over usage of appendToMain.
     * @param {string} toAppend The HTML to append.
     */
    this.appendBeforeFinisher = function (toAppend) {
        var element = $("#" + that.genericFooterId);
        element.before(toAppend);
    };

    /**
     * Builds a generic content container and returns it as a HTML string.
     * The id of this generic content container can be queried via the 
     * genericHeaderId member of this UiController. Recommended to be used as
     * the start of every piece of content.
     * @returns {string} A generic content container as a HTML string.
     */
    this.buildGenericHeader = function () {
        return '<div class="content" id="' + that.genericHeaderId + '">';
    };

    /**
     * Builds a footer to match the generic content container header and returns
     * it as a HTML string. Recommended to be used as the end of every piece of 
     * content.
     * @returns {string} A generic content container footer as a HTML string.
     */
    this.buildGenericFooter = function () {
        return '<div class="finish" id="' + that.genericFooterId + '"></div></div>';
    };

    /**
     * Turns the passed element into a tooltip container.
     * @param {string} mouseOver The element that when moused over should show the tooltip.
     * @param {string} body The body of the tooltip. Required.
     * @param {string} header The header of the tooltip. Optional.
     * @param {string} footer The footer of the tooltip. Optional.
     * @param {string} style Additional style to add to the tooltip. Optional.
     * @returns {string} The tooltipcontainer containing the tooltip.
     */
    this.makeToolTip = function (mouseOver, body, header, footer, style) {
        return `<div class="tooltipcontainer">${createToolTip(body, header, footer, style)}${mouseOver}</div>`;
    };

    /**
     * Creates a static tooltip (that will always be revealed) and returns it.
     * @param {string} body The body of the tooltip. Required.
     * @param {string} header The header of the tooltip. Optional.
     * @param {string} footer The element that when moused over should show the tooltip. Optional.
     * @param {string} style Additional style to add to the tooltip. Optional.
     * @returns {string} The tooltip.
     */
    this.makeStaticToolTip = function (body, header, footer, style) {
        return `${createToolTip(body, header, footer, style)}`;
    };

    /**
     * Fades out all content in the main container.
     * @param {function} fun The function to execute after fading out content.
     */
    this.fadeOutContent = function (fun) {
        //$("#" + mainId).children().fadeOut(400, fun); // Calls fun once for every element hidden.
        $("#" + mainId).children().fadeOut(400).promise().done(fun); // Calls fun once.
    };

    /**
     * Sets the scroll level of the main div to the current sceneScroll.
     */
    this.setScroll = function () {
        $("#" + mainId).scrollTop(that.sceneScroll);
    };

    /**
     * Scrolls the main div to the bottom.
     */
    this.scrollBottom = function () {
        that.sceneScroll = document.getElementById(mainId).scrollHeight;
        that.setScroll();
    };

    /**
     * Sets the scroll level of the main div to 0.
     */
    this.resetScroll = function () {
        that.sceneScroll = 0;
        that.setScroll();
    };

    /**
     * Fades in a fullscreen popup with a single button which will hide the popup
     * once more.
     * @param {string} content The content of the popup window.
     */
    this.showFullscreenPopup = function (content) {
        var popupContent = $("#" + popupContentId);
        popupContent.html(content);
        popupContent.append('<div class="popupfinish"></div>');
        popupContent.append('<div class="popupfinish"></div>');
        $("#" + popupId).fadeIn(400);
        setTimeout(function () { // Firefox fix
            $("#popupcenter").scrollTop(0);
        }, 5);
    };

    /**
     * Applies the passed theme. If no theme is passed, the default theme is applied.
     * Does nothing if the passed theme is already applied.
     * Invalidates previous theme changes.
     * @param {string} theme The name of the theme to apply.
     */
    this.applyTheme = function (theme) {
        theme = global.settings.getThemesDisabled() ? false : theme;
        theme = theme ? theme : "defaulttheme";
        if (currentTheme === theme) {
            return;
        }
        that.cancelThemeChangeTransition();
        currentTheme = theme;
        $("body, body *").addClass("themetransition");
        $(".throbber").removeClass("themetransition");
        themeTimeout = setTimeout(clearThemeChangeClass, 2000);
        $("body").attr('class', 'themetransition');
        $("body").addClass(theme);
        that.themeLoader.load(theme);
    };

    /**
     * Cancels theme change transition and clears up the temporary CSS-class used to facilitate it.
     * In essence: does immediately what applyTheme would have done in two seconds.
     */
    this.cancelThemeChangeTransition = function () {
        clearTimeout(themeTimeout);
        themeTimeout = null;
        clearThemeChangeClass();
    };

    /**
     * Fades out a fullscreen popup.
     */
    this.hideFullscreenPopup = function () {
        $("#" + popupId).fadeOut(400);
    };

    /**
     * Slowly fades in the fullscreen loading screen.
     * Should ideally never be used.
     */
    this.showLoadingScreen = function () {
        $("#loadingscreen").fadeIn(800);
    };

    /**
     * Slowly fades out the fullscreen loading screen.
     * Should ideally only be used during startup.
     */
    this.hideLoadingScreen = function () {
        $("#loadingscreen").fadeOut(800);
    };

    /**
     * Returns a BB-code enabled conditional fail if the settings approve of it.
     * Returns the empty string otherwise.
     * @param {string} fail The failure message.
     * @returns {string} The failure koBBold code or an empty string, depending on the settings.
     */
    this.condFail = function (fail) {
        if (global.settings.getConditionsInvisible()) {
            return "";
        }
        return `[fail]${fail}[/fail]`;
    };

    /**
     * Returns a BB-code enabled conditional success if the settings approve of it.
     * Returns the empty string otherwise.
     * @param {string} succ The success message.
     * @returns {string} The success koBBold code or an empty string, depending on the settings.
     */
    this.condSucc = function (succ) {
        if (global.settings.getConditionsInvisible()) {
            return "";
        }
        return `[succ]${succ}[/succ]`;
    };
}

// END res\js\ui\UiController.js

